// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License.
// See LICENSE.TXT for details.
//

package de.uka.ilkd.asmkey.logic;

import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.op.Equality;
import de.uka.ilkd.key.logic.op.Op;
import de.uka.ilkd.key.logic.sort.*;

/** Sorts for ASM rules. ASM rules are terms of sort {@link #ASMRULE}.
 *
 * @author Hubert Schmid
 */

public final class AsmSort implements Sort {

    /** Sort for ASM rules (terms). */
    public static final Sort ASMRULE = new AsmSort();

    /** field to cache name instance. */
    private final Name name = new Name("asm-rule");

    /** singleton pattern: instance is accessible via @{link ASMRULE}. */
    private AsmSort() { }

    public Name name() {
        return name;
    }

    public SetOfSort extendsSorts() {
        return SetAsListOfSort.EMPTY_SET;
    }

    public boolean extendsTrans(Sort rhs) {
        return SortUtil.extendsTrans(this, rhs);
    }

    public Equality getEqualitySymbol() {
        return Op.EQUALS;
    }

    public String toString() {
        return name.toString();
    }

}
