// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License.
// See LICENSE.TXT for details.
//

package de.uka.ilkd.asmkey.logic;

import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.op.ArrayOfQuantifiableVariable;

/** Term implementation for ASM rules and ASM dynamic logic
 * formulas/terms.
 *
 * @author Hubert Schmid
 */

final class AsmTerm extends Term {

    /** top level operator */
    private final AsmOperator op;

    /** subterms, may be null if this term has no subterms. */
    private final Term[] terms;

    /** variables bound by top level operator of this term. */
    private final ArrayOfQuantifiableVariable boundVariables;

    /** depth of term. (chached value) */
    private final int depth;


    /** create term with given top level operator and no subterms. */
    AsmTerm(AsmOperator op) {
        this(op, TermUtil.EMPTY_TERMS);
    }

    /** create term with given top level operator and given subterms
     * and no bound variables. */
    AsmTerm(AsmOperator op, Term[] terms) {
        this(op, terms, null);
    }

    /** create term with given top level operator, given subterms and
     * bound variables. */
    AsmTerm(AsmOperator op, Term[] terms, ArrayOfQuantifiableVariable vars) {
        super(op, op.sort(terms));
        if (terms == null) {
            throw new IllegalArgumentException("terms may not be null");
        } else {
            for (int i = 0; i < terms.length; ++i) {
                if (terms[i] == null) {
                    throw new IllegalArgumentException("term may not be null");
                }
            }
        }
        this.op = op;
        this.terms = terms;
        this.boundVariables = vars;
        depth = TermUtil.maxDepth(terms, -1) + 1;
	fillCaches();
    }

    /* --- methods --- */

    public int arity() {
        return terms.length;
    }

    public Term sub(int index) {
        return terms[index];
    }

    public int depth() {
        return depth;
    }

    public ArrayOfQuantifiableVariable varsBoundHere(int index) {
        if (op.boundVariables(index)) {
            return boundVariables;
        } else {
            return EMPTY_VAR_LIST;
        }
    }

}
