// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License.
// See LICENSE.TXT for details.
//

package de.uka.ilkd.asmkey.logic;


import de.uka.ilkd.key.logic.Term;


/** Utility class with static methods for ASM rules. */

public final class AsmUtil {

    /** Determines if a given term is 'pure', that means it doens't
     * contain a model operator. */
    public static boolean isPure(Term term) {
        if (term instanceof AsmTerm) {
            return false;
        }
        for (int i = 0; i < term.arity(); ++i) {
            if (!isPure(term.sub(i))) {
                return false;
            }
        }
        return true;
    }


    /** Determines if a given term is 'static', that means it is pure
     * and doens't contain a dynamic funtion. */
    public static boolean isStatic(Term term) {
        if (term instanceof AsmTerm || term.op() instanceof DynamicFunction) {
            // TODO: How should predicate symbols be handled? Are they
            // always rigid?
            return false;
        }
        for (int i = 0; i < term.arity(); ++i) {
            if (!isStatic(term.sub(i))) {
                return false;
            }
        }
        return true;
    }

}
