// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License.
// See LICENSE.TXT for details.
//

package de.uka.ilkd.asmkey.logic;

import de.uka.ilkd.key.logic.sort.*;

/** Utility class with static methods for sorts.
 *
 * @author Hubert Schmid
 */

final class SortUtil {

    /** Determines if the sort 'lhs' extends the sort 'rhs'. */
    protected static boolean extendsTrans(Sort lhs, Sort rhs) {
        if (lhs == rhs) {
            return true;
        }
        for (IteratorOfSort i = lhs.extendsSorts().iterator(); i.hasNext();) {
            Sort s = i.next();
            if (s == rhs || extendsTrans(s, rhs)) {
                return true;
            }
        }
        return false;
    }

    /** Returns 'true' if the given sort is a primitive sort. */
    protected static boolean isPrimitiveSort(Sort sort) {
        return sort instanceof PrimitiveSort;
    }

}
