// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

header {
    package de.uka.ilkd.asmkey.parser.antlr;
}


class AsmKeyLexer extends Lexer;


options {
    // lookahead 3 is required for symbol '<-' versus '<->'.
    k = 3; 
    charVocabulary = '\1'..'\377';
}


tokens {
    /* quantifiers */
    FORALL = "all";
    EXISTS = "ex";

    /* literals */
    TRUE = "true";
    FALSE = "false";

    /* taclets */
    FIND = "find";
    IF = "if";
    CLOSE = "close";
    GOAL = "goal";
    REPLACEWITH = "replacewith";
    ADDRULES = "addrules";
    ADD = "add";
    /* cond */
    VARCOND = "varcond";
    COND = "cond";
    TYPEOF = "typeof";
    NEW = "new";
    OP = "op";
    NOT = "not";
    FREE = "free";
    PURE = "pure";
    DEPENDING = "depending";
    ON = "on";
    
    NONINTERACTIVE = "noninteractive";
	RECURSIVE = "recursive";
    DISPLAYNAME = "displayname";
    HELPTEXT = "helptext";

    /* proof */
    PROOF = "proof";
    TERM = "term";
    BRANCH = "branch";
    INST = "inst";
    IFSEQFORMULA = "ifseqformula";
    INTERACTIVE = "interactive";
    HEUR = "heur";

    PROBLEM = "problem";
    PROCEDURES = "procedures";
    CALL = "call";
    IS = "is";

    /* declarations */
    SORTS = "sorts";
    SCHEMA = "schema";
    VARIABLES = "variables";
    CONSTS = "consts";
    PREDICATES = "predicates";
    FUNCTIONS = "functions";
    FORMULA = "formula";
    RULES = "rules";
	HEURISTICS = "heuristics";
    
    RULE = "rule";

    ASM = "asm";
    DYNAMIC = "dynamic";
    STATIC = "static";
    STATICARGS = "staticargs";

    /* asm */
    ASMDEF = "def";
    ASMUPD = "upd";
    ASMCON = "Con";
    ASMINV = "inv";
    ASMSKIP = "skip";
    ASMEQUI = "equi";
    ASMEQUIS = "equiS";
    ASMTHEN = "then";
    ASMELSE = "else";
    ASMEND = "end";
    ASMIN = "in";
    ASMLET = "let";
    ASMFORALL = "forall";
    ASMWITH = "with";
    ASMDO = "do";
    ASMTRY = "try";
    ASMJOINABLE = "joinable";
    ASMAND = "and";
}


SEMI
    : ';' ;

COLON
    : ':' ;

ASSIGN
    : ":=" ;

DOT
    : '.' ;

COMMA
    : ',' ;

LPAREN
    : '(' ;

RPAREN
    : ')' ;

LBRACE
    : '{' ;

RBRACE
    : '}' ;

LBRACK
    : '[' ;

RBRACK
    : ']' ;

AT
    : '@' ;

OR
    : '|' ;

AND
    : '&' ;

NOT
    : '!' ;

IMP
    : "->" ;

EQV
    : "<->" ;

EQUALS
    : '=' ;

NOTEQUALS
    : "!=" ;

LT
    : '<' ;

GT
    : '>' ;

SEQARROW
    : "==>" ;

PLUS
    : '+' ;

MINUS
    : '-' ;

ASTERIX
    : '*' ;

SLASH
    : '/';

UPDATE
    : "<-" ;




protected
DIGIT
options {
    paraphrase = "a digit";
}
    : '0'..'9' ;


protected
LETTER
options {
    paraphrase = "a letter";
}
    : 'A'..'Z' | 'a'..'z' ;


protected
IDCHAR
options {
    paraphrase = "an admissible character for identifiers";
}
    : LETTER | DIGIT | '_' | '~';


IDENT
options {
    testLiterals = true;
    paraphrase = "an identifer";
}
    : ( IDCHAR | '#' ) ( IDCHAR )*;


/* whitespace */
WS
options {
    paraphrase = "white space";
}
    :
        (
            ' '
        |
            '\t'
        |
            '\n' { newline(); }
        |
            '\r'
        )
        {
            $setType(Token.SKIP);
        }
    ;


/* single line comment */
SL_COMMENT
options {
    paraphrase = "a comment";
}
    :
        "//"
        (~'\n')* '\n'
        {
            $setType(Token.SKIP);
            newline();
        }
    ;


/* multi line comment */
ML_COMMENT
options {
    paraphrase = "a comment";
}
	:
        "/*"
		(
			options {
				generateAmbigWarnings=false;
			}
		:
			{ LA(2) != '/' }? '*'
		|	'\n' { newline(); }
		|	~( '*' | '\n' )
		)*
		"*/"
        {
            $setType(Token.SKIP);
        }
	;


STRING_LITERAL
options {
    paraphrase = "a string in double quotes";
}
    : '"'! ( ESC | ~('"'|'\\') )* '"'! ;


protected
ESC
    :	'\\'
        (	'n'         { $setText("\n"); } 
        |	'r' { $setText("\r"); }
        |	't' { $setText("\t"); }
        |	'b' { $setText("\b"); }
        |	'f' { $setText("\f"); }
        |	'"' { $setText("\""); }
        |	'\'' { $setText("'"); }
        |	'\\' { $setText("\\"); }
        )
    ;


// Local variables:
// mode: Antlr
// End:
