// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License.
// See LICENSE.TXT for details.
//

package de.uka.ilkd.asmkey.parser.ast;


/** Visitor for ASM rules.
 *
 * @see AstAsmRule
 * @author Hubert Schmid
 */

public interface AstAsmRuleVisitor {

    /** Sequential ASM rule: <tt>rule1 ; rule2</tt>. */
    Object visitSeq(Object closure, AstAsmRule rule1, AstAsmRule rule2);

    /** Parallel ASM rule: <tt>rule1 rule2</tt>. */
    Object visitPar(Object closure, AstAsmRule rule1, AstAsmRule rule2);

    /** Skip ASM rule: <tt>skip</tt>. */
    Object visitSkip(Object closure);

    /** Update ASM rule: <tt>lhs := rhs</tt>. */
    Object visitAssign(Object closure, AstTerm lhs, AstTerm rhs);

    /** Call named ASM rule: <tt>id(terms)</tt>. */
    Object visitCall(Object closure, Identifier id, AstTerm[] terms);

    /** If ASM rule: <tt>if formula then thenRule else
     * elseRule</tt>. */
    Object visitIf(Object closure, AstTerm formula, AstAsmRule thenRule, AstAsmRule elseRule);

    /** Let ASM rule: <tt>let var:sort := term in rule</tt>. */
    Object visitLet(Object closure, Identifier var, Identifier sort, AstTerm term, AstAsmRule rule);

    /** Forall ASM rule: <tt>forall var:sort with formula do
     * rule</tt>. */
    Object visitForall(Object closure, Identifier var, Identifier sort, AstTerm formula, AstAsmRule rule);

    /** Try ASM rule: <tt>try tryRule else elseRule</tt>. */
    Object visitTry(Object closure, AstAsmRule tryRule, AstAsmRule elseRule);

    /** SchemaVariable for ASM rule: <tt>var</tt>. */
    Object visitSchemaVariable(Object closure, Identifier var);

    /** Substitution for ASM rule: <tt>{var:sort = subst}
     * term</tt>. */
    Object visitSubstitution(Object closure, Identifier var, Identifier sort, AstTerm subst, AstAsmRule rule);

    /** Metaoperator for ASM rule: <tt>id(rule)</tt>. The Metaoperator
     * has only one argument, an ASM rule. */
    Object visitMeta(Object closure, Identifier id, AstAsmRule rule);

}
