// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License.
// See LICENSE.TXT for details.
//

package de.uka.ilkd.asmkey.parser.ast;


/** Visitor for taclet variable conditions.
 *
 * @see AstCondition
 * @author Hubert Schmid
 */

public interface AstConditionVisitor {

    /** Equality of top level operators. */
    void visitOperatorEquals(boolean equal, Identifier v, Identifier w);

    /** Variable not free in term. */
    void visitNotFreeIn(Identifier v, Identifier w);

    /** Pure term. */
    void visitPure(Identifier v);

    /** Static term. */
    void visitStatic(Identifier v);

    /** Term with static arguments. */
    void visitStaticArgs(Identifier v);

    /** Term with dynamic top level operator (function symbol). */
    void visitDynamic(Identifier v);

    /** Top level operator is an asm named rule call. */
    void visitCall(Identifier v);

    /** Skolem symbol with meta variables. */
    void visitSkolem(Identifier v, Identifier w);

}
