// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License.
// See LICENSE.TXT for details.
//

package de.uka.ilkd.asmkey.parser.ast;


import de.uka.ilkd.key.parser.Location;


/** AST node for declarations. All nodes for declarations
 * (e.g. variables and function symbols, taclets) are instances of
 * subclasses of this class. Every declaration owns an identifier. See
 * {@link #accept} and {@link AstDeclarationVisitor} for more
 * information.
 *
 * @author Hubert Schmid */

public abstract class AstDeclaration extends AstNode {

    /** The name of this declaration (like variable/function name or
     * taclet name. */
    private final Identifier id;


    public AstDeclaration(Location location, Identifier id) {
        super(location);
        this.id = id;
    }


    /** Returns the name of this declaration. */
    public final Identifier getId() {
        return id;
    }

    /** This methods calls the corresponding method in visitor. */
    public abstract void accept(AstDeclarationVisitor visitor);

    /** for debug only */
    public String toString() {
        return "[AstDeclaration:id=" + id + toString2() + "]";
    }

    /** used by toString() (only for debug purpose) */
    protected abstract String toString2();

}
