// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License.
// See LICENSE.TXT for details.
//

package de.uka.ilkd.asmkey.parser.ast;


import java.util.List;

import de.uka.ilkd.asmkey.util.ArrayUtil;
import de.uka.ilkd.asmkey.util.CollectionUtil;
import de.uka.ilkd.key.parser.Location;


/** AST node for problem declaration.
 *
 * @author Hubert Schmid */

public final class AstProblem extends AstNode {

    /** All declarations of the problem (file). */
    private final List decls;

    /** The formula to be proven. */
    private final AstTerm formula;

    /** The saved proof. */
    private final AstProofExpression proof;


    public AstProblem(Location location, AstDeclaration[] decls,
                      AstTerm formula, AstProofExpression proof) {
        super(location);
        this.decls = ArrayUtil.toList(decls);
        this.formula = formula;
        this.proof = proof;
    }


    /** Returns the declarations of this problem. */
    public List getDeclarations() {
        return decls;
    }

    /** Returns the formula to be proven. */
    public AstTerm getFormula() {
        return formula;
    }

    /** Returns the saved (part of the) proof. */
    public AstProofExpression getProof() {
        return proof;
    }

    /** for debug only */
    public String toString() {
        return "[AstProblem:decls=" + CollectionUtil.toString(decls)
            + ",formula=" + formula + ",proof=" + proof + "]";
    }

}
