// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License.
// See LICENSE.TXT for details.
//

package de.uka.ilkd.asmkey.parser.ast;


/** Vistor for ASM terms.
 *
 * @see AstTerm
 * @author Hubert Schmid
 */

public interface AstTermVisitor {

    /* --- Terms of first order logic --- */

    /** Operator term: <tt>op(terms)</tt>. */
    Object visitOperatorTerm(Object closure, AstOperator op, AstTerm[] terms);

    /** Quantifier term: <tt>q var:sort. term</tt>. If sort is null,
     * then var is a schema variable. */
    Object visitQuantifierTerm(Object closure, AstQuantifier q, Identifier var, Identifier sort, AstTerm term);

    /** Funtion term: <tt>id(terms)</tt>. */
    Object visitFunctionTerm(Object closure, Identifier id, AstTerm[] terms);

    /** Substitution term: <tt>{var:sort = subst} term</tt>. If sort
     * is null, then var is a schema variable. */
    Object visitSubstitutionTerm(Object closure, Identifier var, Identifier sort, AstTerm subst, AstTerm term);

    /** Term abbreviation. */
    Object visitAbbreviationTerm(Object closure, Identifier name);


    /* --- Terms of dynamic logic --- */

    /** Term with modal box operator: <tt>[rule] term</tt>. */
    Object visitBox(Object closure, AstAsmRule rule, AstTerm term);

    /** Term with modal diamond operator: <tt>&lt;rule&gt;
     * term</tt>. */
    Object visitDiamond(Object closure, AstAsmRule rule, AstTerm term);

    /** Term with modal def operator: <tt>def(rule)</tt>. */
    Object visitDef(Object closure, AstAsmRule rule);

    /** Term with modal update operator: <tt>upd(lhs := rhs in
     * rule)</tt>. */
    Object visitUpd(Object closure, AstTerm lhs, AstTerm rhs, AstAsmRule rule);

    /** Term with modal Con operator: <tt>Con(rule)</tt>. */
    Object visitCon(Object closure, AstAsmRule rule);

    /** Term with modal inv operator: <tt>inv(term in rule)</tt>. */
    Object visitInv(Object closure, AstTerm term, AstAsmRule rule);

    /** Term with modal joinable operator: <tt>joinable(rule1 and
     * rule2)</tt>. */
    Object visitJoinable(Object closure, AstAsmRule rule1, AstAsmRule rule2);

    /** Term with modal equi operator: <tt>equi(rule1, rule2)</tt>. */
    Object visitEqui(Object closure, AstAsmRule rule1, AstAsmRule rule2);

    /** Term with modal equiS operator: <tt>equiS(rule1,
     * rule2)</tt>. */
    Object visitEquiS(Object closure, AstAsmRule rule1, AstAsmRule rule2);

}
