// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License.
// See LICENSE.TXT for details.
//

package de.uka.ilkd.asmkey.parser.env;


import de.uka.ilkd.asmkey.logic.AsmProcedure;
import de.uka.ilkd.key.logic.op.Operator;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.rule.Heuristic;
import de.uka.ilkd.key.rule.Taclet;


/** Represents the environment necessary for parsing declarations. It
 * refines TermEnvironment.
 *
 * @author Hubert Schmid
 */

public interface DeclarationEnvironment extends TermEnvironment {

    /** Add the given sort to this environment.
     *
     * @throws EnvironmentException if sort already exists.
     */
    void addSort(Sort sort) throws EnvironmentException;

    /** Add schema variable declartion to this environment.
     *
     * @throws EnvironmentException if schema variable already exists.
     */
    void addSchemaVariable(SchemaVariable v) throws EnvironmentException;

    /** Add operator to this environment.
     *
     * @throws EnvironmentException if operator already exists.
     */
    void addOperator(Operator op) throws EnvironmentException;

    /** Add heuristic to this environment.
     *
     * @throws EnvironmentException if heuritic already exists.
     */
    void addHeuristic(Heuristic h) throws EnvironmentException;

    /** Returns the heuritic with the given name.
     *
     * @throws EnvironmentException if no such heuristic exists.
     */
    Heuristic getHeuristic(String id) throws EnvironmentException;

    /** Add Taclet declaration to this environment.
     *
     * @throws EnvironmentException if a taclet with the same name
     * already exists.
     */
    void addTaclet(Taclet t) throws EnvironmentException;

    /** Add procedure (named ASM rule) to this environment.
     *
     * @throws EnvironmentException if proecdure with this name
     * already exists.
     */
    void addProcedure(AsmProcedure p) throws EnvironmentException;

}
