// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License.
// See LICENSE.TXT for details.
//

package de.uka.ilkd.asmkey.parser.env;


import de.uka.ilkd.asmkey.logic.AsmProcedure;
import de.uka.ilkd.key.logic.op.Operator;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.rule.Heuristic;
import de.uka.ilkd.key.rule.Taclet;


/** Implementation of DeclarationEnvironment that forwards (delegates)
 * all queries to another DeclarationEnvironment.
 *
 * @author Hubert Schmid
 */

class DelegateDeclarationEnvironment extends DelegateTermEnvironment implements DeclarationEnvironment {

    /** The environment to which all queries are forwarded. */
    private final DeclarationEnvironment env;


    DelegateDeclarationEnvironment(DeclarationEnvironment env) {
        super(env);
        this.env = env;
    }


    public void addSort(Sort sort) throws EnvironmentException {
        env.addSort(sort);
    }

    public void addSchemaVariable(SchemaVariable v) throws EnvironmentException {
        env.addSchemaVariable(v);
    }

    public void addOperator(Operator op) throws EnvironmentException {
        env.addOperator(op);
    }

    public void addHeuristic(Heuristic h) throws EnvironmentException {
        env.addHeuristic(h);
    }

    public Heuristic getHeuristic(String id) throws EnvironmentException {
        return env.getHeuristic(id);
    }

    public void addTaclet(Taclet t) throws EnvironmentException {
        env.addTaclet(t);
    }

    public void addProcedure(AsmProcedure p) throws EnvironmentException {
        env.addProcedure(p);
    }

}
