// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License.
// See LICENSE.TXT for details.
//

package de.uka.ilkd.asmkey.pp;


import java.util.HashMap;
import java.util.Map;

import de.uka.ilkd.asmkey.logic.*;
import de.uka.ilkd.key.gui.Notation;
import de.uka.ilkd.key.gui.NotationInfo;
import de.uka.ilkd.key.logic.op.Operator;


/** This class extends {@link de.uka.ilkd.key.gui.NotationInfo} and adds {@link de.uka.ilkd.key.gui.Notation}s
 * for ASM terms and rules.
 *
 * @author Hubert Schmid */

public final class AsmNotationInfo extends NotationInfo {

        /**
         * Creates a new Operator Notation table.
         * Every Operator gets assigned its prefix, infix(es), postfix and priority.
         */
        private static Map createNotation() {
                Map map = new HashMap();
                map.put(AsmOperator.BOX, AsmNotation.box());
                map.put(AsmOperator.DIAMOND, AsmNotation.diamond());
                map.put(AsmOperator.DEF, AsmNotation.def());
                map.put(AsmOperator.UPD, AsmNotation.upd());
                map.put(AsmOperator.CON, AsmNotation.Con());
                map.put(AsmOperator.INV, AsmNotation.inv());
                map.put(AsmOperator.JOINABLE, AsmNotation.joinable());
                map.put(AsmOperator.EQUI, AsmNotation.equi());
                map.put(AsmOperator.EQUIS, AsmNotation.equiS());
                map.put(AsmOperator.SEQ, AsmNotation.seq());
                map.put(AsmOperator.PAR, AsmNotation.par());
                map.put(AsmOperator.SKIP, AsmNotation.skip());
                map.put(AsmOperator.ASSIGN, AsmNotation.assign());
                map.put(AsmOperator.IF, AsmNotation.if_());
                map.put(AsmOperator.LET, AsmNotation.let());
                map.put(AsmOperator.FORALL, AsmNotation.forall());
                map.put(AsmOperator.TRY, AsmNotation.try_());
                map.put(AsmProcedure.Operator.class, AsmNotation.named());
                return map;
        }


        /** Stores only the ASM notations. */
        private final Map map;


        public AsmNotationInfo() {
                map = createNotation();
        }


        public Notation getNotation(Operator op) {
                /* First check if a matching ASM notation exists. Else
                 * forward the call to the super class. */
                if (map.containsKey(op)) {
                        return (Notation) map.get(op);
                } else if (map.containsKey(op.getClass())) {
                        return (Notation) map.get(op.getClass());
                } else {
                        return super.getNotation(op);
                }
        }

}
