// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License.
// See LICENSE.TXT for details.
//

package de.uka.ilkd.asmkey.proof;


import java.util.List;

import de.uka.ilkd.asmkey.util.ArrayUtil;
import de.uka.ilkd.key.logic.PosInTerm;


/** A ProofRule represents an application of an taclet. The proof rule
 * stores all required information, e.g. taclet name, selected term
 * and user instantiations.
 *
 * @author Hubert Schmid */

public final class ProofRule {

    /** name of the taclet */
    private final String name;

    /** selected formula */
    private final int formula;

    /** selected subterm */
    private final PosInTerm pos;

    /** interactive instantiations */
    private final List instantiations;

    private final boolean interactive;

    /** selected heuristics */
    private final List heuristics;

    /** formulas for if-clauses */
    private final List ifSeqFormulas;


    ProofRule(String name, int formula, PosInTerm pos,
              ProofInstantiation[] instantiations, boolean interactive,
              String[] heuristics, Integer[] ifSeqFormulas) {
        this.name = name;
        this.formula = formula;
        this.pos = pos;
        this.instantiations = ArrayUtil.toList(instantiations);
        this.interactive = interactive;
        this.heuristics = ArrayUtil.toList(heuristics);
        this.ifSeqFormulas = ArrayUtil.toList(ifSeqFormulas);
    }


    /** Name of used taclet. */
    public String getName() {
        return name;
    }

    /** Selected formula. */
    public int getFormula() {
        return formula;
    }

    /** Selected subterm. */
    public PosInTerm getPosition() {
        return pos;
    }

    /** Interactive instantiations. */
    public List getInstantiations() {
        return instantiations;
    }

    public boolean isInteractive() {
        return interactive;
    }

    /** Active heuristics */
    public List getHeuristics() {
        return heuristics;
    }

    /** formulas for if-clauses */
    public List getIfSeqFormulas() {
        return ifSeqFormulas;
    }

}
