// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License.
// See LICENSE.TXT for details.
//

package de.uka.ilkd.asmkey.rule;


import de.uka.ilkd.asmkey.logic.AsmProcedure;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.rule.VariableCondition;
import de.uka.ilkd.key.rule.inst.SVInstantiations;


/** Taclet condition for terms. The conditions checks, if the top
 * level operator is an asm named rule call.
 *
 * @author Hubert Schmid
 */

public final class CallCondition implements VariableCondition {

    /** the instantiation of this schema variable is checked. */
    private final SchemaVariable v;


    /** create a call condition for the given schema variable. */
    public CallCondition(SchemaVariable v) {
        this.v = v;
    }


    /** Check if the top level operator of the term is a invocation of
     * a named asm rule. */
    public boolean check(SchemaVariable var, Term term, SVInstantiations svInst) {
        /* Only check, if the schema variable matches. Else this
         * method is called later. */
        if (var == v) {
            return term.op() instanceof AsmProcedure.Operator;
        } else {
            return true;
        }
    }

}
