// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General Public License.
// See LICENSE.TXT for details.
//

package de.uka.ilkd.asmkey.rule;


import de.uka.ilkd.asmkey.logic.DynamicFunction;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.rule.VariableCondition;
import de.uka.ilkd.key.rule.inst.SVInstantiations;


/** Taclet condition for terms with dynamic top level operator
 * (function symbol).
 *
 * @see StaticCondition
 * @see PureCondition
 * @author Hubert Schmid
 */

public final class DynamicCondition implements VariableCondition {

    /** the instantiation of this schema variable is checked on taclet
     * application. */
    private final SchemaVariable v;


    /** create a condition for the given schema variable. */
    public DynamicCondition(SchemaVariable v) {
        this.v = v;
    }


    /** Check if the term has a dynamic top level operator (function
     * symbol). */
    public boolean check(SchemaVariable var, Term term, SVInstantiations svInst) {
        /* Only check, if the schema variable matches. Else this
         * method is called later. */
        if (var == v) {
            return term.op() instanceof DynamicFunction;
        } else {
            return true;
        }
    }

}
