// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.casetool;

import tudresden.ocl.check.types.*;
import tudresden.ocl.check.OclTypeException;

import com.togethersoft.openapi.sci.*;
import com.togethersoft.openapi.sci.enum.*;

import java.util.HashMap;

import recoder.service.*;
import recoder.abstraction.Package;
import recoder.list.*;
import recoder.convenience.*;
import recoder.io.*;
import recoder.util.*;
import recoder.java.*;
import de.uka.ilkd.key.casetool.*;

public class JavaOCLTypeMap {

    private HashMapOfClassifier classes;

    private JavaOCLTypeMap() {
    }

    public JavaOCLTypeMap(HashMapOfClassifier cls) {
	this.classes = cls;
    }

    private Type getBasicTypeFor(String t) {
	if (t.equals("byte") || t.equals("short") || 
	    t.equals("char") || t.equals("int")  || 
	    t.equals("long")) {
	    return Basic.INTEGER;
	} else if (t.equals("float") || t.equals("double")) {
	    return Basic.REAL;
	} else if (t.equals("boolean")) {
	    return Basic.BOOLEAN;
	} else if (t.equals("void")) {
	    return Any.VOID;
	} else
	    throw new OclTypeException("Cannot find OCL type"+
				       "for Java type \""+t+"\"");
    }
    
    private boolean isBasicType(recoder.abstraction.Type type){
	if ((type instanceof recoder.abstraction.PrimitiveType)
	    || (type instanceof recoder.abstraction.NullType)) {
	    //???            || (type == null)) {
	    return true;
	} else  {
	    return false;
	}
    }
    
    public Type getTypeFor(recoder.abstraction.Type type, 
			   HashMapOfClassifier classes){
        String result;
        if (type == null) {
	    return getBasicTypeFor("void");
        } else if (isBasicType(type)){
	    // basic type
	    return getBasicTypeFor(type.getName());
	} else if (type.getFullName().equals("java.lang.String")){
	    return Basic.STRING;
	} else {
	    String refClass = type.getName();
	    if (classes.getClassifier(refClass) != null 
		&& refClass!=null) {
		return classes.getClassifier(refClass);
	    }
	    else {
		return null;
		//		return new UMLOCLClassifier(type.getFullName());
            }
        }
    }
    
}
