// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.casetool.together;

import junit.framework.*;
import de.uka.ilkd.key.casetool.*;
import de.uka.ilkd.key.java.declaration.*;
import de.uka.ilkd.key.java.abstraction.*;
import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.op.*;

import de.uka.ilkd.key.ocl.ocldltranslator.*;
import de.uka.ilkd.key.ocl.ocldltranslator.heuristics.*;
import de.uka.ilkd.key.ocl.ocldltranslator.control.*;
import de.uka.ilkd.key.ocl.ocldltranslator.normalize.*;
import de.uka.ilkd.key.ocl.ocldltranslator.transform.*;

public class TestTgFacade extends TestCase{

    static final String testClassString = 
	"package qwe.rty; public class X {"
	+" private int x = 3;"
	+" public String y(int x) {"
	+"  return \"\"+x;"
	+" }"
	+" public String v() {"
	+"  int h=9;"
	+" }"
	+"}";

    TgFacade facade;
    KeyOclConstraintTranslator transl;

    public TestTgFacade(String name) {
	super(name);
    }

    public void setUp() {
	ReprModelClass mc = new TogetherReprModelClass(null,null,null) {
		public String getText() {
		    return TestTgFacade.testClassString;
		}
	    };
	try {
	    facade=TgFacade.createFacadeForTest(mc);
	} catch (RuntimeException e) {
	    e.printStackTrace();
	}
	transl=new KeyOclConstraintTranslator(facade);
    }

    public void test1() {
	JavaInfo ji=facade.javaInfo();
	KeYJavaType t=(KeYJavaType)ji.getKeYJavaType("qwe.rty.X");
	assertNotNull(ji.getProgramMethod(t,"v",SLListOfType.EMPTY_LIST)
		      .getMethodDeclaration().getBody());
	KeYJavaType kjt=ji.getKeYJavaType("qwe.rty.X");
	assertNotNull(ji.getProgramMethod(kjt,"v",SLListOfType.EMPTY_LIST)
		      .getMethodDeclaration().getBody());
	ProgramVariable pv=new ProgramVariable(new ProgramElementName("PV"),kjt);
	assertNotNull(ji.getProgramMethod(pv,"v", new ProgramVariable[0])
		      .getMethodDeclaration().getBody());	
	KeYJavaType kjtint=ji.getKeYJavaType("int");
	ProgramVariable intpv=new ProgramVariable(new ProgramElementName("pvi"),kjtint);
	System.out.println(ji.getProgramMethod(pv,"y", new ProgramVariable[]{intpv}).getMethodDeclaration().getBody());
	
    }



}
