// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

/* Generated by Together */

package de.uka.ilkd.key.casetool.together;

import de.uka.ilkd.key.casetool.ReprModelMethod;
import de.uka.ilkd.key.casetool.together.TogetherReprModelClass;
import de.uka.ilkd.key.casetool.ReprModelClass;

import com.togethersoft.openapi.rwi.RwiMember;
import com.togethersoft.openapi.rwi.RwiModel;
import com.togethersoft.openapi.rwi.RwiDiagram;
import com.togethersoft.openapi.rwi.RwiElement;
import com.togethersoft.openapi.rwi.RwiNode;
import com.togethersoft.openapi.rwi.RwiProperty;


public class TogetherReprModelMethod extends TogetherReprModel 
    implements ReprModelMethod{

    // which element of the model do I represent
    private RwiMember orig;
    private RwiModel currRwiModel;
    private RwiDiagram currRwiDiagram;
    
    private RwiMember origParent;
    private boolean origParentAlreadySet = false;

    public TogetherReprModelMethod(RwiMember theOrig, RwiModel model, 
				   RwiDiagram diag) { 
	orig = theOrig;
	currRwiModel = model;
	currRwiDiagram = diag;
    }

    public ReprModelClass getContainingClass(){
 	    RwiNode containingClass = orig.getContainingNode();
	    return new TogetherReprModelClass(containingClass, currRwiModel, 
					      currRwiDiagram);
       }

    public String getContainingPackage(){
	return getContainingClass().getContainingPackage();
    }

    public String getContainingClassName(){
	return getContainingClass().getClassName();
	
    }

    public boolean isVoid() {
	String retTypeString = orig.getProperty(RwiProperty.RETURN_TYPE);
	return (retTypeString.equals("void"));
    }


    public String getCallSignature() {
	return getCallSignature(true);
    }
 
    // returnvalue is adapted to the needs of OCL-parser from dresden
   
    public String getCallSignature(boolean translateJavaType2OCLTypes){
	String argString = orig.getProperty(RwiProperty.PARAMETERS_TEXT);
	// argString has the form type1 arg1, type2 arg2, ...
	// parser expects of form arg1: type1; arg2: type2
	String parserString = shuffleForParser
	    (argString, translateJavaType2OCLTypes);
	    
	String retTypeString = orig.getProperty(RwiProperty.RETURN_TYPE);
	String retSuffix = null;
	if (retTypeString == null){
	    // We have a constructor method ...
	    // Leave it empty (because of the syntax used by the OCL parser ...
	    retSuffix = "";
	} else 
	    if (retTypeString.equals("void")){
		// Leave it empty (because of the syntax used by the OCL parser ...
		retSuffix = "";
	    } else {
		if (translateJavaType2OCLTypes) 
		    retSuffix = ":" + transformTypeJava2OCL(retTypeString);
		else 
		    retSuffix = ":" + retTypeString;
	    }
	return orig.getProperty(RwiProperty.NAME)+"("+ parserString +")" + retSuffix;
    }

    public String getName(){ return orig.getProperty(RwiProperty.NAME); }


    // besides shuffling the arg with the type we also
    // transform some java-types to ocl-types
    private String shuffleForParser(String argString, 
				    boolean translateJavaType2OCLTypes){
	if (argString.equals("")){return "";}
	String resString = "";
	int actPos = 0;
	int nextSpace, nextColon;
	boolean through = false;
	String optColon = "";
	String actArgument, actTypeJava, actTypeOcl;
	while (!through){
	    nextSpace = argString.indexOf(' ', actPos);
	    nextColon = argString.indexOf(',', actPos);
	    if (nextColon == -1){
		// end of argString 
		through = true;
		nextColon = argString.length();
	    }
	    if (actPos > 0) {optColon = "; ";}
	    actArgument = argString.substring(nextSpace+1, nextColon);
	    actTypeJava = argString.substring(actPos, nextSpace);
	    if (translateJavaType2OCLTypes)
		actTypeOcl = transformTypeJava2OCL(actTypeJava);
	    else
		actTypeOcl = actTypeJava;
	    resString = resString + optColon + actArgument + ": " + actTypeOcl;
	    actPos = nextColon + 2;
	}
	return resString;
    }

    private String transformTypeJava2OCL(String aJavaType){
	if ("int".equals(aJavaType) || 
	    "byte".equals(aJavaType) ||
 	    "short".equals(aJavaType) ||
	    "long".equals(aJavaType)){
	    return "Integer";
	}
	else if ("boolean".equals(aJavaType)) {return "Boolean";}
	else {return aJavaType;}
    }





    // looks if orig has a parent and set it if needed
    // parent means for methods the next occurrence of the method
    //  in the inheritance hierarchy which has an own pre/post specification
    private RwiMember getOrigParent(){
	if (origParentAlreadySet){
	    return origParent;
	}else{
	    // we ask our containingNode to do the job
	    RwiNode containingClass = orig.getContainingNode();
	    TogetherReprModelClass contClassRepr = 
		new TogetherReprModelClass(containingClass, currRwiModel, 
					   currRwiDiagram);
	    String myUniqueName = orig.getUniqueName();
	    RwiMember candParent;
 	    // findParentMeth returns the next Meth-Declaration
	    // in the inheritance hierarchy above
	    candParent= contClassRepr.findParentMeth(myUniqueName);
	    if (candParent==null){
		return null;
	    }
	    if ((candParent.getProperty("preconditions") != null || 
		 candParent.getProperty("postconditions") != null)){
		// candParent has its own specification
 		origParent = candParent;
	    }else{
 		// otherwise we ask candParent for its parent
		origParent=
		    (new TogetherReprModelMethod(candParent, currRwiModel, 
						 currRwiDiagram)).getOrigParent();
	    }
	    origParentAlreadySet = true;
	}
	return origParent;
    }





    public String getMyOrParentPostCond() {
    	String myPre = getMyPreCond();
	String myPost = getMyPostCond();
	if (myPre.equals("") && myPost.equals("")){
	    // ask parent
	    TogetherReprModelMethod parentRepr = 
		new TogetherReprModelMethod(getOrigParent(), currRwiModel, 
					    currRwiDiagram);
	    return parentRepr.getMyPostCond();
	}else{
	    return myPost;
	}
    }


    // we ask the parent only when method does not have neither precond
    // nor postcond.  
    
    public String getMyOrParentPreCond() {
	String myPre = getMyPreCond();
	String myPost = getMyPostCond();
	if (myPre.equals("") && myPost.equals("")){
	    // ask parent
	    TogetherReprModelMethod parentRepr = 
		new TogetherReprModelMethod(getOrigParent(), currRwiModel, 
					    currRwiDiagram);
	    return parentRepr.getMyPreCond(); 
	}else{
	    return myPre;
	}
    }

    public String getMyPreCond() { 
	if (orig.getProperty("preconditions")==null){
	    return "";
	}else{
	    return orig.getProperty("preconditions");
	}
    }

	public void setMyPreCond(String precond)
	{
		orig.setProperty("preconditions", precond);
	}
 
    public String getParentPreCond() {
	// ask parent
	TogetherReprModelMethod parentRepr = 
	    new TogetherReprModelMethod(getOrigParent(), currRwiModel, 
					currRwiDiagram);
	return parentRepr.getMyPreCond();
    }

    public String getMyPostCond() {
	if (orig.getProperty("postconditions")==null){
	    return "";
	}else{
	    return orig.getProperty("postconditions");
	}
    }
    
    public void setMyPostCond(String postcond) 
    {
    	orig.setProperty("postconditions", postcond);
    }

    public String getParentPostCond() {
	// ask parent
	TogetherReprModelMethod parentRepr = 
	    new TogetherReprModelMethod(getOrigParent(), currRwiModel, 
					currRwiDiagram);
	return parentRepr.getMyPostCond();
    }
    
    public String getMyGFAbs()
    {
    	String result = orig.getProperty("GFAbstractSyntax");
    	if (result == null)
    		return "";
    	else
    		return result;
    }
    
    public void setMyGFAbs(String abs)
    {
    	orig.setProperty("GFAbstractSyntax", abs);
    }

    public boolean hasOrigParent() {
        return (getOrigParent() != null);
    }
    
    
    public String toString() {
       return getName();
    }
}
