// The KeY system is protected by the GNU General 
// Public License. See LICENSE.TXT for details.
//

package de.uka.ilkd.key.casetool.together;

import de.uka.ilkd.key.casetool.*;
import de.uka.ilkd.key.casetool.together.TogetherReprModelClass;
import tudresden.ocl.check.types.Type;
import java.io.IOException;
import java.util.Iterator;

import com.togethersoft.openapi.rwi.RwiModel;
import com.togethersoft.openapi.rwi.RwiNode;
import com.togethersoft.openapi.rwi.RwiModelAccess;
import com.togethersoft.openapi.rwi.RwiPackage;
import com.togethersoft.openapi.rwi.RwiProperty;
import com.togethersoft.openapi.rwi.enum.RwiPackageEnumeration;
import com.togethersoft.openapi.rwi.enum.RwiNodeEnumeration;
import com.togethersoft.openapi.sci.SciModel;
import com.togethersoft.openapi.sci.SciModelAccess;

import recoder.abstraction.Method; 
import recoder.abstraction.Field; 
import recoder.abstraction.ClassType; 
import recoder.list.CompilationUnitList;
import recoder.list.MethodList;
import recoder.list.FieldList;
import recoder.list.StringArrayList;
import recoder.list.TypeList;
import recoder.list.ClassTypeList;
import recoder.java.declaration.TypeDeclaration; 
import recoder.io.SourceFileRepository;
import recoder.io.DefaultSourceFileRepository;
import recoder.util.FileCollector;
import recoder.java.CompilationUnit;
import recoder.service.DefaultSourceInfo;


public class UMLOCLTogetherModel implements UMLOCLModel {
    //debug flag
    private static final boolean DEBUG = false;

    private SourceFileRepository sfr;
    private String projectRoot;
    private HashMapOfClassifier classifiers;
    private HashMapOfAssociations associations;
    private JavaOCLTypeMap typeMap = null;

    private UMLOCLTogetherModel() {
    }

    public UMLOCLTogetherModel(String projectRoot) {
	this.projectRoot = projectRoot;
	this.classifiers = new HashMapOfClassifier();
	this.associations = new HashMapOfAssociations();
	initRecoder();
    }

    public HashMapOfClassifier getUMLOCLClassifiers() {
	      
	String[] files = collectJavaFiles(projectRoot);
	try{
	    for (int i=0; i<files.length; i++) {
                if (files[i] != null) {
		    sfr.getCompilationUnitFromFile(files[i]);
                }
            }
	} catch (recoder.ParserException e){
            System.out.println("  Exception occured while "+
			       "reading in Java sources: " + e);
        }
	CompilationUnitList cuList = sfr.getCompilationUnits();
	
	// find classes
	findAllClasses(cuList);
	if (DEBUG) {
	    System.out.println("------------- found the following "+
			       "classes ------------------");
	    Iterator it = classifiers.values().iterator();
	    UMLOCLClassifier c;
	    while (it.hasNext()) {
		c = (UMLOCLClassifier)it.next();
		System.out.println(c.getName());
	    }
	}

	typeMap = new JavaOCLTypeMap(classifiers);

	// find attributes and methods
	CompilationUnit unit;
	for (int i=0; i < cuList.size(); i++) {
	    unit = cuList.getCompilationUnit(i);
	    TypeDeclaration td;
	    for (int j = 0; j < unit.getTypeDeclarationCount(); j++) {
		td = unit.getTypeDeclarationAt(j);	
		findAllAttributes(td);
		findAllMethods(td);
	    }
	}
	if (DEBUG) {
	    System.out.println("------------- found the following "+
			       "methods and attributes ------------------");
	    Iterator cit = classifiers.values().iterator();
	    UMLOCLClassifier c;
	    while (cit.hasNext()) {
		c = (UMLOCLClassifier)cit.next();
		HashMapOfFeatures feat = c.features();
		Iterator it = feat.values().iterator();
		UMLOCLFeature f;
		while (it.hasNext()) {
		    f = (UMLOCLFeature)it.next();
		    if (f instanceof UMLOCLBehaviouralFeature)
			System.out.println("Method    "+c.getName()+
					   " :: "+f.getName());
		    if (f instanceof UMLOCLStructuralFeature)
			System.out.println("Attribute "+c.getName()+
					   " :: "+f.getName());
		} 
	    }
	}

	// find associations
	RwiModel rwiModel = RwiModelAccess.getModel();		
	RwiPackageEnumeration rwiRoots=rwiModel.
	    rootPackages(RwiProperty.MODEL);
	while (rwiRoots.hasMoreElements()) {
	    RwiPackage rwiPackage = rwiRoots.nextRwiPackage();
	    processAssociations(rwiPackage);
	}

	if (DEBUG)
	    System.out.println("------------- found the following "+
			       "associations ------------------");

	Iterator cit = classifiers.values().iterator();
	UMLOCLClassifier c;
	while (cit.hasNext()) {
	    c = (UMLOCLClassifier)cit.next();
	    Iterator it = c.getAssociations().values().iterator();
	    UMLOCLAssociation assoc;
	    while (it.hasNext()) {
		assoc = (UMLOCLAssociation)it.next();
		if (DEBUG)
		    System.out.println("assoc from: "+assoc.getSource().getName()+
				       " to: "+assoc.getTarget().getName());
	    }
	}
	return classifiers;
    }

    protected void processAssociations(RwiPackage rwiPackage){
	SciModel sciModel = SciModelAccess.getModel();
	RwiModel rwiModel = RwiModelAccess.getModel();

	RwiNodeEnumeration rwiNodesEnum = rwiPackage.nodes();
	RwiNode rwiNode;
	TogetherReprModelClass cls;
	while (rwiNodesEnum.hasMoreElements()){
	    rwiNode = rwiNodesEnum.nextRwiNode();
	    cls = new TogetherReprModelClass(rwiNode, rwiModel, null);
	    cls.getAssociations(classifiers);
	}	
	RwiPackageEnumeration subpackages = rwiPackage.subpackages();
	while (subpackages.hasMoreElements()) {
	    RwiPackage subpackage = subpackages.nextRwiPackage();
	    processAssociations(subpackage);
	}
    }


    private void findAllClasses(CompilationUnitList cuList) {
	UMLOCLClassifier supertype;
	ClassType st;
	UMLOCLClassifier cls;
	for (int i=0; i < cuList.size(); i++) {
	    CompilationUnit pa = cuList.getCompilationUnit(i);
	    for (int j = 0; j < pa.getTypeDeclarationCount(); j++) {
		TypeDeclaration td = pa.getTypeDeclarationAt(j);
		if (!classifiers.containsKey(td.getName())) {			
		    cls = new UMLOCLClassifier(td.getName(), td.getFullName());
		    classifiers.putClassifier(td.getName(),cls);
		}
		else 
		    cls = classifiers.getClassifier(td.getName());

		// now find all supertypes of this class
		ClassTypeList supertypes = td.getAllSupertypes();
		for (int k = 1; k < supertypes.size(); k++) {
		    st = supertypes.getClassType(k);
		    if (classifiers.containsKey(st.getName())) {			
			cls.addSupertype(st.getName(), 
					 classifiers.getClassifier(st.getName()));
		    } else {
			supertype = new UMLOCLClassifier(st.getName(), st.getFullName());
			classifiers.putClassifier(st.getName(), supertype);
			cls.addSupertype(st.getName(), supertype);
		    }
		}

	    }	
	}
    }

    private void findAllMethods(TypeDeclaration td) {
	String nameString;
	UMLOCLClassifier cls = classifiers.getClassifierByFullName(td.getFullName());

	MethodList ml = td.getAllMethods();
	Method meth;
	for (int l=0; l < ml.size(); l++) {
	    meth = ml.getMethod(l);
	    recoder.abstraction.Type retType = meth.getReturnType();
	    tudresden.ocl.check.types.Type ret2 = 
		typeMap.getTypeFor(meth.getReturnType(), 
				   classifiers);
	    
	    nameString = new String(meth.getName() + "(");
	    TypeList ctl = meth.getSignature();
	    Type[] paramsArray = new Type[ctl.size()];
	    recoder.abstraction.Type par;
	    for (int k=0; k < ctl.size(); k++) {		
		par = ctl.getType(k);
		if (k < ctl.size() - 1) {
		    nameString = nameString + par.getFullName() + ",";
		} else {
		    nameString = nameString + par.getFullName();
		}
	       
		if (typeMap.getTypeFor(par, classifiers) != null) {
		    paramsArray[k] = typeMap.getTypeFor(par, 
							classifiers);
		} else{
		    UMLOCLClassifier c = new 
			UMLOCLClassifier(par.getName(), par.getFullName());
		    classifiers.putClassifier(par.getName(), c);
		    paramsArray[k] = typeMap.getTypeFor(par, 
							classifiers);
		}
	    }
	    nameString = nameString + ")";
	    
	    if (retType!=null) {
		cls.addFeature(nameString, new UMLOCLBehaviouralFeature
		    (nameString,ret2,paramsArray));
	    } else {
		cls.addFeature(nameString, new UMLOCLBehaviouralFeature
		    (nameString,paramsArray));
	    }	    
	}	
    }
    
	
    private void findAllAttributes(TypeDeclaration td) {
	FieldList fl = td.getAllFields();
	Field field;
	recoder.abstraction.Type fieldType;
	for (int l=0; l < fl.size(); l++) {
	    field = fl.getField(l);
	    fieldType = field.getType();
	    UMLOCLClassifier cls = classifiers.
		getClassifierByFullName(td.getFullName());

	    Type type = typeMap.getTypeFor(fieldType, classifiers);
	    
	    if (type == null) {
		UMLOCLClassifier c = new 
		    UMLOCLClassifier(fieldType.getName(), 
				     fieldType.getFullName());
		classifiers.putClassifier(fieldType.getName(), c);
		type=c;
	    }

	    classifiers.getClassifierByFullName(td.getFullName()).
		addFeature(field.getName(), new UMLOCLStructuralFeature
		    (field.getName(), type));
	}		
    }

    private void initRecoder() {
	de.uka.ilkd.key.java.Services service = 
	    new de.uka.ilkd.key.java.Services();
	de.uka.ilkd.key.java.Recoder2KeY r2k = 
	    new de.uka.ilkd.key.java.Recoder2KeY(projectRoot, 
						 service.getJavaInfo().rec2key(), 
						 service.getTypeConverter());
	de.uka.ilkd.key.java.recoderext.KeYCrossReferenceServiceConfiguration 
	    servConf = r2k.getServiceConfiguration();
	sfr = servConf.getSourceFileRepository();
	DefaultSourceInfo sfi = new DefaultSourceInfo(servConf);
    }

    private String[] collectJavaFiles(String dir) {
	FileCollector col = new FileCollector(dir);
	StringArrayList list = new StringArrayList();
	while (col.next(DefaultSourceFileRepository.JAVA_FILENAME_FILTER)) {
	    String path;
	    try {
		path = col.getFile().getCanonicalPath();
	    } catch (IOException ioe) {
		path = col.getFile().getAbsolutePath();
	    }
	    list.add(path);
	}
	String[] result = new String[list.size()];
	for (int i = list.size() - 1; i >= 0; i -= 1) {
	    result[i] = list.getString(i);
	}
	return result;
	
    }


}
