// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.casetool.together.scripts.menuextension;

import de.uka.ilkd.key.util.Debug;
import java.util.Properties;
import java.util.LinkedList;
import java.util.Iterator;
import java.util.Vector;
import com.togethersoft.openapi.ide.command.IdeCommandItem;
import de.uka.ilkd.key.ocl.ocldltranslator.detailedocl.*;
import de.uka.ilkd.key.gui.*;

public class OCLRefinementSettings implements Settings {

    /** the key values for the properties */
    private static final String EXCTHROWN = "[OCLRef]ExcThrown";
    private static final String USENULL = "[OCLRef]Null";

    private static OCLMod[] EMPTY=new OCLMod[0];
    
    /** the listener to this settings entry */
    private LinkedList listenerList = new LinkedList();
    /** the different OCLModifications that are set*/
    private boolean excModSet;
    private boolean nullModSet;

    /** the different OCLModifications have to be set*/
    private boolean excModSetNew;
    private boolean nullModSetNew;

    private IdeCommandItem guiComp;
    
    public OCLRefinementSettings() {
	restore();
    }

        
    /** restores old values */
    public void restore() {
	excModSetNew=excModSet;
	nullModSetNew=nullModSet;
    }

    public void store() {
	excModSet=excModSetNew;
	nullModSet=nullModSetNew;
	fireSettingsChanged();
    }

    public boolean excModSet() {
	return excModSet;
    }


    public boolean nullModSet() {
	return nullModSet;
    }


    public void setExcModSet(boolean b) {
	excModSetNew=b;
    }


    public void setNullModSet(boolean b) {
	nullModSetNew=b;
    }

    public void setGUIComp(IdeCommandItem item) {
	this.guiComp=item;
	updateEnabledGUI();
    }

    public void updateEnabledGUI() {
	guiComp.setEnabled(maySet());
    }

    public boolean maySet() {
	//	return xmiModelChosen(); doesn't work due to TogetherCC
	return true;
    }

    private boolean xmiModelChosen() {
	return "0".equals(ProofSettings.THE_SETTINGS
			  .getModelSourceSettings().getModelSource());
    }

    public OCLMod[] getOCLMods() {
	if (!xmiModelChosen()) {
	    return EMPTY;
	}
	Vector v=new Vector();
	if (excModSetNew) v.add(ExcThrownMod.SINGLETON);
	if (nullModSetNew) v.add(NullMod.SINGLETON);
	OCLMod[] result=new OCLMod[v.size()];
	int i=0;
	Iterator it=v.iterator();
	while (it.hasNext()) {
	    result[i++]=(OCLMod)it.next();
	}
	return result;
    }

    /** gets a Properties object and has to perform the necessary
     * steps in order to change this object in a way that it
     * represents the stored settings
     */
    public void readSettings(Properties props) {
	excModSet  = Boolean.valueOf(props.getProperty(EXCTHROWN)).booleanValue();
	nullModSet  = Boolean.valueOf(props.getProperty(USENULL)).booleanValue();
	restore();
    }

    /*
    /** The settings to store are written to the given Properties object.
     * @param props the Properties object where to write the settings as (key, value)
     *  pair
     */
    public void writeSettings(Properties props) {
	props.setProperty(EXCTHROWN, ""+excModSetNew);
	props.setProperty(USENULL, ""+nullModSetNew);
    }

    public void changeFor(OCLMod mod, boolean to) {
	if (mod==ExcThrownMod.SINGLETON) setExcModSet(to);
	if (mod==NullMod.SINGLETON) setNullModSet(to);
    }

    /** sends the message that the state of this setting has been
     * changed to its registered listeners (not thread-safe)
     */
    protected void fireSettingsChanged() {
	Iterator it = listenerList.iterator();
	while (it.hasNext()) {
	    ((SettingsListener)it.next()).settingsChanged(new GUIEvent(this));
	}
    }

    /** adds a listener to the settings object 
     * @param l the listener
     */
    public void addSettingsListener(SettingsListener l) {
	listenerList.add(l);
    }


}
