// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

/** common superclass of TacletIfSelectionDialog and TacletMatchCompletionDialog */
package de.uka.ilkd.key.gui;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import de.uka.ilkd.key.proof.ApplyTacletDialogModel;
import de.uka.ilkd.key.logic.*;

public abstract class ApplyTacletDialog extends JDialog {


    // buttons
    protected JButton cancelButton;
    protected JButton applyButton;
   
    private KeYMediator mediator;
    protected boolean checkAfterEachInput=true;

    protected ApplyTacletDialogModel[] model; 
    private JTextArea statusLabel;
    private JPanel statusLine;

    public ApplyTacletDialog(ApplyTacletDialogModel[] model,
			     KeYMediator mediator) { 

        super(mediator.mainFrame(), "Choose Taclet Instantiation", false);
//	setSize(800,700);
//	setLocation(70,50);

	this.mediator = mediator;
	this.model = model;

	applyButton  = new JButton("Apply");
	cancelButton = new JButton("Cancel");
    
	mediator.requestModalAccess(this); 
	addWindowListener(new WindowAdapter() {
		public void windowClosed(WindowEvent e) {
		    ApplyTacletDialog.this.closeDlg();		    
		}

		public void windowClosing(WindowEvent e) {
		    ApplyTacletDialog.this.closeDlg();
		}
	    });
    }
    
    protected KeYMediator mediator() {
	return mediator;
    }

    private int countLines(String s) {
	int i=0;
	int p=0;
	while ((p=s.indexOf("\n",p))>=0) {
	    i++;
	    p++;
	}
	return i+1;
    }
    
    protected JPanel createInfoPanel() {
	ListOfNamed vars=model[0].var_ns().elements();
	JPanel panel = new JPanel(new GridLayout(1,1));
	panel.setBorder(new TitledBorder("Sequent program variables"));       
	JScrollPane scroll = new JScrollPane();
	JTextArea text;
	if (vars.size()>0) {
	    text=new JTextArea(vars.toString(),2,40);
	} else {
	    text=new JTextArea("none",1,40);
	}
	scroll.setViewportView(text);
	text.setEditable(false);
	panel.add(scroll, BorderLayout.CENTER);
	return panel;
    }

    protected JPanel createTacletDisplay() {
	JPanel panel = new JPanel(new BorderLayout());	
	panel.setBorder(new TitledBorder("Selected Taclet - "+model[0].taclet().name()));
	panel.setAlignmentY(panel.TOP_ALIGNMENT);
	// show taclet
	JScrollPane scroll = new JScrollPane();
	int nolines=countLines(model[0].taclet().toString())+1;
	if (nolines>10) nolines=11;
	JTextArea text=new JTextArea(model[0].taclet().toString(),nolines,40);
	text.setEditable(false);
	scroll.setViewportView(text);
	panel.add(scroll, BorderLayout.CENTER);

	return panel;
    }

    protected abstract void pushAllInputToModel();
    protected abstract int current();

    public boolean checkAfterEachInput() {
	return checkAfterEachInput;
    }

    protected JPanel createStatusPanel() {
	statusLabel=new JTextArea("Status information", 5, 40);	
	statusLabel.setEditable(false);
	statusLine = new JPanel();
	//	statusLine.setLayout(new GridLayout(1,2));
	JCheckBox selectCheckBox=new JCheckBox("Check after each input",
							true);
	selectCheckBox.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
// 		    JButton checkNow=(JButton)((JPanel)
// 					       ((JCheckBox)e.getSource())
// 					       .getParent()).getComponent(0);
		    checkAfterEachInput=(((JCheckBox)e.getSource())
				      .isSelected());
		    if (checkAfterEachInput) {
			pushAllInputToModel();
			setStatus(model[current()].getStatusString());
			//			checkNow.setEnabled(true);
		    } else {
			setStatus("");
			//			checkNow.setEnabled(false);
		    }
		}
	    });

	selectCheckBox.setAlignmentX(0);
	JPanel activePanel=new JPanel();
	activePanel.add(selectCheckBox);
	JButton checkNow=new JButton("Check now");
	checkNow.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {		    
		    pushAllInputToModel();
		    setStatus(model[current()].getStatusString());
		}
	    });	

	//	activePanel.add(checkNow);
	statusLine.add(activePanel);
	statusLine.add(new JScrollPane(statusLabel,
		       JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, 
		       JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED));
	statusLabel.setBorder(new BevelBorder(BevelBorder.LOWERED));
	setStatus(model[current()].getStatusString());
	return statusLine;
    }

    protected JPanel createButtonPanel(ActionListener buttonListener) {
	JPanel panel=new JPanel(new GridBagLayout());
	GridBagConstraints c = new GridBagConstraints();

	cancelButton.addActionListener(buttonListener);
	applyButton.addActionListener(buttonListener);
	c.insets = new Insets(5,20,5,20);
	c.gridx = 0;
	panel.add(cancelButton, c);    

	c.gridx = 1;
	panel.add(applyButton, c);    
	panel.setAlignmentY(panel.BOTTOM_ALIGNMENT);

	return panel;
    }

    protected void setStatus(String s) {
	if (statusLabel != null) {
	    statusLabel.setText(s); 
	}
    }

    protected void closeDlg() {
	mediator.freeModalAccess(this);
    }

}
