// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.gui;


import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;


public abstract class GUIAbstractTreeNode implements TreeNode {
    
    public abstract TreeNode getChildAt(int childIndex);

    public abstract int getChildCount();

    public abstract TreeNode getParent();
    
    public abstract boolean isLeaf();


    public int getIndex(TreeNode node) {
	for ( int i=0; i<getChildCount(); i++ ) {
	    if ( getChildAt(i).equals(node) ) {
		return i;
	    }
	}
	return -1;
    }

    public boolean getAllowsChildren() {
	return ! isLeaf();
    }

    public Enumeration children() {
	return new ChildEnumeration();
    }


    public Object[] getPath() {
	LinkedList path = new LinkedList();
	TreeNode n = this;
	//	System.out.println("1: "+n);
	path.addFirst(n);
	while (n.getParent()!=null) {
	    n = n.getParent();
	    //	    System.out.println("2: "+n+ "    "+path);
	    path.addFirst(n);
	}
	return path.toArray();
    }

    private class ChildEnumeration implements Enumeration {
	int current = 0;

	public boolean hasMoreElements() {
	    return current < getChildCount();
	}

	public Object nextElement() {
	    return getChildAt(current++);
	}
	
    }    
}

