// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.gui;
/** this class implements a TreeModel that can be displayed using the
 * JTree class framework 
 */
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;

import de.uka.ilkd.key.proof.Proof;
import de.uka.ilkd.key.proof.Node;
import de.uka.ilkd.key.proof.IteratorOfNode;

class GUIBranchNode extends GUIAbstractTreeNode 
                    implements TreeNode {


    private GUIProofTreeModel tree;
    private Node subTree;
    private Object label;
    
    public GUIBranchNode(GUIProofTreeModel tree,
			 Node subTree,
			 Object   label) {
	this.tree = tree;
	this.subTree = subTree;
	this.label = label;
	for (int i=0; i<getChildCount(); i++) {
	    getChildAt(i);
	}
    }

    
    public TreeNode getChildAt(int childIndex) {
	int count = 0;
	int ncc = 0;
	Node n = subTree;
	while ( childIndex != count 
	        && (ncc = n.childrenCount()) == 1 ) {
	    count++;
	    n = n.child(0);
	}
	if ( childIndex == count ) {
	    return tree.getProofTreeNode(n);
	} else {
            Object label;
            Node newNode = n.child(childIndex-count-1);
            label = newNode.name();
            if (label == null) label = "Case "+(childIndex-count);
	    return tree.getBranchNode(newNode, label);
	}
    }

    public int getChildCount() {
	int count = 1;
	int ncc = 0;
	Node n = subTree;
	while ( (ncc = n.childrenCount()) == 1 ) {
	    count++;
	    n = n.child(0);
	}
	return count + ncc;
    }
    
    
    public TreeNode getParent() {
	Node n = subTree.parent();
	if ( n==null ) {
	    return null;
	} else {
	    while (n.parent()!=null
		   && n.parent().childrenCount()==1) {
		n = n.parent();
	    }
	    return tree.findBranch(n);
	}
    }

    public Node getNode() {
	return subTree;
    }

    public void setLabel(String s) {
	label=s;
	subTree.setName(s);
    }

    public boolean isLeaf() {
	return false;
    }

    public String toString() {
	return label.toString();
    }


}
