// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.gui;

import java.awt.Font;
import java.awt.Color;
import javax.swing.JTextArea;
import javax.swing.UIManager;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.proof.Node;
import de.uka.ilkd.key.rule.RuleApp;
import de.uka.ilkd.key.rule.TacletApp;
import de.uka.ilkd.key.rule.inst.GenericSortInstantiations;

public class NonGoalInfoView extends JTextArea {
    
    public NonGoalInfoView (Node node, KeYMediator mediator) {
	SequentPrintFilter filter =
	    new ConstraintSequentPrintFilter ( node.sequent (),
					       mediator.getUserConstraint ()
					       .getConstraint () );
	SequentPrinter printer = new SequentPrinter
	    (new ProgramPrinter(null), 
	     mediator.getNotationInfo(),
	     filter);
	//	node.sequent().prettyprint(printer);
	printer.printSequent ();
	String s = printer.toString();
	RuleApp app = node.getAppliedRuleApp();
	if ( app != null ) {
	    s = s + "\n \nUpcoming rule application: \n";
	    s = s + app.rule();
	    if ( app instanceof TacletApp ) {
		TacletApp tapp = (TacletApp)app;
		if ( tapp.instantiations ().getGenericSortInstantiations () !=
		     GenericSortInstantiations.EMPTY_INSTANTIATIONS ) {
		    s = s + "\n\nWith sorts:\n";
		    s = s +
			tapp.instantiations ().getGenericSortInstantiations ();
		}
	    }
	}

	Config.DEFAULT.addConfigChangeListener(
	    new ConfigChangeListener() {
		    public void configChanged(ConfigChangeEvent e) {
			updateUI();
		    }
		});

	updateUI();
	setText(s);
	setEditable(false);
    }


    public void updateUI() {
	super.updateUI();
	Font myFont = UIManager.getFont(Config.KEY_FONT_INNER_NODE_VIEW);
	if (myFont != null) {
	    setFont(myFont);
	} else {
	    Debug.out("KEY-INNER_NODE_FONT not available, use standard font.");
	}
    }

}
