// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.gui;

/**
 * prints a sequent and its parts for one given sequent as SequentPrinter
 * does but does not compute the PositionTable. The class merely
 * overwrites the methods that compute the position table in
 * SequentPrinter with empty method bodies.
 */
public class PureSequentPrinter extends SequentPrinter {

    /**
     * creates a SequentPrinter and sets the sequent in this SequentPrinter
     * as well as a ProgramPrinter that prints Java programs and a
     * NotationInfo with additional information on how to denote operators
     * (e.g., in infix or prefix notation)
     * @param prgPrinter the ProgramPrinter that prints Java programs
     * @param notationInfo the NotationInfo with additional information on
     * denoting operators
     * @param seq the sequent that this SequentPrinter can print
     */
    public PureSequentPrinter(ProgramPrinter prgPrinter, 
			  NotationInfo notationInfo, 
			  de.uka.ilkd.key.logic.Sequent seq) {
	super(prgPrinter, notationInfo, seq);
    }
 
    /** does nothing as this printer does not build any position information
     */
    protected void markStartSub() {
    }

    /** does nothing as this printer does not build any position information
     */
    protected void markEndSub() {
    }
    
    /**
     * does nothing as this printer does not build any position information
     *
     * @param i an arbirary int
     */
    protected void startTerm(int i) {
    }

    /**
     * returns null as no PositionTable representing position information on the
     * sequent of this SequentPrinter is built.
     */
    public InitialPositionTable getPositionTable() {
	return null;
    }


}
