// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.gui;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.op.*;
import javax.swing.*;
import de.uka.ilkd.key.java.PrettyPrinter;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.Recoder2KeY;
import de.uka.ilkd.key.logic.sort.*;

public class TestSequentPrinter {
 
    private ConstrainedFormula[] con;
    private Sequent seq;
    
    public void setUp() { 
	TermFactory tf=TermFactory.DEFAULT;
	Sort s0=new PrimitiveSort(new Name("s0"));
       	Function p=new Function(new Name("pe"),Sort.FORMULA,new Sort[]{s0,s0});
	Function q=new Function(new Name("q"),Sort.FORMULA,new Sort[]{s0});
	Function r=new Function(new Name("r"),Sort.FORMULA,new Sort[]{s0,s0});
	Function f=new Function(new Name("f"), s0, new Sort[]{s0});
	Function c=new Function(new Name("c"), s0, new Sort[]{});
	LogicVariable x=new LogicVariable(new Name("x"), s0);
	LogicVariable y=new LogicVariable(new Name("y"), s0);

	Term t_c=tf.createFunctionTerm(c, new Term[]{});
	Term t_fc=tf.createFunctionTerm(f, new Term[]{t_c});
	Term t_ffc=tf.createFunctionTerm(f, new Term[]{t_fc});
	Term t_p=tf.createFunctionTerm(p, new Term[]{t_fc,t_c});
	Term t_q=tf.createFunctionTerm(q, new Term[]{t_ffc});
	Term t_r=tf.createFunctionTerm(r, new Term[]{t_ffc,t_fc});
	Term t_allxr=tf.createQuantifierTerm(Op.ALL, 
					    new QuantifiableVariable[]{x,y},
					     t_r);
	Term t_substq=tf.createSubstitutionTerm(Op.SUBST, x, t_fc, t_q);
	JavaInfo ji=new JavaInfo();
	Recoder2KeY c2k=new Recoder2KeY(ji.rec2key(), ji.getTypeConverter());
	JavaBlock jb=c2k.readBlock("{for (int i=0;i<3; i++) i=5;}",
						 c2k.createEmptyContext());
	Term t_dia=tf.createDiamondTerm(jb, t_r);
	Term t_true=tf.createJunctorTerm(Op.TRUE,new Term[]{});

	con=new ConstrainedFormula[5];
	con[0]=new ConstrainedFormula(t_p, Constraint.BOTTOM);
	con[1]=new ConstrainedFormula(t_substq, Constraint.BOTTOM);
	con[2]=new ConstrainedFormula(t_allxr, Constraint.BOTTOM);
	con[3]=new ConstrainedFormula(t_true, Constraint.BOTTOM);
	con[4]=new ConstrainedFormula(t_dia, Constraint.BOTTOM);
	Semisequent ss0=Semisequent.EMPTY_SEMISEQUENT;
	Semisequent ss1=Semisequent.EMPTY_SEMISEQUENT;
	ss0=ss0.insert(0,con[0]).semisequent();
	ss0=ss0.insert(1,con[1]).semisequent();
	ss0=ss0.insert(2,con[2]).semisequent();
	ss1=ss1.insert(0,con[3]).semisequent();
	ss1=ss1.insert(1,con[4]).semisequent();
	seq=Sequent.createSequent(ss0,ss1);
	//	seq=Sequent.EMPTY_SEQUENT;
    }
       
    public void test1() {
	ProgramPrinter pp=new ProgramPrinter(null);
	SequentPrintFilter filter = new ConstraintSequentPrintFilter ( seq,
								       Constraint.BOTTOM );
	SequentPrinter sp=new SequentPrinter(pp, NotationInfo.createInstance(), filter);
	seq.prettyprint(sp);
	System.out.println(sp.toString());
	InitialPositionTable pt=sp.getPositionTable();
	System.out.println(pt);
	for (int i=-1; i<sp.toString().length()+2;i++) {
	    PosInSequent p=pt.getPosInSequent(i,filter);
	    System.out.println(i+"  "+pt.getPosInSequent(i,filter)
			       +p.getStart()+","+p.getEnd());
	}
    }


    public void test2() {
	TestSequentPrinter test=new TestSequentPrinter();
	ProgramPrinter pp=new ProgramPrinter(null);
	SequentPrinter printer=new SequentPrinter(pp, 
						  NotationInfo.createInstance(), 
						  seq);
	seq.prettyprint(printer);
	
	JFrame main_win = new JFrame("Sequentviewer - alpha version");	
	SequentView fv  = new SequentView(new KeYMediator(null));
	fv.setPrinter(printer);
	main_win.getContentPane().add(fv);
	main_win.setSize(400,200);
	main_win.show();
    }


    public static void main(String[] args) {
	TestSequentPrinter test=new TestSequentPrinter();
	test.setUp();
	test.test1();
	test.test2();
    }




}
