// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.java;

import recoder.java.declaration.*;
import recoder.list.*;
import recoder.java.*;
import de.uka.ilkd.key.java.recoderext.KeYCrossReferenceServiceConfiguration;

/** this class stores recoder related contextinformation used to parse
 * in program parts in which non-declared variables are used
 */
class Context {

    private static int counter=0;
    private recoder.java.CompilationUnit compilationUnitContext;
    private ClassDeclaration classContext;
    private static final ImportMutableList emptyImport=new ImportArrayList(0);
    private KeYCrossReferenceServiceConfiguration servConf;
    
    /** creates a new context object
     * @param compilationUnitContext a
     * recoder.java.CompilationUnit
     * @param classContext a recoder.java.declaration.ClassDeclaration
     */
    public Context(KeYCrossReferenceServiceConfiguration servConf, 
		   recoder.java.CompilationUnit compilationUnitContext, 
		   ClassDeclaration classContext) {
	this.compilationUnitContext = compilationUnitContext;
	this.classContext = classContext;
	this.servConf=servConf;
    }

    /** creates a new context object
     * @param compilationUnitContext a
     * recoder.java.declaration.CompilationUnit
     */
    public Context(KeYCrossReferenceServiceConfiguration servConf,
		   recoder.java.CompilationUnit compilationUnitContext) {
	this(servConf, compilationUnitContext, 
	     createClassDecl(servConf));
	this.servConf=servConf;
    }
    
    private static recoder.java.declaration.ClassDeclaration createClassDecl
	(KeYCrossReferenceServiceConfiguration servConf) {
	recoder.ProgramFactory pf=servConf.getProgramFactory();
	recoder.java.Identifier id= new recoder.java.Identifier("KeYSpecialParsing");
	recoder.java.declaration.ClassDeclaration cd
	    = pf.createClassDeclaration(null, id, null, null, null);
	return cd;
    }



    /** creates a new context object
     * @param classContext a recoder.java.declaration.ClassDeclaration
     */
    public Context(KeYCrossReferenceServiceConfiguration servConf, 
		   ClassDeclaration classContext) {
	this(servConf, createCompUnit(classContext),
	     classContext);
    }

    private static recoder.java.CompilationUnit createCompUnit
	(ClassDeclaration classContext) {
	recoder.java.CompilationUnit cu 
	    = new recoder.java.CompilationUnit
		(null, emptyImport,
		 inList(classContext));
	//	cu.setDataLocation(new ContextDataLocation("tmp"+counter++));
	return cu;
    }

    /** creates the empty context */
//     public Context() {
// 	this(
// 	     //new recoder.java.CompilationUnit(), 
// 	     new ClassDeclaration());
//     }

    public static TypeDeclarationMutableList inList(TypeDeclaration td) {
	TypeDeclarationMutableList tdml=new TypeDeclarationArrayList();
	tdml.add(td);
	return tdml;
    }

    /** returns the compilation context */
    public recoder.java.CompilationUnit getCompilationUnitContext() {
	return compilationUnitContext;
    }

    /** returns the compilation context */
    public ClassDeclaration getClassContext() {
	return classContext;
    }

    
}
