// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

// This file is part of the COMPOST library, (c) 1999 Universitaet Karlsruhe.

package de.uka.ilkd.key.java;

import java.io.*;

import de.uka.ilkd.key.util.ExtList;

/**
 *  Top level implementation of a Java {@link NonTerminalProgramElement}.
 * taken from COMPOST and changed to achieve an immutable structure
 */

public abstract class JavaNonTerminalProgramElement extends JavaProgramElement
 implements NonTerminalProgramElement {

    /**
     * Java program element.
     */

    public JavaNonTerminalProgramElement() {
    }

    /**
     * Java program element.
     * @param list as ExtList with children of the node
     */

    public JavaNonTerminalProgramElement(ExtList list) {
	super(list);
    }

    /**
     *        Extracts the index of a child from its position code.
     *        @param positionCode the position code.
     *        @return the index of the given position code.
     *        @see NonTerminalProgramElement#getChildPositionCode(ProgramElement)
     */
    public int getIndexOfChild(int positionCode) {
	return positionCode >> 4;
    }

    /**
 *        Extracts the role of a child from its position code.
 *        @param positionCode the position code.
 *        @return the role code of the given position code.
 *        @see NonTerminalProgramElement#getChildPositionCode(ProgramElement)
    */
    public int getRoleOfChild(int positionCode) {
	return positionCode & 15;
    }
    
    /**
 *        Returns the index of the given child, or <CODE>-1</CODE> if there is 
 *        no such child. The child is searched for by identity:
 *        <CODE>getChildAt(getIndexOfChild(x)) == x</CODE>.
 *        @param child the exact child to look for.
 *        @return the index of the given child, or <CODE>-1</CODE>.
    */
    public int getIndexOfChild(ProgramElement child) {
        int i;
        for (i = getChildCount() - 1; i >= 0 && (getChildAt(i) != child); i--);
        return i;
    }
    
    /**
     * returns the index of element el in array arr
     * @param arr the array where the element is looked for
     * @param el the element to look for 
     * @return the index of the element (-1 if not found)
     */
    protected int getArrayPos(ArrayOfProgramElement arr, ProgramElement el) {
	for (int i=0;i<arr.size();i++) {
	    if (arr.getProgramElement(i)==el) return i;
	} 
	return -1;
    } 

    /** commented in interface SourceElement. Overwrites the default
     * method implementation in ProgramElement by descending down to
     * the children.
     */
    public boolean equalsModRenaming(SourceElement se, 
				     NameAbstractionTable nat) {
	if (this.getClass()!=se.getClass()) {
	    return false;
	}
	if (((JavaNonTerminalProgramElement)se).
	    getChildCount()!=getChildCount()) {
	    return false;
	}	
	for (int i = 0; i<getChildCount(); i++) {
	    if (!(getChildAt(i)).equalsModRenaming
		(((JavaNonTerminalProgramElement)se).getChildAt(i), nat)) {
		return false;
	    }
	}
	return true;
    }
        
}
