// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

// This file is part of the COMPOST library, (c) 1999 Universitaet Karlsruhe.

package de.uka.ilkd.key.java;

import java.io.*;
import de.uka.ilkd.key.util.ExtList;

/**
 *  Top level implementation of a Java {@link ProgramElement}.
 * taken from COMPOST and changed to achieve an immutable structure
 */

public abstract class JavaProgramElement extends JavaSourceElement
                                         implements ProgramElement {

    /**
 *        Comments.
    */

    protected final Comment[] comments;


    /**
 *      Java program element.
     */

    public JavaProgramElement() {
	comments=null;
    }

    /**
     * Java program element. 
     * @param list ExtList with comments
     */

    public JavaProgramElement(ExtList list) {
	super(list);
	comments=(Comment[])list.collect(Comment.class);
    }

    /**
     *      Get comments.
     *      @return the comments.
     */

    public Comment[] getComments() {
        return comments;
    }

    /**
     *      Template Method. Prints DocComments if existing, calls
     *      prettyPrintMain and prints other Comments.
     */

    public void prettyPrint(PrettyPrinter w) throws IOException {
        int s = (comments != null) ? comments.length : 0;
        int t = 0;
        for (int i = 0; i < s; i += 1) {
            Comment c = comments[i];
            if (c.isPrefixed()) {
                c.prettyPrint(w);
            } else {
                t += 1;
            }
        }
        prettyPrintMain(w);
        if (t > 0) {
            for (int i = 0; i < s; i += 1) {
                Comment c = comments[i];
                if (!c.isPrefixed()) {
                    if (c instanceof SingleLineComment) {
                        w.scheduleComment((SingleLineComment)c);
                    } else {
                        c.prettyPrint(w);
                    }
                }
            }
        }
    }

    /**
     *      Prints main content of current node and all syntactical children.
     *      Hook method of prettyPrint; defaults to do nothing.
     */
    protected void prettyPrintMain(PrettyPrinter w) throws IOException {}


   /**
    * returns a copy of the given array
    * @param arr the array to be copied
    * @return the copied array
    */
    protected static Object[] copyArray(Object[] arr) {	
	if (arr!=null) {
	    Object res= java.lang.reflect.Array.newInstance(
			   arr.getClass().getComponentType(), 
			   arr.length);
	    System.arraycopy(arr, 0, res, 0, arr.length);
	    return (Object[])res;
	}
	return null;
    } 

    /** commented in interface SourceElement. The default equals
     * method compares  two elements by testing if they have the 
     * same type and calling the default equals method.
     */
    public boolean equalsModRenaming(SourceElement se, 
				     NameAbstractionTable nat) {
	if (this.getClass() != se.getClass()) {
	    return false;
	}
	return this.equals(se);
    }

}
