// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

// This file is part of the COMPOST library, (c) 1999 Universitaet Karlsruhe.

package de.uka.ilkd.key.java;

/**
 *  Non terminal program element.
 * taken from COMPOST and changed to achieve an immutable structure
 */

public interface NonTerminalProgramElement extends ProgramElement {

 /**
 *      Returns the number of children of this node.
 *      @return an int giving the number of children of this node
 */
    int getChildCount();

 /**
 *      Returns the child at the specified index in this node's "virtual"
 *      child array.
 *      @param index an index into this node's "virtual" child array
 *      @return the program element at the given position
 *      @exception ArrayIndexOutOfBoundsException if <tt>index</tt> is out
 *                 of bounds
 */
    ProgramElement getChildAt(int index);
    
 /**
 *        Returns the index of the given child, or <CODE>-1</CODE> if there is 
 *        no such child. The child is searched for by identity:
 *        <CODE>getChildAt(getIndexOfChild(x)) == x</CODE>.
 *        @param child the exact child to look for.
 *        @return the index of the given child, or <CODE>-1</CODE>.
 */
    int getIndexOfChild(ProgramElement child);
    
    /**
 *        Returns the positional code of the given child, or <CODE>-1</CODE> if 
 *        there is no such child. The result contains an encoding of the
 *        relative position of the child as well as the role it has been
 *        playing in this parent element. This information is required for
 *        proper undo of transformations and is to be delivered to the
 *        detached method of the ChangeHistory.
 *        @see recoder.service.ChangeHistory#detached
 *        @param child the exact child to look for.
 *        @return the positional code of the given child, or <CODE>-1</CODE>.
    */
    int getChildPositionCode(ProgramElement child);

    /**
 *        Extracts the index of a child from its position code.
 *        @param positionCode the position code.
 *        @return the index of the given position code.
 *        @see NonTerminalProgramElement#getChildPositionCode(ProgramElement)
    */
    int getIndexOfChild(int positionCode);


    /**
 *        Extracts the role of a child from its position code.
 *        @param positionCode the position code.
 *        @return the role code of the given position code.
 *        @see NonTerminalProgramElement#getChildPositionCode(ProgramElement)
    */
    int getRoleOfChild(int positionCode);
}
