// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.java;
import de.uka.ilkd.key.java.abstraction.Field;
import de.uka.ilkd.key.java.abstraction.ListOfField;
import de.uka.ilkd.key.java.abstraction.SLListOfField;
import de.uka.ilkd.key.java.abstraction.IteratorOfField;
import de.uka.ilkd.key.java.abstraction.Type;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;
import de.uka.ilkd.key.java.abstraction.PrimitiveType;
import de.uka.ilkd.key.java.abstraction.ClassType;
import de.uka.ilkd.key.java.abstraction.ArrayType;
import de.uka.ilkd.key.java.abstraction.NullType;
import de.uka.ilkd.key.java.declaration.*;
import de.uka.ilkd.key.java.declaration.modifier.*;
import de.uka.ilkd.key.java.expression.*;
import de.uka.ilkd.key.java.expression.literal.*;
import de.uka.ilkd.key.java.expression.operator.*;
import de.uka.ilkd.key.java.statement.*;
import de.uka.ilkd.key.java.recoderext.ImplicitIdentifier;
import de.uka.ilkd.key.java.recoderext.KeYCrossReferenceSourceInfo;
import de.uka.ilkd.key.java.recoderext.KeYCrossReferenceServiceConfiguration;
import de.uka.ilkd.key.java.recoderext.RecoderModelTransformer;
import de.uka.ilkd.key.java.recoderext.ImplicitFieldAdder;
import de.uka.ilkd.key.java.recoderext.ImplicitMethodAdder;
import de.uka.ilkd.key.java.reference.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.sort.*;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.parser.*;
import de.uka.ilkd.key.util.ExtList;
import de.uka.ilkd.key.util.Debug;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Iterator;
import java.io.*;

public class Recoder2KeY implements JavaReader{
    
    private final static ProgramElementName createObjectName = new ProgramElementName("<createObject>");
    private final static ProgramElementName createArrayName = new ProgramElementName("<createArray>");
    private final static ProgramElementName prepareName = new ProgramElementName("<prepare>");
    private final static ProgramElementName prepareParentName = new ProgramElementName("<prepareParent>");

    protected HashMap implicitMethodCache = new HashMap();

    /**
     *   mapping from recoder.abstraction.Type to KeYJavaType
     */
    protected HashMap type2KeYType = new HashMap();

    /**
     *  methodsDeclaring contains the recoder method declarations as keys 
     *  that have been started to convert but are not yet finished.
     *  The mapped value is the reference to the later completed 
     *  ProgramMethod.
     */
    protected HashMap methodsDeclaring = new HashMap();

    protected KeYRecoderMapping rec2key = new KeYRecoderMapping();
    protected TypeConverter typeConverter;
    protected KeYCrossReferenceServiceConfiguration servConf;
    protected recoder.java.ProgramElement progElem;

    private NamespaceSet namespaces = new NamespaceSet ();

    private static int interactCounter = 0;

    public Recoder2KeY(KeYCrossReferenceServiceConfiguration servConf,
		       KeYRecoderMapping rec2key, TypeConverter tc) {
	this(servConf, null, null, rec2key, tc, new NamespaceSet());
    }

    public Recoder2KeY(KeYRecoderMapping rec2key, TypeConverter tc) {
	this(new KeYCrossReferenceServiceConfiguration(), 
	     null, null, rec2key, tc, new NamespaceSet());
    }

    public Recoder2KeY(KeYRecoderMapping rec2key, NamespaceSet nss,
		       TypeConverter tc) {
	this(new KeYCrossReferenceServiceConfiguration(),
	     null, null, rec2key, tc, nss);
    }

    public Recoder2KeY(KeYCrossReferenceServiceConfiguration servConf,
		       KeYRecoderMapping rec2key, NamespaceSet nss,
		       TypeConverter tc) {
	this(servConf, null, null, rec2key, tc, nss);
    }


    public Recoder2KeY(KeYCrossReferenceServiceConfiguration servConf,
			  recoder.java.ProgramElement progElem,
			  String classPath,
			  KeYRecoderMapping rec2key,
			  TypeConverter tc,
			  NamespaceSet nss) {
	this.servConf = servConf;
	this.progElem = progElem;
	this.typeConverter = tc;
	this.rec2key = rec2key;		
	
	if(classPath != null) {
	    servConf.getProjectSettings().setProperty
		(recoder.io.PropertyNames.INPUT_PATH,
		 classPath);
	}
	if (servConf.getProjectSettings().
	    ensureSystemClassesAreInPath()) {
	} else {
 	    System.err.println("System classes not found in path.");
	}
	namespaces=nss;	
    }


    public Recoder2KeY(recoder.java.ProgramElement progElem,
		       String classPath, 
		       KeYRecoderMapping rec2key,
		       TypeConverter tc) {
	this(new KeYCrossReferenceServiceConfiguration(), 
	     progElem, classPath, rec2key, tc, new NamespaceSet());
    }

    public Recoder2KeY(recoder.java.ProgramElement progElem,
		       KeYRecoderMapping rec2key,
		       TypeConverter tc) {
	this(progElem, null, rec2key, tc);
    }

    public Recoder2KeY(String classPath, KeYRecoderMapping rec2key,
		       TypeConverter tc) {
	this(null, classPath, rec2key, tc);
    }

    public Recoder2KeY(String classPath,
		       KeYRecoderMapping rec2key,
		       TypeConverter tc, NamespaceSet namespaces) {
	this(null, classPath, rec2key, tc);
	 this.namespaces = namespaces;
    }
    
    private TypeConverter typeConverter() {
	return typeConverter;
    }
    
    /**
     * returns a javaInfo object with this mapping produced by recoder2key
     */
    public JavaInfo createJavaInfo(Services s) {
	KeYProgModelInfo kpmi=new KeYProgModelInfo(getServiceConfiguration(),
						   rec2key(),
						   typeConverter());
	s.getJavaInfo().setKeYProgModelInfo(kpmi);	
	return s.getJavaInfo();
    }


    public KeYCrossReferenceServiceConfiguration getServiceConfiguration() {
	return servConf;
    }
    
    
    public KeYRecoderMapping rec2key() {
	return rec2key;
    }

    
    /**
     * wraps a RECODER StatementBlock in a method
     * @param block the recoder.java.StatementBlock to wrap
     * @return the enclosing recoder.java.MethodDeclaration
     */
    protected recoder.java.declaration.MethodDeclaration 
	embedBlock(recoder.java.StatementBlock block) {

	/* 
	   MethodDeclaration(modifiers,return type,Identifier, parameters,
	                     throws, StatementBlock)
	 */
	recoder.java.declaration.MethodDeclaration mdecl=new 
	    recoder.java.declaration.MethodDeclaration
	    (null, null, new recoder.java.Identifier("SOMEMETHOD"), 
	     null, null, block);
	mdecl.makeParentRoleValid();
	return mdecl;
    }  


    /**
     * wraps a RECODER MethodDeclaration in a class
     * @param mdecl the recoder.java.declaration.MethodDeclaration to wrap
     * @param classContext the recoder.java.declaration.ClassDeclaration
     * where the method has to be embedded
     * @return the enclosing recoder.java.declaration.ClassDeclaration
     */
    protected recoder.java.declaration.ClassDeclaration
	embedMethod(recoder.java.declaration.MethodDeclaration mdecl,
		    Context context) {

	recoder.java.declaration.ClassDeclaration classContext = 
	    context.getClassContext();

	// add method to memberdeclaration list
	recoder.list.MemberDeclarationMutableList memberList = 
	    classContext.getMembers();
	if (memberList == null) {
	    memberList = new recoder.list.MemberDeclarationArrayList(1); 
	    classContext.setMembers(memberList);
	} 
	for (int i=0; i<memberList.size(); i++) {
	    if (memberList.getMemberDeclaration(i) 
		instanceof recoder.java.declaration.MethodDeclaration) {
		recoder.java.declaration.MethodDeclaration olddecl
		    = (recoder.java.declaration.MethodDeclaration) 
		    memberList.getMemberDeclaration(i);
		if (olddecl.getName().equals(mdecl.getName())) {
		    memberList.remove(i);
		}
	    }
	}
	memberList.add(mdecl);

	// add method to class

	classContext.setProgramModelInfo(servConf.getCrossReferenceSourceInfo()); 
	classContext.makeParentRoleValid();	
	return classContext;
    }  


    protected recoder.list.CompilationUnitMutableList recoderCompilationUnits
	(String[] cUnitStrings) {
	recoder.util.Debug.setLevel(500);
	parseSpecialClasses();
	recoder.list.CompilationUnitMutableList cUnits
	    = new recoder.list.CompilationUnitArrayList();	
	int i = 0;
	try {
	    for (i=0; i<cUnitStrings.length; i++) {
		Debug.out("Reading ", cUnitStrings[i]);
		cUnits.add(servConf.getProgramFactory().parseCompilationUnit
			   (new StringReader(cUnitStrings[i]))); 
	    }
	    // run cross referencer
	    for (i=0; i<cUnits.size(); i++) {
		cUnits.getCompilationUnit(i).makeAllParentRolesValid();
		servConf.getChangeHistory().attached(cUnits.getCompilationUnit(i));
	    }
	    servConf.getChangeHistory().updateModel();
 	    // transform program
	    transformModel(cUnits);
	} catch(IOException ioe) {
	    Debug.out("recoder2key: IO Error when reading"+
		      "compilation unit (unit, exception) ", 
		      cUnitStrings[i], ioe);		
	    throw new ConvertException
		("IOError when trying to read"+
		 " a java program " + cUnitStrings[i] +
		 " Thrown Exception:" + ioe.toString());
	} catch(recoder.ParserException pe) {
	    Debug.out("recoder2key: Recoder Parser Error when" +
		      "reading a comiplation unit (unit, exception)",
		      cUnitStrings[i], pe);		
	    //e.printStackTrace();
	    throw new ConvertException
		("Parser error when reading the java program "+
		 cUnitStrings[i] + " Thrown Exception:" + pe);
	}
	
		
	return cUnits;
    }

    protected recoder.list.CompilationUnitMutableList 
	recoderCompilationUnitsAsFiles(String[] cUnitStrings) {
	recoder.list.CompilationUnitMutableList cUnits
	    = new recoder.list.CompilationUnitArrayList();
	parseSpecialClasses();
	try {
	    cUnits=servConf.getProgramFactory().parseCompilationUnits(cUnitStrings);
	    for (int i=0; i<cUnits.size(); i++) {
		cUnits.getCompilationUnit(i).makeAllParentRolesValid();
		servConf.getChangeHistory().attached(cUnits.getCompilationUnit(i));
	    }
 	    servConf.getChangeHistory().updateModel();
	    // transform program
	    transformModel(cUnits);
	} catch (recoder.service.AmbiguousDeclarationException ade) {
	    Debug.out("recoder2key: Ambigious declaration.\n", ade);		
 	    Debug.printStackTrace(ade);
	    throw new ConvertException
		("A class/field/variable has been declared more than once." +
		 ade);
	} catch(recoder.ParserException pe) {
	    Debug.out("recoder2key: Recoder Parser Error when" +
		      "reading comiplation units. Exception:", pe);		
 	    Debug.printStackTrace(pe);
	    throw new ConvertException
		("Parser error when reading java files."+
		 "Thrown Exception:" + pe);
	}
	return cUnits;
    }

    /**
     * as we do not support lemmats yet we need the following implementations
     */
    // %%% HACK FME 
    private final String[] basicClasses() {
	return new String[] 
	{"class KeYSpecialParsing { Exception e; }",

	 "package java.lang; public interface Cloneable {}",

	 "package java.lang; public class Throwable "+
	 "extends Object { public Throwable(){} }",

	 "package java.lang; public class Exception "+
	 "extends java.lang.Throwable { public Exception(){} }",

	 "package java.lang; public class RuntimeException "+
	 "extends Exception { public RuntimeException(){} }",

	 "package java.lang; public class NegativeArraySizeException "+
	 "extends RuntimeException { public NegativeArraySizeException(){} }",

	 "package java.lang; public class NullPointerException "+
	 "extends RuntimeException { public NullPointerException(){} }",

	 "package java.lang; public class IndexOutOfBoundException "+
	 "extends RuntimeException { public IndexOutOfBoundException(){} }",

	 "package java.lang; public class ArrayIndexOutOfBoundsException "+
	 "extends IndexOutOfBoundException { public ArrayIndexOutOfBoundsException(){} }"	,

	 "package java.lang; public class IllegalArgumentException "+
	 "extends RuntimeException { public IllegalArgumentException(){} }"	,

	 "package java.lang; public class IllegalStateException "+
	 "extends RuntimeException { public IllegalStateException(){} public IllegalStateException(String message){} }"	,

	 "package java.lang; public class ArithmeticException "+
	 "extends RuntimeException { public ArithmeticException(){} }",

	 "package java.io; public interface Serializable {}"
	};
    }

    /**
     * adds a special compilation unit containing references to types that
     * have to be available in Recoder and KeY form, e.g. Exceptions
     */
    private recoder.list.CompilationUnitMutableList parseSpecial() {
	recoder.ProgramFactory pf = servConf.getProgramFactory();
	recoder.list.CompilationUnitMutableList rcuList =
	    new recoder.list.CompilationUnitArrayList();
	recoder.java.CompilationUnit rcu = null;
	try {
	    String[] classes = basicClasses();
	    for (int i = 0; i<classes.length; i++) {
		rcu = pf.parseCompilationUnit(classes[i]); 
		rcu.makeAllParentRolesValid();
		rcuList.add(rcu);		
	    }
	} catch (recoder.ParserException e) {
	    System.err.println("recoder2key: Error while parsing specials");
	    System.err.println("recoder2key: Try to continue...");
	}
	return rcuList;
    }

    /** 
     * if not parsed yet the special classes are read in and converted 
     */
    public void parseSpecialClasses() {
	if (rec2key.parsedSpecial()) {
	    return;
	}
	recoder.list.
	    CompilationUnitMutableList specialClasses = parseSpecial();
	for (int i=0; i<specialClasses.size(); i++) {
	    specialClasses.getCompilationUnit(i).makeAllParentRolesValid();
	    servConf.getChangeHistory().attached(specialClasses.getCompilationUnit(i));
	}
	servConf.getChangeHistory().updateModel();
	transformModel(specialClasses);
	for (int i=0; i<specialClasses.size(); i++) {
	    callConvert(specialClasses.getCompilationUnit(i));
	}
	rec2key().parsedSpecial(true);
    }
    

    public CompilationUnit[] readCompilationUnits(String[] cUnitStrings) {
	recoder.list.CompilationUnitMutableList cUnits
	= recoderCompilationUnits(cUnitStrings);
	CompilationUnit[] result = new CompilationUnit[cUnits.size()];
	for (int i=0; i<cUnits.size(); i++) {
	    result[i]=(CompilationUnit) convert(cUnits.getCompilationUnit(i));
	}
	return result;
    }

    public CompilationUnit[] readCompilationUnitsAsFiles(String[] cUnitStrings) {
	recoder.list.CompilationUnitMutableList cUnits
	= recoderCompilationUnitsAsFiles(cUnitStrings);
	CompilationUnit[] result = new CompilationUnit[cUnits.size()];
	for (int i=0; i<cUnits.size(); i++) {
	    //	    Debug.out("R2K: ", cUnitStrings[i]);
	    result[i]=(CompilationUnit) convert(cUnits.getCompilationUnit(i));
	}
	return result;
    }

    public CompilationUnit readCompilationUnit(String cUnitString) { 
	recoder.java.CompilationUnit cc=recoderCompilationUnits
	    (new String[]{cUnitString}).getCompilationUnit(0);
	return (CompilationUnit) convert(cc);
    }


    private String trim(String s) {
	if (s.length()>100) return s.substring(0,150)+"[...]";
	return s;
    }


    /** 
     * parses a given JavaBlock using the context to determine the right 
     * references and returns a statement block of recoder.
     * @param block a String describing a java block
     * @param context recoder.java.CompilationUnit in which the block has 
     * to be interpreted
     * @return the parsed and resolved recoder statement block
     */
    protected recoder.java.StatementBlock recoderBlock(String block,
						       Context context) {
	recoder.java.StatementBlock bl = null;       
	parseSpecialClasses();
	try {
	    bl = servConf.getProgramFactory().
		parseStatementBlock(new StringReader(block)); 
	    bl.makeAllParentRolesValid();
	    embedMethod(embedBlock(bl), context);
	    context.getCompilationUnitContext().makeParentRoleValid();
 	    servConf.getCrossReferenceSourceInfo().register(bl);
 	    servConf.getChangeHistory().attached(context.getCompilationUnitContext());   	
 	    servConf.getChangeHistory().updateModel();
	} catch (recoder.service.UnresolvedReferenceException e) {
            e.printStackTrace();        
	    // Attention: this highly depends on Recoders exception messages!
	    String errorMessage = e.getMessage(); 
	    String pos = errorMessage.substring(errorMessage.indexOf("@")+1);
	    pos = pos.substring(0, pos.indexOf(" ")); 
	    int line   = -1;
	    int column = -1;
	    try {
		line   = Integer.parseInt(pos.substring(0,pos.indexOf("/")));
		column = Integer.parseInt(pos.substring(pos.indexOf("/")+1));
	    } catch (NumberFormatException nfe) {
		Debug.out("recoder2key:unresolved reference at "+
			  "line:"+line+" column:"+column);
		throw new ConvertException
		    ("Some program elements are undeclared."+e);
	    }
	    throw new PosConvertException
		("Program element undeclared.", line, column);
	} catch (recoder.parser.ParseException pe) {
	    throw new ConvertException(pe);
	} catch (recoder.ParserException e) {
	    throw new ConvertException
		("Recoder parser threw exception:"+e);
	} catch (IOException e) {
	    Debug.out("recoder2key: IOException detected. "+
		      "(parsed program, IOException)", block, e);
	    if (block.length()>20) block = block.substring(0,20)+"...";
	    throw new ConvertException
		("When trying to read "+block+": "+trim(e.getMessage()));
	} 
	return (recoder.java.StatementBlock)bl;
    } 

    /** parses a given JavaBlock using the context to determine the right 
     * references
     * @param block a String describing a java block
     * @param context recoder.java.CompilationUnit in which the block has 
     * to be interprested
     * @return the parsed and resolved JavaBlock
     */
    public JavaBlock readBlock(String block,
			       Context context) { 
	JavaBlock jb = JavaBlock.createJavaBlock
	    ((JavaProgramElement)callConvert(recoderBlock
					     (block, context)));
	return jb;
    }

    /** parses a given JavaBlock using the context to determine the right 
     * references using an empty context
     * @param block a String describing a java block
     * @return the parsed and resolved JavaBlock
     */
    public JavaBlock readBlockWithEmptyContext(String block) { 
	return readBlock(block, createEmptyContext());
    }

    public JavaBlock readBlockWithProgramVariables(Namespace varns, String s) {
	IteratorOfNamed it = varns.allElements().iterator();
	ListOfProgramVariable pvs = SLListOfProgramVariable.EMPTY_LIST;
	while (it.hasNext()) {
	    Named n = it.next();
	    if (n instanceof ProgramVariable) {
		pvs=pvs.prepend((ProgramVariable)n);
	    }
	}
	Context ctx = createContext(pvs);
	return readBlock(s, ctx);
    }

    protected void insertToMap(recoder.ModelElement r,
			     ModelElement k) {
	
	if (r != null) {
	    rec2key.put(r, k);
	} else {
	    // commented out because they caused an exception (they were thrown at array references)
// 	    Debug.out("Rec2Key.insertToMap : Omitting entry  (r = " + r + " -> k = " + k + ")");
// 	    Debug.out("Maybe there is a bug !!");
	}
    }


    /** determines the right convert method using reflection
     * @param pe the recoder.java.JavaProgramElement to be converted
     * @return the converted element
     */
    protected Object callConvert(recoder.java.ProgramElement pe) { 

	Debug.assertTrue(pe!=null);
	Method m = null;
	Class[] context = new Class[]{pe.getClass()};

	while (m == null && context[0]!=null) {	    
	    try {
		m = getClass().getMethod("convert", context);
	    } catch (NoSuchMethodException nsme) {
		context[0] = context[0].getSuperclass();
		Debug.out("recoder2key: method not found. " +
			  "Next try with ", context[0]);		
	    }
	}
	Object o = null;
	try {
	    o = m.invoke(this, new Object[]{pe});
	} catch (IllegalAccessException iae) {
	    Debug.out("recoder2key: cannot access method ", iae);
	    throw new ConvertException("recoder2key: cannot access method" + iae);
	} catch (IllegalArgumentException iarg) {
	    Debug.out("recoder2key: wrong method arguments ", iarg);
	    throw new ConvertException("recoder2key: wrong method arguments" + iarg);
	} catch (InvocationTargetException ite) {
	    Debug.out("recoder2key: called method threw exception ", 
		      ite.getTargetException());
	    if (ite.getTargetException() instanceof ConvertException) {
		throw (ConvertException)ite.getTargetException();
	    } else {
		ite.getTargetException().printStackTrace();
		throw new ConvertException
		    ("recoder2key: called method "+ m + " threw exception:" + 
		     ite.getTargetException());
	    }
	}
	return o;
    }

    protected ProgramElement doConvert(recoder.java.JavaProgramElement pe) {
	ExtList el = new ExtList();
	el.add(callConvert(pe));       
	return (ProgramElement)el.get(0);
    }

    /** 
     * constructs the name of the corresponding KeYClass
     * @param recoderClass Class that is the original recoder 
     * @return String containing the KeY-Classname
     */
    protected String getKeYName(Class recoderClass) {
	// value of recoderPrefixLength is: "recoder.".length()
	final int recoderPrefixLength = 8;
	return "de.uka.ilkd.key." + 
	    recoderClass.getName().substring(recoderPrefixLength);
    }

    
    /** gets the KeY-Class related to the recoder one
     * @param recoderClass the original Class
     * @return the related KeY Class 
     */
    protected Class getKeYClass(Class recoderClass) {
	try {
	    return Class.forName(getKeYName(recoderClass));
	} catch (ClassNotFoundException cnfe) {
	    Debug.out("There is an AST class missing at KeY.", cnfe);
	    throw new ConvertException
		("There is an AST class missing at KeY. " + cnfe);
	} catch (ExceptionInInitializerError initErr) {
	    Debug.out("recoder2key: Failed initializing class ", initErr);
	    Debug.fail();
	} catch (LinkageError le) {
	    Debug.out("recoder2key: Linking class failed.", le);
	    Debug.fail();
	}
	return null;
    }

    /** determines the right standard constructor of the KeYClass
     * @param recoderClass the Class of the recoder AST object
     * @return the Constructor of the right KeY-Class
     */
    protected Constructor getKeYClassConstructor(Class recoderClass) {
	Constructor result=null;
	try {
	    result=getKeYClass(recoderClass).getConstructor
		(new Class[]{ExtList.class}); 
	} catch (NoSuchMethodException nsme) {
	    Debug.out("recoder2key: constructor not found. ", nsme);
	} catch (SecurityException se) {
	    Debug.out("recoder2key: access denied. ", se);
	}
	return result;
    }

//     /** collects children and adds their converted KeY-counterpart to
//      * the list of childrem
//      * @param pe the JavaNonTerminalProgramElement that needs its
//      * children before being converted
//      * @param children an ExtList with the children of pe
//      * @return the list of children after conversion
//      */
//     protected ExtList
// 	collectChildren(recoder.java.JavaNonTerminalProgramElement pe) { 
// 	return collectChildren((recoder.java.NonTerminalProgramElement) pe);
//     }

    /** collects children and adds their converted KeY-counterpart to
     * the list of childrem
     * @param pe the NonTerminalProgramElement that needs its
     * children before being converted
     * @param children an ExtList with the children of pe
     * @return the list of children after conversion
     */
    protected ExtList collectChildren
	(recoder.java.NonTerminalProgramElement pe) {
	ExtList children = new ExtList();
	for (int i=0; i<pe.getChildCount(); i++) {
	    children.add(callConvert(pe.getChildAt(i)));
	}
	children.add(positionInfo(pe));
	return children;
    }

    protected PositionInfo positionInfo(recoder.java.SourceElement se) {
	Position relPos   = new Position(se.getRelativePosition().getLine(),
				     se.getRelativePosition().getColumn());
	Position startPos = new Position(se.getStartPosition().getLine(),
				       se.getStartPosition().getColumn());
	Position endPos   = new Position(se.getEndPosition().getLine(),
				     se.getEndPosition().getColumn());
	return new PositionInfo(relPos, startPos, endPos);
    }

    /**
     * the standard case. 
     * @param pe the recoder.java.ProgramElement to be converted 
     * @return the converted de.uka.ilkd.key.java.JavaProgramElement 
     */
    public ProgramElement convert(recoder.java.JavaProgramElement pe) {
	ProgramElement result = null;
	ExtList parameter; 

	if (pe instanceof recoder.java.JavaNonTerminalProgramElement) {
	    parameter = 
		collectChildren((recoder.java.JavaNonTerminalProgramElement) pe);
	} else {
	    parameter = new ExtList();
	}
	try {
	    result = (ProgramElement) getKeYClassConstructor(pe.getClass()).
		newInstance(new Object[]{parameter});
	} catch (InstantiationException e) {       
	    Debug.out("recoder2key: invocation of constructor failed.", e);
	} catch (InvocationTargetException ite) {       
	    Debug.out("recoder2key: invocation of constructor failed.",
		      ite.getTargetException());
	} catch (IllegalAccessException iae) {       
	    Debug.out("recoder2key: access denied.", iae);
	} catch (IllegalArgumentException iae) {       
	    Debug.out("recoder2key: illegal arguments.", iae);
	}
	return result;
    }

    // ------------------- operators ----------------------
    
    /** 
     * converts the recoder.java.expression.operator.NewArray
     * node to the KeYDependance
     */
    public NewArray convert(recoder.java.expression.operator.NewArray newArr){
	// first we need to collect all children
	ExtList children=collectChildren(newArr);
	// now we have to extract the array initializer
	// is stored separately and must not appear in the children list
	ArrayInitializer arrInit=(ArrayInitializer)
	    children.get(ArrayInitializer.class);
	children.remove(arrInit);	
		
	recoder.abstraction.Type javaType = getServiceConfiguration().
	    getCrossReferenceSourceInfo().getType(newArr);

	return new NewArray(children, getKeYJavaType(javaType), arrInit, 
			    newArr.getDimensions());
    }

    /** 
     * converts the recoder.java.expression.operator.CopyAssignment
     * node to the KeYDependance
     */
    public CopyAssignment convert(recoder.java.expression.operator.CopyAssignment ass){
	// first we need to collect all children
	ExtList children = collectChildren(ass);
	return new CopyAssignment(children);
    }

    /** 
     * converts the recoder.java.expression.operator.PostIncrement
     * node to the KeYDependance
     */
    public PostIncrement convert
	(recoder.java.expression.operator.PostIncrement postInc) {
	// first we need to collect all children
	ExtList children = collectChildren(postInc);
	return new PostIncrement(children);
    }

    /** 
     * converts the recoder.java.expression.operator.PreIncrement
     * node to the KeYDependance
     */
    public PreIncrement convert
	(recoder.java.expression.operator.PreIncrement preInc){
	// first we need to collect all children
	ExtList children = collectChildren(preInc);
	return new PreIncrement(children);
    }

    /** 
     * converts the recoder.java.expression.operator.PostDecrement
     * node to the KeYDependance
     */
    public PostDecrement convert
	(recoder.java.expression.operator.PostDecrement postDec) {
	// first we need to collect all children
	ExtList children = collectChildren(postDec);
	return new PostDecrement(children);
    }

    /** 
     * converts the recoder.java.expression.operator.PreDecrement
     * node to the KeYDependance
     */
    public PreDecrement convert
	(recoder.java.expression.operator.PreDecrement preDec){
	// first we need to collect all children
	ExtList children = collectChildren(preDec);
	return new PreDecrement(children);
    }

    /** 
     * converts the recoder.java.expression.operator.Minus
     * node to the KeYDependance
     */
    public Minus convert
	(recoder.java.expression.operator.Minus minus){
	// first we need to collect all children
	ExtList children = collectChildren(minus);
	return new Minus(children);
    }


    /** 
     * converts the recoder.java.expression.operator.Plus
     * node to the KeYDependance
     */
    public Plus convert
	(recoder.java.expression.operator.Plus plus){
	// first we need to collect all children
	ExtList children = collectChildren(plus);
	return new Plus(children);
    }

    /** 
     * converts the recoder.java.expression.operator.Times
     * node to the KeYDependance
     */
    public Times convert
	(recoder.java.expression.operator.Times times){
	// first we need to collect all children
	ExtList children = collectChildren(times);
	return new Times(children);
    }

    /** 
     * converts the recoder.java.expression.operator.Divide
     * node to the KeYDependance
     */
    public Divide convert
	(recoder.java.expression.operator.Divide div){
	// first we need to collect all children
	ExtList children = collectChildren(div);
	return new Divide(children);
    }


    /** 
     * converts the recoder.java.expression.operator.PlusAssignment
     * node to the KeYDependance
     */
    public PlusAssignment convert
	(recoder.java.expression.operator.PlusAssignment plus){
	// first we need to collect all children
	ExtList children = collectChildren(plus);
	return new PlusAssignment(children);
    }

    /** 
     * converts the recoder.java.expression.operator.MinusAssignment
     * node to the KeYDependance
     */
    public MinusAssignment convert
	(recoder.java.expression.operator.MinusAssignment minus){
	// first we need to collect all children
	ExtList children = collectChildren(minus);
	return new MinusAssignment(children);
    }

    /** 
     * converts the recoder.java.expression.operator.TimesAssignment
     * node to the KeYDependance
     */
    public TimesAssignment convert
	(recoder.java.expression.operator.TimesAssignment times){
	// first we need to collect all children
	ExtList children = collectChildren(times);
	return new TimesAssignment(children);
    }

    /** 
     * converts the recoder.java.expression.operator.DivideAssignment
     * node to the KeYDependance
     */
    public DivideAssignment convert
	(recoder.java.expression.operator.DivideAssignment div){
	// first we need to collect all children
	ExtList children = collectChildren(div);
	return new DivideAssignment(children);
    }

    /** 
     * converts the recoder.java.expression.operator.LessThan
     * node to the KeYDependance
     */
    public LessThan convert
	(recoder.java.expression.operator.LessThan op){
	// first we need to collect all children
	ExtList children = collectChildren(op);
	return new LessThan(children);
    }

    /** 
     * converts the recoder.java.expression.operator.LessOrEquals
     * node to the KeYDependance
     */
    public LessOrEquals convert
	(recoder.java.expression.operator.LessOrEquals op){
	// first we need to collect all children
	ExtList children = collectChildren(op);
	return new LessOrEquals(children);
    }

    /** 
     * converts the recoder.java.expression.operator.GreaterThan
     * node to the KeYDependance
     */
    public GreaterThan convert
	(recoder.java.expression.operator.GreaterThan op){
	// first we need to collect all children
	ExtList children = collectChildren(op);
	return new GreaterThan(children);
    }

    /** 
     * converts the recoder.java.expression.operator.GreaterOrEquals
     * node to the KeYDependance
     */
    public GreaterOrEquals convert
	(recoder.java.expression.operator.GreaterOrEquals op){
	// first we need to collect all children
	ExtList children = collectChildren(op);
	return new GreaterOrEquals(children);
    }

    /** 
     * converts the recoder.java.expression.operator.Equals
     * node to the KeYDependance
     */
    public Equals convert
	(recoder.java.expression.operator.Equals op){
	// first we need to collect all children
	ExtList children = collectChildren(op);
	return new Equals(children);
    }

    /** 
     * converts the recoder.java.expression.operator.NotEquals
     * node to the KeYDependance
     */
    public NotEquals convert
	(recoder.java.expression.operator.NotEquals op){
	// first we need to collect all children
	ExtList children = collectChildren(op);
	return new NotEquals(children);
    }


    //------------------- literals --------------------------------------
    

    /** convert a recoder IntLiteral to a KeY IntLiteral */
    public IntLiteral convert
	(recoder.java.expression.literal.IntLiteral intLit) {
	// if there are comments to take into consideration 
	// change parameter to ExtList
	return new IntLiteral(intLit.getValue());
    }
    
    /** convert a recoder BooleanLiteral to a KeY BooleanLiteral */
    public BooleanLiteral 
	convert(recoder.java.expression.literal.BooleanLiteral booleanLit) {

	// if there are comments to take into consideration 
	// change parameter to ExtList
	return (booleanLit.getValue() ? BooleanLiteral.TRUE : BooleanLiteral.FALSE);
    }


    /** convert a recoder StringLiteral to a KeY StringLiteral */
    public StringLiteral 
	convert(recoder.java.expression.literal.StringLiteral stringLit) {

	// if there are comments to take into consideration 
	// change parameter to ExtList
	return new StringLiteral(stringLit.getValue());
    }

    /** convert a recoder DoubleLiteral to a KeY DoubleLiteral */
    public DoubleLiteral 
	convert(recoder.java.expression.literal.DoubleLiteral doubleLit) {

	// if there are comments to take into consideration 
	// change parameter to ExtList
	return new DoubleLiteral(doubleLit.getValue());
    }

    /** convert a recoder FloatLiteral to a KeY FloatLiteral */
    public FloatLiteral 
	convert(recoder.java.expression.literal.FloatLiteral floatLit) {

	// if there are comments to take into consideration 
	// change parameter to ExtList
	return new FloatLiteral(floatLit.getValue());
    }

    /** convert a recoder LongLiteral to a KeY LongLiteral */
    public LongLiteral 
	convert(recoder.java.expression.literal.LongLiteral longLit) {

	// if there are comments to take into consideration 
	// change parameter to ExtList
	return new LongLiteral(longLit.getValue());
    }

    /** convert a recoder CharLiteral to a KeY CharLiteral */
    public CharLiteral 
	convert(recoder.java.expression.literal.CharLiteral charLit) {

	// if there are comments to take into consideration 
	// change parameter to ExtList

	return new CharLiteral(charLit.getValue());
    }


    /** convert a recoder NullLiteral to a KeY NullLiteral */
    public NullLiteral 
	convert(recoder.java.expression.literal.NullLiteral nullLit) {

	// if there are comments to take into consideration 
	// change parameter to ExtList
	return NullLiteral.NULL;
    }



    //----------------------------------------------------------

    
    /** convert a recoder EmptyStatement to a KeY EmptyStatement*/
    public EmptyStatement convert(recoder.java.statement.EmptyStatement
				       eStmnt) {	
	// may change if comments are implemented, then 
	// new EmptyStatement(children);
	return new EmptyStatement();
    }
	
    /** convert a recoder EmptyStatement to a KeY EmptyStatement*/    
    public ProgramElementName convert(recoder.java.Identifier id) {

	return new ProgramElementName(id.getText());
    }

    //------------------- operators -------------------------------- //

    public Instanceof convert
	(recoder.java.expression.operator.Instanceof rio) {       
	return new Instanceof((Expression)doConvert
			      ((recoder.java.JavaProgramElement)
			       rio.getExpressionAt(0)),
			      (TypeReference)doConvert(rio.getTypeReference()));

    }

    //------------------- declaration -------------------------------- //


    /**
     * creates a method reference with prefix <code>super</code>. The
     * name of the referenced method, a reference to the class type
     * where the method reference occurs (execution context) and a
     * reference to the object on which the method is invoked are given
     * as parameters.
     * @param name a String with the method's name
     * @param classRef a TypeReference to the class where the method
     * reference is placed
     * @param invocationTarget a Reference to the object on which the
     * method is invoked.
     * @return the constructed method reference
     */
    protected final MethodReference methodReference
	(ProgramElementName name, TypeRef classRef, 
	 ReferencePrefix invocationTarget) {
	return new MethodReference
	    (new ArrayOfExpression(new Expression[0]), 
	     name, 
	     new SuperReference((ReferencePrefix)null, 
				new ExecutionContext(classRef, 
						     invocationTarget)), 
	     null);
    }
    
    /** 
     * creates an assignment <code> lhs:=rhs </code>
     */
    protected final Statement assign(Expression lhs, Expression rhs) {
	return new CopyAssignment(lhs, rhs);
    }

    /**
     * creates an attribute access <code>prefix.field </code>
     */
    protected final Expression attribute(ReferencePrefix prefix, 
					 ProgramVariable field) {
	return new FieldReference(field, prefix);
    }

    /**
     * creates a local variable declaration <code> typeRef name; </code>     
     */
    protected final LocalVariableDeclaration declare
	(String name, TypeReference typeRef) {
	return new LocalVariableDeclaration
	    (typeRef, 
	     new VariableSpecification
		 (new ProgramVariable(new ProgramElementName(name), 
				      typeRef.getKeYJavaType())));
    }

    /**
     * extracts all fields out of fielddeclaration
     * @param field the FieldDeclaration of which the field
     * specifications have to be extracted
     * @return a ListOfField the includes all field specifications found
     * int the field declaration of the given list
     */
    private final ListOfField filterField(FieldDeclaration field) {
	ListOfField result = SLListOfField.EMPTY_LIST;
	ArrayOfFieldSpecification spec = field.getFieldSpecifications();
	for (int i = spec.size()-1; i>=0; i--) {
	    result = result.prepend(spec.getFieldSpecification(i));
	} 
	return result;
    }

    /**
     * extracts all field specifications out of the given
     * list. Therefore it descends into field declarations.
     * @param list the ExtList with the members of a type declaration
     * @return a ListOfField the includes all field specifications found
     * int the field declaration of the given list
     */
    private ListOfField filterField(ExtList list) {
	ListOfField result = SLListOfField.EMPTY_LIST;
	Iterator it = list.iterator();
	while (it.hasNext()) {
	    Object pe = it.next();
	    if (pe instanceof FieldDeclaration) {
		result = result.prepend
		    (filterField((FieldDeclaration)pe));
	    }
	}
	return result;
    }

    /**
     * extracts all implicit field specifications out of the given
     * list 
     * @param list the ListOfField from which the implicit ones have to
     * be selected
     * @return a list with all implicit fields found in 'list'
     */
    private ListOfField filterImplicitFields(ListOfField list) {
	ListOfField result = SLListOfField.EMPTY_LIST;
	IteratorOfField it = list.iterator();
	while (it.hasNext()) {
	    Field field = it.next();
	    if (field instanceof ImplicitFieldSpecification) {
		result = result.append(field);
	    } 	
	}
	return result;
    }


    /**
     * retrieves a field with the given name out of the list
     * @param name a String with the name of the field to be looked for
     * @param fields the ListOfField where we have to look for the field
     * @return the program variable of the given name or null if not
     * found
     */
    private final ProgramVariable find(String name, ListOfField fields) {
	IteratorOfField it = fields.iterator();
	while (it.hasNext()) {
	    Field field = it.next();
	    if (name.equals(field.getName())) {
		return (ProgramVariable)
		    ((FieldSpecification)field).getProgramVariable();
	    }
	}
	return null;
    }

    /** 
     * returns the default value of the given type 
     * according to JLS 4.5.5
     * @return the default value of the given type 
     * according to JLS 4.5.5
     */
    private Expression getDefaultValue(Type type) {
	if (type instanceof PrimitiveType) {
	    return type.getDefaultValue();
	} 
	return NullLiteral.NULL;	    
    }


    /**
     * Creates the <code>&lt;prepareParent&gt;</code> method. The
     * purpose of this method is to have an initial call to the
     * <code>&lt;prepare&gt;</code> method. We could just call
     * <code>prepare()</code> on the new created object itself, which
     * would lead to a split in case distinction in order to decide
     * which of the <code>prepare</code> methods have to be taken. This
     * holds only for the first call as the succeeding calls are
     * performed using <code>super.&lt;prepare()&gt;</code> and
     * therefore are statically bound. 
     */
    public ProgramMethod createPrepareParentMethod(TypeRef classRef) {
	StatementBlock block;
	if (classRef.getKeYJavaType().getJavaType() != 
	    ClassDeclaration.objectClassDecl) {
	    // we can access the implementation	    	    
	    block = new StatementBlock
		(methodReference(prepareName, classRef, 
				 new ThisReference()));
	} else {
	    block = new StatementBlock();
	}
	MethodDeclaration md = new MethodDeclaration
	    (new Modifier[]{new Private()}, classRef, 
	    prepareParentName, new ParameterDeclaration[0], null,
	    block, false);	

	return new ProgramMethod(md, classRef.getKeYJavaType(), null);
    }

    /**
     * creates the statements which take the next object out of the list of
     * available objects
     * @return the statements which take the next object out of the list of
     * available objects
     */
    private LinkedList createObject(LocalVariableDeclaration local, 
				    ListOfField fields) {
	LinkedList result = new LinkedList();
	ProgramVariable newObject = (ProgramVariable) local.getVariables().
	    getVariableSpecification(0).getProgramVariable();		
	ListOfField implicitFields = filterImplicitFields(fields);
	// retrieves implicit fields; optimized for readability not for speed
	// if necessary find all attributes at once
	ProgramVariable nextToCreate = find("<nextToCreate>", implicitFields);
	ProgramVariable created      = find("<created>", implicitFields);
	ProgramVariable initialized  = find("<initialized>", implicitFields);
	ProgramVariable next         = find("<next>", implicitFields);

	result.addLast(local);
	result.addLast(assign(newObject, new FieldReference(nextToCreate, null)));
	result.addLast(assign
		       (attribute(newObject, created), 
			BooleanLiteral.TRUE));
	result.addLast(assign(new FieldReference(nextToCreate, null), 
			      attribute(newObject, next)));

	result.addLast(assign(attribute(newObject, initialized), 
			      BooleanLiteral.FALSE));	
	return result;
    }

    
    /**
     * creates the body for the implicit <create> method
     */
    private StatementBlock createCreateObjectBody
	(TypeRef classRef, ListOfField fields) {

	LocalVariableDeclaration local = declare("newObject", classRef);	
	LinkedList body                = createObject(local, fields);
	ProgramVariable newObject      = (ProgramVariable) local.getVariables().
	    getVariableSpecification(0).getProgramVariable();		

	body.addAll
	    (createPrepareBodyList(classRef, newObject, fields));

	/* 
	   would be nicer, but causes unnecessary case distinctions
	   new MethodReference(null, prepareName, newObject, 
					 createPrepareMethod(classType, newObject, 
					 javaInfo)));
	*/

	body.addLast(new Return(newObject));

	return new StatementBlock(new ArrayOfStatement
	    ((Statement[])body.toArray(new Statement[body.size()])));
    }

    
    /**
     * creates the implicit method <code>&lt;createObject&gt;</code>
     */
    public ProgramMethod createCreateObjectMethod(TypeRef classRef, 
						  ListOfField fields) {
	Modifier[] modifiers = new Modifier[]{new Protected(),
					      new Static()};	

	MethodDeclaration md = new MethodDeclaration
	    (modifiers, 
	     classRef, 
	     createObjectName, 
	     new ParameterDeclaration[0], null,
	     createCreateObjectBody(classRef, fields), false);

	return new ProgramMethod(md, classRef.getKeYJavaType(), null);
    }



    /**
     * returns the explicit method with the given name declared in the
     * given type
     */
    protected ProgramMethod getImplicitMethod
	(KeYJavaType classType, ProgramElement name, ListOfField fields) {

	ProgramMethod result = null;
	HashMap type2method = (HashMap)implicitMethodCache.get(name);
	if (type2method != null) {
	    result = (ProgramMethod)type2method.get(classType);
	} else {
	    type2method = new HashMap();
	    implicitMethodCache.put(name, type2method);
	}
	if (result == null) {
	    TypeRef classRef = new TypeRef(new ProgramElementName
		(""+classType.getSort().name()), 0, null, classType);
	    if (name.equals(prepareName)) {
		result = createPrepareMethod(classRef, fields);	    
	    } else if (name.equals(prepareParentName)) {
		result = createPrepareParentMethod(classRef);	    		
	    } else if (name.equals(createObjectName)) {
		result = createCreateObjectMethod(classRef, fields);	    		
	    } else {
		Debug.out("recoder2key: unknown implicit method", name);
	    }
	    type2method.put(classType, result);
	}
	return result;
    }


    /**
     *  creates the assignments of the field variables to their default values
     * and inserts them to the given body list
     *
     * @return the same list body that has been handed over as parameter
     */
    private LinkedList defaultSettings(ListOfField fields, 
				       ReferencePrefix invocationTarget,
				       LinkedList body) {
	LinkedList result = body;

	IteratorOfField it = fields.iterator();
	while (it.hasNext()) {
	    Field field = it.next();
	    if (!(field instanceof ImplicitFieldSpecification || 
		  field.isStatic())) {
		ProgramVariable var = (ProgramVariable)
		    field.getProgramVariable();
 		result.addLast
 		    (assign(new FieldReference(var, invocationTarget), 
 			    getDefaultValue(var.getKeYJavaType().getJavaType())));
	    }
	}
	return result;
    }

    /** 
     * creates an implicit method called 'prepare', that sets all attributes to
     * their default values
     */
    private LinkedList createPrepareBodyList
	(TypeRef     classRef, 
	 ReferencePrefix invocationTarget,
	 ListOfField fields) {

	LinkedList prepareBody = new LinkedList();

	ClassDeclaration classDecl = 
	    (ClassDeclaration)classRef.getKeYJavaType().getJavaType();


	if (ClassDeclaration.objectClassDecl != classDecl) {
	    prepareBody.addFirst(methodReference(prepareName, 
						 classRef, 
						 invocationTarget));
	} 

	prepareBody = defaultSettings(fields, invocationTarget, prepareBody);

	return prepareBody;
    }

    /** 
     * creates an implicit method called 'prepare', that sets all attributes to
     * their default values
     */
    private StatementBlock createPrepareBody(TypeRef classRef, 
					     ListOfField fields) {
	LinkedList body = createPrepareBodyList(classRef, 
						new ThisReference(), 
						fields);
	return new StatementBlock(new ArrayOfStatement
	    ((Statement[]) body.toArray(new Statement[body.size()])));
    }

    /**
     * adds the prepare method 
     */
    private ProgramMethod createPrepareMethod(TypeRef classRef, 
					      ListOfField fields) {

	MethodDeclaration md = new MethodDeclaration
	    (new Modifier[]{new Protected()}, classRef, prepareName, 
	    new ParameterDeclaration[0], null,
		createPrepareBody(classRef, fields), false);
	
	ProgramMethod result = new ProgramMethod
	    (md, classRef.getKeYJavaType(), null); 
	return result;
    } 


    /**
     * this method is called to add implicit methods to a key class
     */
    protected void addImplicitMethods(KeYJavaType classType, 
				      ExtList members) {
	ListOfField fields = filterField(members);
	members.add(getImplicitMethod(classType, prepareName, fields));
	members.add(getImplicitMethod(classType, prepareParentName, fields));
	members.add(getImplicitMethod(classType, createObjectName, fields));
    }


    // END OF IMPLICIT METHODS 


    public ClassDeclaration convert
	(recoder.java.declaration.ClassDeclaration td) {

	recoder.list.MemberDeclarationMutableList members = td.getMembers();


	KeYJavaType kjt = getKeYJavaType(td);
	ExtList classMembers = collectChildren(td);       

	addImplicitMethods(kjt, classMembers);

	ClassDeclaration keYClassDecl = new ClassDeclaration(classMembers);	    	    

	kjt.setJavaType(keYClassDecl);
	return keYClassDecl;	
    }
            
    public InterfaceDeclaration convert
	(recoder.java.declaration.InterfaceDeclaration td) {


	KeYJavaType kjt =  getKeYJavaType(td);
	ExtList members = collectChildren(td);       
	InterfaceDeclaration keYInterfaceDecl = new InterfaceDeclaration(members);	    
	kjt.setJavaType(keYInterfaceDecl);

	return keYInterfaceDecl;	
    }

    
    /**
     * creates an implicit field of the given type and name
     */
    private recoder.java.declaration.FieldDeclaration
	createImplicitRecoderField(String typeName, String fieldName,
				   boolean isStatic) {
	
	recoder.list.ModifierMutableList modifiers = new 
	    recoder.list.ModifierArrayList(1 + (isStatic ? 1 : 0));

	modifiers.add(new recoder.java.declaration.modifier.Private());
	if (isStatic) {
	    modifiers.add(new recoder.java.declaration.modifier.Static());
	}
	return new recoder.java.declaration.FieldDeclaration
	    (modifiers, new recoder.java.reference.TypeReference
		(new recoder.java.Identifier(typeName)), 
	     new ImplicitIdentifier(fieldName), null);

    }
        
    /** 
     * converts a recoder LocalVariableDeclaration to a KeY
     * LocalVariableDeclaration
     * (especially the declaration type of its parent is determined
     * and handed over)
     */
    public LocalVariableDeclaration
	convert(recoder.java.declaration.LocalVariableDeclaration lvd) {  
	 LocalVariableDeclaration local = 
	     new LocalVariableDeclaration(collectChildren(lvd)); 

	 return local;
    }


    /** 
     * converts a recoder ParameterDeclaration to a KeY
     * ParameterDeclaration
     * (especially the declaration type of its parent is determined
     * and handed over)
     */
    public ParameterDeclaration
	convert(recoder.java.declaration.ParameterDeclaration pd) {    
	 ParameterDeclaration param = 
	     new ParameterDeclaration(collectChildren(pd), pd.getASTParent()
				      instanceof 
				      recoder.java.declaration
				      .InterfaceDeclaration); 
	 return param;
    }


     /** convert a recoder FieldDeclaration to a KeY
      * FieldDeclaration
      * (especially the declaration type of its parent is determined
      * and handed over)
      */
     public FieldDeclaration
 	convert(recoder.java.declaration.FieldDeclaration fd) {    
	 FieldDeclaration field = 
	     new FieldDeclaration(collectChildren(fd), fd.getASTParent()
				  instanceof recoder.java.declaration
				  .InterfaceDeclaration); 
	 return field;
     }

     /** convert a recoder ConstructorDeclaration to a KeY
      * ConstructorDeclaration
      * (especially the declaration type of its parent is determined
      * and handed over)
      */
     public ConstructorDeclaration
 	convert(recoder.java.declaration.ConstructorDeclaration cd) {    

	 ConstructorDeclaration consDecl
	     =new ConstructorDeclaration
	     (collectChildren(cd), cd.getASTParent() instanceof
	      recoder.java.declaration.InterfaceDeclaration);
	insertToMap(cd, consDecl);
	return consDecl;
     }

    public TypeCast
	convert(recoder.java.expression.operator.TypeCast c) {
	ExtList children=collectChildren(c);
	TypeReference tr=(TypeReference)children.get(0);
	Expression e=(Expression)children.get(1);
	return new TypeCast(e, tr);	
    }

    private KeYJavaType lookup(recoder.abstraction.Type t) {
	return (KeYJavaType) rec2key.toKeY(t);
    }
    
    private LinkedList directSuperClasses
	(recoder.java.declaration.TypeDeclaration tdl){

	recoder.service.CrossReferenceSourceInfo crsc
	    = getServiceConfiguration().getCrossReferenceSourceInfo();
	LinkedList sc=new LinkedList();
	if (tdl instanceof recoder.java.declaration.InterfaceDeclaration) {
	    recoder.java.declaration.Extends exts 
		= ((recoder.java.declaration.InterfaceDeclaration)
		   tdl).getExtendedTypes();
	    for (int i=0; i<exts.getTypeReferenceCount(); i++) {
		recoder.java.reference.TypeReference ref 
		    = exts.getTypeReferenceAt(i);
		sc.add((recoder.abstraction.ClassType) crsc.getType(ref));
	    }
	    return sc;
	} else if (tdl instanceof recoder.java.declaration.ClassDeclaration) {
	    recoder.java.declaration.Implements impls 
		= ((recoder.java.declaration.ClassDeclaration)tdl)
		.getImplementedTypes();
	    for (int i=0; i<impls.getTypeReferenceCount(); i++) {
		recoder.java.reference.TypeReference ref 
		    = impls.getTypeReferenceAt(i);
		sc.add((recoder.abstraction.ClassType) crsc.getType(ref));
	    }
	    sc.add((recoder.abstraction.ClassType) crsc.getType
		      (((recoder.java.declaration.ClassDeclaration)tdl)
		       .getExtendedTypes()
		       .getTypeReferenceAt(0)));
	    return sc;
	}
	Debug.fail();
	return null;
    }

    private boolean isObject(recoder.abstraction.ClassType ct) {
	return "java.lang.Object".equals(ct.getFullName())
	    || "Object".equals(ct.getName());
    }

    private void createObjectSort(recoder.abstraction.ClassType ct, 
				  SetOfSort supers) {
	Sort newSort=null;
	if (!isObject(ct)) {
	    newSort = new ClassInstanceSortImpl(new Name(ct.getFullName()), supers);
	} else {
	    newSort = typeConverter().getObjectSort();
	}
	addKeYJavaType(ct, newSort);
    }


    private boolean containsType(recoder.abstraction.Type t) {
	return rec2key.mapped(t);
    }

    private SetOfSort directSuperSorts
	(recoder.abstraction.ClassType classType) {

	recoder.list.ClassTypeList supers=classType.getSupertypes();
	SetOfSort ss=SetAsListOfSort.EMPTY_SET;
	for (int i=0; i<supers.size(); i++) {
	    if (!containsType(supers.getClassType(i))) {
		createObjectSort(supers.getClassType(i), 
				 directSuperSorts(supers.getClassType(i)));
	    }
	    ss=ss.add((Sort)getKeYJavaType(supers.getClassType(i)).getSort());	    
	}
	if (ss==SetAsListOfSort.EMPTY_SET && !isObject(classType)) {
	    ss=ss.add(typeConverter().getObjectSort());
	}
	return ss;
    }

    private KeYJavaType getKeYJavaType(recoder.abstraction.Type t) {		
	if (t==null) {
	    return null; //this can originate from 'void'
	}
	KeYJavaType kjt = lookup(t);	    
	if (kjt != null) {
	    return kjt;
	}
	// create a new KeYJavaType
	if (t instanceof recoder.abstraction.PrimitiveType) {
	    Sort s = typeConverter.getPrimitiveSort
		(PrimitiveType.getPrimitiveType(t.getFullName()));
	    if (s==null) {
		s=new PrimitiveSort(new Name(t.getFullName()));
	    }
	    addKeYJavaType(t, s);
	} else if (t instanceof recoder.abstraction.ClassType) {
	    recoder.abstraction.ClassType ct=(recoder.abstraction.ClassType)t;
	    createObjectSort(ct, directSuperSorts(ct));	    
	} else if (t instanceof recoder.abstraction.ArrayType){
	    recoder.abstraction.Type bt 
		= ((recoder.abstraction.ArrayType)t).getBaseType();
	    kjt = getKeYJavaType(bt);
	    Sort s = ArraySortImpl.getArraySort(kjt.getSort());
	    addKeYJavaType(t, s);
	}
	return lookup(t);
    }
    
    private KeYJavaType addKeYJavaType(recoder.abstraction.Type t, Sort s) {
	KeYJavaType result = null;       
	if (!(t instanceof recoder.java.declaration.TypeDeclaration)) {
	    Type type = null;
	    if (t instanceof recoder.abstraction.PrimitiveType) {
		type = PrimitiveType.getPrimitiveType(t.getFullName());
		result = typeConverter.getKeYJavaType(type);		
		if (result == null) {
		    Debug.out("recoder2key: create new KeYJavaType for", t);
		    Debug.out("recoder2key: this should not happen");
		    result = new KeYJavaType(type, s);
		}
	    } else if (t instanceof recoder.abstraction.NullType) {
		type = NullType.JAVA_NULL;
		result = new KeYJavaType(type);
	    } else if (t instanceof recoder.bytecode.ClassFile) {
		    recoder.bytecode.ClassFile cf = (recoder.bytecode.ClassFile)t;
		    if (!cf.getName().equals("Object")) {
			if (cf.isInterface()) {
			    type = new InterfaceDeclaration(cf.getFullName());
			} else { 
			    type = new ClassDeclaration(cf.getFullName());
			}
			result = new KeYJavaType(type, s);
		    } else {			
			result = typeConverter().getObjectKeYJavaType();
		    }
		    setUpSort(s);
	    } else if (t instanceof recoder.abstraction.ArrayType) {
		setUpSort(s);
		result = new KeYJavaType(s);		    
	    } else {
		Debug.out("recoder2key: unknown type", t);
		result = new KeYJavaType();		
	    }
	    	    	    
	} else {
	    setUpSort ( s );
	    if (s!=typeConverter().getObjectSort()) {
		result = new KeYJavaType(s);		    
	    } else {
		result = typeConverter().getObjectKeYJavaType();
	    }
	}
	insertToMap(t, result);

	// delayed creation of virtual array declarations
	// to avoid cycles
	if (t instanceof recoder.abstraction.ArrayType) {
	    recoder.abstraction.Type recBaseType 
		= ((recoder.abstraction.ArrayType)t).getBaseType();
	    result.setJavaType(createArrayType(getKeYJavaType(recBaseType), 
					       (KeYJavaType) rec2key.toKeY(t)));
	}
	
	return (KeYJavaType) rec2key.toKeY(t); //usually this equals result, 
	//sometimes however, there is a 'legacy' type in the mapping, 
	//which has priority
    }

    /**
     * Insert sorts into the namespace, add symbols that may have been
     * defined by a sort to the function namespace (e.g. functions for
     * collection sorts)
     */
    protected void setUpSort ( Sort s ) {
	namespaces.sorts ().add(s);
	if (s instanceof NonCollectionSort) {
	    NonCollectionSort ns
		= (NonCollectionSort)s;
	    namespaces.sorts ().add(ns.getSetSort());
	    namespaces.sorts ().add(ns.getSequenceSort());
	    namespaces.sorts ().add(ns.getBagSort());
	}
	if ( s instanceof SortDefiningSymbols ) {
	    ((SortDefiningSymbols)s).createSymbols ( namespaces.functions (), 
						     namespaces.sorts () );
	}
    }

    // ----------------- references ----------------------------------- //

    /**
     * converts a recoder this  constructor reference.
     * @return the this reference in the KeY data structures
     */
     public ThisConstructorReference convert
	 (recoder.java.reference.ThisConstructorReference tcr) {	 

	 recoder.java.NonTerminalProgramElement ntpe = tcr.getASTParent();

	 while (!(ntpe instanceof
		  recoder.java.declaration.ConstructorDeclaration || 
		  ntpe == null)) {
	     ntpe = ntpe.getASTParent();
	 }

	 if (!(ntpe instanceof recoder.java.declaration.ConstructorDeclaration)) {
	     Debug.fail("recoder2key: this constructor reference without"+
			"enclosing constructor");
	 }

	 recoder.abstraction.Type recoderType = 
	     ((recoder.java.declaration.ConstructorDeclaration)ntpe).getContainingClassType();



	 ExtList children = collectChildren(tcr);
	 ExecutionContext execContext = new ExecutionContext
	     (new TypeRef(new ProgramElementName(recoderType.getFullName()),
			  0, null, getKeYJavaType(recoderType)), 
	      new ThisReference());

	 return new ThisConstructorReference(children, execContext);
     }


    /**
     * converts a SpecialConstructorReference. 
     * Special handling because the initializing
     * Expressions and the ReferencePrefix accessPath might not be disjunct.
     */
    public SuperConstructorReference 
	convert(recoder.java.reference.SuperConstructorReference scr) {

	 recoder.java.NonTerminalProgramElement ntpe = scr.getASTParent();
	 
	 while (!(ntpe instanceof
		  recoder.java.declaration.ConstructorDeclaration || 
		  ntpe == null)) {
	     ntpe = ntpe.getASTParent();
	 }

	 if (!(ntpe instanceof recoder.java.declaration.ConstructorDeclaration)) {
	     Debug.fail("recoder2key: super constructor reference without"+
			"enclosing constructor");
	 }

	 recoder.abstraction.Type recoderType = 
	     ((recoder.java.declaration.ConstructorDeclaration)ntpe).getContainingClassType();

	ExtList children=collectChildren(scr);
	ReferencePrefix prefix=null;
	int prefixPos=getPosition(scr,scr.getReferencePrefix());	     
	if (prefixPos!=-1) {
	    prefix=(ReferencePrefix)children.get(prefixPos);
	    children.remove(prefixPos);
	}

	ExecutionContext execContext = new ExecutionContext
	    (new TypeRef(new ProgramElementName(recoderType.getFullName()),
			 0, null, getKeYJavaType(recoderType)), 
	     new ThisReference());

	return new SuperConstructorReference(children, prefix, execContext);
    }


    public SuperReference 
	convert(recoder.java.reference.SuperReference sr) {	
	
	recoder.java.NonTerminalProgramElement ntpe = sr.getASTParent();
	 
	 while (!(ntpe instanceof
		  recoder.java.declaration.MethodDeclaration || 
		  ntpe == null)) {
	     ntpe = ntpe.getASTParent();
	 }

	 if (!(ntpe instanceof recoder.java.declaration.MethodDeclaration)) {
	     Debug.fail("recoder2key: super reference without"+
			"enclosing method declaration");
	 }

	 recoder.abstraction.Type recoderType = 
	     ((recoder.java.declaration.MethodDeclaration)ntpe).getContainingClassType();

	 ExtList children=collectChildren(sr);
	 ReferencePrefix prefix=null;
	 int prefixPos=getPosition(sr,sr.getReferencePrefix());	     
	 if (prefixPos!=-1) {
	     prefix=(ReferencePrefix)children.get(prefixPos);
	     children.remove(prefixPos);
	 }
	 
	 ExecutionContext execContext = new ExecutionContext
	     (new TypeRef(new ProgramElementName(recoderType.getFullName()),
			  0, null, getKeYJavaType(recoderType)), 
	      new ThisReference());

	 SuperReference srkey = new SuperReference(children, execContext);
// 	 insertToMap(sr, srkey);
// 	 System.out.println("inserting "+sr+" --> "+srkey);
	 return srkey;
    }


    /** convert a recoder VariableSpecification to a KeY
     * VariableSpecification
     * (checks dimension and hands it over and insert in hashmap)
     */
    public VariableSpecification
	convert(recoder.java.declaration.VariableSpecification recoderVarSpec){	

	VariableSpecification varSpec
	    = (VariableSpecification)rec2key.toKeY(recoderVarSpec);
	

	if (varSpec == null) {
	    recoder.abstraction.Type recoderType = 
		(servConf.getSourceInfo()).getType(recoderVarSpec);

	    ProgramElementName name = new ProgramElementName
		(recoderVarSpec.getName());
	    ProgramVariable pv = 
		new ProgramVariable(name, getKeYJavaType(recoderType), false);
	    ExtList children = collectChildren(recoderVarSpec);
	    varSpec = new VariableSpecification
		(children, pv, recoderVarSpec.getDimensions(), 
		 pv.getKeYJavaType());

	    insertToMap(recoderVarSpec, varSpec);
	}
	return varSpec;
    }


    /** convert a recoder MethodDeclaration to a KeY
     * ProgramMethod
     * (especially the declaration type of its parent is determined
     * and handed over)
     */
    public ProgramMethod
	convert(recoder.java.declaration.MethodDeclaration md) {     
	ProgramMethod result=null;

	// methodsDeclaring contains the recoder method declarations as keys 
	// that have been started to convert but are not yet finished.
	// The mapped value is the reference to the later completed 
	// ProgramMethod.
	if (methodsDeclaring.containsKey(md)) {
	    // a recursive call from a method reference
	    return (ProgramMethod) methodsDeclaring.get(md);
	                                    //reference that will later be set.
	} 
	methodsDeclaring.put(md, result);
	if (!rec2key.mapped(md)) {
	    MethodDeclaration methDecl
		= new MethodDeclaration
		    (collectChildren(md), md.getASTParent() instanceof 
		     recoder.java.declaration.InterfaceDeclaration);
	    recoder.abstraction.ClassType cont = 
		servConf.getCrossReferenceSourceInfo().
		getContainingClassType((recoder.abstraction.Member)md);

	    result = new ProgramMethod
		(methDecl, getKeYJavaType(cont), getKeYJavaType(md.getReturnType()));
	    insertToMap(md, result);
	}
	methodsDeclaring.remove(md);
	result = (ProgramMethod)rec2key.toKeY(md);
	return result;
    }


    /** 
     * convert a recoder FieldSpecification to a KeY FieldSpecification
     * (checks dimension and hands it over and insert in hash map)
     */
    public FieldSpecification
 	convert(recoder.java.declaration.FieldSpecification recoderVarSpec){

	if (recoderVarSpec == null) { //%%%%%%%%%%%%%
	    return new FieldSpecification();
	}

	FieldSpecification varSpec
	    = (FieldSpecification)rec2key.toKeY(recoderVarSpec);

	if (varSpec==null) {
	    recoder.abstraction.Type recoderType =
		(servConf.getSourceInfo()).getType(recoderVarSpec);

	    if (recoderVarSpec.getIdentifier() instanceof ImplicitIdentifier) {
		// the modelled field is an implicit one, we have to handle this one
		// explicit 
		varSpec = new ImplicitFieldSpecification 
		    (new ProgramVariable
			(new ProgramElementName(recoderVarSpec.getName()),
			 getKeYJavaType(recoderType),
			 recoderVarSpec.isStatic()), 
			 getKeYJavaType(recoderType));
	    } else {	    
		varSpec = new FieldSpecification
		    (collectChildren(recoderVarSpec), 
		     new ProgramVariable
			 (new ProgramElementName(recoderVarSpec.getName()), 
			  getKeYJavaType(recoderType), 
			  recoderVarSpec.isStatic()), 
		     recoderVarSpec.getDimensions(), 
		     getKeYJavaType(recoderType));
	    }

	    insertToMap(recoderVarSpec, varSpec);
	}

	return varSpec;
    }


    /** 
     * convert a recoder TypeReference to a KeY TypeReference
     * (checks dimension and hands it over)
     */
    public TypeReference
	convert(recoder.java.reference.TypeReference tr) {
	recoder.abstraction.Type rType
	    = getServiceConfiguration().getSourceInfo().getType(tr);
	if (rType==null) return null; // because of 'void'

	KeYJavaType kjt = getKeYJavaType(rType);	
	ExtList children = collectChildren(tr);
	TypeReference result = 
	    new TypeRef((ProgramElementName)children.
			get(ProgramElementName.class),
			tr.getDimensions(), 
			(ReferencePrefix)children.
			get(ReferencePrefix.class), kjt);
	return result;
    }

    /** 
     * if an UncollatedReferenceQualifier appears throw a
     * ConvertExceception because these qualifiers have to be resolved
     * by running the CrossReferencer
     */
    public JavaProgramElement 
	convert(recoder.java.reference.UncollatedReferenceQualifier urq) {    
	if ("<prepareParent>".equals(urq.getName())) {	    
	    ReferencePrefix prefix = null;
	    if (urq.getReferencePrefix() != null) {
		prefix = (ReferencePrefix)callConvert(urq.getReferencePrefix());
	    } 
	    return new MethodReference(new ArrayOfExpression(new Expression[0]),
				       new ProgramElementName("<prepareParent>"),
				       prefix, null);
	}
	throw new ConvertException("recoder2key: Qualifier "+urq.getName()+
				   " not resolvable.");
    }


    protected recoder.java.declaration.VariableSpecification 
	getRecoderVarSpec(recoder.java.reference.VariableReference vr) {

	recoder.service.SourceInfo sourceInfo = servConf.getSourceInfo();
	return sourceInfo.getVariableSpecification(sourceInfo.getVariable(vr));
    }

    /**
     * converts a recoder variable reference. A ProgramVariable is created
     * replacing the variable reference.
     * @param fr the recoder variable reference.
     */
     public ProgramVariable convert
	 (recoder.java.reference.VariableReference vr) {

	 recoder.java.declaration.VariableSpecification recoderVarspec
	     = getRecoderVarSpec(vr);	 
	 if (rec2key.mapped(recoderVarspec)) {
	     VariableSpecification keyVarspec = (VariableSpecification) 
		 rec2key.toKeY(recoderVarspec);
	     return (ProgramVariable)keyVarspec.getProgramVariable();
	 } else {
	     VariableSpecification keyVarspec = convert(recoderVarspec);
	     ProgramVariable progvar = (ProgramVariable) 
		 keyVarspec.getProgramVariable();
	     insertToMap(recoderVarspec, keyVarspec);
	     return progvar;	     
	 }
     }

    

    /**
     * extracts all field specifications out of the given
     * list. Therefore it descends into field declarations.
     * @param list the ArrayOfMemberDeclaration with the members of a
     * type declaration
     * @return a ListOfField the includes all field specifications found
     * int the field declaration of the given list
     */
    private ListOfField filterField(ArrayOfMemberDeclaration list) {
	ListOfField result = SLListOfField.EMPTY_LIST;
	for (int i = list.size()-1; i >= 0; i--) {
	    MemberDeclaration pe = list.getMemberDeclaration(i);
	    if (pe instanceof FieldDeclaration) {
		result = result.append
		    (filterField((FieldDeclaration)pe));
	    }
	}
	return result;
    }


    /**
     * converts a recoder array length reference to a usual KeY field
     * reference
     */
    public FieldReference convert
	(recoder.java.reference.ArrayLengthReference alr) {
	recoder.abstraction.Type recoderType = servConf.
	    getCrossReferenceSourceInfo().getType(alr.getReferencePrefix());
	ArrayDeclaration ad =
	    (ArrayDeclaration)getKeYJavaType(recoderType).getJavaType();

	ProgramVariable length  = find("length", filterField(ad.getMembers()));
	// the invocation of callConvert should work well as each array
	// length reference must have a reference prefix (at least this
	// is what i think)
	return new FieldReference
	    (length, (ReferencePrefix)doConvert
	     ((recoder.java.JavaProgramElement)alr.getReferencePrefix()));
    }

    /**
     * converts a recoder field reference. A ProgramVariable is created
     * replacing the field reference.
     * @param fr the recoder field reference.
     */
     public FieldReference convert(recoder.java.reference.FieldReference fr) {
	 ProgramVariable pv;
	 recoder.java.declaration.FieldSpecification recoderVarSpec
	     = (recoder.java.declaration.FieldSpecification)
	         getRecoderVarSpec(fr);

	 ReferencePrefix prefix = null;	
	 if (fr.getReferencePrefix() != null) {
	     prefix = (ReferencePrefix)doConvert
		 ((recoder.java.JavaProgramElement)fr.getReferencePrefix());
	 }

	 if (recoderVarSpec == null) { 
	     // null means only bytecode available for this 
	     // field %%%
 	    recoder.abstraction.Type recoderType = 
 		servConf.getByteCodeInfo().getType
		(servConf.getSourceInfo().getField(fr));	    

	    pv = new ProgramVariable
		(new ProgramElementName(fr.getName()), getKeYJavaType(recoderType),
		 servConf.getSourceInfo().getField(fr).isStatic());
	    insertToMap
		(new recoder.java.declaration.
		    FieldSpecification(fr.getIdentifier()), 			 
		 new FieldSpecification(pv));	     
	    
	    return new FieldReference(pv, prefix);
	 } 

	 VariableSpecification keyVarSpec;
	 if (rec2key.mapped(recoderVarSpec)) {
	     keyVarSpec = (VariableSpecification) 
		 rec2key.toKeY(recoderVarSpec);
	 } else {
	     keyVarSpec = convert(recoderVarSpec);
	     insertToMap(recoderVarSpec, keyVarSpec);
	 }
	 pv = (ProgramVariable) keyVarSpec.getProgramVariable();	     

	 recoder.abstraction.ClassType ct = 
	     servConf.getSourceInfo().getContainingClassType(fr);

	 if (ct != null && 
 	     ct.getFullName().startsWith("KeY_interactive_input_")) {
 	     return new EmbededVariableReference(pv, prefix);
 	 } else {
	     return new FieldReference(pv, prefix);
 	 }
     }



    /**
     * converts a recoder method reference. A
     * de.uka.ilkd.key.logic.op.ProgramMethod is created 
     * replacing the method reference.
     * @param mr the recoder method reference.
     * @return the Method the KeY Dependance
     */
     public MethodReference convert(recoder.java.reference.MethodReference mr) {

	 recoder.service.SourceInfo sourceInfo = servConf.getSourceInfo();
	 recoder.abstraction.Method method = null;
	
	 try {
	     method = sourceInfo.getMethod(mr);
	 } catch (recoder.service.UnresolvedReferenceException ure) {
	     if ("<prepare>".equals(mr.getName())) {	    
		 ReferencePrefix prefix = null;
		 if (mr.getReferencePrefix() != null) {
		     prefix = (ReferencePrefix)callConvert(mr.getReferencePrefix());
		 } 
		 return new MethodReference(new ArrayOfExpression(new Expression[0]),
					    new ProgramElementName("<prepare>"),
					    prefix, null);
	     } else {
		 throw ure;
	     }
	 }
	 
	 if (method.isAbstract()) {
	     method=null;
	 } else {
	     if (!rec2key.mapped(method)) {
		 if (method instanceof recoder.java.declaration.MethodDeclaration) {
		     // method reference before method decl, also recursive calls.
		     //	do not use: 
		     ProgramMethod keyMethodDecl =
			 convert((recoder.java.declaration.MethodDeclaration)method); 
		     // because of cycles when reading recursive programs		     
		 }
	     }
	 }

	 ExtList children=collectChildren(mr);
	 // convert reference prefix separately
	 ReferencePrefix prefix=null;
	 int prefixPos=getPosition(mr, mr.getReferencePrefix());
	 if (prefixPos!=-1) {
	     prefix=(ReferencePrefix)children.get(prefixPos);
	     children.remove(prefixPos);
	 }
	 return new MethodReference
	     (children, prefix, (ProgramMethod)rec2key.toKeY(method));
     }



    //--------------Special traetment because of ambiguities ----------


    public LabeledStatement convert
	(recoder.java.statement.LabeledStatement l) {

	ExtList children=collectChildren(l);
	Label lab=null;
	int labPos=getPosition(l,l.getIdentifier());	
	if (labPos!=-1) {
	    lab=(Label)children.get(labPos);
	    children.remove(labPos);
	}
	return new LabeledStatement(children, lab);
    }

    /**
     * converts a For.
     * @param f the For of recoder
     * @return the For of KeY
     */
    public For convert(recoder.java.statement.For f) {
	return new For(convertLoopInitializers(f), 
		       convertGuard(f), 
		       convertUpdates(f), 
		       convertBody(f));
    }

    /**
     * converts a While.
     * @param f the While of recoder
     * @return the While of KeY
     */
    public While convert(recoder.java.statement.While w) {
	return new While(convertGuard(w).getExpression(), 
			 convertBody(w));
    }

    /**
     * converts a Do.
     * @param f the Do of recoder
     * @return the Do of KeY
     */
    public Do convert(recoder.java.statement.Do d) {
	return new Do(convertGuard(d).getExpression(), 
		      convertBody(d));
    }

    /**
     * helper for convert(x) with x a LoopStatement. Converts the body of x.
     */
    protected Statement convertBody(recoder.java.statement.LoopStatement ls) {
	Object body = null;	
	if (ls.getBody() != null) {
	     body = callConvert(ls.getBody());
	}
	return (Statement) body;
    }

    /**
     * helper for convert(x) with x a LoopStatement. Converts the guard of x.
     */
    protected Guard convertGuard(recoder.java.statement.LoopStatement ls) {
	Object guard=null;
	if (ls.getGuard()!=null) {
	    guard = callConvert(ls.getGuard());
	}
	return new Guard((Expression)guard);
    }

    /**
     * helper for convert(x) with x a LoopStatement. Converts the updates of x.
     */
    protected ForUpdates
	convertUpdates(recoder.java.statement.LoopStatement ls) {
	    ExtList updates=new ExtList();
	    if (ls.getUpdates()!=null) {
		for (int i=0; i<ls.getUpdates().size(); i++) {
		    updates.add
			(callConvert(ls.getUpdates().getExpression(i)));
		}	    
	    } else {
		return null;
	    }
	    return new ForUpdates(updates);
	}

    /**
     * helper for convert(x) with x a LoopStatement. Converts the loop
     * initializers of x. 
     */
    protected LoopInit
	convertLoopInitializers(recoder.java.statement.LoopStatement ls) {
	    ExtList inits=new ExtList();
	    if (ls.getInitializers()!=null) {
		for (int i=0; i<ls.getInitializers().size(); i++) {
		    inits.add(callConvert(ls.getInitializers().
					  getLoopInitializer(i)));

		}	    
	    } else {
		return null;	 
	    }
	   
	    LoopInitializer[] result=new LoopInitializer[inits.size()];
	    for (int j=0; j<inits.size(); j++) {
		result[j]=(LoopInitializer)inits.get(j);
	    }
	    return new LoopInit(result);
	}

 

    /**
     * converts an ArrayReference. Special handling because the initializing
     * Expressions and the ReferencePrefix accessPath might not be disjunct.
     */
    public ArrayReference convert(recoder.java.reference.ArrayReference ar) {
	ExtList children=collectChildren(ar);
	ReferencePrefix prefix=null;

	int prefixPos=getPosition(ar,ar.getReferencePrefix());
	if (prefixPos!=-1) {
	    prefix=(ReferencePrefix)children.get(prefixPos);
	    children.remove(prefixPos);
	}
	return new ArrayReference(children, prefix);
    }



    /**
     * converts a Case. 
     * Special handling because the initializing
     * Expression and Statements might not be disjunct.
     */
    public Case convert(recoder.java.statement.Case c) {
	ExtList children=collectChildren(c);
	Expression expr=null;
	int exprPos=getPosition(c,c.getExpression());	
	if (exprPos!=-1) {
	    expr=(Expression)children.get(exprPos);
	    children.remove(exprPos);
	}
	return new Case(children, expr);
    }


    /**
     * converts a New. 
     * Special handling because the ReferencePrefix and the TypeReference
     *  might not be disjunct.
     */
    public New convert(recoder.java.expression.operator.New n) {

	ExtList children = collectChildren(n);
	ReferencePrefix rp = null;
	int rpPos=getPosition(n,n.getReferencePrefix());
	if (rpPos!=-1) {
	    rp=(ReferencePrefix)children.get(rpPos);
	    children.remove(rpPos);
	}
	return new New(children, rp);
    }


    //-----------------------------------------------------------


    public Import convert(recoder.java.Import im) {
	return new Import(collectChildren(im), 
			  im.isMultiImport());
    }

    /**
     * returns the position of pe2 in the virtual child array of pe1
     */
    protected static int getPosition(recoder.java.NonTerminalProgramElement 
				   pe1, 
				   recoder.java.ProgramElement pe2) {
	int n=pe1.getChildCount();
	int i=0;
	while ((i<n) && (pe1.getChildAt(i)!=pe2)) i++;
	return (i==n) ? -1 : i; 
    }

    
    public JavaProgramElement convert() {
	return (JavaProgramElement) callConvert(progElem);
    }

    private recoder.java.declaration.ClassDeclaration interactClassDecl() {
	recoder.java.declaration.ClassDeclaration classContext = 
 	    new recoder.java.declaration.ClassDeclaration
  		(null, new recoder.java.Identifier("KeY_interactive_input_"
						   +interactCounter++), 
		 null, null, null);
 	classContext.setProgramModelInfo(servConf.getCrossReferenceSourceInfo());
	return classContext;
    }

    /** creates an empty RECODER compilation unit 
     * @return the recoder.java.CompilationUnit 
     */
    public Context createEmptyContext() {
 	recoder.java.declaration.ClassDeclaration classContext 
	    = interactClassDecl();
 	return new Context(servConf, classContext);
    }

    private VariableSpecification lookupVarSpec(ProgramVariable pv) {
	Iterator it=rec2key.elemsKeY().iterator();
	while (it.hasNext()) {
	    Object o=it.next();
	    if ((o instanceof VariableSpecification) 
		&& ((VariableSpecification)o).getProgramVariable()==pv){
		
		return (VariableSpecification)o;
	    }
	}
	return null;
    }


    public void addProgramVariablesToClassContext
	(recoder.java.declaration.ClassDeclaration classContext,
	 ListOfProgramVariable vars, recoder.service.CrossReferenceSourceInfo csi) {
	
	IteratorOfProgramVariable it = vars.iterator();
	java.util.HashSet names=new java.util.HashSet();
	recoder.list.MemberDeclarationMutableList list = classContext.getMembers();
	if (list == null) {
	    list = new recoder.list.MemberDeclarationArrayList();
	    classContext.setMembers(list);
	}
	l: while (it.hasNext()) {
 	    VariableSpecification keyVarSpec;
	    ProgramVariable var = it.next();
	    if (names.contains(var.name().toString())) {
		continue l;}
	    names.add(var.name().toString());
	    keyVarSpec=lookupVarSpec(var);
	    if (keyVarSpec==null) {
		keyVarSpec=new FieldSpecification(var);
	    }
	    if (var.getKeYJavaType()==null) {
		throw new IllegalArgumentException("Variable "+var+" has no type");
	    }

	    recoder.java.Identifier typeId
		=new recoder.java.Identifier(var.getKeYJavaType().getFullName()) {
		    protected void setText(String s) {
			id=s; //this is a hack because of a Recoder bug
			      // Recoder does not want to take "int[]"
		    }
		};

	    recoder.java.declaration.FieldDeclaration recVar=
		new recoder.java.declaration.FieldDeclaration
		    (null, new recoder.java.reference.TypeReference(typeId),
		     new recoder.java.Identifier(keyVarSpec.getName()),
		     null);
 	    list.add(recVar);
 	    classContext.makeAllParentRolesValid();
	    recoder.java.declaration.VariableSpecification rvarspec
		= recVar.getVariables().getVariableSpecification(0);
	    rvarspec.setProgramModelInfo(csi);
	    insertToMap(recVar.getVariables()
			.getVariableSpecification(0), keyVarSpec);
	}
	servConf.getChangeHistory().updateModel();
    }

    

    public Context createContext(ListOfProgramVariable pvs) {
	return createContext(pvs, servConf.getCrossReferenceSourceInfo());
    }

    public Context createContext(ListOfProgramVariable vars, 
				 recoder.service.CrossReferenceSourceInfo csi) {
 	recoder.java.declaration.ClassDeclaration classContext 
	    = interactClassDecl();
	addProgramVariablesToClassContext(classContext, vars, csi);	
	return new Context(servConf, classContext);
    }
    
    // invoke model transformers
    protected void transformModel
	(recoder.list.CompilationUnitMutableList cUnits) {
	RecoderModelTransformer[] transformer = 
	    new RecoderModelTransformer[] { 
		new ImplicitFieldAdder(servConf, cUnits)//, 
		//		new ImplicitMethodAdder(servConf, cUnits) 
	    };
	for (int i = 0; i<transformer.length; i++) {
	    transformer[i].execute();
	    servConf.getChangeHistory().updateModel();
	}

    }


    // array type creation


    /**
     * create a local variable declaration <br></br>
     *   <code>type name = init; </code>
     */
    protected final LocalVariableDeclaration declare
	(String name, Expression init, KeYJavaType type) {
	return new LocalVariableDeclaration
	    (new TypeRef(type), 
	     new VariableSpecification
		 (new ProgramVariable(new ProgramElementName(name), type), 
		  init, type));
    }


    /**
     * add the prepare method for arrays
     */
    private ProgramMethod createPrepareArrayMethod(TypeRef arrayRef, 
						   ProgramVariable length,
						   Expression defaultValue, 
						   ListOfField fields) {
	KeYJavaType integerType = getKeYJavaType
	    (new recoder.abstraction.PrimitiveType
		("int", servConf.getSourceInfo()));
	IntLiteral zero = new IntLiteral(0);

	LocalVariableDeclaration forInit = declare("i", zero, integerType);

	ProgramVariable pv = (ProgramVariable)forInit.getVariables().
	    getVariableSpecification(0).getProgramVariable();
	
	For forLoop = new For
	    (new LoopInitializer[]{forInit}, 
	     new LessThan(pv, attribute(new ThisReference(), length)),
	     new Expression[]{new PostIncrement(pv)}, 
	     (Statement)assign(new ArrayReference
		 (new ThisReference(), new Expression[]{pv}),
			       defaultValue));
	
	StatementBlock body = new StatementBlock(forLoop);

	MethodDeclaration md = new MethodDeclaration
	    (new Modifier[]{new Private()}, arrayRef, prepareName, 
	    new ParameterDeclaration[0], null,
		body, false);
	
	ProgramMethod result = new ProgramMethod
	    (md, arrayRef.getKeYJavaType(), null); 

	return result;
    } 


    /**
     * creates the body of method <code>&lt;createArray(int
     * paramLength)&gt;</code> therefore it first adds the statements
     * responsible to take the correct one out of the list, then the
     * arrays length attribute is set followed by a call to
     * <code>&lt;prepare&gt;()</code> setting the arrays fields on their
     * default value. 
     * @param arrayRef the TypeReference to the array the method belongs
     *   to
     * @param length the final public ProgramVariable
     * <code>length</length> of the array 
     * @param paramLength the ProgramVariable which is the parameter of
     *   the <code>&lt;createArray&gt;</code> method
     * @param prepare the ProgramMethod <code>&lt;prepare&gt;()</code>
     *   which will be called when executing
     * <code>&lt;createArray&gt;</code> 
     * @param fields the ListOfFields of the current array
     * @return the StatementBlock which is the method's body <br></br>
     *     <code>
     *  { arrayRef newObject;
     *    newObject = arrayRef.<nextToCreate>;
     *    arrayRef.<nextToCreate> = arrayRef.<nextToCreate>.<next>;
     *    newObject.<initialized> = false;
     *    newObject.<created>     = true;
     *    newObject.length = paramLength;
     *    newObject.<prepare>();
     *    newObject.<initialized> = true;
     *    return newObject;
     *   }
     *   </code>
     */
    private StatementBlock createCreateArrayBody
	(TypeReference arrayRef, ProgramVariable length,						 
	 ProgramVariable paramLength, ProgramMethod prepare,
	 ListOfField fields) {
	LocalVariableDeclaration local = declare("newObject", arrayRef);	
	LinkedList body                = createObject(local, fields);
	ProgramVariable newObject      = (ProgramVariable) local.
	    getVariables().getVariableSpecification(0).getProgramVariable();	
	
	body.add(assign(attribute(newObject, length), paramLength));

	body.add(new MethodReference
	    (null, prepareName, newObject, prepare));	
	body.add(assign(attribute(newObject, 
				  find("<initialized>", fields)),
			BooleanLiteral.TRUE));
	body.add(new Return(newObject));
 	return new StatementBlock
	    ((Statement[])body.toArray(new Statement[body.size()]));
    }
    
    /**
     * creates the implicit method <code>&lt;createArray&gt;</code>
     * it fulfills a similar purpose as
     * <code>&lt;createObject&gt;</code> in addition it sets the arrays
     * length and calls the prepare method
     */
    private ProgramMethod createCreateArrayMethod(TypeRef arrayRef, 
						  ProgramVariable length,
						  ProgramMethod prepare,
						  ListOfField fields) {

	Modifier[] modifiers = new Modifier[] { new Protected(),
						new Static() };
	
	KeYJavaType integerType = getKeYJavaType
	    (new recoder.abstraction.PrimitiveType("int", 
						   servConf.getSourceInfo()));
	
	VariableSpecification paramLength = 
	    new VariableSpecification
		(new ProgramVariable
		    (new ProgramElementName("length"), integerType));
	
	ParameterDeclaration param = new ParameterDeclaration
	    (new Modifier[0], new TypeRef(integerType), paramLength, false);


	MethodDeclaration md = new MethodDeclaration
	    (modifiers, arrayRef, createArrayName,
	     new ParameterDeclaration[]{param}, null,
	     createCreateArrayBody
	     (arrayRef, length, 
	      (ProgramVariable)paramLength.getProgramVariable(), 
	      prepare, fields), false);

	return new ProgramMethod(md, arrayRef.getKeYJavaType(), null);
    }

    /**
     * creates an implicit field of the given name and type
     * @param name a String with the name of the implicit field
     * @param typeRef a TypeReference refering to the type as which the
     * new field has to be declared
     * @param isStatic a boolean that forces a field to become static or
     *  non static 
     * @return the new created FieldDeclaration <br></br>
     *     </code>private (static) typeRef name</code>
     */
    private FieldDeclaration createImplicitArrayField
	(String name, TypeReference typeRef, boolean isStatic) {
	ImplicitFieldSpecification varSpec = 
	    new ImplicitFieldSpecification
	    (new ProgramVariable(new ProgramElementName(name),
				 typeRef.getKeYJavaType(), isStatic), 
	     typeRef.getKeYJavaType());	
	// no recoder dependance
	// insertToMap(recoderVarSpec, varSpec);
	Modifier[] modifiers = new Modifier[isStatic ? 2 : 1];
	modifiers[0] = new Private();
	if (isStatic) {
	    modifiers[1] = new Static();
	}
	return new FieldDeclaration
	    (modifiers, typeRef, new FieldSpecification[]{varSpec},
	     false);
    }



    /**
     * Adds several implicit fields and methods to given list of members.
     * @param members an ExtList with the members of parent
     * @param parent the KeYJavaType of the array to be enriched by its
     * implicit members
     * @param baseType the KeYJavaType of the parent's element type
     */
    private void addImplicitArrayMembers(ExtList members, 
					 KeYJavaType parent, 
					 KeYJavaType baseType) {
	
	Type base = baseType.getJavaType();
	int dimension = base instanceof ArrayType ? 
	    ((ArrayType)base).getDimension() + 1 : 1;
	TypeRef parentReference = new TypeRef(new ProgramElementName
					      (""+parent.getSort().name()), 
					      dimension, null, parent); 

	members.add(createImplicitArrayField("<first>", parentReference, true));
	members.add(createImplicitArrayField("<nextToCreate>", parentReference, 
					     true));
	KeYJavaType booleanKJT = getKeYJavaType
	    (new recoder.abstraction.PrimitiveType
	     ("boolean", servConf.getSourceInfo()));
	// instance
	members.add(createImplicitArrayField("<created>", 
					     new TypeRef(booleanKJT), 
					     false));
	members.add(createImplicitArrayField("<initialized>", 
					     new TypeRef(booleanKJT), 
					     false));
	members.add(createImplicitArrayField("<next>", parentReference, 
					     false));    
	

	// add methods
	// the only situation where base can be null is in case of a
	// reference type
	Expression defaultValue = (base != null ? 
				   base.getDefaultValue() : 
				   NullLiteral.NULL);
	ListOfField fields = filterField(members);
	ProgramVariable length = find("length", fields);
	ProgramMethod prepare = createPrepareArrayMethod
	    (parentReference, length, defaultValue, fields);
	members.add(prepare);
	members.add(createCreateArrayMethod(parentReference, 
					    length, prepare,
					    fields));
   }

    /**
     * creates the field declaration for the public final integer field
     * <code>length</code>
     */
    private FieldDeclaration createArrayLength() {
	KeYJavaType integerType = getKeYJavaType
	    (new recoder.abstraction.PrimitiveType
		("int", servConf.getSourceInfo()));
	FieldSpecification specLength = new FieldSpecification
	    (new ProgramVariable
		(new ProgramElementName("length"), integerType, false));
	return new FieldDeclaration
	    (new Modifier[]{new Public(), new Final()},
	     new TypeRef(integerType),
	     new FieldSpecification[]{specLength}, false);
    }

    public ArrayDeclaration createArrayType(KeYJavaType baseType, 
					    KeYJavaType arrayType) {
	ExtList members = new ExtList();
	if (baseType.getJavaType() != null) {
	    members.add(new TypeRef(baseType));
	} else {
	    members.add(new TypeRef
		(new ProgramElementName
		    (""+baseType.getSort().name()), 0, 
		 null, baseType));
	}
	members.add(createArrayLength());	

	addImplicitArrayMembers(members, arrayType, baseType);	

	return new ArrayDeclaration(members);
    }

}
