// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.java;

import de.uka.ilkd.key.java.abstraction.KeYJavaType;
import de.uka.ilkd.key.java.abstraction.PrimitiveType;
import de.uka.ilkd.key.java.declaration.LocalVariableDeclaration;
import de.uka.ilkd.key.java.declaration.Modifier;
import de.uka.ilkd.key.java.declaration.VariableSpecification;
import de.uka.ilkd.key.java.expression.operator.Instanceof;
import de.uka.ilkd.key.java.reference.FieldReference;
import de.uka.ilkd.key.java.reference.SchematicFieldReference;
import de.uka.ilkd.key.java.reference.ArrayLengthReference;
import de.uka.ilkd.key.java.reference.ReferencePrefix;
import de.uka.ilkd.key.java.statement.*;
import de.uka.ilkd.key.java.recoderext.SchemaJavaProgramFactory;
import de.uka.ilkd.key.java.recoderext.SchemaCrossReferenceServiceConfiguration;
import de.uka.ilkd.key.java.recoderext.KeYCrossReferenceServiceConfiguration;
import de.uka.ilkd.key.java.recoderext.TypeSVWrapper;
import de.uka.ilkd.key.java.recoderext.ProgramVariableSVWrapper;
import de.uka.ilkd.key.logic.NamespaceSet;
import de.uka.ilkd.key.logic.Namespace;
import de.uka.ilkd.key.logic.ProgramElementName;
import de.uka.ilkd.key.logic.op.IProgramVariable;
import de.uka.ilkd.key.logic.op.ProgramVariable;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.logic.sort.ProgramSVSort;
import de.uka.ilkd.key.rule.ContextTerm;
import de.uka.ilkd.key.rule.metaconstruct.*;
import de.uka.ilkd.key.util.ExtList;
import de.uka.ilkd.key.util.Debug;
import java.io.IOException;
import java.io.StringReader;
import java.util.Iterator;

public class SchemaRecoder2KeY extends Recoder2KeY implements SchemaJavaReader{
    
    /** the namespace containing the program schema variables allowed here*/
    protected Namespace svns;
    public static KeYJavaType typeSVType = new KeYJavaType
	(PrimitiveType.PROGRAM_SV, ProgramSVSort.TYPE);
    private static SchemaCrossReferenceServiceConfiguration servConf 
	= new SchemaCrossReferenceServiceConfiguration();

    public SchemaRecoder2KeY(KeYRecoderMapping krm, TypeConverter tc, 
			     NamespaceSet nss) {
	super(servConf, null, null, krm, tc, nss);
    }

    public SchemaRecoder2KeY(recoder.java.JavaProgramElement progElem,
			     String classPath, KeYRecoderMapping krm, 
			     TypeConverter tc) {
	super((de.uka.ilkd.key.java.recoderext
			  .KeYCrossReferenceServiceConfiguration)
	      new SchemaCrossReferenceServiceConfiguration(), 
	      progElem, classPath, krm, tc, new NamespaceSet());
    }

    public SchemaRecoder2KeY(recoder.java.JavaProgramElement progElem,
			     KeYRecoderMapping krm, TypeConverter tc) {
	this(progElem, null, krm, tc);
    }

    public SchemaRecoder2KeY(String classPath, KeYRecoderMapping krm, 
			     TypeConverter tc) {
	this(null, classPath, krm, tc);
    }

    public void setSVNamespace(Namespace svns) {
	this.svns=svns;
    }

    /** creates an empty RECODER compilation unit 
     * @return the recoder.java.CompilationUnit 
     */
    public Context createEmptyContext() {
	recoder.ProgramFactory pf=servConf.getProgramFactory();
	recoder.java.Identifier id= new recoder.java.Identifier("KeYSpecialParsing");
	recoder.java.declaration.ClassDeclaration cd
	    = pf.createClassDeclaration(null, id, null, null, null);
	return new Context(servConf, new recoder.java.CompilationUnit(), cd);
    }

    public ProgramMetaConstruct 
	convert(de.uka.ilkd.key.java.recoderext.RKeYMetaConstruct mc) {

	ExtList list = new ExtList();
	String mcName = mc.getName();
	list.add(callConvert(mc.getChild()));
	if("#switch-to-if".equals(mcName)){
	    return new SwitchToIf((SchemaVariable)list.get(SchemaVariable.class));
	}
	if ("#unwind-loop".equals(mcName)) {
	    return new UnwindLoop
		((LoopStatement)list.get(LoopStatement.class));	    
	} else if ("#do-break".equals(mcName)) {	
	    return new DoBreak((LabeledStatement)list.get(LabeledStatement.class));
	} else if ("#expand-method-body".equals(mcName)) {	
	    return new ExpandMethodBody((SchemaVariable)list.
					get(SchemaVariable.class));
	} else if ("#method-call".equals(mcName)) {	
	    if (mc.getVariableSV() == null) {
		return new MethodCall
		    ((Expression)list.get(Expression.class));
	    } else {
		return new MethodCall
		    (mc.getVariableSV().getSV(),
		     (Expression)list.get(Expression.class));
	    }
	} else if ("#constructor-call".equals(mcName)) {	
	    return new ConstructorCall
		(mc.getVariableSV().getSV(), 
		 (Expression)list.get(Expression.class));
	} else if ("#special-constructor-call".equals(mcName)) {	
	    return new SpecialConstructorCall
		((Expression)list.get(Expression.class));
	} else if ("#post-work".equals(mcName)) {	
	    return new PostWork
		(mc.getVariableSV().getSV(), 
		 (Expression)list.get(Expression.class));
	} else if ("#resolve-multiple-var-decl".equals(mcName)) {	
	    return new MultipleVarDecl
		((SchemaVariable)list.get(SchemaVariable.class));
	} else if ("#init-array-creation".equals(mcName)) {	
	    return new InitArrayCreation
		    (mc.getVariableSV().getSV(), 
		     (Expression)list.get(Expression.class));
	} else {	
	    throw new ConvertException("Program meta construct "
				       +mc.toString()
				       +" unknown.");	
	}
    }
 	

    public ProgramMetaConstruct 
	convert(de.uka.ilkd.key.java.recoderext.RKeYMetaConstructExpression mc) {
	ExtList list=new ExtList();
	String mcName = mc.getName();
	list.add(callConvert(mc.getChild()));
	if ("#static-evaluate".equals(mcName)) {
	    return new InstanceOfEval
		((Instanceof)list.get(Instanceof.class));	    
	} else if ("#mul".equals(mcName)) {
	    return new InstanceOfEval
		((Instanceof)list.get(Instanceof.class));	    
	} else if ("#create-object".equals(mcName)) {
	    return new CreateObject((Expression)list.get(Expression.class));
	} else if ("#isstatic".equals(mc.getName())) {
	    return new IsStatic((Expression)list.get(Expression.class));	    
	} else if ("#length-reference".equals(mcName)) {
	    return new ArrayLength((Expression)list.get(Expression.class));
	} else {
	    throw new ConvertException("Program meta construct "
				       + mc.toString()
				       + " unknown.");	
	}
    }


    public ProgramMetaConstruct 
	convert(de.uka.ilkd.key.java.recoderext.RKeYMetaConstructType mc) {
	ExtList list = new ExtList();
	list.add(callConvert(mc.getChild()));
	if ("#typeof".equals(mc.getName0())) {
	    return new TypeOf
		((Expression)list.get(Expression.class));
	} else {
	    throw new ConvertException("Program meta construct "
				       +mc.toString()
				       +" unknown.");	
	}
    }

    public MethodCallStatement 
	convert(de.uka.ilkd.key.java.recoderext.RMethodCallStatement l) {
	ExtList list = collectChildren(l);
	return new MethodCallStatement(list, null);
    }
 

    public ContextTerm
	convert(de.uka.ilkd.key.java.recoderext.ContextStatementBlock csb) {
	ExtList children = collectChildren(csb);
	return new ContextTerm(children);
    }


    public SchemaVariable
	convert(de.uka.ilkd.key.java.recoderext.ExpressionSVWrapper svw) {

	return svw.getSV();
    }


    public SchemaVariable
	convert(de.uka.ilkd.key.java.recoderext.StatementSVWrapper svw) {

	return svw.getSV();
    }

    public SchemaVariable
	convert(de.uka.ilkd.key.java.recoderext.LabelSVWrapper svw) {

	return svw.getSV();
    }


    public SchemaVariable
	convert(de.uka.ilkd.key.java.recoderext.TypeSVWrapper svw) {

	return svw.getSV();
    }

    public SchemaVariable
	convert(de.uka.ilkd.key.java.recoderext.CatchSVWrapper svw) {
	return svw.getSV();
    }

    public SchemaVariable
	convert(de.uka.ilkd.key.java.recoderext.ProgramVariableSVWrapper svw) {

	return svw.getSV();
    }
    
    /**
     * wraps a RECODER ClassDeclaration in a compilation unit
     * @param classDecl the recoder.java.ClassDeclaration to wrap
     * @param cUnit the recoder.java.CompilationUnit where the class is wrapped
     * @return the enclosing recoder.java.CompilationUnit
     */
    protected recoder.java.CompilationUnit embedClass
	(recoder.java.declaration.ClassDeclaration classDecl,
	 Context context) { 
	
	recoder.java.CompilationUnit cUnit = 
	    context.getCompilationUnitContext();

	// add class to compilation unit
	recoder.list.TypeDeclarationMutableList typeDecls = 	    
	    cUnit.getDeclarations();

	if (typeDecls==null) {
	    typeDecls=new recoder.list.TypeDeclarationArrayList(0); 
	} else {
	    typeDecls = (recoder.list.TypeDeclarationMutableList) 
		typeDecls.deepClone();
	}
	typeDecls.add(classDecl);

 	recoder.java.CompilationUnit compUnitContext
 	    = (recoder.java.CompilationUnit)cUnit.deepClone();
	
	compUnitContext.setDeclarations(typeDecls);
	compUnitContext.makeParentRoleValid();
	servConf.getChangeHistory().attached(compUnitContext);
	servConf.getChangeHistory().updateModel();
	return compUnitContext;
    }  


    /** parses a given JavaBlock using the context to determine the right 
     * references and returns a statement block of recoder.
     * @param block a String describing a java block
     * @param context recoder.java.CompilationUnit in which the block has 
     * to be interpreted
     * @return the parsed and resolved recoder statement block
     */
    protected recoder.java.StatementBlock 
	recoderBlock(String block, Context context) { 
	recoder.java.StatementBlock bl = null;

	SchemaJavaProgramFactory factory
	    = (SchemaJavaProgramFactory) servConf.getProgramFactory();
	factory.setSVNamespace(svns);
	try {
	    bl=factory.parseStatementBlock(new StringReader(block));
	} catch (recoder.ParserException e) {
	    Debug.out("readSchemaJavaBlock(Reader,CompilationUnit)"
		      + " caused the " +
		      "exception:\n", e);
	    Debug.printStackTrace(e);
	    throw new ConvertException
		("Parsing: \n **** BEGIN ****\n "+ block + 
		 "\n **** END ****\n failed. Thrown Exception:" +
		 e.toString());
	} catch (IOException ioe) {
	    Debug.out("readSchemaJavaBlock(Reader,CompilationUnit)"
		      + " caused the IO exception:\n", ioe);
	    Debug.printStackTrace(ioe);
	    throw new ConvertException
		("IO Error when parsing: \n **** BEGIN ****\n "+ block +
		 "\n **** END ****\n failed. Thrown IOException:" + 
		 ioe.toString());
	}	

	embedClass(embedMethod(embedBlock(bl), context), context);

	return (recoder.java.StatementBlock)bl;
    }


    /**
     * converts a For.
     * @param f the For of recoder
     * @return the For of KeY
     */
    public For convert(recoder.java.statement.For f) {

	ILoopInit li;
	IForUpdates ifu;
	IGuard iGuard;
	if (f.getInitializers()!=null && 
	    f.getInitializers().getLoopInitializer(0) 
	    instanceof de.uka.ilkd.key.java.recoderext.ExpressionSVWrapper) {
	    de.uka.ilkd.key.java.recoderext.ExpressionSVWrapper esvw
		= (de.uka.ilkd.key.java.recoderext.ExpressionSVWrapper)
		f.getInitializers().getLoopInitializer(0); //brrrr!
	    li = esvw.getSV();
	} else {
	    li = convertLoopInitializers(f);
	}
	
	if (f.getGuard() instanceof 
	    de.uka.ilkd.key.java.recoderext.ExpressionSVWrapper) {
	    de.uka.ilkd.key.java.recoderext.ExpressionSVWrapper esvw
		= (de.uka.ilkd.key.java.recoderext.ExpressionSVWrapper)
		f.getGuard();
	    iGuard = esvw.getSV();
	} else {
	    iGuard = convertGuard(f);
	}

       
	if (f.getUpdates() != null && f.getUpdates().getExpression(0) 
	    instanceof de.uka.ilkd.key.java.recoderext.ExpressionSVWrapper) {
	    de.uka.ilkd.key.java.recoderext.ExpressionSVWrapper esvw
		= (de.uka.ilkd.key.java.recoderext.ExpressionSVWrapper)
		f.getUpdates().getExpression(0);
	    ifu = esvw.getSV();
	} else {
	    ifu = convertUpdates(f);
	}

	return new For(li, iGuard, ifu, convertBody(f));
    }


//     protected VariableSpecification
// 	convert(recoder.java.declaration.VariableSpecification recoderVarspec){
// 	return super.convert(recoderVarspec);
//     }


     protected VariableSpecification convertVarSpecWithSVType
 	(recoder.java.declaration.VariableSpecification recoderVarspec){
	 VariableSpecification varspec=
 	    (VariableSpecification) rec2key().toKeY(recoderVarspec);
 	if (varspec==null) {	    
 	    ExtList l=collectChildren(recoderVarspec);	    
	    ProgramElement pv
		= ProgramSVSort.VARIABLE.getSVWithSort(l, 
						       ProgramElementName.class);
	    if (pv instanceof ProgramElementName) {  //sth. like #type i;
		KeYJavaType kjt=new KeYJavaType(typeSVType);
		pv=new ProgramVariable((ProgramElementName)pv, kjt);
	    }
	    ProgramElement init
		= ProgramSVSort.VARIABLEINIT.getSVWithSort
		(l, Expression.class);
 	    varspec = new VariableSpecification((IProgramVariable)pv, 
						recoderVarspec.getDimensions(),
						(Expression)init,
						typeSVType);
 	    insertToMap(recoderVarspec, varspec);
 	}
 	return varspec;
     }

     public LocalVariableDeclaration
 	convert(recoder.java.declaration.LocalVariableDeclaration lvd) {  
 	if (lvd.getTypeReference() instanceof TypeSVWrapper) {
 	    recoder.list.VariableSpecificationList rspecs = lvd.getVariables();
 	    VariableSpecification[] varspecs
 		= new VariableSpecification[rspecs.size()];
 	    for (int i=0; i<rspecs.size(); i++) {
 		varspecs[i] = convertVarSpecWithSVType
 		    (rspecs.getVariableSpecification(i));
 	    }
 	    SchemaVariable typesv
 		= ((TypeSVWrapper)lvd.getTypeReference()).getSV();
 	    return new LocalVariableDeclaration(new Modifier[0], typesv, varspecs);
 	} else {
 	    return super.convert(lvd);
 	}
     }



     /** convert a recoder VariableSpecification to a KeY
      * VariableSpecification
      * (checks dimension and hands it over and insert in hashmap)
      */
     public VariableSpecification
 	convert(recoder.java.declaration.VariableSpecification recoderVarspec){	

 	if (! (recoderVarspec.getIdentifier() instanceof ProgramVariableSVWrapper)) {
 	    return super.convert(recoderVarspec);
 	}
 	VariableSpecification varSpec
 	    = (VariableSpecification)rec2key().toKeY(recoderVarspec);
 	if (varSpec==null) {

 	    ProgramElementName name=new ProgramElementName
 		(recoderVarspec.getName());
 	    ExtList children = collectChildren(recoderVarspec);
 	    IProgramVariable progvar 
		= (IProgramVariable) children.get(SchemaVariable.class);

	    children.remove(progvar);
 	    varSpec = new VariableSpecification(children, 
 					    progvar,
 					    recoderVarspec.getDimensions(),
 					    null);
 	    insertToMap(recoderVarspec, varSpec);

 	}
 	return varSpec;
     }

    public FieldReference convert
	 (recoder.java.reference.FieldReference fr) {

	 ReferencePrefix prefix = null;	
	 if (fr.getReferencePrefix() != null) {
	     prefix = (ReferencePrefix)doConvert
		 ((recoder.java.JavaProgramElement)fr.getReferencePrefix());
	 }

         SchemaVariable suffix = (SchemaVariable) doConvert
	     ((recoder.java.JavaProgramElement) fr.getIdentifier());
         
         return new SchematicFieldReference(suffix, prefix);
    }

} 
