// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.java.abstraction;

import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.logic.op.Function;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.java.expression.Literal;

/**
 * The KeY java type realises a tupel (sort, type) of a logic sort and
 * the java type (for example a class declaration). 
 * In contrast to other classes the KeYJavaType is <emph>not</emph>
 * immutable, so use it with care. 
 */
public class KeYJavaType implements Type {


    /** the AST type */
    private Type javaType=null;
    /** the logic sort */
    private Sort sort=null;

    private Function typeConst=null;
    
    /** creates a new KeYJavaType */
    public KeYJavaType() {
    }

    /** creates a new KeYJavaType */
    public KeYJavaType(Type javaType, Sort sort) {
	this.javaType = javaType;
	this.sort = sort;
	typeConst = new Function(sort.name(), Sort.TYPE, new Sort[0]);
    }

    /** creates a new KeYJavaType */
    public KeYJavaType(Sort sort) {
	this.sort = sort;
	typeConst=new Function(sort.name(), Sort.TYPE, new Sort[0]);
    }

    /** creates a new KeYJavaType */
    public KeYJavaType(Type type) { 
	this.javaType = type;
	typeConst=new Function(new Name(type.getFullName()), 
			       Sort.TYPE, new Sort[0]);
    }

    public Function typeConst() {
	return typeConst;
    }

     public void setJavaType(Type type) {
	 javaType = type;
     }
    
    public void setSort(Sort s) {
 	sort = s;
    }

    public Type getJavaType() {
	return javaType;
    }

    public Sort getSort() {
	return sort;
    }

    /** 
     * returns the default value of the given type 
     * according to JLS 4.5.5 
     * @return the default value of the given type 
     * according to JLS 4.5.5
     */
    public Literal getDefaultValue() {
	return javaType.getDefaultValue();
    }
    
    public String toString() {
	if (javaType == null) return "KeYJavaType:null,"+sort;
	return "(type, sort): ("+javaType.getName()+","+sort+")"; 
    }

    public String getFullName() {
	return getJavaType().getFullName();
    }

    public String getName() {
	return getJavaType().getName();
    }

}

