// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//


package de.uka.ilkd.key.java.expression;

import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.util.ExtList;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;


/** 
 *  An assignment is an operator with side-effects.
 */

public abstract class Assignment extends Operator 
    implements ExpressionStatement {

    /**
     * Constructor for the transformation of COMPOST ASTs to KeY.
     * @param children the children of this AST element as KeY classes.
     * In this case the order of the children is IMPORTANT. 
     * 	May contain:
     * 		2 of Expression (the first Expression as left hand
     * 			side, the second as right hand side), 
     * 		Comments
     */
    public Assignment(ExtList children) {
	super(children);
    }

    public Assignment() {

    }

    /**
       Assignment.
       @param lhs an expression.
       @param rhs an expression.
    */
    public Assignment(Expression lhs, Expression rhs) {
        super(lhs, rhs);
    }

    /**
       Unary Assignment (e.g. +=, ++).
       @param lhs an expression.
    */
    public Assignment(Expression lhs) {
        super(lhs);
    }

    /**
     *        Checks if this operator is left or right associative. Assignments
     *        are right associative.
     *        @return <CODE>true</CODE>, if the operator is left associative,
     *        <CODE>false</CODE> otherwise.
     */

    public boolean isLeftAssociative() {
        return false;
    }


    public KeYJavaType getKeYJavaType(Services javaServ) {
	return javaServ.getTypeConverter().getKeYJavaType
	    (getExpressionAt(0));
    }
}
