// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

// This file is part of the COMPOST library, (c) 1999 Universitaet Karlsruhe.

package de.uka.ilkd.key.java.expression.literal;

import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.expression.*;
import de.uka.ilkd.key.util.ExtList;
import de.uka.ilkd.key.java.visitor.Visitor;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;
import de.uka.ilkd.key.java.abstraction.PrimitiveType;


/**
 *  Boolean literal.
 *  @author <TT>AutoDoc</TT>
 */

public class BooleanLiteral extends Literal {

    public static BooleanLiteral TRUE = new BooleanLiteral(true);
    public static BooleanLiteral FALSE = new BooleanLiteral(false);


    protected final boolean value;


    /**
 *      Boolean literal.
 *      @param value a boolean value.
     */

    public BooleanLiteral(boolean value) {
        this.value=value;
    }

    /**
     *      Boolean literal.
     *      @param children list with all children
     *       May contain: Comments
     *      @param value a boolean value.
     */

    public BooleanLiteral(ExtList children, boolean value) {
	super(children);
        this.value=value;
    }

    /**
 *      Get value.
 *      @return the string.
     */

    public boolean getValue() {
        return value;
    }

    /**
 *      Get value.
 *      @return the string.
     */

    public String getName() {
        return (value ? "true" : "false") ;
    }

    /** tests if equals
     */
    public boolean equals(Object o) {
	if (!(o instanceof BooleanLiteral)) {
	    return false;
	}
	return ((BooleanLiteral)o).getValue() == getValue(); 
    }



    /** calls the corresponding method of a visitor in order to
     * perform some action/transformation on this element
     * @param v the Visitor
     */
    public void visit(Visitor v) {
	v.performActionOnBooleanLiteral(this);
    }

    public void prettyPrint(PrettyPrinter p) throws java.io.IOException {
        p.printBooleanLiteral(this);
    }


    public KeYJavaType getKeYJavaType(Services javaServ) {
	return javaServ.getJavaInfo().getKeYJavaType(PrimitiveType.JAVA_BOOLEAN);
    }

}
