// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

// This file is part of the COMPOST library, (c) 1999 Universitaet Karlsruhe.

package de.uka.ilkd.key.java.expression.literal;

import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.expression.*;
import de.uka.ilkd.key.util.ExtList;
import de.uka.ilkd.key.java.visitor.Visitor;
import de.uka.ilkd.key.java.abstraction.PrimitiveType;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;

/**
 *  Long literal.
 *  @author <TT>AutoDoc</TT>
 */

public class LongLiteral extends Literal {

    /**
 *      Textual representation of the value.
     */

    protected final String value;


    /**
 *      Long literal.
 *      @param value a long value.
     */

    public LongLiteral(long value) {
        this.value="" + value + 'L';
    }

    /**
     *      Long literal.
     *      @param list with children(comments)
     *      @param value a string.
     */

    public LongLiteral(ExtList children, String value) {
	super(children);
        this.value=(value.endsWith("L") || value.endsWith("l")) ? value : (value + 'L');
    }

    /**
 *      Long literal.
 *      @param value a string.
     */

    public LongLiteral(String value) {
        this.value=(value.endsWith("L") || value.endsWith("l")) ? value : (value + 'L');
    }

    /** tests if equals
     */
    public boolean equals(Object o) {
	if (!(o instanceof LongLiteral)) {
	    return false;
	}
	return ((LongLiteral)o).getValue().equals(getValue()); 
    }

    /**
 *      Get value.
 *      @return the string.
     */

    public String getValue() {
        return value;
    }

    /** calls the corresponding method of a visitor in order to
     * perform some action/transformation on this element
     * @param v the Visitor
     */
    public void visit(Visitor v) {
	v.performActionOnLongLiteral(this);
    }

    public void prettyPrint(PrettyPrinter p) throws java.io.IOException {
        p.printLongLiteral(this);
    }


    public KeYJavaType getKeYJavaType(Services javaServ) {
	return javaServ.getJavaInfo().getKeYJavaType(PrimitiveType.JAVA_LONG);
    }

}
