// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

// This file is part of the COMPOST library, (c) 1999 Universitaet Karlsruhe.

package de.uka.ilkd.key.java.expression.literal;

import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.util.ExtList;
import de.uka.ilkd.key.java.expression.*;
import de.uka.ilkd.key.java.reference.*;
import de.uka.ilkd.key.java.visitor.Visitor;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;

/**
 *  String literal.
 *  @author <TT>AutoDoc</TT>
 */

public class StringLiteral extends Literal implements ReferencePrefix {

    /**
 *      The value.
     */

    protected final String value;


    /**
 *      String literal.
 *      @param value a string.
     */

    public StringLiteral(String value) {
        this.value=value;
    }

    /**
 *      String literal.
 *      @param childen an ExtList with children(here:comments)
 *      @param value a string.
     */

    public StringLiteral(ExtList children, String value) {
	super(children);
        this.value=value;
    }


    /** tests if equals
     */
    public boolean equals(Object o) {
	if (!(o instanceof StringLiteral)) {
	    return false;
	}
	return ((StringLiteral)o).getValue().equals(getValue()); 
    }

    /**
 *      Get value.
 *      @return the string.
     */

    public String getValue() {
        return value;
    }

    /** calls the corresponding method of a visitor in order to
     * perform some action/transformation on this element
     * @param v the Visitor
     */
    public void visit(Visitor v) {
	v.performActionOnStringLiteral(this);
    }

    public void prettyPrint(PrettyPrinter p) throws java.io.IOException {
        p.printStringLiteral(this);
    }


    /**
     * We do not have a prefix, so fake it!
     * This way we implement ReferencePrefix
     * @author VK
     */
    public ReferencePrefix getReferencePrefix() {
	return null;
    }

    public ReferencePrefix setReferencePrefix(ReferencePrefix r) {
	return this;
    }

    public KeYJavaType getKeYJavaType(Services javaServ) {
	//implement me!!!!%%%
	throw new IllegalStateException("Not implemented: StringLiteral did "
					+"not know how to "
					+"determine its KeYJavaType");
    }


}
