// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

// This file is part of the COMPOST library, (c) 1999 Universitaet Karlsruhe.

package de.uka.ilkd.key.java.expression.operator;

import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.abstraction.*;
import de.uka.ilkd.key.java.expression.*;
import de.uka.ilkd.key.util.ExtList;
import java.io.*;
import de.uka.ilkd.key.java.visitor.Visitor;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;

/** The most weird ternary C operator ?: */

public class Conditional extends Operator {


    /**
     *      Conditional.
     *      @param children list of children the first one is the guard expression, 
     *      the second one the then expression and the last one the else expr.
     */

    public Conditional(ExtList children) {
	super(children);
    }


    /**
 *      Get arity.
 *      @return the int value.
     */

    public int getArity() {
        return 3;
    }

    /**
 *      Get precedence.
 *      @return the int value.
     */

    public int getPrecedence() {
        return 12;
    }

    /**
 *      Get notation.
 *      @return the int value.
     */

    public int getNotation() {
        return INFIX;
    }

    /**
 *        Checks if this operator is left or right associative. Conditionals
 *        are right associative.
 *        @return <CODE>true</CODE>, if the operator is left associative,
 *        <CODE>false</CODE> otherwise.
     */

    public boolean isLeftAssociative() {
        return false;
    }

    /** calls the corresponding method of a visitor in order to
     * perform some action/transformation on this element
     * @param v the Visitor
     */
    public void visit(Visitor v) {
	v.performActionOnConditional(this);
    }

    public void prettyPrint(PrettyPrinter p) throws java.io.IOException {
        p.printConditional(this);
    }

    public KeYJavaType getKeYJavaType(Services javaServ) {
	TypeConverter typeConverter = javaServ.getTypeConverter();
	KeYJavaType type1 = typeConverter.
	    getKeYJavaType(getExpressionAt(1));
	KeYJavaType type2 = typeConverter.
	    getKeYJavaType(getExpressionAt(2));
	if (typeConverter.isIdentical(type1, type2))
	    return type1;

	// numeric types
	if (typeConverter.isArithmeticType(type1) &&
	    typeConverter.isArithmeticType(type2) ) {
	    if (type1.getJavaType() == PrimitiveType.JAVA_BYTE &&
		type2.getJavaType() == PrimitiveType.JAVA_SHORT || 
		type1.getJavaType() == PrimitiveType.JAVA_SHORT &&
		type2.getJavaType() == PrimitiveType.JAVA_BYTE)
		return javaServ.getJavaInfo().
		    getKeYJavaType(PrimitiveType.JAVA_SHORT);
	    if (typeConverter.isImplicitNarrowing(getExpressionAt(1),
						  (PrimitiveType)type2.getJavaType()))
		return type2;
	    if (typeConverter.isImplicitNarrowing(getExpressionAt(2),
						  (PrimitiveType)type1.getJavaType()))
		return type1;
	    return typeConverter.getPromotedType(type1, type2);
	}

	// reference types
	if (typeConverter.isNullType(type1) &&
	    typeConverter.isReferenceType(type2))
	    return type2;
	if (typeConverter.isNullType(type2) &&
	    typeConverter.isReferenceType(type1))
	    return type1;
	if (typeConverter.isAssignableTo(type1, type2))
	    return type2;
	if (typeConverter.isAssignableTo(type2, type1))
	    return type1;

	throw new RuntimeException("Could not determine type of conditional "+
				   "expression\n"+this+". This usually means that "+
				   "the Java program is not compilable.");
    }

}
