// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//


package de.uka.ilkd.key.java.expression.operator;

import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.expression.*;
import de.uka.ilkd.key.util.ExtList;
import de.uka.ilkd.key.java.visitor.Visitor;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;

/**
 *  Divide.
 */

public class Divide extends Operator {

    /**
     *      Divide.
     *      @param children an ExtList with all children of this node
     *      the first children in list will be the one on the left
     *      side, the second the one on the  right side.
     */

    public Divide(ExtList children) {
        super(children);
    }

    public Divide(Expression lhs, Expression rhs) {
	super(lhs, rhs);
    }


    /**
     *      Get arity.
     *      @return the int value.
     */

    public int getArity() {
        return 2;
    }

    /**
     *      Get precedence.
     *      @return the int value.
     */

    public int getPrecedence() {
        return 2;
    }

    /**
 *      Get notation.
 *      @return the int value.
     */

    public int getNotation() {
        return INFIX;
    }

    /** calls the corresponding method of a visitor in order to
     * perform some action/transformation on this element
     * @param v the Visitor
     */
    public void visit(Visitor v) {
	v.performActionOnDivide(this);
    }

    public void prettyPrint(PrettyPrinter p) throws java.io.IOException {
        p.printDivide(this);
    }


    public KeYJavaType getKeYJavaType(Services javaServ) {
	TypeConverter tc=javaServ.getTypeConverter();
	    return tc.getPromotedType
		(tc.getKeYJavaType((Expression)getChildAt(0)),
		 tc.getKeYJavaType((Expression)getChildAt(1)));
    }
}
