package de.uka.ilkd.key.java.recoderext;

import de.uka.ilkd.key.util.Debug;
import recoder.*;
import recoder.abstraction.*;
import recoder.java.*;
import recoder.java.declaration.*;
import recoder.java.declaration.modifier.*;
import recoder.java.expression.operator.*;
import recoder.java.expression.literal.*;
import recoder.java.reference.*;
import recoder.java.statement.*;
import recoder.kit.TwoPassTransformation;
import recoder.list.ClassTypeList;
import recoder.list.FieldList;

import recoder.list.CompilationUnitMutableList;
import recoder.list.FieldSpecificationList;
import recoder.list.MemberDeclarationMutableList;
import recoder.list.ModifierMutableList;
import recoder.list.ModifierArrayList;
import recoder.list.ParameterDeclarationArrayList;
import recoder.list.StatementArrayList;
import recoder.list.StatementMutableList;
import java.util.*;

/**
 * The Java DL requires some implicit fields and methods, that are
 * available in each Java class. The name of the implicit fields/methods
 * is usually enclosed between two angle brackets. To access them in a
 * uniform way, they are added as usual fields to the classes, in
 * particular this makes it possible to parse them in a natural way.
 *  The ImplicitMethodAdder make use of two builders that encapsulate
 * the information about the implementation of the implicit
 * methods. These builder classes are described in
 *   <ul>
 *     <li> {@link CreateObjectBuilder}  </li>
 *     <li> {@link PrepareObjectBuilder} </li>
 *   </ul>
 * @see ImplicitFieldAdder
 */
public class ImplicitMethodAdder extends RecoderModelTransformer {

    private ImplicitMethodBuilder[] builder;

    /**
     * creates a transformation that adds the implicit methods
     * <code>&lt;createObject&gt;</code>, <code>&lt;prepare&gt;</code>
     * and <code>&lt;prepareParent&gt;</code>
     * @param services the CrossReferenceServiceConfiguration to access
     * model information
     * @param units the array of CompilationUnits describing the model
     * to be transformed
     */
    public ImplicitMethodAdder
	(CrossReferenceServiceConfiguration services, 
	 CompilationUnitMutableList units) {	

	super(services, units);

	builder = new ImplicitMethodBuilder[] {
	    new PrepareObjectBuilder(services), 
	    new CreateObjectBuilder(services)
		};
    }

    /** 
     * attaches a method declaration to the declaration of type td at
     * position idx
     * @param md the MethodDeclaration to insert
     * @param td the TypeDeclaration that becomes parent of the new
     * method
     * @param idx the position where to add the method
     */
    public void attach(MethodDeclaration md, TypeDeclaration td, 
		       int idx) {
	super.attach(md, td, idx);
    }
    
    /**
     * 
     */
    protected void makeExplicit(TypeDeclaration td) {
	if (td instanceof ClassDeclaration) {
	    for (int i = 0; i<builder.length; i++) {
		MethodDeclaration md = builder[i].createMethod(this, td);
		attach(md, td, 0);
	    }
	    /* 	    inner classes are not supported
		    int typeCount = td.getTypeDeclarationCount();
		    for (int j = 0; j<typeCount; j++) {
		    makeExplicit(td.getTypeDeclarationAt(j));
		    } 
	    */
	    td.makeAllParentRolesValid();
	}
    }
}
