package de.uka.ilkd.key.java.recoderext;

import de.uka.ilkd.key.util.Debug;
import recoder.*;
import recoder.abstraction.*;
import recoder.java.*;
import recoder.java.declaration.*;
import recoder.java.declaration.modifier.*;
import recoder.java.expression.operator.*;
import recoder.java.expression.literal.*;
import recoder.java.reference.*;
import recoder.java.statement.*;
import recoder.kit.TwoPassTransformation;
import recoder.list.ClassTypeList;
import recoder.list.FieldList;

import recoder.list.CompilationUnitMutableList;
import recoder.list.FieldSpecificationList;
import recoder.list.MemberDeclarationMutableList;
import recoder.list.ModifierMutableList;
import recoder.list.ModifierArrayList;
import recoder.list.ParameterDeclarationArrayList;
import recoder.list.StatementArrayList;
import recoder.list.StatementMutableList;
import java.util.*;

/**
 * 
 */
public class PrepareObjectBuilder extends ImplicitMethodBuilder {

    public PrepareObjectBuilder
	(CrossReferenceServiceConfiguration services) {	
	super(services);
    }

    /** 
     * returns the default value of the given type 
     * according to JLS 4.5.5
     * @return the default value of the given type 
     * according to JLS 4.5.5
     */
    private Expression getDefaultValue(Type type) {
	if (type instanceof ClassType || type instanceof ArrayType) {
	    return new NullLiteral();
	} else if (type instanceof PrimitiveType) {
	    if ("boolean".equals(type.getName())) {
		return new BooleanLiteral(false);
	    } else if ("byte".equals(type.getName())  ||
		       "short".equals(type.getName()) || 
		       "int".equals(type.getName())) {
		return new IntLiteral(0);
	    } else if ("long".equals(type.getName())) {
		return new LongLiteral((long)0);
	    } else if ("char".equals(type.getName())) {
		return new CharLiteral((char)0);
	    } else if ("float".equals(type.getName())) {
		return new FloatLiteral((float)0.0);
	    } else if ("double".equals(type.getName())) {
		return new DoubleLiteral((double)0.0);
	    }
	}
	Debug.fail("makeImplicitMembersExplicit: unknown primitive type"+type);
	return null;
    }
    

    /**
     *  creates the assignments of the field variables to their default values
     * and inserts them to the given body list
     * @return the same list body that has been handed over as parameter
     */
    private LinkedList defaultSettings(ReferencePrefix prefix,
				       FieldList fields,
				       LinkedList body) {
	LinkedList result = body;

	for (int i = 0; i<fields.size(); i++) {
	    Field field = fields.getField(i);
	    if (!field.isStatic()) {
		result.addLast(assign(
		    (attribute(prefix, 
			       (FieldSpecification)field)),
		     getDefaultValue
		     (services.getCrossReferenceSourceInfo().getType(field))));
	    }
	}
	return result;
    }


    /** 
     * creates an implicit method called 'prepare', that sets all attributes to
     * their default values
     */
    protected StatementBlock createPrepareBody
	(ReferencePrefix prefix, TypeDeclaration classType) {

	LinkedList prepareBody = new LinkedList();

	if (!"java.lang.Object".equals(classType.getFullName())) {
	    // we can access the implementation	    	    
	    prepareBody.addFirst
		(new MethodReference(new SuperReference(), 
				     new ImplicitIdentifier("<prepare>")));
	}

	prepareBody = defaultSettings
	    (prefix, services.getCrossReferenceSourceInfo().getAllFields(classType),
	     prepareBody);

	if (prepareBody.size()>0) {
	    StatementMutableList content = new
		StatementArrayList((Statement[])prepareBody.toArray
				   (new Statement[prepareBody.size()]));

	    return new StatementBlock(content);
	} else {
	    return new StatementBlock();
	}
    }
    
    /**
     * creates the implicit <code>&lt;prepare&gt;</code> method that
     * sets the fields of the given type to its default values
     * @param type the TypeDeclaration for which the
     * <code>&lt;prepare&gt;</code> is created
     * @return the implicit <code>&lt;prepare&gt;</code> method
     */
    public MethodDeclaration createMethod(recoder.kit.TwoPassTransformation pass,
					  TypeDeclaration type) {
	ModifierMutableList modifiers = new ModifierArrayList(1);
	modifiers.add(new Protected());	
	return new MethodDeclaration
	    (modifiers, null, new ImplicitIdentifier("<prepare>"), 
	     new ParameterDeclarationArrayList(0), null,
	     createPrepareBody(new ThisReference(), type));
    }    
}
