// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//


package de.uka.ilkd.key.java.reference;


import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.declaration.*;
import de.uka.ilkd.key.logic.op.ProgramVariable;
import de.uka.ilkd.key.util.ExtList;
import de.uka.ilkd.key.java.visitor.Visitor;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;

/**
 *  context variable reference.
 */
public class EmbededVariableReference extends FieldReference {


    public EmbededVariableReference(ProgramVariable pv, ReferencePrefix prefix) {
	super(pv, prefix);
    }

    public EmbededVariableReference(ExtList children, ReferencePrefix prefix) {
	super(children, prefix);
    }



    /** pretty print */
    public void prettyPrint(PrettyPrinter p) throws java.io.IOException {
        p.printEmbededVariableReference((EmbededVariableReference)this);
    }



    /** calls the corresponding method of a visitor in order to
     * perform some action/transformation on this element
     * @param v the Visitor
     */
    public void visit(Visitor v) {
	v.performActionOnEmbededVariableReference(this);
    }

    public boolean equals(Object cmp) {
	Object pv=cmp;
	if (cmp instanceof EmbededVariableReference) {
	    pv=((EmbededVariableReference)cmp).getProgramVariable();
	}
	return pv==getProgramVariable();
    }

    public KeYJavaType getKeYJavaType(Services javaServ) {
	return getProgramVariable().getKeYJavaType();
    }


}
