// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//


package de.uka.ilkd.key.java.reference;

import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.abstraction.Constructor;
import de.uka.ilkd.key.java.declaration.*;
import de.uka.ilkd.key.java.visitor.Visitor;
import de.uka.ilkd.key.logic.op.ProgramVariable;
import de.uka.ilkd.key.util.ExtList;


/**
 *  This constructor reference.
 * 
 */

public class ThisConstructorReference extends SpecialConstructorReference {

    /**
     *      This constructor reference.
     */
    public ThisConstructorReference() {
	super();
    }

    /**
     *      This constructor reference.
     *      @param arguments an expression mutable list.
     */

    public ThisConstructorReference(Expression[] arguments) {
        super(arguments);
    }

    /**
     * This constructor reference.
     * @param arguments an expression mutable list.
     * @param executionContext the ExecutionContext to the execution
     *        context where this reference is placed
     */
    public ThisConstructorReference(Expression[] arguments, 
				    ExecutionContext executionContext) {
        super(arguments, executionContext);
    }

    /**
     * This constructor reference.
     * @param arguments an expression mutable list.
     * @param executionContext the ExecutionContext to the execution
     *        context where this reference is placed
     */
    public ThisConstructorReference(ArrayOfExpression arguments, 
				    ExecutionContext executionContext) {
        super(arguments, executionContext);
    }

    /**
     * Constructor for the transformation of COMPOST ASTs to KeY.
     * @param children the children of this AST element as KeY classes.
     * @param executionContext the ExecutionContext to the execution
     *        context where this reference is placed
     * May contain: 
     * 	several of Expression (as initializers of the array),
     *  Comments
     */ 
    public ThisConstructorReference(ExtList children, 
				    ExecutionContext executionContext) {
	super(children, executionContext);
    }

    /**
     * Constructor for the transformation of COMPOST ASTs to KeY.
     * @param children the children of this AST element as KeY classes.
     * May contain: 
     * 	several of Expression (as initializers of the array),
     *  Comments
     * Must contain:
     *  execution context
     */ 
    public ThisConstructorReference(ExtList children) {
	super(children);
    }


    public int getChildPositionCode(ProgramElement child) {
	// role 0: executionContext
	// role 1: object that is initialised
        // role 2: (IDX): parameters
        if (child == executionContext) {
            return 0;
        }
        if (arguments != null) {
            int index = getArrayPos(arguments,child);
            if (index >= 0) {
                return (index << 4) | 1;
            }
        }
        return -1;
    }

    /** calls the corresponding method of a visitor in order to
     * perform some action/transformation on this element
     * @param v the Visitor
     */
    public void visit(Visitor v) {
	v.performActionOnThisConstructorReference(this);
    }

    public void prettyPrint(PrettyPrinter p) throws java.io.IOException {
        p.printThisConstructorReference(this);
    }
}
