// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
/**
 * This class encapsulates a guard for a loop
 */

package de.uka.ilkd.key.java.statement;

import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.util.ExtList;
import de.uka.ilkd.key.java.visitor.Visitor;

public class Guard extends JavaNonTerminalProgramElement 
    implements IGuard {

    Expression expr;

    public Guard(Expression expression) {
	expr=expression;
    }

    public Guard(ExtList children) {
	expr=(Expression)children.get(Expression.class);
    }

    public Expression getExpression() {
	return expr;
    }

    public void visit(Visitor v) {
	v.performActionOnGuard(this);
    }

    public int getChildCount() {
	return 1;
    }

    public ProgramElement getChildAt(int index) {
	if (index==0) return expr;
	return null;
    }

    public int getChildPositionCode(ProgramElement child) {
	if (child==expr) return 0;
	return -1;
    }


}
