// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.java.visitor;

import java.util.Stack;
import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.expression.*;
import de.uka.ilkd.key.java.expression.operator.*;
import de.uka.ilkd.key.java.expression.literal.*;
import de.uka.ilkd.key.java.reference.*;
import de.uka.ilkd.key.java.declaration.*;
import de.uka.ilkd.key.java.declaration.modifier.*;
import de.uka.ilkd.key.java.statement.*;
import de.uka.ilkd.key.util.ExtList;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.java.abstraction.Type;
import java.util.Iterator;
import de.uka.ilkd.key.rule.metaconstruct.ProgramMetaConstruct;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.logic.sort.ProgramSVSort;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.rule.inst.SVInstantiations;

/** Walks through a java AST in depth-left-fist-order. 
 * This walker is used to transform a program according to the given 
 * SVInstantiations.
 */
public class ProgramReplaceVisitor extends CreatingASTVisitor {


    private ProgramElement result = null;

    private JavaInfo javaInfo;
    private SVInstantiations svinsts;
    private Type lastType = null;

    /** 
     * create the  ProgramReplaceVisitor
     * @param root the ProgramElement where to begin
     */
    public ProgramReplaceVisitor(ProgramElement root, 
				 JavaInfo javaInfo, 
				 SVInstantiations svi) {
	super(root);
	this.javaInfo = javaInfo;
	svinsts=svi;
    }

    /** the action that is performed just before leaving the node the
     * last time 
     */
    protected void doAction(ProgramElement node) {
	node.visit(this);
    }
    
    /** starts the walker*/
    public void start() {	
	stack.push(new ExtList());		
	walk(root());
	ExtList el=(ExtList)stack.peek();
	int i=0;
	while (!(el.get(i) instanceof ProgramElement)) {
	    i++;
	}
	result=(ProgramElement) ((ExtList)stack.peek()).get(i);
    }

    public ProgramElement result() { 	
	return result;
    }
    
    /** walks through the AST. While keeping track of the current node
     * @param node the JavaProgramElement the walker is at 
     */
    protected void walk(ProgramElement node) {	
	stack.push(new ExtList());
	/*
	if (node instanceof LocalVariableDeclaration) {
	    LocalVariableDeclaration decl=(LocalVariableDeclaration) node;
	    lastType = decl.getVariables().getVariableSpecification(0).getType();
	    if (lastType instanceof SchemaVariable) {
		lastType = (Type) svinsts.getInstantiation
		    ((SchemaVariable)lastType);
	    }
	} 
	*/      
	super.walk(node);	
    }

    public String toString() {
	return stack.peek().toString();
    }

    /** the implemented default action is called if a program element is,
     * and if it has children all its children too are left unchanged
     */
    protected void doDefaultAction(SourceElement x) {
	addChild(x);
    }


    
    public void performActionOnSchemaVariable(SchemaVariable sv) {
	Object inst = svinsts.getInstantiation(sv);
	if (inst instanceof ProgramElement) {
	    Debug.out("ProgramReplace SV:", sv);
	    Debug.out("ProgramReplace:", inst);
	    addChild((ProgramElement)inst);
	} else if (inst instanceof ArrayOfProgramElement) {
	    addChildren((ArrayOfProgramElement)inst);	
	} else if ((inst instanceof Term) 
		   && (((Term)inst).op() instanceof ProgramInLogic)){
	    TypeConverter tc=javaInfo.getTypeConverter();
	    addChild(tc.convertToProgramElement((Term)inst));
	} else {
	    Debug.fail("Schema Variable "+sv+" should be instantiated"
			   +" to program(s)");
	}
	changed();
    }


    public void performActionOnProgramMetaConstruct(ProgramMetaConstruct x) {
	ProgramReplaceVisitor trans = new ProgramReplaceVisitor(x.body(), javaInfo,
								svinsts);
	trans.start();
	ProgramElement result=(ProgramElement)trans.result();
	result=x.symbolicExecution(result, javaInfo, svinsts);
	addChild(result);
	changed();
    }

    public void xxxperformActionOnVariableSpecification(VariableSpecification x)     {
  	DefaultAction def=new DefaultAction() {		
  		ProgramElement createNewElement(ExtList changeList) {
		    Type type=lastType;
		    lastType=null;

		    if (changeList.size()==1) {
			return new VariableSpecification
			    ((ProgramVariable)changeList.get(0),0,null,type);
		    } else {
			
			return new VariableSpecification
			    ((ProgramVariable)changeList.get(0),0,
			     (Expression)changeList.get(1),type);
		    }

  		}
  	    };
  	def.doAction(x);	
    }
        
}
