// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.java.visitor;

import java.util.Stack;
import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.expression.*;
import de.uka.ilkd.key.java.expression.operator.*;
import de.uka.ilkd.key.java.expression.literal.*;
import de.uka.ilkd.key.java.reference.*;
import de.uka.ilkd.key.java.declaration.*;
import de.uka.ilkd.key.java.declaration.modifier.*;
import de.uka.ilkd.key.java.statement.*;
import de.uka.ilkd.key.util.ExtList;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.java.abstraction.Type;
import java.util.Iterator;
import de.uka.ilkd.key.logic.op.ProgramVariable;

/**
 * Walks through a java AST in depth-left-fist-order. This visitor
 * replaces ThisReferences by a variable this.
 */
public class ThisReplaceVisitor extends CreatingASTVisitor {

    private ProgramElement result=null;

    private ProgramVariable thisVariable = null;

    /** create the  ThisReplaceVisitor
     * @param root the ProgramElement where to begin
     */
    public ThisReplaceVisitor(ProgramElement root, ProgramVariable thisVar) {
	super(root);
	thisVariable=thisVar;
    }

    /** the action that is performed just before leaving the node the
     * last time 
     */
    protected void doAction(ProgramElement node) {
	node.visit(this);
    }
    
    /** starts the walker*/
    public void start() {	
	stack.push(new ExtList());		
	walk(root());
	ExtList el=(ExtList)stack.peek();
	int i=0;
	while (!(el.get(i) instanceof ProgramElement)) {
	    i++;
	}
	result=(ProgramElement) ((ExtList)stack.peek()).get(i);
    }

    public ProgramElement result() { 	
	return result;
    }
    
    /** walks through the AST. While keeping track of the current node
     * @param node the JavaProgramElement the walker is at 
     */
    protected void walk(ProgramElement node) {	
	stack.push(new ExtList());
	super.walk(node);	
    }



    public void performActionOnThisReference(ThisReference x) {
	addChild(thisVariable);
	changed();
    }

}
