// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.java.visitor;

import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.expression.*;
import de.uka.ilkd.key.java.expression.operator.*;
import de.uka.ilkd.key.java.expression.literal.*;
import de.uka.ilkd.key.java.reference.*;
import de.uka.ilkd.key.java.declaration.*;
import de.uka.ilkd.key.java.declaration.modifier.*;
import de.uka.ilkd.key.java.statement.*;
import de.uka.ilkd.key.java.SourceElement.*;
import de.uka.ilkd.key.rule.metaconstruct.ProgramMetaConstruct;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.rule.ContextTerm;
import de.uka.ilkd.key.logic.ProgramElementName;

/**
 * This class is implemented by visitors/walkers.
 * Each AST node implements a visit(Visitor) method that
 * calls the  doActionAt<NodeType> method. Similar to the pretty print
 * mechanism.
 */
public interface Visitor {

    public void performActionOnProgramElementName(
			    ProgramElementName x);

    public void performActionOnProgramVariable(
			    ProgramVariable x);

    public void performActionOnSchemaVariable(
			    SchemaVariable x);
    
    public void performActionOnProgramMethod(
			    ProgramMethod x);

    public void performActionOnProgramMetaConstruct(
			    ProgramMetaConstruct x);

    public void performActionOnContextTerm(
			    ContextTerm x);

    public void performActionOnIntLiteral(IntLiteral x); 

    public void performActionOnBooleanLiteral(BooleanLiteral x); 

    public void performActionOnStringLiteral(StringLiteral x); 

    public void performActionOnNullLiteral(NullLiteral x); 

    public void performActionOnCharLiteral(CharLiteral x); 

    public void performActionOnDoubleLiteral(DoubleLiteral x); 

    public void performActionOnLongLiteral(LongLiteral x); 

    public void performActionOnFloatLiteral(FloatLiteral x); 

    public void performActionOnPackageSpecification(PackageSpecification x); 

    public void performActionOnTypeReference(TypeReference x); 

    public void performActionOnPackageReference(PackageReference x);     

    public void performActionOnThrows(Throws x); 

    public void performActionOnArrayInitializer(ArrayInitializer x); 

    public void performActionOnCompilationUnit(CompilationUnit x); 

    public void performActionOnArrayDeclaration(ArrayDeclaration x);

    public void performActionOnClassDeclaration(ClassDeclaration x); 

    public void performActionOnInterfaceDeclaration(InterfaceDeclaration x); 

    public void performActionOnFieldDeclaration(FieldDeclaration x); 

    public void performActionOnLocalVariableDeclaration(LocalVariableDeclaration x); 

    public void performActionOnVariableDeclaration(VariableDeclaration x); 

    public void performActionOnParameterDeclaration(ParameterDeclaration x); 

    public void performActionOnMethodDeclaration(MethodDeclaration x); 

    public void performActionOnClassInitializer(ClassInitializer x); 

    public void performActionOnStatementBlock(StatementBlock x); 

    public void performActionOnBreak(Break x); 

    public void performActionOnContinue(Continue x); 

    public void performActionOnReturn(Return x); 

    public void performActionOnThrow(Throw x);

    public void performActionOnDo(Do x); 

    public void performActionOnFor(For x);

    public void performActionOnWhile(While x); 

    public void performActionOnIf(If x); 

    public void performActionOnSwitch(Switch x); 

    public void performActionOnTry(Try x); 

    public void performActionOnLabeledStatement(LabeledStatement x); 

    public void performActionOnMethodCallStatement(MethodCallStatement x); 

    public void performActionOnMethodBodyStatement(MethodBodyStatement x); 

    public void performActionOnSynchronizedBlock(SynchronizedBlock x); 

    public void performActionOnImport(Import x);

    public void performActionOnExtends(Extends x);

    public void performActionOnImplements(Implements x);

    public void performActionOnVariableSpecification(VariableSpecification x); 

    public void performActionOnFieldSpecification(FieldSpecification x); 

    public void performActionOnImplicitFieldSpecification(ImplicitFieldSpecification x); 

    public void performActionOnBinaryAnd(BinaryAnd x); 

    public void performActionOnBinaryAndAssignment(BinaryAndAssignment x); 

    public void performActionOnBinaryOrAssignment(BinaryOrAssignment x); 

    public void performActionOnBinaryXOrAssignment(BinaryXOrAssignment x); 

    public void performActionOnCopyAssignment(CopyAssignment x); 

    public void performActionOnDivideAssignment(DivideAssignment x);

    public void performActionOnMinusAssignment(MinusAssignment x); 

    public void performActionOnModuloAssignment(ModuloAssignment x);

    public void performActionOnPlusAssignment(PlusAssignment x); 

    public void performActionOnPostDecrement(PostDecrement x); 

    public void performActionOnPostIncrement(PostIncrement x); 

    public void performActionOnPreDecrement(PreDecrement x); 

    public void performActionOnPreIncrement(PreIncrement x); 

    public void performActionOnShiftLeftAssignment(ShiftLeftAssignment x); 

    public void performActionOnShiftRightAssignment(ShiftRightAssignment x); 

    public void performActionOnTimesAssignment(TimesAssignment x);

    public void performActionOnUnsignedShiftRightAssignment 
	(UnsignedShiftRightAssignment x); 

    public void performActionOnBinaryNot(BinaryNot x); 

    public void performActionOnBinaryOr(BinaryOr x); 

    public void performActionOnBinaryXOr(BinaryXOr x);

    public void performActionOnConditional(Conditional x);
	
    public void performActionOnDivide(Divide x);

    public void performActionOnEquals(Equals x); 

    public void performActionOnGreaterOrEquals(GreaterOrEquals x);

    public void performActionOnGreaterThan(GreaterThan x); 

    public void performActionOnLessOrEquals(LessOrEquals x);

    public void performActionOnLessThan(LessThan x); 

    public void performActionOnNotEquals(NotEquals x); 

    public void performActionOnNewArray(NewArray x); 

    public void performActionOnInstanceof(Instanceof x);

    public void performActionOnExactInstanceof(ExactInstanceof x);

    public void performActionOnNew(New x); 

    public void performActionOnTypeCast(TypeCast x); 

    public void performActionOnLogicalAnd(LogicalAnd x);

    public void performActionOnLogicalNot(LogicalNot x);

    public void performActionOnLogicalOr(LogicalOr x);

    public void performActionOnMinus(Minus x); 

    public void performActionOnModulo(Modulo x);

    public void performActionOnNegative(Negative x);

    public void performActionOnPlus(Plus x); 

    public void performActionOnPositive(Positive x); 

    public void performActionOnShiftLeft(ShiftLeft x);

    public void performActionOnShiftRight(ShiftRight x);

    public void performActionOnTimes(Times x); 

    public void performActionOnUnsignedShiftRight(UnsignedShiftRight x); 

    public void performActionOnArrayReference(ArrayReference x); 

    public void performActionOnMetaClassReference(MetaClassReference x); 

    public void performActionOnMethodReference(MethodReference x); 

    public void performActionOnFieldReference(FieldReference x); 

    public void performActionOnSchematicFieldReference(SchematicFieldReference x); 

    public void performActionOnEmbededVariableReference
	(EmbededVariableReference x);

    public void performActionOnVariableReference(VariableReference x); 

    public void performActionOnMethod(ProgramMethod x); 

    public void performActionOnSuperConstructorReference(SuperConstructorReference x); 

    public void performActionOnExecutionContext(ExecutionContext x); 

    public void performActionOnConstructorDeclaration(ConstructorDeclaration x); 

    public void performActionOnThisConstructorReference(ThisConstructorReference x); 

    public void performActionOnSuperReference(SuperReference x); 

    public void performActionOnThisReference(ThisReference x); 

    public void performActionOnArrayLengthReference(ArrayLengthReference x); 

    public void performActionOnThen(Then x);

    public void performActionOnElse(Else x);

    public void performActionOnCase(Case x);

    public void performActionOnCatch(Catch x);

    public void performActionOnDefault(Default x);

    public void performActionOnFinally(Finally x);

    public void performActionOnModifier(Modifier x); 

    public void performActionOnEmptyStatement(EmptyStatement x);

    public void performActionOnComment(Comment x); 

    public void performActionOnParenthesizedExpression(ParenthesizedExpression x); 

    public void performActionOnForUpdates(ForUpdates x); 

    public void performActionOnGuard(Guard x); 

    public void performActionOnLoopInit(LoopInit x); 


}
