// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

/* Generated by Together */

package de.uka.ilkd.key.logic;

import de.uka.ilkd.key.java.JavaProgramElement;
import de.uka.ilkd.key.java.ProgramElement;
import de.uka.ilkd.key.java.NamedModelElement;
import de.uka.ilkd.key.java.StatementBlock;
import de.uka.ilkd.key.java.Statement;
import de.uka.ilkd.key.java.PrettyPrinter;
import de.uka.ilkd.key.java.NameAbstractionTable;
import java.io.StringWriter;
import java.io.IOException;

public class JavaBlock implements java.io.Serializable {
    
    public static final JavaBlock EMPTY_JAVABLOCK=new JavaBlock();
    private final JavaProgramElement prg;


    /** create a new empty JavaBlock as singleton 
     */
    private JavaBlock() {
	this.prg=null;
    }

    /** create a new JavaBlock 
     * @param prg the root JavaProgramElement for this JavaBlock
     */
    private JavaBlock(JavaProgramElement prg) {
	this.prg=prg;
    }

    /** create a new JavaBlock 
     * @param prg the root JavaProgramElement for this JavaBlock
     */
    public static JavaBlock createJavaBlock(JavaProgramElement prg) {
	if (prg==null) {
	    return EMPTY_JAVABLOCK;	   
	} 
	return new JavaBlock(prg);

    }

    public boolean isEmpty() {
	if ((program() instanceof StatementBlock))  {
	    return ((StatementBlock)program()).isEmpty();
	}
	return this==EMPTY_JAVABLOCK;
    }
    
    public Namespace namespace() {
	return null;
    }
    
    /** converts a parser generated namespace into a KeY namespace */
    public Namespace convertNamespace() {
	return null;
    }

    /** returns true iff the program elements are equal */
    public boolean equals(Object o) {
	if (!(o instanceof JavaBlock) || o == null) {
	    return false;
	}
	JavaBlock cmp = (JavaBlock)o;
	return equalsModRenaming(cmp.program(), 
				 new NameAbstractionTableDisabled());
    }

    /** returns true if the given ProgramElement is equal to the
     * one of the JavaBlock modulo renaming (see comment in SourceElement)
     */ 
    public boolean equalsModRenaming(Object o, 
				     NameAbstractionTable nat) {
	if (!(o instanceof JavaBlock) || o == null) {
	    return false;
	}
	JavaBlock cmp = (JavaBlock)o;
	return equalsModRenaming(cmp.program(), nat);
    }


    /** returns true if the given ProgramElement is equal to the
     * one of the JavaBlock modulo renaming (see comment in SourceElement)
     */ 
    private boolean equalsModRenaming(JavaProgramElement pe,
				     NameAbstractionTable nat) {
	if (pe == null && program() == null) {
	    return true;
	} else if ((pe == null && program()!=null) ||
	    (program() == null && pe != null)) {	    
	    return false;
	}
	return program().equalsModRenaming(pe, nat);
    }

    /** returns the java program 
     * @return the stored JavaProgramElement
     */
    public JavaProgramElement program() {
	return prg;
    }

    /** toString */
    public String toString() {
	if (this==EMPTY_JAVABLOCK) return "";
	StringWriter sw=new StringWriter();
	try {
	    PrettyPrinter pp=new PrettyPrinter(sw);
	    pp.setIndentationLevel(3);
	    prg.prettyPrint(pp);
	} catch (IOException e) {
	    System.err.println("toString of JavaBlock failed due to :"+e);
	    e.printStackTrace();
	    //	    System.exit(-1);
	}
	return sw.toString();
    }



    /** this class is used by method call. As in this case we do not
     * want to abstract from  names
     */
    static class NameAbstractionTableDisabled extends NameAbstractionTable {
	
	public void add(de.uka.ilkd.key.java.SourceElement pe1, 
			de.uka.ilkd.key.java.SourceElement pe2) {
	}

	public boolean contains(de.uka.ilkd.key.java.SourceElement pe) {
	    return false;
	    
	}
	
	public boolean sameAbstractName(de.uka.ilkd.key.java.SourceElement pe1, 
					de.uka.ilkd.key.java.SourceElement pe2) {
	    return false;
	}
    }

}
