// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.logic;

import de.uka.ilkd.key.logic.op.Op;
import de.uka.ilkd.key.logic.op.Operator;
import de.uka.ilkd.key.logic.op.Modality;
import de.uka.ilkd.key.logic.op.ArrayOfQuantifiableVariable;
/*
 * A ProgramTerm represents a term with a modality operator like
 * the diamond or box operator together with a JavaBlock. Instances
 * should never be accessed via this interface, use the interface of
 * the superclass Term and construct instances only via a TermFactory
 * instead. 
 */

class ProgramTerm extends Term {
    /** the program
     * @link aggregation
     * @supplierCardinality 0..1
     */
    private final JavaBlock javaBlock;

    /**  sub term */
    private final Term subTerm; 


    /** 
     * creates a diamond term, so there is an additional
     * parameter javaBlock 
     */
    ProgramTerm(Operator modality, 
		JavaBlock javaBlock, 
		Term subTerm) {
	super(modality, ((Modality)modality).sort(subTerm));
	this.subTerm   = subTerm;
	this.javaBlock = javaBlock;
	fillCaches();
    }


    /** @return n-th subterm (always the only one)*/    
    public Term sub(int n) {
	return subTerm;
    }	
   
    /** @return arity of the quantifier term 1 as int */
    public int arity() {
	return 1;
    } 

    /**@return depth of the term */
    public int depth() {
	return subTerm.depth() + 1;
    }

    /** @return JavaBlock if term has diamond */
    public JavaBlock javaBlock() {
	return javaBlock;
    }
    
    /** @return an empty variable list */
    public ArrayOfQuantifiableVariable varsBoundHere(int n) {
	return EMPTY_VAR_LIST;
    }

    public String toString() {
	StringBuffer sb = new StringBuffer();
	if ( op() == Op.DIA ) {
	    sb.append("<").append(javaBlock).append("> (");
	} else if ( op() == Op.BOX ) {
	    sb.append("[").append(javaBlock).append("] (");
	} else {
	    sb.append("???Some Strange Modality???").append(javaBlock);
	}
	sb.append(subTerm).append(")");
	return sb.toString();
    }

}
