// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.logic;

import junit.framework.*;
import de.uka.ilkd.key.proof.*;
import de.uka.ilkd.key.logic.sort.*;
import de.uka.ilkd.key.logic.op.*;

/** class tests the PosInOccurrence
*/


public class TestPosInOcc extends TestCase {

    public TestPosInOcc(String name) {
	super(name);
    }

    public void testAntecSuccWithSequentGiven() {
	TermFactory tf=TermFactory.DEFAULT;
	Sort sort1=new PrimitiveSort(new Name("S1"));
	LogicVariable x=new LogicVariable(new Name("x"),sort1);  
	LogicVariable y=new LogicVariable(new Name("y"),sort1);
	Function p=new Function(new Name("p"),Sort.FORMULA,new Sort[]{sort1});
	Term t_x=tf.createFunctionTerm(x, new Term[0]);
	Term t_px=tf.createFunctionTerm(p, new Term[]{t_x});
	Term t_y=tf.createFunctionTerm(y, new Term[0]);    	
	Term t_py=tf.createFunctionTerm(p, new Term[]{t_y}); 
	ConstrainedFormula cf1=new ConstrainedFormula(t_px, Constraint.BOTTOM);
	ConstrainedFormula cf2=new ConstrainedFormula(t_py, Constraint.BOTTOM);
	ConstrainedFormula cf3=new ConstrainedFormula(t_py, Constraint.BOTTOM);
	Sequent seq=Sequent.createSequent
	    (Semisequent.EMPTY_SEMISEQUENT.insert(0,cf1).semisequent().insert(1,cf2).semisequent(), 
	     Semisequent.EMPTY_SEMISEQUENT.insert(0,cf3).semisequent());
	PosInOccurrence pos1=new PosInOccurrence(cf1, 
						 PosInTerm.TOP_LEVEL, seq);
	PosInOccurrence pos2=new PosInOccurrence(cf2, 
						 PosInTerm.TOP_LEVEL, seq);
	PosInOccurrence pos3=new PosInOccurrence(cf3, 
						 PosInTerm.TOP_LEVEL, seq);

	assertTrue("p(x)[cf1] should be in antecedent", pos1.isInAntec());
	assertTrue("p(y)[cf2] should be in antecedent", pos2.isInAntec());
	assertTrue("p(y)[cf3] should be in succedent", !pos3.isInAntec());
	
    }

    public void testAntecSuccWithProofGiven() {
	TermFactory tf=TermFactory.DEFAULT;
	Sort sort1=new PrimitiveSort(new Name("S1"));
	LogicVariable x=new LogicVariable(new Name("x"),sort1);  
	LogicVariable y=new LogicVariable(new Name("y"),sort1);
	LogicVariable z=new LogicVariable(new Name("z"),sort1);
	Function p=new Function(new Name("p"),Sort.FORMULA,new Sort[]{sort1});
	Term t_x=tf.createFunctionTerm(x, new Term[0]);
	Term t_px=tf.createFunctionTerm(p, new Term[]{t_x});
	Term t_y=tf.createFunctionTerm(y, new Term[0]);    	
	Term t_py=tf.createFunctionTerm(p, new Term[]{t_y});
	Term t_z=tf.createFunctionTerm(z, new Term[0]);    	
	Term t_pz=tf.createFunctionTerm(p, new Term[]{t_z}); 
	ConstrainedFormula cf1=new ConstrainedFormula(t_px, Constraint.BOTTOM);
	ConstrainedFormula cf2=new ConstrainedFormula(t_py, Constraint.BOTTOM);
	ConstrainedFormula cf3=new ConstrainedFormula(t_py,
						    Constraint.BOTTOM);
	ConstrainedFormula cf4=new ConstrainedFormula(t_pz, Constraint.BOTTOM);
	Sequent seq=Sequent.createSequent(
					  Semisequent.EMPTY_SEMISEQUENT,
					  Semisequent.EMPTY_SEMISEQUENT.insert(0,cf3).semisequent());

	Proof proof=new Proof();
	proof.setRoot(new Node(proof, seq));
	proof.root().add(new Node
	    (proof, Sequent.createSequent(Semisequent.EMPTY_SEMISEQUENT.insert(0,cf2).semisequent(),   
					  Semisequent.EMPTY_SEMISEQUENT.insert(0,cf3).semisequent())));	
	Node n1=new Node(proof, Sequent.createSequent(
						Semisequent.EMPTY_SEMISEQUENT.insert(0,cf1).semisequent(),  
						Semisequent.EMPTY_SEMISEQUENT.insert(0,cf3).semisequent()));
	n1.add(new Node
	    (proof, Sequent.createSequent(Semisequent.EMPTY_SEMISEQUENT.
					  insert(0,cf1).semisequent().
					  insert(1,cf4).semisequent(), 
					  Semisequent.EMPTY_SEMISEQUENT.
					  insert(0,cf3).semisequent())));
	proof.root().add(n1);

	PosInOccurrence pos1=new PosInOccurrence(cf1, 
						 PosInTerm.TOP_LEVEL, proof);
	PosInOccurrence pos2=new PosInOccurrence(cf2, 
						 PosInTerm.TOP_LEVEL, proof);
	PosInOccurrence pos3=new PosInOccurrence(cf3, 
						 PosInTerm.TOP_LEVEL,
						 proof);
	PosInOccurrence pos4=new PosInOccurrence(cf4, 
						 PosInTerm.TOP_LEVEL, proof);

	assertTrue("p(x)[cf1] should be in antecedent", pos1.isInAntec());
	assertTrue("p(y)[cf2] should be in antecedent", pos2.isInAntec());
	assertTrue("p(y)[cf3] should be in succedent", !pos3.isInAntec());
	assertTrue("p(z)[cf4] should be in antecedent",  pos4.isInAntec());
		
    }
}
