// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

/** this class is used to keep track of used variables and offer
 * services like creating and retuning variables that are new in a
 * sequent 
*/
package de.uka.ilkd.key.logic;

import java.util.HashSet;
import de.uka.ilkd.key.proof.*;
import de.uka.ilkd.key.logic.op.LogicVariable;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.logic.op.*;

public class VariableDeliverer {

    /** the sequent the variabledeliver belongs to */
    private Sequent seq;
    /** keeps seq variables in mind */ 
    private HashSet set;
        
    /**
     * creates a new VariableDeliverer for a given sequent
     * @param seq the Sequent this deliverer relates to
     */
    public VariableDeliverer(Sequent seq) {
	this.seq=seq;
	set=new HashSet();	
    }
    
    
    private void union(Semisequent semi) 
    {
	IteratorOfConstrainedFormula it=semi.iterator();       
	LVRCollector vcv=new LVRCollector();
	while (it.hasNext()) {
	    it.next().formula().execPostOrder(vcv);	    
	}
	
	IteratorOfLogicVariableRepresentative varIt=vcv.varIterator();
	while (varIt.hasNext()) {
	    set.add(varIt.next().name());	    
	}	
    }
    

    
    /**
     * creates a new logicvariable with name of the given variable plus an index
     * and of course same sort
     * @param var the bound SchemaVariable used as template 
     * @return the new LogicVariable
     */
    private LogicVariable newVar(SchemaVariable var) {
	// get all used vars 
	union(seq.antecedent());	
	union(seq.succedent());	

	int index=0;
	String varNameStr=var.name().toString();	
	Name newVarName=var.name();	
	while (set.contains(newVarName)) {
	    newVarName=new Name(varNameStr+index);	    
	    index++;	    
	}
	
	return new LogicVariable(newVarName, var.sort());	
    }


        
//     public TacletApp instantiateNewVars(TacletApp app) {
// 	SVInstantiations svInst = app.instantiations();
// 	IteratorOfSchemaVariable it=((Taclet)app.rule()).varsNew();
// 	while (it.hasNext()) {
// 	    SchemaVariable var=(SchemaVariable)it.next();
// 	    LogicVariable newvar = newVar(var);
// 	    svInst = svInst.add(var, TermFactory.DEFAULT.createFunctionTerm(newvar,
// 		    new Term[0]));
// 	}
// 	if (app.rule() instanceof FindTaclet) {
// 	    return PosTacletApp.createPosTacletApp((FindTaclet)app.taclet(), 
// 				  svInst, app.posInOccurrence());
// 	} else {
// 	    return NoPosTacletApp.createNoPosTacletApp((NoFindTaclet)app.taclet(), 
// 						   svInst);
// 	}
//     }

}
