// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.logic.ldt;

import de.uka.ilkd.key.java.expression.*;
import de.uka.ilkd.key.java.expression.operator.*;
import de.uka.ilkd.key.java.expression.literal.*;
import de.uka.ilkd.key.java.abstraction.Type;
import de.uka.ilkd.key.java.abstraction.ListOfType;
import de.uka.ilkd.key.java.abstraction.SLListOfType;
import de.uka.ilkd.key.java.abstraction.IteratorOfType;
import de.uka.ilkd.key.java.Expression;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.util.ExtList;
import java.io.File;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Namespace;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.proof.RuleSource;

/** 
 * This class inherits from LDT and implements all method that are
 * necessary to handle common operations on java.lang.Object
 */
public class ObjectLDT extends LDT {
        
    public ObjectLDT() {
	super(new Name("Object"));
        ruleSource = RuleSource.initRuleFile("objectRules.key");
    }

    /** returns the name of the target sort of the LDT 
     * is needed to find the right Sort when constructing the LDT
     * @return the Name of the target sort
     */
    public Name getSortName() {
	return targetSort() == null ? new Name("Object") : targetSort().name();
    }


    public void init(Namespace func_ns, Sort targetSort, ListOfType type) {
	super.init(func_ns, targetSort, type);
	Namespace functions = functions();
    }


    /** returns the function symbol for the given operation 
     * null if no function is found for the given operator
     * @return  the function symbol for the given operation 
    */
    public Function getFunctionFor
	(de.uka.ilkd.key.java.expression.Operator op) {
	return null;
    }

    /**
     * returns true if the LDT offers an operation for the given java
     * operator and the logic subterms; here: the operator is == and the 
     * subterms are two and have subsorts of Object.
     * @param op the de.uka.ilkd.key.java.expression.Operator to
     * translate
     * @param subs the logic subterms of the java operator
     * @return  true if the LDT offers an operation for the given java
     * operator and the subterms 
     */
    public boolean isResponsible
	(de.uka.ilkd.key.java.expression.Operator op, Term[] subs) {
	return false;
    }


    /** returns true if the LDT offers an operation for the given
     * binary java operator and the logic subterms 
     * @param op the de.uka.ilkd.key.java.expression.Operator to
     * translate
     * @param left the left subterm of the java operator
     * @param right the right subterm of the java operator
     * @return  true if the LDT offers an operation for the given java
     * operator and the subterms 
     */
    public boolean isResponsible
	(de.uka.ilkd.key.java.expression.Operator op, Term left, Term right) {
	return isResponsible(op, new Term[]{left, right});
    }

    /** returns true if the LDT offers an operation for the given
     * unary java operator and the logic subterms 
     * @param op the de.uka.ilkd.key.java.expression.Operator to
     * translate
     * @param sub the logic subterms of the java operator
     * @return  true if the LDT offers an operation for the given java
     * operator and the subterm
     */
    public boolean isResponsible
	(de.uka.ilkd.key.java.expression.Operator op, Term sub) {
	return false;
    }

    /** translates a given literal to its logic counterpart 
     * @param lit the Literal to be translated
     * @return the Term that represents the given literal in its logic
     * form
     */ 
    public Term translateLiteral(Literal lit) {
	throw new IllegalStateException("ObjectLDT not responsible for any"
					+" literal: "+lit);
    }

    public boolean hasLiteralFunction(Function f) {
	return false;
    }

    public Expression translateTerm(Term t, ExtList children) {
	throw new RuntimeException("ObjectLDT: Cannot convert term to program: "
				   +t);
    }
} 
