 // This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.logic.op;

import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.ldt.*;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.logic.sort.ArraySort;
import de.uka.ilkd.key.logic.sort.ProgramSVSort;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.gui.ProofSettings;
import de.uka.ilkd.key.util.ExtList;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;

public class ArrayOp extends AccessOp{

    /**
     * creates an ArrayOp
     */
    ArrayOp(Name name){
        super(name);
    }
    
    /**
     * determines the sort of the {@link Term} if it would be created using this
     * Operator as top level operator and the given terms as sub terms. The
     * assumption that the constructed term would be allowed is not checked.
     * @param term an array of Term containing the subterms of a (potential)
     * term with this operator as top level operator
     * @return sort of the term with this operator as top level operator of the
     * given substerms
     */
    public Sort sort(Term[] term) {
	Sort as = term[0].sort();
	if(as instanceof ProgramSVSort){ 
	    return ProgramSVSort.NONSIMPLEEXPRESSION;
	}
	Sort s = ((ArraySort)as).elementSort();
        return s;
    }
    
    /** checks whether the top level structure of the given @link Term
     * is syntactically valid, given the assumption that the top level
     * operator of the term is the same as this Operator. The
     * assumption that the top level operator and the term are equal
     * is NOT checked.
     * @return true iff the top level structure of
     * the @link Term is valid.
     */
    public boolean validTopLevel(Term term) {
	if(!(term.sub(0).sort() instanceof ArraySort || term.sub(0).sort() instanceof ProgramSVSort)){
	    return false;
	}
	if(!(term.sub(1).sort() instanceof ProgramSVSort)){
	    if(ProofSettings.THE_SETTINGS.getLDTSettings().getNaturalNumberLDT() instanceof ArithIntLDT){
		if(!term.sub(1).sort().name().equals(((de.uka.ilkd.key.logic.ldt.
						       ArithIntLDT)ProofSettings.THE_SETTINGS.
						      getLDTSettings().getNaturalNumberLDT()).
						     getSortName())){
		    return false;
		}
	    }
	    if(ProofSettings.THE_SETTINGS.getLDTSettings().getNaturalNumberLDT() instanceof ArithIntLDTIgnoringOF){
		if(!term.sub(1).sort().name().equals(((de.uka.ilkd.key.logic.ldt.
						       ArithIntLDTIgnoringOF)ProofSettings.THE_SETTINGS.
						      getLDTSettings().getNaturalNumberLDT()).
						     getSortName())){
		    return false;
		}
	    }
	}
        return term.arity()==2;
    }

    public KeYJavaType getKeYJavaType(Term t) {
	return ((ProgramVariable) t.op()).getKeYJavaType();
    }

    public de.uka.ilkd.key.java.Expression convertToProgram(Term t, 
							    ExtList el) {
	return null; //%%%%%
    }
    
}

