// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//


package de.uka.ilkd.key.logic.op;

import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.gui.SequentPrinter;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Term;

public class Metavariable extends TermSymbol
    implements ParsableVariable, Comparable {

    // Used to define an alternative order of all existing
    // metavariables
    private static int maxSerial = 0;
    private        int serial;
    
    private synchronized void setSerial () {
	serial = maxSerial++;
    }

    public Metavariable(Name name, Sort sort) {
	super(name, sort);
	if ( sort == Sort.FORMULA ) {
	    throw new RuntimeException(
		 "Attempt to create metavariable of type formula");
	}
	setSerial ();
    }


    /** @return true iff number of subterms of term is 0
     */
    public boolean validTopLevel(Term term){
        return term.arity()==0;
    }

    public String toString() {
	return name()+":"+sort();
    }

   /** @return arity of the Metavariable as int */
    public int arity() {
	return 0;
    }

    public int compareTo ( Object p ) {
	if ( p == this )
	    return 0;
	if ( p == null )
	    throw new NullPointerException ();

	// throws a ClassCastException on error
	int t = name ().toString ().compareTo
	    ( ((Metavariable)p).name ().toString () );
	if ( t == 0 )
	    return serial < ((Metavariable)p).serial ? -1 : 1;
	return t;
    }
}
