// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//


package de.uka.ilkd.key.logic.op;

import de.uka.ilkd.key.java.abstraction.KeYJavaType;
import de.uka.ilkd.key.java.Expression; 
import de.uka.ilkd.key.java.expression.operator.NewArray; 
import de.uka.ilkd.key.java.expression.*;
import de.uka.ilkd.key.util.ExtList;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.sort.PrimitiveSort;
import de.uka.ilkd.key.logic.sort.ArraySort;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.logic.sort.ArraySortImpl;
import de.uka.ilkd.key.logic.ProgramInLogic;
import de.uka.ilkd.key.logic.sort.ArrayOfSort;


/**
 * represents an array in a program.
 */
public class ProgramArray extends Function implements ProgramInLogic{

    private KeYJavaType kjt;
    private NewArray newArray=null;
    private ArrayInitializer ai=null;


    public ProgramArray(Name name, ArraySort s){
	super(name, s, new PrimitiveSort[0]);
    }  


    /** creates a ProgramArray
     * @param name String with name of the array
     */
    public static ProgramArray createProgramArray(Name name, KeYJavaType kjt, NewArray newArray) {
	ProgramArray na;
	Sort s=kjt.getSort();
	na = new ProgramArray(name, (ArraySort) s); 
	na.kjt = kjt;
	na.newArray = newArray;
	return na; 
    }

    public static ProgramArray createProgramArray(Name name, KeYJavaType kjt, ArrayInitializer ai) {
	ProgramArray na;
	Sort s=kjt.getSort();
	for(ArrayInitializer i=ai;i instanceof ArrayInitializer && i!=null &&
		i.getChildCount()!=0 && i.getChildAt(0) instanceof ArrayInitializer;
	    i=(ArrayInitializer)i.getExpressionAt(0)){
	}
	na = new ProgramArray(name, (ArraySort) s); 
	na.kjt = kjt;
	na.ai = ai;  
	return na;
    }


    public NewArray getNewArray(){
        return newArray;
    }		   

    public KeYJavaType getKeYJavaType(Term t) {
	return kjt;
    }

    public Expression convertToProgram(Term t, ExtList list) {
	return newArray;
    }

    public String toString(){
	return name().toString()+":"+sort().toString();
    }

}
