// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//


package de.uka.ilkd.key.logic.op;

import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.logic.sort.ArrayOfSort;
import de.uka.ilkd.key.logic.sort.SortDefiningSymbols;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Term;

public class SortDependingFunction extends Function
    implements SortDependingSymbol {
    
    Name kind;
    Sort sortDependingOn;

    /** creates a Function 
     * @param name String with name of the function
     * @param sort the Sort of the function (result type)
     * @param kind name of the kind this object belongs to
     * @param sortDependingOn sort this object is depending on, this
     * is the difference between various objects of the same kind
     * @return argsorts array of type Sort with the allowed sorts of
     * the function arguments */   
    public SortDependingFunction ( Name   name,
				   Sort   sort,
				   Sort[] argSorts,
				   Name   kind,
				   Sort   sortDependingOn ) {
	super ( name, sort, argSorts );
	this.kind            = kind;
	this.sortDependingOn = sortDependingOn;
    }

    public SortDependingFunction ( Name        name,
				   Sort        sort,
				   ArrayOfSort argSorts,
				   Name        kind,
				   Sort        sortDependingOn ) {
	super ( name, sort, argSorts );
	this.kind            = kind;
	this.sortDependingOn = sortDependingOn;
    }

    /**
     * @return the sort this object has been instantiated for
     */
    public Sort    getSortDependingOn () {
	return sortDependingOn;
    }

    /**
     * Compares "this" and "p"
     * @param p object to compare
     * @return true iff this and p are instances of the same kind of
     * symbol, but possibly instantiated for different sorts
     */
    public boolean isSimilar          ( SortDependingSymbol p ) {
	return getKind ().equals ( p.getKind () );
    }

    /**
     * Assign a name to this term symbol, independant of concrete
     * instantiations for different sorts
     * @return the kind of term symbol this object is an instantiation
     * for
     */
    public Name    getKind            () {
	return kind;
    }

    /**
     * Get the instance of this term symbol defined by the given sort
     * "p_sort"
     * @return a term symbol similar to this one, or null if this
     * symbol is not defined by "p_sort"
     *
     * POSTCONDITION: result==null || (this.isSimilar(result) &&
     * result.getSortDependingOn()==p_sort)
     */
    public SortDependingSymbol getInstanceFor ( SortDefiningSymbols p_sort ) {
	return p_sort.lookupSymbol ( getKind () );
    }

}
