// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.logic.sort;

import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.op.*;

public class NullSortImpl extends ClassInstanceSortImpl implements NullSort {


    /** creates a NullSort*/
    public NullSortImpl(Name name) {
	super(name);
    }

    public de.uka.ilkd.key.logic.sort.Sort elementSort() {
	return null;
    }

    /**
     * This is called from "AbstractNonCollectionSort" to define the
     * symbols specified by this collection sort
     * @return list of the symbols, null if something has gone wrong
     * (e.g. information about LDTs is not yet available within
     * "p_tc")
     */
    public ListOfSortDependingSymbol createSymbols () {
	return SLListOfSortDependingSymbol.EMPTY_LIST;
    }

    /**
     * returns true iff the given sort is an object sort
     */
    public boolean extendsTrans(Sort s) {
	return (s instanceof ObjectSort);
    }

    /** @return equality symbol of this sort */
    public Equality getEqualitySymbol(){
	return Op.EQUALS;
    }


    /**
     * @return an object of this class with elementSort().equals(p),
     * or null if such an object cannot be constructed (as p is an
     * incompatible sort)
     */
    public Sort cloneFor ( Sort p ) {
	return this;
    }

}
