// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.logic.sort;

import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.abstraction.*;
import de.uka.ilkd.key.java.reference.*;
import de.uka.ilkd.key.java.declaration.*;
import de.uka.ilkd.key.java.expression.*;
import de.uka.ilkd.key.java.expression.operator.*;
import de.uka.ilkd.key.java.expression.literal.*;
import de.uka.ilkd.key.java.statement.*;
import de.uka.ilkd.key.util.ExtList;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.ProgramElementName;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.java.visitor.JavaIterator;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.logic.op.ProgramVariable;
import de.uka.ilkd.key.logic.op.ProgramMethod;

import java.util.Iterator;

public class ProgramSVSort extends de.uka.ilkd.key.logic.sort.PrimitiveSort {

    public static final ProgramSVSort STATEMENT
	= new StatementSort();

    public static final ProgramSVSort EXPRESSION
	= new ExpressionSort();

    public static final ProgramSVSort STATEMENTEXPRESSION
	= new ProgramSVSort(new Name("StatementExpression"));

    public static final ProgramSVSort SIMPLEEXPRESSION 
	= new SimpleExpressionSort();

    public static final ProgramSVSort SIMPLEJAVABOOLEXPRESSION 
 	= new SimpleExpressionSpecialPrimitiveTypeSort
 	("SimpleExpressionOfTypeBoolean", new
 	    PrimitiveType[]{PrimitiveType.JAVA_BOOLEAN});

    public static final ProgramSVSort SIMPLEJAVABYTEEXPRESSION 
	= new SimpleExpressionSpecialPrimitiveTypeSort
	("SimpleExpressionOfTypeByte", new
	    PrimitiveType[]{PrimitiveType.JAVA_BYTE});

    public static final ProgramSVSort SIMPLEJAVACHAREXPRESSION 
	= new SimpleExpressionSpecialPrimitiveTypeSort
	("SimpleExpressionOfTypeChar", new
	    PrimitiveType[]{PrimitiveType.JAVA_CHAR});

    public static final ProgramSVSort SIMPLEJAVASHORTEXPRESSION 
	= new SimpleExpressionSpecialPrimitiveTypeSort
	("SimpleExpressionOfTypeShort", new
	    PrimitiveType[]{PrimitiveType.JAVA_SHORT});

    public static final ProgramSVSort SIMPLEJAVAINTEXPRESSION 
	= new SimpleExpressionSpecialPrimitiveTypeSort
	("SimpleExpressionOfTypeInt", new
	    PrimitiveType[]{PrimitiveType.JAVA_INT});

    public static final ProgramSVSort SIMPLEJAVALONGEXPRESSION 
	= new SimpleExpressionSpecialPrimitiveTypeSort
	("SimpleExpressionOfTypeLong", new
	    PrimitiveType[]{PrimitiveType.JAVA_LONG});

    public static final ProgramSVSort SIMPLEJAVABYTESHORTINTEXPRESSION
	= new SimpleExpressionSpecialPrimitiveTypeSort
	("SimpleExpressionOfTypeLong", new
	    PrimitiveType[]{PrimitiveType.JAVA_BYTE,
			     PrimitiveType.JAVA_SHORT,PrimitiveType.JAVA_INT});

    public static final ProgramSVSort SIMPLEANYJAVATYPEEXPRESSION 
	= new SimpleExpressionSpecialPrimitiveTypeSort
	("SimpleExpressionOfTypeLong", new
	    PrimitiveType[]{PrimitiveType.JAVA_BYTE,
			     PrimitiveType.JAVA_SHORT,PrimitiveType.JAVA_INT,
			     PrimitiveType.JAVA_LONG});

    public static final ProgramSVSort NONSIMPLEEXPRESSION 
	= new NonSimpleExpressionSort();

    public static final ProgramSVSort BOOLEANEXPRESSION
	= new BooleanExpressionSort();

    public static final ProgramSVSort NONSIMPLEBOOLEANEXPRESSION
	= new NonSimpleBooleanExpressionSort();

    public static final ProgramSVSort LEFTHANDSIDE = new LeftHandSideSort();

    public static final ProgramSVSort LOOPINIT = new LoopInitSort();

    public static final ProgramSVSort GUARD = new GuardSort();

    public static final ProgramSVSort FORUPDATES = new ForUpdatesSort();

    public static final ProgramSVSort LABEL
	= new ProgramSVSort(new Name("Label"));

    public static final ProgramSVSort TYPE
	= new ProgramSVSort(new Name("Type"));

    public static final ProgramSVSort VARIABLE
	= new ProgramVariableSort();

    // implict field match
    public static final ProgramSVSort IMPLICITREFERENCE
	= new ImplicitFieldReferenceSort();

    public static final ProgramSVSort IMPLICITNEXT
	= new ImplicitFieldSort("<next>");

    public static final ProgramSVSort IMPLICITNEXTTOCREATE
	= new ImplicitFieldSort("<nextToCreate>");

    public static final ProgramSVSort ARRAYLENGTH
	= new FieldSort("length");

    public static final ProgramSVSort BOOLEANVARIABLE
	= new ProgramSVSort(new Name("BooleanVariable"));

    public static final ProgramSVSort BRANCH
	= new BranchSort();

    public static final ProgramSVSort CATCH
	= new CatchSort();

    public static final ProgramSVSort SWITCH
	= new SwitchSVSort(); 

    public static final ProgramSVSort VARIABLEINIT
	= new ProgramSVSort(new Name("VariableInitializer"));

    public static final ProgramSVSort NEW = new NewSVSort();

    public static final ProgramSVSort NEWARRAY = new NewArraySVSort();

    public static final ProgramSVSort ARRAYINITIALIZER = 
	new ArrayInitializerSVSort();

    public static final ProgramSVSort METHODREFERENCE = 
	new MethodReferenceSort();

    public static final ProgramSVSort SPECIALCONSTRUCTORREFERENCE = 
	new SpecialConstructorReferenceSort();

    public static final ProgramSVSort MULTIPLEVARDECL
	= new MultipleVariableDeclarationSort();

    public static final ProgramSVSort CLASSTYPE
	= new ClassTypeSort();

    public static final ProgramSVSort INTERFACETYPE
	= new InterfaceTypeSort();

    public static final ProgramSVSort ARRAYTYPE
	= new ArrayTypeSort();

    public static final ArraySVSort ARRAY
	= new ArraySVSort();

    public static final ProgramSVSort CLASSTYPEEXPRESSION
	= new ClassTypeExpressionSort();

    public static final ProgramSVSort INTERFACETYPEEXPRESSION
	= new InterfaceTypeExpressionSort();

    public static final ProgramSVSort METHODBODY
	= new MethodBodySort();

    public ProgramSVSort(Name name) {
	super(name);
    }

    public boolean canStandFor(Term t) {
 	return false;
     }

    protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	return true;
    }

    public boolean canStandFor(ProgramElement check,
			       JavaInfo javaInfo) {
	JavaIterator it = new JavaIterator(check);
	ProgramElement pe = it.element();
	while (it.hasNext()) {
	    if (!allowed(pe, javaInfo)) {
		return false;
	    }
	    pe = it.next();
	}
	return true;
    }

    static boolean methodConstrReference(ProgramElement pe) {
	    return ((pe instanceof MethodReference)
		|| (pe instanceof ConstructorReference)
		|| (pe instanceof SpecialConstructorReference));
    }

    public ProgramElement getSVWithSort(ExtList l, Class alternative) {	
	Iterator it=l.iterator();
	while (it.hasNext()) {
	    Object o=it.next();
	    if (o instanceof SchemaVariable
		&& (((SchemaVariable)o).sort()==this)) {
		    return (ProgramElement) o;
	    }
	    else if ((alternative.isInstance(o)) 
		     && (! (o instanceof SchemaVariable))) {
		return (ProgramElement) o;
	    }
	}
	return null;
    }

    static private class SwitchSVSort extends ProgramSVSort{
	public SwitchSVSort(){
	    super(new Name("Switch"));
	}

	public boolean canStandFor(ProgramElement pe, 
				   JavaInfo javaInfo) {
	    return (pe instanceof Switch);
	}
    }

    static private class LeftHandSideSort extends ProgramSVSort{

	public LeftHandSideSort() {
	    super(new Name("LeftHandSide"));
	}

	public boolean canStandFor(ProgramElement pe, 
				   JavaInfo javaInfo) {
	    if (methodConstrReference(pe)) {
		return false;  //last component of access path must be without sideeffects
	    }
	    return super.canStandFor(pe, javaInfo);
	}

	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    return (!(pe instanceof de.uka.ilkd.key.java.expression.Operator));
	}
    }

    static private class MethodReferenceSort extends ProgramSVSort{

	public MethodReferenceSort() {
	    super(new Name("MethodReference"));
	}

	public boolean canStandFor(ProgramElement pe,
				   JavaInfo javaInfo) {
	    return (pe instanceof MethodReference);
	}

	public boolean canStandFor(Term t) {
	    return (t.op() instanceof ProgramMethod);
	}
    }


    static private class SpecialConstructorReferenceSort 
	extends ProgramSVSort{

	public SpecialConstructorReferenceSort() {
	    super(new Name("SpecialConstructorReference"));
	}

	public boolean canStandFor(ProgramElement pe,
				   JavaInfo javaInfo) {
	    if (pe instanceof SpecialConstructorReference) {
		return true;
	    }
	    return false;
	}

	public boolean canStandFor(Term t) {
	    return (t.op() instanceof ProgramMethod);
	}
    }


    static private class MultipleVariableDeclarationSort extends ProgramSVSort{

	public MultipleVariableDeclarationSort() {
	    super(new Name("MultipleVariableDeclaration"));
	}

	public boolean canStandFor(ProgramElement pe,
				   JavaInfo javaInfo) {
	    if (pe instanceof VariableDeclaration &&
		((VariableDeclaration)pe).getVariables().size() > 1) 
		return true;	    
	    return false;
	}	

    }

    static private class ClassTypeSort extends ProgramSVSort{

	public ClassTypeSort() {
	    super(new Name("ClassType"));
	}

	public boolean canStandFor(ProgramElement pe,
				   JavaInfo javaInfo) {
	    recoder.abstraction.Type type = javaInfo.getKeYProgModelInfo().
		getType(((de.uka.ilkd.key.java.reference.TypeReference)pe));

	    if (type instanceof recoder.java.declaration.ClassDeclaration)
		return true;
	    if (type instanceof recoder.bytecode.ClassFile && 
		!((recoder.bytecode.ClassFile)type).isInterface())
		return true;
	    return false;
	}
    }

    static private class InterfaceTypeSort extends ProgramSVSort{

	public InterfaceTypeSort() {
	    super(new Name("InterfaceType"));
	}


	public boolean canStandFor(ProgramElement pe,
				   JavaInfo javaInfo) {
	    recoder.abstraction.Type type = javaInfo.getKeYProgModelInfo().
		getType(((de.uka.ilkd.key.java.reference.TypeReference)pe));

	    if (type instanceof recoder.java.declaration.InterfaceDeclaration)
		return true;
	    if (type instanceof recoder.bytecode.ClassFile && 
		((recoder.bytecode.ClassFile)type).isInterface())
		return true;
	    return false;
	}
    }

    static private class ArrayTypeSort extends ProgramSVSort{

	public ArrayTypeSort() {
	    super(new Name("ArrayType"));
	}


	public boolean canStandFor(ProgramElement pe,
				   JavaInfo javaInfo) {
	    recoder.abstraction.Type type = javaInfo.getKeYProgModelInfo().
		getType(((de.uka.ilkd.key.java.reference.TypeReference)pe));

	    if (type instanceof recoder.abstraction.ArrayType)
		return true;
	    return false;
	}
    }


    static private class ClassTypeExpressionSort extends ExpressionSort{
	
	public ClassTypeExpressionSort() {
	    super(new Name("ClassTypeExpression"));
	}
	
	public boolean canStandFor(ProgramElement pe,
				   JavaInfo javaInfo) {
	    if (!super.canStandFor(pe, javaInfo)) {
		return false;
	    }
	    
	    Type jt =((Expression)pe).getKeYJavaType
		(javaInfo.getServices()).getJavaType();
	    
	    if (jt instanceof ClassType &&
		!((ClassType)jt).isInterface()) 
		return true;

	    return false;
	}
    }

    static private class InterfaceTypeExpressionSort extends ExpressionSort{
	
	public InterfaceTypeExpressionSort() {
	    super(new Name("InterfaceTypeExpression"));
	}
	
	public boolean canStandFor(ProgramElement pe,
				   JavaInfo javaInfo) {
	    if (!super.canStandFor(pe, javaInfo)) {
		return false;
	    }
	    
	    Type jt =((Expression)pe).getKeYJavaType
		(javaInfo.getServices()).getJavaType();
	    
	    if (jt instanceof ClassType &&
		((ClassType)jt).isInterface()) 
		return true;

	    return false;
	}
    }

    static private class ProgramVariableSort extends ProgramSVSort{

	public ProgramVariableSort() {
	    super(new Name("ProgramVariable"));
	}

	public boolean canStandFor(Term t) {
	    if (t.op() instanceof ProgramVariable) {
		return true;
	    }
	    return false;
	}

	public boolean canStandFor(ProgramElement pe,
				   JavaInfo javaInfo) {
	    if (pe instanceof ProgramVariable
		|| pe instanceof VariableSpecification
		|| pe instanceof TypeReference
		|| pe instanceof ThisReference
		|| pe instanceof SuperReference) {
		return true;
	    }
	    if(pe instanceof VariableReference){
		if(!(pe instanceof FieldReference)){
		    return true;
		}else{
		    if(((FieldReference)pe).getProgramVariable().isStatic()){
			boolean res=((FieldReference)pe).getReferencePrefix() == null ?
			    true : VARIABLE.canStandFor(((FieldReference)pe).getReferencePrefix(), javaInfo);
			return res;
		    }
		    return ((FieldReference)pe).getChildCount()<2;	    
		}
	    }
	    return false;
	}
    }
  
    static private class FieldSort extends ProgramSVSort{

	protected final ProgramElementName field;
	
	public FieldSort(Name name) {
	    super(name);
	    this.field = null;
	}

	public FieldSort(Name name, String field) {
	    super(name);
	    this.field = new ProgramElementName(field);
	}

	public FieldSort(String field) {
	    super(new Name("#field"));
	    this.field = new ProgramElementName(field);
	}

	// %%% we should move information from the variable
	// specification to the program variable %RB
	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof ProgramVariable) {
		return ((ProgramVariable)pe).name().equals(field);
	    }
	    return false;
	}


	public boolean canStandFor(Term t) {
	    if (t.op() instanceof ProgramVariable) {
		return ((ProgramVariable)t.op()).name().equals(field);
	    }
	    return false;
	}

	public boolean canStandFor(ProgramElement pe,
				   JavaInfo javaInfo) {
	    return allowed(pe, javaInfo);
	}	
    }

    static private class ImplicitFieldSort extends FieldSort {
	
	public ImplicitFieldSort(Name name) {
	    super(name);
	}

	public ImplicitFieldSort() {
	    this(new Name("ImplicitField"));
	}

	public ImplicitFieldSort(String field) {
	    super(new Name("ImplicitField"), field);
	}
	

	// %%% we should move information from the variable
	// specification to the program variable %RB
	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    return super.allowed(pe, javaInfo)||  
		(field == null && pe instanceof ProgramVariable && 		  
		 ((ProgramVariable)pe).name().toString().charAt(0) =='<');
	}


	public boolean canStandFor(Term t) {
	    if (t.op() instanceof ProgramVariable) {
		return allowed((ProgramVariable)t.op(), null);
	    }
	    return false;
	}

	public boolean canStandFor(ProgramElement pe,
				   JavaInfo javaInfo) {
	    ProgramElement var = pe;
	    if (pe instanceof ImplicitFieldSpecification) {
		var = ((ImplicitFieldSpecification)pe).getProgramVariable();
	    } else if (pe instanceof FieldReference) {
		var = ((FieldReference)pe).getProgramVariable();
	    }
	    return allowed(var, javaInfo);		
	}
    }


    static private class ImplicitFieldReferenceSort 
	extends ImplicitFieldSort {

	public ImplicitFieldReferenceSort() {
	    super(new Name("ImplicitFieldReference"));
	}

	// %%% we should move information from the variable
	// specification to the program variable %RB
	// implicit fields are not of array type so we do not check for
	// this reference type
	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    return super.allowed(pe, javaInfo) &&
		((ProgramVariable)pe).getKeYJavaType().getJavaType() instanceof ClassType;
	}
    }


    static private class SimpleExpressionSort extends ProgramSVSort{

	public SimpleExpressionSort() {
	    super(new Name("SimpleExpression"));
	}

	protected SimpleExpressionSort(Name n) {
	    super(n);
	}
	/*
	protected boolean allowed(ProgramElement pe,
				  JavaInfo javaInfo) {
	    if (pe instanceof Negative) {
 		return ((Negative)pe).getChildAt(0) instanceof Literal;
 	    }	   
	    if (pe instanceof Literal) {
		return true;
	    }
	    return (pe instanceof ProgramVariable
		    || pe instanceof VariableSpecification);
		    }*/

	public boolean canStandFor(Term t) {
	    return true; //%%%%%%%%
	}
	
	public boolean canStandFor(ProgramElement pe,
				   JavaInfo javaInfo) {
	    if(!(pe instanceof Expression)){
		return false;
	    }
	    if (pe instanceof Negative) {
 		return ((Negative)pe).getChildAt(0) instanceof Literal;
 	    }	   
	    if (pe instanceof Literal) {
		return true;
	    }
	    if (pe instanceof MethodReference) {
		return false;
	    }
	    if (pe instanceof Instanceof) {
		return (VARIABLE.canStandFor(((Instanceof) pe).getChildAt(0), javaInfo) &&
			!methodConstrReference(((Instanceof) pe).getChildAt(1))); 
	    }
	    return VARIABLE.canStandFor(pe, javaInfo);    
	}
    }

    static private class SimpleExpressionSpecialPrimitiveTypeSort 
	extends SimpleExpressionSort{

	private PrimitiveType[] allowed_types;

	public SimpleExpressionSpecialPrimitiveTypeSort(String name, PrimitiveType[] allowed_types) {
	    super(new Name(name));
	    this.allowed_types = allowed_types;
	}


	public boolean canStandFor(ProgramElement check, 
				   JavaInfo javaInfo) {
	    if (!super.allowed(check, javaInfo)) {
		return false;
	    }
	    KeYJavaType kjt =
		javaInfo.getPrimitiveKeYJavaType((Expression)check);
	    if (kjt == null)
		return false;
	    for (int i=0; i<allowed_types.length; i++){
		if (kjt.getJavaType() == allowed_types[i])
		    return true;
	    }
	    return false;
	}
    }


    static private class NonSimpleExpressionSort extends ProgramSVSort{

	public NonSimpleExpressionSort() {
	    super(new Name("NonSimpleExpression"));
	}

	public boolean canStandFor(ProgramElement check, 
				   JavaInfo javaInfo) {
	    return !SIMPLEEXPRESSION.canStandFor(check,javaInfo);
	}


    }


    static private class BooleanExpressionSort extends ProgramSVSort{

	public BooleanExpressionSort() {
	    super(new Name("BooleanExpression"));
	}

	private boolean checkArrayPrefix(ReferencePrefix prefix) {
	    if (prefix == null || 
		prefix instanceof NewArray ||
		prefix instanceof MethodReference) {	    
		return true;
	    } else {
		return false;
	    }
	}

	private boolean checkDimensionExpressions
	    (ArrayOfExpression dims, JavaInfo javaInfo) {
	    for (int i = 0; i < dims.size(); i++) {
		if(canStandFor(dims.getExpression(i), javaInfo)) {
		    return true;
		}
	    }
	    return false;
	} 

	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof ParenthesizedExpression) {
		return true;
	    }
	    if (pe instanceof BooleanLiteral) {
		return true;
	    }
	    if (pe instanceof ProgramVariable) {
		return true;
	    }
	    if (pe instanceof Instanceof) {
		return true;
	    }
	    if (pe instanceof Equals) {
 		return true;
            }
	    if (pe instanceof NotEquals) {
 		return true;
            }
	    if (pe instanceof LessThan) {
 		return true;
            }
	    if (pe instanceof LessOrEquals) {
 		return true;
            }
	    if (pe instanceof GreaterThan) {
 		return true;
            }
	    if (pe instanceof GreaterOrEquals) {
 		return true;
            }
	    if (pe instanceof ArrayReference) {
		ArrayReference array = (ArrayReference) pe;				
		return checkArrayPrefix(array.getReferencePrefix()) &&
		    checkDimensionExpressions(array.getDimensionExpressions(),
					      javaInfo);
	    }
	    if ((pe instanceof MethodReference)
		|| (pe instanceof ConstructorReference)
		|| (pe instanceof SpecialConstructorReference)) {
		return true;
	    }
	    return false;
	}

	public boolean canStandFor(ProgramElement check, 
				   JavaInfo javaInfo) {
	    KeYJavaType kjt =
		javaInfo.getPrimitiveKeYJavaType((Expression)check);
	    if (kjt.getJavaType() != PrimitiveType.JAVA_BOOLEAN)
		return false;
	    JavaIterator it = new JavaIterator(check);
	    ProgramElement pe = it.element();
	    while (it.hasNext()) {
		if (allowed(pe, javaInfo)) {
		    return true;
		}
		pe = it.next();
	    }
	    return false;
	}


    }

    static private class NonSimpleBooleanExpressionSort extends ProgramSVSort{

	public NonSimpleBooleanExpressionSort() {
	    super(new Name("NonSimpleBooleanExpression"));
	}

	private boolean checkArrayPrefix(ReferencePrefix prefix) {
	    if (prefix == null || 
		prefix instanceof NewArray ||
		prefix instanceof MethodReference) {	    
		return true;
	    } else {
		return false;
	    }
	}

	private boolean checkDimensionExpressions
	    (ArrayOfExpression dims, JavaInfo javaInfo) {
	    for (int i = 0; i < dims.size(); i++) {
		if(canStandFor(dims.getExpression(i), javaInfo)) {
		    return true;
		}
	    }
	    return false;
	} 

	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof ParenthesizedExpression) {
		return true;
	    }
	    if (pe instanceof Instanceof) {
		return true;
	    }
	    if (pe instanceof Equals) {
 		return true;
            }
	    if (pe instanceof NotEquals) {
 		return true;
            }
	    if (pe instanceof LessThan) {
 		return true;
            }
	    if (pe instanceof LessOrEquals) {
 		return true;
            }
	    if (pe instanceof GreaterThan) {
 		return true;
            }
	    if (pe instanceof GreaterOrEquals) {
 		return true;
            }
 	    if (pe instanceof LogicalAnd) {
 		return true;
            }
 	    if (pe instanceof LogicalOr) {
 		return true;
            }
	    if (pe instanceof ArrayReference) {
		ArrayReference array = (ArrayReference) pe;				
		return checkArrayPrefix(array.getReferencePrefix()) &&
		    checkDimensionExpressions(array.getDimensionExpressions(),
					      javaInfo);
	    }
	    if ((pe instanceof MethodReference)
		|| (pe instanceof ConstructorReference)
		|| (pe instanceof SpecialConstructorReference)) {
		return true;
	    }
	    return false;
	}

	public boolean canStandFor(ProgramElement check, 
				   JavaInfo javaInfo) {
	    KeYJavaType kjt =
		javaInfo.getPrimitiveKeYJavaType((Expression)check);
	    if (kjt.getJavaType() != PrimitiveType.JAVA_BOOLEAN)
		return false;
	    JavaIterator it = new JavaIterator(check);
	    ProgramElement pe = it.element();
	    while (it.hasNext()) {
		if (allowed(pe, javaInfo)) {
		    return true;
		}
		pe = it.next();
	    }
	    return false;
	}


    }
 

    static private class JavaTypeByteSort extends ProgramSVSort{

	public JavaTypeByteSort() {
	    super(new Name("JavaTypeByte"));
	}
	// to be changed!!!
	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof Assignment) {
		return false;
	    }
	    if (pe instanceof Instanceof) {
		return false;
	    }
	    return !methodConstrReference(pe);
	}

    }

    static private class JavaTypeShortSort extends ProgramSVSort{

	public JavaTypeShortSort() {
	    super(new Name("JavaTypeShort"));
	}
	// to be changed!!!
	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof Assignment) {
		return false;
	    }
	    if (pe instanceof Instanceof) {
		return false;
	    }
	    return !methodConstrReference(pe);
	}

    }

    static private class JavaTypeIntSort extends ProgramSVSort{

	public JavaTypeIntSort() {
	    super(new Name("JavaTypeInt"));
	}
	// to be changed!!!
	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof Assignment) {
		return false;
	    }
	    if (pe instanceof Instanceof) {
		return false;
	    }
	    return !methodConstrReference(pe);
	}

    }

    static private class JavaTypeLongSort extends ProgramSVSort{

	public JavaTypeLongSort() {
	    super(new Name("JavaTypeLong"));
	}
	// to be changed!!!
	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof Assignment) {
		return false;
	    }
	    if (pe instanceof Instanceof) {
		return false;
	    }
	    return !methodConstrReference(pe);
	}

    }


    static private class IntegerLiteralSort extends ProgramSVSort{

	public IntegerLiteralSort() {
	    super(new Name("IntegerLiteral"));
	}

	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof IntLiteral) {  

		return true;
	    }
	    return false;
	}

    }

    static private class LongLiteralSort extends ProgramSVSort{

	public LongLiteralSort() {
	    super(new Name("LongLiteral"));
	}

	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof LongLiteral) {

		return true;
	    }
	    return false;
	}

    }


    static private class CatchSort extends ProgramSVSort{

	public CatchSort() {
	    super(new Name("Catch"));
	}

	
	public boolean canStandFor(ProgramElement pe, JavaInfo javaInfo) {
	    if (pe instanceof Catch) {
		return true;
	    } 
	    return false;
	}
    }

    static private class StatementSort extends ProgramSVSort{

	public StatementSort() {
	    super(new Name("Statement"));
	}

	public boolean canStandFor(ProgramElement pe, JavaInfo javaInfo) {
	    return allowed(pe, javaInfo);
	}

	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    return (pe instanceof Statement);
	}
    }

    static private class ExpressionSort extends ProgramSVSort{

	public ExpressionSort(Name n) {
	    super(n);
	}

	public ExpressionSort() {
	    super(new Name("Expression"));
	}

	public boolean canStandFor(ProgramElement pe, JavaInfo javaInfo) {
	    return allowed(pe, javaInfo);
	}

	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    return (pe instanceof Expression);
	}
    }


    static private class NewSVSort extends ProgramSVSort{

	public NewSVSort() {
	    super(new Name("New"));
	}

	public boolean canStandFor(ProgramElement check, 
				   JavaInfo javaInfo) {
	    if (check instanceof New) {
		return true;
	    }
	    return false;
        }
    }


    static private class NewArraySVSort extends ProgramSVSort{

	public NewArraySVSort() {
	    super(new Name("NewArray"));
	}

	public boolean canStandFor(ProgramElement check, 
				   JavaInfo javaInfo) {
	    if (check instanceof NewArray) {
		return true;
	    }
	    return false;
        }
    }

    static private class ArrayInitializerSVSort extends ProgramSVSort{

	public ArrayInitializerSVSort() {
	    super(new Name("ArrayInitializer"));
	}

	public boolean canStandFor(ProgramElement check, 
				   JavaInfo javaInfo) {
	    if (check instanceof ArrayInitializer) {
		return true;
	    }
	    return false;
        }
    }

    
    

    static private class MethodBodySort extends ProgramSVSort{

	public MethodBodySort() {
	    super(new Name("MethodBody"));
	}

	public boolean canStandFor(ProgramElement check, JavaInfo javaInfo) {
	    return allowed(check, javaInfo);
	}

	protected boolean allowed(ProgramElement pe, JavaInfo javaInfo) {
	    return (pe instanceof MethodBodyStatement);
	}
    }

    static private class BranchSort extends ProgramSVSort{

	public BranchSort() {
	    super(new Name("Branch"));
	}


	public boolean canStandFor(ProgramElement pe,
				   JavaInfo javaInfo) {
	    if (pe instanceof Branch) {
		return true;
	    }
	    return false;
	}
    }

    static private class ArraySVSort extends ProgramSVSort{
	public ArraySVSort() {
	    super(new Name("ArraySVSort"));
	}
	
	public boolean canStandFor(ProgramElement check,
			       JavaInfo javaInfo) {
	    return (check instanceof ArrayReference);
        }
    }


    static private class LoopInitSort extends ProgramSVSort{
	public LoopInitSort() {
	    super(new Name("LoopInitSort"));
	}
	
	public boolean canStandFor(ProgramElement check, JavaInfo javaInfo) {
	    return allowed(check, javaInfo);
	}

	public boolean allowed(ProgramElement check,
			       JavaInfo javaInfo) {
	    return (check instanceof LoopInit);
        }
    }

    static private class GuardSort extends ProgramSVSort{
	public GuardSort() {
	    super(new Name("GuardSort"));
	}
	
	public boolean canStandFor(ProgramElement check, JavaInfo javaInfo) {
	    return allowed(check, javaInfo);
	}
 
	public boolean allowed(ProgramElement check,
			       JavaInfo javaInfo) {
	    return (check instanceof Guard);
        }
    }

    static private class ForUpdatesSort extends ProgramSVSort{
	public ForUpdatesSort() {
	    super(new Name("ForUpdatesSort"));
	}
	
	public boolean canStandFor(ProgramElement check, JavaInfo javaInfo) {
	    return allowed(check, javaInfo);
	}

 	public boolean allowed(ProgramElement check,
			       JavaInfo javaInfo) {
	    return (check instanceof ForUpdates);
        }
    }

    

}
