// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.logic.sort;

import de.uka.ilkd.key.logic.op.SortDependingSymbol;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Namespace;


/**
 * Interface for sorts having specific symbols
 * (e.g. functions/predicates of collection sorts)
 */
public interface SortDefiningSymbols extends Sort {

    /**
     * Create the symbols defined by this sort, insert them into the
     * namespace "p_func_ns"
     * @param p_func_ns Namespace to which functions and predicates
     * should be added to
     */
    public void createSymbols ( Namespace p_func_ns, Namespace sort_ns );

    /**
     * Lookup the symbol of kind "p_name", which is a sort
     * independent identifier for this symbol
     * @return Symbol with (kind) name "p_name"
     * ("ret.getKind().equals(p_name)"), null if no such object exists
     */
    public SortDependingSymbol lookupSymbol  ( Name      p_name );

}
