// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001 Universitaet Karlsruhe, Germany
//                    and Chalmers University of Technology, Sweden
//
// The KeY system is protected by the GNU General
// Public License. See LICENSE.TXT for details.
//

package de.uka.ilkd.key.ocl.gf;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;
import java.io.PrintStream;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;

import de.uka.ilkd.key.casetool.*;


/** @author Kristofer Johannisson */

/** utilities for creating GF grammar judgements 
  * (based on information from a UML model)  */
class GrammarCreator
{
    static boolean PRINTDEBUG = false; // print debug info to System.out? 

    /** vector of lines (strings) of the abstract syntax */
    private Vector abs = new Vector();
    /** vector of lines (strings) of the OCL concrete syntax */
    private Vector ocl = new Vector();
    /** vector of lines (strings) of the English concrete syntax */
    private Vector eng = new Vector();

    private HashMap key2gf;

    GrammarCreator()
    {
	//  set up the mapping of KeY's names for OCLtypes into GF's names for OCLtypes
        key2gf = new HashMap();
        key2gf.put("Boolean", "BoolC");
        key2gf.put("Integer", "Integer");
        key2gf.put("Real", "Real");
        key2gf.put("String", "StringC");
	// key2gf.put("java.lang.String", "String");
      key2gf.put("OCLType", "OCLType");
        key2gf.put("OCLAny", "OCLAny");
        key2gf.put("OCLState", "OCLState");
        key2gf.put("OCLExpression", "OCLExpression");
        key2gf.put("Enumeration", "Enumeration");
	// also own conversion of Java types into OCL
        key2gf.put("boolean", "BoolC");
        key2gf.put("int", "Integer");
        key2gf.put("float", "Real");
        key2gf.put("double", "Real");
        // what about Collection types?
    }

    private void abs(String s) {abs.add(s);}
    private void ocl(String s) {ocl.add(s);}
    private void eng(String s) {eng.add(s);}

    /** map KeY's names for OCLtypes into GF's names */
    /*     private String toGF(String name) */
    /*     { */
    /*         String result = (String) key2gf.get(name); */
    /*  */
    /*         if (result != null) return result; */
    /*         else return (name); */
    /*     } */

    //    /** translate some java types into OCL types */
//     public String java2ocl(String s)
//     {
// 	if (s.equals("int")) return "Integer";
// 	if (s.equals("float")) return "Real";
// 	if (s.equals("double")) return "Real";
// 	if (s.equals("boolean")) return "Boolean";
// 	return s;
//     }
    
    /** create (hopefully) unique identifiers from class names */
    private String classId(String clss)
    {
        String result = (String) key2gf.get(clss);

        if (result != null) return result;
        else return (idFilter(clss)+"C");
    }
	
    /** filter out characters that GF doesn't like in identifiers */
    private String idFilter(String s) 
    {
	char c;
	String result = new String();
	for (int i = 0; i < s.length(); i++) { 
	    c = s.charAt(i);
	    switch(c) {
	    case '.': result += "_"; break;
	    case '(': result += "_"; break;
	    case ')':  break;
	    case ' ': break;
	    case '[': result += "_"; break;
	    case ']': result += "_"; break;
	    case ',': result += "_"; break;
	    default: result += String.valueOf(c);
	    }
	}
	return result;
    }
	
    /** create unique id for a operation from strings like "doSomething(type1, type2, ...)" 
	  * and the parameter types */
    public String operId(String clss, String oper, String[] params)
    {
	String result = idFilter(clss) +"_"+ idFilter(stripParams(oper));
	for (int i=0; i < params.length; i++) 
	    result += "_"+classId(params[i]);
	return result;
    }

    /** transform strings like "doSomething(x, y)" into "doSomething" */
    public static String stripParams(String nameWithParams)
    {
    	// System.out.println(nameWithParams);
        int i = nameWithParams.indexOf("(");

        if (i >= 0)
            return nameWithParams.substring(0, i);
        else
            return nameWithParams;
    }
    
    /** create a unique id for the constraint-building function of a class */
    public String classConstraint(String clss)
    {
    	return (idFilter(clss)+"_inv");
    }

    /** create a unique id for the constraint-building function of an operation */
    public String opConstraint(String clss, String oper, String[] params)
    {
	return operId(clss, oper, params)+"Constr";
    }

    /** produce GF judgements for a class */
    public void mkClass(String clss)
    {
	abs("fun " + classId(clss) + " : Class;");
	this.ocl("lin " + classId(clss) + " = stdClass \"" + clss + "\";");
		
	String noun;
	if (clss.length() > 1)
	    noun = clss.substring(0,1).toLowerCase() + clss.substring(1,clss.length());
	else
	    noun = clss.toLowerCase();
	eng("lin " + classId(clss) + " = mkClass \"" + noun +"\" \"" + clss + "\";");
		
	// invariants for this class:
	abs("fun " + classConstraint(clss) + " : (VarSelf " + classId(clss) + " -> ConstraintBody) -> Constraint;");
	ocl("lin " + classConstraint(clss) + " body = contextClass \"" + clss + "\" body.s;");
	eng("lin " + classConstraint(clss) + " body = contextClass \"" + clss + "\""+engConstraintBodyL("body") +";");
	ocl("printname " + classConstraint(clss) + " = \"[constraint for the class " + clss + "]\";");	
	eng("printname " + classConstraint(clss) + " = \"[constraint for the class " + clss + "]\";");	
	

	// invariants with context hidden:
	abs("fun " + classConstraint(clss)+"HC : (VarSelf " + classId(clss) + " -> ConstraintHCInvariant) -> ConstraintHideContext;");
	ocl("lin " + classConstraint(clss)+"HC body = body;");
	eng("lin " + classConstraint(clss)+"HC body = body;");
		
    }

    /** check if name is of kind "isMarried", return "married" (if true) or null (if false) */
    private String isIsWhatever(String name)
    {
	if (name.length() > 2 && name.substring(0,2).equals("is"))
	    return name.substring(2, name.length()).toLowerCase();
	else
	    return null;
    }

    /** produce GF judgements for an attribute */
    public void mkAttribute(String clss, String name, String type)
    {
		String gfName = idFilter(clss) + "_" + idFilter(name);
	if (classId(type).equals("BoolC")) { 
	    // this attribute will correspond to a sentence in English
	    abs("fun " + gfName + " : " + "Instance " + classId(clss) + " -> "
		 + "Sent;");
	    String pred = isIsWhatever(name);	    
	    if (pred != null) {
		// e.g. "the person is married"
		eng("lin " + gfName + " i = mkIsAttr i \"" + pred + "\";");
	    } else {
		// e.g. "the property prop holds for the person"
		eng("lin " + gfName + " i = mkSentAttr i \"" + name + "\";");
	    }

	} else {
	    // a standard attribute
	    this.abs("fun " + gfName + " : " + "Instance " + classId(clss) + " -> "
		     + "Instance " + classId(type) + ";");
	    eng("lin " + gfName + " i = mkAttr i \"" + name + "\";");
	}

	// no difference between Bool and sentences in OCL:
        this.ocl("lin " + gfName + " i = dotAttr i \"" + name + "\";");
    }

    /** produce GF judgements for a query operation */
    public void mkQuery(String clss, String name, String returns, String[] params)
    {
    	String id = operId(clss, name, params);
	String opC = opConstraint(clss, name, params);
        name = stripParams(name);
        String s = new String();

        s += ("fun " +id + " : Instance " + classId(clss));
        
        // the function which corresponds to the query used in a expression
        // abstract syntax:
        for (int i=0; i < params.length; i++)
            s += (" -> Instance " + classId(params[i]));
        if (classId(returns).equals("BoolC")) { // convert Bool to sentence
	    abs(s + " -> Sent;");
	} else {
	    this.abs(s + " -> Instance " + classId(returns) + ";");
	}
       	// ocl:
       	this.ocl("lin " + id + " i " + oclArgList("x", params.length) +
		 " = mkQuery i \""+ name + "\" (" + oclParamList("x", params.length) + ");");

       	// English
	if (classId(returns).equals("BoolC")) { // e.g the property prop(x,y) holds for the object
	    eng("lin " + id + " i " + oclArgList("x", params.length) +
		" = mkSentQuery i \""+ name + "\" (" + oclParamList("x", params.length) + ");");
	} else {
	    eng("lin " + id + " i " + oclArgList("x", params.length) +
		" = mkQuery i \""+ name + "\" (" + oclParamList("x", params.length) + ");");
	}
		       	

        // the constraint-building function for this query:
        // abstract syntax:
        s = ("fun " + opC + " : (VarSelf " + classId(clss) + " -> " +
	     "VarResult " + classId(returns));
        for (int i=0; i < params.length; i++)
            s += (" -> Instance " + classId(params[i]));
        this.abs(s + " -> ConstraintBody) -> Constraint;");
        
        // ocl concrete syntax:
	this.ocl(oclContextOp(clss, id, name, params, returns));
		
	// english concrete syntax:
	eng(engContextOp(clss, id, name, params, returns));
    }

    /** produce GF judgements for a non-query operation */
    public void mkNonQuery(String clss, String name, String returns, String[] params)
    {
	String id = operId(clss, name, params);
	String opC = opConstraint(clss, name, params);
        name = stripParams(name);
	String s = new String();

        // the constraint-building function for this operation:
        s += ("fun " + opC + " : (VarSelf " + classId(clss));
        if (returns != null)
            s += (" -> VarResult " + classId(returns));
        for (int i=0; i < params.length; i++)
            s += (" -> Instance " + classId(params[i]));
        this.abs(s + " -> ConstraintBody) -> Constraint;");
    	
    	ocl(oclContextOp(clss, id, name, params, returns));
    	eng(engContextOp(clss, id, name, params, returns));
    }
    
    /** given an operation, construct a syntax tree which can be filled in with a constraint
      * @param oper has the form e.g. "income(d:Date; x:Bla):Integer"
      * the parameter types are assumed to be OCL types
      */
    public String mkOpConstrSyntaxTree(String clss, String oper)
    {
    	
    	String id = ""; // should be "income(Date,Bla)"
    	String params = "";
    	Vector paramTypes = new Vector();
    	String rtrn = ""; // return type
    	char[] c = oper.toCharArray();
    	
    	// traverse oper from left to right
    	int i = 0;
    	while (c[i] != '(') {
	    id += String.valueOf(c[i]);
	    i++;
    	}
	// consumed oper name, i.e. (d:Date, x:Bla):Integer is left
    	i++; id += "("; // skip '('
    	while (c[i] != ')') {
	    String var = "";
	    while (c[i] != ':' ) {
		var += String.valueOf(c[i]);
		i++;
	    }
	    i++; // skip ':'
	    String type = "";
	    while (c[i] != ';' && c[i] != ')') {
		type += String.valueOf(c[i]);
		id += c[i];
		i++;
	    }
	    // add the type to the vector
	    paramTypes.add(type.trim());
	    // add the parameter to the list N.B.: no type info
	    if (params.equals(""))
		params += "\\"+var.trim(); //+ " : Instance " + classId(type.trim());
	    else
		params += " -> " + "\\"+var.trim(); // + " : Instance " + classId(type.trim());
	    if (c[i] == ')')
		break;
	    i++; id += ","; // skip ','
    	}
    	// done with parameters, ):Integer is left
    	
    	if (c[i]==')')
	    i++; // skip ')'
    	
    	// return type? 
	while (i < c.length && c[i] != ':' ) 
	    i++; // skip ':'
	i++;
	while (i < c.length) {
	    rtrn += c[i];
	    i++;
	}
		
	// convert Vector paramTypes to String[]
	Object[] paramTypesO = paramTypes.toArray();
	String[] paramTypesS = new String[paramTypesO.length];
	for (int j=0; j < paramTypesO.length; j++)
	    paramTypesS[j] = (String) paramTypesO[j];
			
	String result = opConstraint(clss,id, paramTypesS) + " (\\GFvarself"; //: VarSelf "+classId(clss);
	if (!rtrn.equals(""))
	    result += " -> \\GFvarresult"; // : VarResult " + classId(rtrn.trim());
	if (!params.equals(""))
	    result += " -> " + params;
	result += " -> preC ? (postC ? emptyC))";
		
    	return result;
    }
	
    /** produce GF judgements stating that superclss is a superclass of clss */
    public void mkSuperType(String clss, String superclss)
    {
	String sub = classId(clss);
	String sup = classId(superclss); 
	if (!sub.equals(sup)) { // reflexivity is trivial and already implicit in grammar
	    abs("fun " + sub+"Conforms2"+sup + " : Subtype " + sub + " " + sup + ";");
	    ocl("pattern " + sub+"Conforms2"+sup + "= [\"" + clss + " is a subtype of " + superclss + "\"];");
	    eng("pattern " + sub+"Conforms2"+sup + "= [\"" + clss + " is a subtype of " + superclss + "\"];");
	}
    }
	
    /** produce GF judgements for a simple named, typed association with multiplicity. 
	  * the isOrdered argument is currently ignored */
    public void mkAssoc(String clss, String name, String type, 
			UMLOCLMultiplicity mult, boolean isOrdered)
    {
	if (name == null) {
	    // make the first letter of type into lower case
	    // as suggested in "Missing Rolenames" in OCL 1.4 spec
	    if (type.length() > 1)
		name = type.substring(0,1).toLowerCase() + type.substring(1,type.length());
	    else
		name = type.toLowerCase();
	}
	
	String gfName = idFilter(clss) + "_" + idFilter(name);
		
	abs("fun " + gfName+"Assoc" + " : Instance " + classId(clss) + " -> Instance (Set " +
		 classId(type) + ");");
	ocl("lin " + gfName+"Assoc i = dotAssoc i \""+name+"\";");
	eng("lin " + gfName+"Assoc = mkAssoc \""+name+"\";");
	
// 	if (mult.getMax() == 1) {
// 	    // add shortcut to directy refer to the single element of the assoc.
// 	    this.abs("fun "  + clss+"_"+name+"Assoc1" + " : Instance " + classId(clss) + " -> Instance " +
// 		     classId(type) + ";");
// 	    //ocl("lin "  + clss+"_"+name+"Assoc1 i = " 
// 	}
    }

    
    /** build a string like: 
      * lin clss_nameC body = contextOpXX "clss" "name" "returns" (params) body.s */
    private static String oclContextOp(String clss, String id, String name, String[] params, String returns)
    {
    	String s;
    	
    	if (returns == null) {
	    if (params == null || params.length == 0) // returns void, no params
		return ("lin " + id + "Constr body = contextOp00 \"" + clss +
    	    		"\" \"" + name + "\" body.s;");
    	    else // returns void, has params
    	    	return ("lin " +id + "Constr body = contextOp10 \"" + clss +
    	    		"\" \"" + name +"\"" + "(" + oclConstrParams(params,1) +")" +  "body.s;");
    	}
    	else {
	    if (params == null || params.length == 0) // returns something, no params
		return ("lin " +id + "Constr body = contextOp01 \"" + clss +
    	    		"\" \"" + name + "\" \"" + returns +"\" body.s;");
    	    else // returns something, has params
    	    	return ("lin " +  id + "Constr body = contextOp11 \"" + clss +
    	    		"\" \"" + name + "\" (" + oclConstrParams(params,2) + ") \"" + returns 
    	    		+"\" body.s;");
    	}
    }
    
    /** for removing record fields which should not have to be removed? */
    /** this GF bug should be fixed now? */
    private static String engConstraintBodyL(String b)
    {
    	return "{s0 = "+b+".s0; s1="+b+".s1; s2="+b+".s2; invC="+b+".invC; preC="+b+".preC; postC="+b+".postC}" ;
    }   
	 
    /** build a string like: 
      * lin clss_nameC body = contextOpXX "clss" "name" "returns" (params) ...body... */
    private static String engContextOp(String clss, String id, String name, String[] params, String returns)
    {
    	String s;
    	String body = engConstraintBodyL("body");// "{s0 = body.s0; s1=body.s1; s2=body.s2; invC=body.invC; preC=body.preC; postC=body.postC}" ;
    	
    	if (returns == null) {
	    if (params == null || params.length == 0) // returns void, no params
		return ("lin " + id + "Constr body = contextOp00 \"" + clss +
    	    		"\" \"" + name + "\" " + body +";");
    	    else // returns void, has params
    	    	return ("lin " + id + "Constr body = contextOp10 \"" + clss +
    	    		"\" \"" + name + "\" (" + oclConstrParams(params,1) +") " + body +";");
    	}
    	else {
	    if (params == null || params.length == 0) // returns something, no params
		return ("lin " + id + "Constr body = contextOp01 \"" + clss +
    	    		"\" \"" + name + "\" \"" + returns +"\"" + body +";");
    	    else // returns something, has params
    	    	return ("lin " + id + "Constr body = contextOp11 \"" + clss +
    	    		"\" \"" + name + "\" (" + oclConstrParams(params,2) + ") \"" + returns 
    	    		+"\"" + body + ";");
    	}
    }


    /** build a string like: x1 x2 x3 */
    private static String oclArgList(String var, int n)
    {
    	if (n < 0)  /* throw exception? */
	    return "*** java method oclArgList was called with a negative argument ***";    	
    	if (n == 0) return "";
    	else 
	    if (n == 1) return var + "1";
	    else return oclArgList(var, n-1) + " " + var + n;
    }  
    
    /** build a string like: argCons x3 (argCons x2 (argCons x1 argNil) argNil) argNil */
    private static String oclParamList(String var, int n)
    {
    	if (n < 0)  /* throw exception? */
	    return "*** java method oclParamList was called with a negative argument ***";    	
    	if (n == 0) return "argNil";
    	else return "argCons " + var + n + " (" + oclParamList(var, n-1) + ")";
    }
    		

    /** build a string like:  body.v2 ++ [": Type1"] ++ "," ++ body.v3 ++ [": Type2"]... */
    private static String oclConstrParams(String[] types, int base)
    {
	String s = "body.v"+base+" ++ [\": " + types[0] + "\"]";
		
	for (int i=1; i < types.length; i++)
	    s += " ++ \",\" ++ body.v" + (base+i) + " ++ [\": " + types[i] + "\"]";
		
	return s;
    }


	
    /** output the resulting grammar 
     * @param projPath is the path to the project 
     * @param tgf holds information about paths to copies of the static OCL grammars */
    public void flush(String projPath, TempGrammarFiles tgf)
    {
	if (PRINTDEBUG) {
	    System.out.println("\n *** And now for some GF judgements... ***\n");
	    System.out.println("-- Abstract syntax:");
	    tempPrint(System.out, abs);
	    System.out.println("-- OCL Conrete syntax:");
	    tempPrint(System.out, ocl);
	    System.out.println("-- English Concrete syntax:");
	    tempPrint(System.out, eng);
	}
        
        try {
	    PrintStream absStream = new PrintStream(new FileOutputStream(projPath + "/userDefined.Abs.gf", false));
	    PrintStream oclStream = new PrintStream(new FileOutputStream(projPath + "/userDefined.OCL.gf", false));        	
	    PrintStream engStream = new PrintStream(new FileOutputStream(projPath + "/userDefined.Eng.gf", false));
	    tempPrint(absStream, abs);
	    tempPrint(oclStream, ocl);
	    tempPrint(engStream, eng);
	    PrintStream wrapAbsStream = new PrintStream(new FileOutputStream(projPath + "/specs.Abs.gf", false));
	    PrintStream wrapOCLStream = new PrintStream(new FileOutputStream(projPath + "/specs.OCL.gf", false));
	    PrintStream wrapEngStream = new PrintStream(new FileOutputStream(projPath + "/specs.Eng.gf", false));
	    if (tgf==null)
		System.out.println("GC: tgf is null");
	    printWrapper(wrapAbsStream, "Abs", tgf.getAbsPaths());
	    printWrapper(wrapOCLStream, "OCL", tgf.getOclPaths());
	    printWrapper(wrapEngStream, "Eng", tgf.getEngPaths());

	    absStream.close();
	    oclStream.close();
	    engStream.close();
	    wrapAbsStream.close();
	    wrapOCLStream.close();
	    wrapEngStream.close();
	} catch (FileNotFoundException e) { 
	    System.err.println("GrammarCreator: Could not write grammar to files in directory " + projPath);      
	    System.err.println(
			       "e.getMessage(): " + e.getMessage());
	    System.err.println(
			       "e.getLocalizedMessage(): " +
			       e.getLocalizedMessage());
	    System.err.println("e.toString(): " + e);
	    System.err.println("e.printStackTrace():");
	    e.printStackTrace(System.err);
	}
    }

    private static void tempPrint(PrintStream ps, Vector v)
    {
	Iterator it = v.iterator();
	while(it.hasNext()) 
	    ps.println(it.next());
    }
	
    /** create a "wrapper grammar file" which imports all the necessary grammars 
     * @param lang should be either "Abs", "Eng" or "OCL" 
     * @param path is the path to the static grammar files */
    private static void printWrapper(PrintStream ps, String lang, String[] paths)
    {
	// 	if (PRINTDEBUG) 
	// 	    System.out.println("printWrapper uses this path: " + path);
	ps.println("include ");
	if (!lang.equals("Abs"))
	    ps.println("\"specs.Abs.gf\";");
	for (int i=0; i < paths.length; i++) {
	    String path = new String(paths[i]);
	    if (path == null || path.equals(""))
		System.err.println("GrammarCreator: Path to GF OCL grammars is not defined.");
	    else {
		// make sure path ends with '/'
		// 		if (path.charAt(path.length()-1) != '/')
		// 		    path += '/';
		ps.println("    \"" + path + "\";");
	    }
	}
	ps.println("    " + "\"userDefined." + lang + ".gf\";");
    }
		

}
