// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.ocl.ocldltranslator;

import de.uka.ilkd.key.logic.*;
import tudresden.ocl.check.KeyTypeChecker;

/**
 * A class that is used in order to modifiy a string
 * that represents an OCL constraint to be translated
 * and that does nor conform to the parsers grammer 
 * (altough the constraint should be correct), so that
 * it can be parsed by the parser.
 * The modification will be resolved later on in the
 * tree representation of the parsed constraint.
 */


public class ConstraintModifier  {
    
    /**
     * Creates a parsable version of the given constraint
     * text if possible.
     * 
     * Currently only constructor method names (starting with an
     * uppercase letter) will be prepended by some default 
     * prefix (starting with a lowercase letter).
     * This prefix will be removed later in by the KeyTypeChecker.
     */
    public String modifyConstraintText(String constraintText){
//System.err.println("==="+    constraintText);
	String result;

	de.uka.ilkd.key.util.Debug.out("Modifying constraint:\n ", constraintText);

	// replace type short by Integer in the context of the
	// constraint, e.g. in the signature of a method, because the
	// parser wants the context in ocl-notation.
	//xxx This is only a test. If we pursue this approach other types have to 
	//be replaced as well
	int indexOfShort=constraintText.indexOf("short");
	while (indexOfShort!=-1){
	    constraintText=constraintText.substring(0,indexOfShort)+"Integer"+
		constraintText.substring(indexOfShort+5,constraintText.length());    
	    indexOfShort=constraintText.indexOf("short");
	}

	int invTokenIndex = constraintText.indexOf(" inv");
	int openParenIndex = constraintText.indexOf("(");
	if (invTokenIndex == -1 || (openParenIndex != -1 && openParenIndex < invTokenIndex)) {
	    // We have an operation constraint ...

	    // Check whether we have a constructor specification, that
	    // means that the name of the method and the name of
	    // the corresponding context class coincides ...

	    String startString =  constraintText.substring(0, openParenIndex);
	    String restString = constraintText.substring(openParenIndex);
	    int dcolonIndex =  startString.lastIndexOf("::");
	    String methodName = startString.substring(dcolonIndex+2).trim();
	    String prefixString = startString.substring(0, dcolonIndex);
	    int classNameStartIndex;
	    int i = prefixString.lastIndexOf("::");
	    if (i != -1){
		classNameStartIndex = i+2;
	    } else {
		classNameStartIndex = prefixString.lastIndexOf(" ")+1;
	    }
	    String className = prefixString.substring(classNameStartIndex).trim();
	    if (className.equals(methodName)){
		// Add a default prefix string (that starts with a lowercase letter) 
		// in front of the constructor name ...
		System.err.println("ADDING PREFIX for CONSTRUCTOR NAME ...");
		
		result = prefixString + "::" + KeyTypeChecker.KEY_PREFIX_LOWERCASE 
		    + methodName + restString;
	    } else {
		result = constraintText;
	    }
	} else {
	    result = constraintText;
	}

	de.uka.ilkd.key.util.Debug.out("Modified constraint:\n ", result);

	return result;
	
    }

}








