// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.ocl.ocldltranslator.detailedocl;

import de.uka.ilkd.key.ocl.ocldltranslator.*;
import de.uka.ilkd.key.ocl.ocldltranslator.test.*;
import de.uka.ilkd.key.TogetherFacade;

public class DetTranslationTester extends TranslationTester{
    
    OCLMod[] mods;

    protected DetTranslationTester(OCLMod[] mods) {
	this.mods=mods;
    }

    protected String prepareXMI(String xmiFileName) {
	DetailedPrep prep = new DetailedPrep(xmiFileName);
	prep.addListeners(mods);
	prep.transform();
	prep.writeToFile(xmiFileName+".det");
	return xmiFileName+".det";
    }

    protected String prepareConstraint(String orig) {
	DetailedConstraintPrep prep = new DetailedConstraintPrep(orig);
	prep.addListeners(mods);
	prep.transform();
	return prep.result();
    }

    protected KeyOclConstraintTranslator getKeyOclConstraintTranslator
	(TogetherFacade fac) {

	DetailedConstrTransl dct = new DetailedConstrTransl(fac, null);
	dct.addListeners(mods);
	return dct;
    }

    public static void main(String[] args){
	if (args.length == 2){
	    String inputFileName = args[0];
	    String outputFileName = args[1];
	    DetTranslationTester aTransTester 
		= new DetTranslationTester(new OCLMod[]{new NullMod(), 
							new ExcThrownMod()});
	    aTransTester.executeTestSuite(inputFileName, outputFileName);
	} else if (args.length == 3){
	    String options =  args[0];
	    String inputFileName = args[1];
	    String outputFileName = args[2];
	    
	    boolean logParsing = false;
	    boolean translationDetails = false;
	    if (options.indexOf("v") != -1){
		logParsing = true;
	    }

	    if (options.indexOf("d") != -1){
		translationDetails = true;
	    }
	    
	    DetTranslationTester aTransTester 
		= new DetTranslationTester(new OCLMod[]{new NullMod(), 
							new ExcThrownMod()});
	    aTransTester.enableLogging(logParsing);
	    aTransTester.showTranslationDetails(translationDetails);
	    aTransTester.executeTestSuite(inputFileName, outputFileName);
	    
	} else {
	    System.err.println("Wrong syntax !\nUse 'java de.uka.ilkd.key."
			       +"ocl.ocldltranslation.detailedocl."
			       +"DetTranslationTester [-v] testSuiteFileName"
			       +"  resultFileName'");
	    System.err.println("Arguments :");
	    System.err.println("-----------");
	    System.err.println("testSuiteFileName : The file that contains all"
			       +" informations for the translation.");
	    System.err.println("resultFileName : The file in which the results"
			       +" of the translation will be witten.\n");
	    System.err.println("Options :");
	    System.err.println("---------");
	    System.err.println("v : Show details of the XML parsing process for"
			       +" the test suite file.");
	}

    }


}


