// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.ocl.ocldltranslator.detailedocl;

import de.uka.ilkd.key.ocl.ocldltranslator.*;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import java.io.*;
import java.net.*;
import java.util.Vector;
import java.util.Iterator;

public class DetailedPrep extends DefaultHandler{
        
    protected org.apache.xerces.parsers.SAXParser xmlParser;
    protected InputSource src;
    private static final String CLASS_FEATURE_TAG = "Foundation.Core.Classifier.feature";
    private static final String NAMESPACE_TAG = "Foundation.Core.Namespace.ownedElement";
    private static final String DATATYPE_TAG="Foundation.Core.DataType";
    private StringBuffer result=new StringBuffer("");
    private String lastElem="";
    private boolean chr=false;
    private boolean insertNullAttr=false;
    private Vector classIds=new Vector();
    private String currentModElemId="";
    private String currentDTId="";
    private Vector listeners=new Vector();
    private XMIInfo xmiInfo;

    private static final String stringDataType = 
	"  <Foundation.Core.DataType xmi.id = 'keytxmiid9' >\n"+
	"    <Foundation.Core.ModelElement.name>string"+
	"</Foundation.Core.ModelElement.name>\n"+
	"    <Foundation.Core.ModelElement.visibility xmi.value = 'private'/>\n"+
	"    <Foundation.Core.GeneralizableElement.isRoot"+
	" xmi.value = 'false'/>\n"+
	"    <Foundation.Core.GeneralizableElement.isLeaf"+
	" xmi.value = 'false'/>\n"+
	"    <Foundation.Core.GeneralizableElement.isAbstract"+
	" xmi.value = 'false'/>\n"+
	"  </Foundation.Core.DataType>";


    private static final String booleanDataType =
	"  <Foundation.Core.DataType xmi.id = 'txmiid1a' >\n"+
	"    <Foundation.Core.ModelElement.name>boolean"+
	"</Foundation.Core.ModelElement.name>\n"+
	"    <Foundation.Core.ModelElement.visibility xmi.value = 'private'/>\n"+
	"    <Foundation.Core.GeneralizableElement.isRoot"+
	" xmi.value = 'false'/>\n"+
	"    <Foundation.Core.GeneralizableElement.isLeaf"+
	" xmi.value = 'false'/>\n"+
	"    <Foundation.Core.GeneralizableElement.isAbstract"+
	" xmi.value = 'false'/>\n"+
	"  </Foundation.Core.DataType>";

    public DetailedPrep(String xmiFileName) {	
	this.xmiInfo=new XMIInfo(xmiFileName);	
	src = new InputSource(xmiFileName);
	xmlParser = new org.apache.xerces.parsers.SAXParser();
	xmlParser.setContentHandler(this);
    }

    public XMIInfo getXMIInfo() {
	return xmiInfo;
    }

    public void addListener(OCLMod listener) {
	listeners.add(listener);	
	listener.setXMIInfo(getXMIInfo());
    }


    public void addListeners(OCLMod[] l) {
	for (int i=0; i<l.length; i++) {
	    addListener(l[i]);
	}
    }

    public void transform() {
	try {
	    xmlParser.parse(src);
	} catch (SAXException se) {
	    System.err.println("Parse error in " + src);
	    se.printStackTrace();
	} catch (IOException ioe){
	    System.err.println("I/O exception at " + src);
	    ioe.printStackTrace();
	} finally {
	    try {
		xmlParser.reset();
	    } catch (Exception e){
		System.err.println("Error while setting back parser.");
		e.printStackTrace();
	    }
	}

	
    }

    public void writeToFile(String xmiFileName) {
	PrintWriter w=null;
	try {
	    w = new PrintWriter
		(new BufferedWriter(new FileWriter((new URL("file://"+xmiFileName)).getFile())));
	} catch (java.io.IOException e) {
	    System.err.println("Preparing Detailed Model: Writing to "
			       +xmiFileName+" failed.");
	    e.printStackTrace();
	}
	w.print(result.toString());
	w.close();
    }

    public String result() {
	return result.toString();
    }

    public static String replace(String s, String ch, String with) {
	int l=s.lastIndexOf(ch);
	while (l>=0) {
	    s=s.substring(0, l)+with+s.substring(l+1);
	    l=s.lastIndexOf(ch);
	}
	return s;
    }

    public void writeString(StringBuffer sb, 
					 char[] ch, int start, int length) {
	for (int i=start; i<start+length; i++) {
	    if (ch[i]=='>') sb.append("&gt;");
	    else if (ch[i]=='<') sb.append("&lt;");
	    else if (ch[i]=='&') sb.append("&amp;");
	    else sb.append(ch[i]);
	}
    }

    public void characters(char[] ch, int start, int length) {
	chr=true;       	
	writeString(result, ch, start, length);
	if ((new String(ch, start, length)).equals("String")) {
	    insertNullAttr=true;
	}
    }
    
    public void ignorableWhitespace(char[] ch, int start, int length)
	throws SAXException {		
	result=result.append(new StringBuffer().append(ch,start,length));
    }



    public void endElement(String namespaceURI, String localName,
			   String qName) {
 	Iterator it=listeners.iterator();
 	while (it.hasNext()) {
	    OCLMod currentMod=(OCLMod)it.next();
 	    chr=currentMod.insertXMIAtEndElement(result, localName, 
						 currentModElemId) || chr;
 	}
// 	if (localName.equals(DATATYPE_TAG) && insertNullAttr) {
// 	    result=result.append(XMIFactory.getClassFeature
// 				 (XMIFactory.getAttribute("null","'"
// 							  +currentDTId+"'")));
// 	    insertNullAttr=false;
// 	    chr=true;
// 	}
// 	if (localName.equals(CLASS_FEATURE_TAG)) {
// 	    result=result.append(excThrown());
// 	    result=result.append(nullAttr(currentModElemId));
// 	    chr=true;
// 	}
// 	if (localName.equals(NAMESPACE_TAG)) {
// 	    result=result.append(nullClass());
// 	    result=result.append(nullGeneralization());
// 	    chr=true;
// 	}
	if (localName.equals("Foundation.Core.Namespace.ownedElement")) {
	    //	    result=result.append(stringDataType);
	    //	    result=result.append(booleanDataType);
	}
  	if (lastElem.equals(localName) && !chr 
 	    && !(localName.endsWith(".name"))
 	    && !(localName.endsWith(".body"))
 	    && !(localName.endsWith(".language"))
 	    && !(localName.endsWith(".parameter"))
 	    && !(localName.endsWith(".specification"))) {
 	    if (result.length()>1) {
 		result=result.insert(result.length()-1,"/");
 	    }
 	} else {
 	    result=result.append("</"+localName+">");	
 	}
 	chr=false;
 	classIds.add(localName);
    }

    public void startDocument() {
	result=result.append("<?xml version = '1.0' encoding = 'ISO-8859-1' ?>"
			     +"\n<!DOCTYPE XMI SYSTEM  'uml.dtd' \n>");
    }

    public void startElement(java.lang.String uri, java.lang.String localName, 
			     java.lang.String qName, Attributes attributes) {       
	result=result.append("<"+localName);
	for (int i=0; i<attributes.getLength(); i++) {	    
	    result=result.append(" "+attributes.getLocalName(i)+" = '"
				 +attributes.getValue(i)+"'");
	    if (localName.equals(XMIFactory.CLASS) 
		&& attributes.getLocalName(i).equals("xmi.id")) {
		currentModElemId=attributes.getValue(i);		
	    }
	    if (localName.equals(DATATYPE_TAG) 
		&& attributes.getLocalName(i).equals("xmi.id")) {
		currentDTId=attributes.getValue(i);		
	    }
	}
	result=result.append(">");
	lastElem=localName;
	chr=false;
    }

    public static void main (String args[]) {
	System.out.println(replace("dasd < dsad << dfsa > sad >> ",">","&gt;"));
    }

}
