// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.ocl.ocldltranslator.detailedocl;

import de.uka.ilkd.key.ocl.ocldltranslator.*;
import tudresden.ocl.parser.node.Node;
import tudresden.ocl.check.types.Type;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.sort.*;
import tudresden.ocl.OclTree;
import java.util.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.declaration.*;
import de.uka.ilkd.key.java.statement.*;
import de.uka.ilkd.key.java.abstraction.*;
import de.uka.ilkd.key.java.expression.literal.BooleanLiteral;
import de.uka.ilkd.key.util.ExtList;
import de.uka.ilkd.key.TogetherFacade;
import de.uka.ilkd.key.logic.op.ProgramVariable;

public class ExcThrownMod extends OCLMod{

    public static final ExcThrownMod SINGLETON=new ExcThrownMod();

    private String constraint="";

    private HashMap excTypes2Vars=null;
    
    private boolean excThrownOcc=false;

    private Term trueTerm=null;

    private String booleanId=null;

    protected String booleanId() {	
	return xmiInfo().booleanId();
    }        


    protected String stringId() {
	return xmiInfo().stringId();
    }        

    protected String excThrown() {
	return XMIFactory.getOpParam("excThrown", ""+booleanId(), 
				     "type", ""+stringId());
	//return XMIFactory.getOp("excThrown", ""+booleanId());
    }

    public boolean insertXMIAtEndElement(StringBuffer result, String elemName, 
					 String currentModElemId) {
	if (elemName.equals(XMIFactory.CLASSIF_FEATURE)) {
 	    result=result.append(excThrown());
	    return true;
	}
	return false;
    }


    public JavaBlock getMethodCallJavaBlock(KeyOclConstraintTranslator 
					    consTransl, String methodReturnType,
					    Vector methodParameterNames, 
					    Vector methodParameterTypes, 
					    String constrainedOperation, 
					    String constrainedType) {

	if (excThrownOcc) {
	    KeyOclExpressionTranslator exprTransl=consTransl.exprTransl();
	    TogetherFacade togetherFacade = consTransl.getTogetherFacade();
	    if (trueTerm==null) {
		initializeBooleanTerm(togetherFacade.javaInfo());		
	    }
	    boolean voidMethodCall = (methodReturnType == null);
	    ProgramVariable[] argProgVars 
		= new ProgramVariable[methodParameterNames.size()];
	    ProgramVariable currentProgVar;
	    String currentParamName, currentParamTypeName;
	    KeYJavaType currentParamType;
	    for(int i=0;i<methodParameterNames.size();i++){
		currentParamName = (String) methodParameterNames.elementAt(i);
		currentParamTypeName=(String) methodParameterTypes.elementAt(i);
		currentParamType = exprTransl
		    .getKeYJavaTypeForNonCollectionType(currentParamTypeName);
		currentProgVar = consTransl.getTogetherFacade()
		    .getProgramVariable(currentParamName,currentParamType);
		argProgVars[i] = currentProgVar;
	    }
		
	    JavaBlock methodCallJavaBlock = null; 
	    KeYJavaType calledObjectVarType = exprTransl
		.getKeYJavaTypeForNonCollectionType(constrainedType);
		
	    ProgramVariable calledObjectVar = consTransl.getTogetherFacade()
		.getProgramVariable("self",calledObjectVarType);

	    VarAndType[] excProgVars = getProgVarsForExcTypes();
	    initializeProgVarsForExcTypes();  //delete for next call
	    if (!voidMethodCall) {
		// we do have a specified return type for the method ...
		    
		// Create a program variable for the result of the method ...
		KeYJavaType methodResultType 
		    = exprTransl.getKeYJavaTypeForNonCollectionType
		    (methodReturnType);
		ProgramVariable resultProgVar 
		    = togetherFacade.getProgramVariable("result",
							methodResultType);
		    
		methodCallJavaBlock 
		    = togetherFacade.getMethodCall(calledObjectVar,
						   constrainedOperation,
						   resultProgVar,
						   argProgVars, excProgVars);
	    } else {
		// we do not have a specified return type for the method ...
		methodCallJavaBlock 
		    = togetherFacade.getVoidMethodCall(calledObjectVar,
						       constrainedOperation,
						       argProgVars, 
						       excProgVars);
	    }
	    return methodCallJavaBlock;
	}
	return null;
    }

    private boolean treeContainsExcThrown(OclTree oclTree) {
	return (oclTree.toString().indexOf("excThrown")>0);
    }

    public TranslationResult getTranslationFor(DetailedConstrTransl
					       consTransl,
					       OclTree oclTree, 
					       boolean useDiamondForm, 
					       boolean ignorePost,
					       boolean 
					       oclTreeIsSimpleExpression){
	if (treeContainsExcThrown(oclTree)) {
	    excThrownOcc=true;
	    return consTransl.getSuperTranslationFor(oclTree, 
						     useDiamondForm, ignorePost,
						     oclTreeIsSimpleExpression);
	} else {
	    excThrownOcc=false;
	    return null;
	}
    }


    public ProgramVariable getVariableForExcType(String excKind, 
						 KeyOclExpressionTranslator 
						 exprTransl) {
	if (excTypes2Vars==null) {
	    initializeProgVarsForExcTypes();  
	}
	if (!excTypes2Vars.containsKey(excKind)) {
	    ProgramVariable pv = new ProgramVariable
		    (new ProgramElementName("thrown"+excKind), 
		     trueTerm.sort());
	    KeYJavaType kjt 
		= exprTransl.getKeYJavaTypeForNonCollectionType(excKind);
	    VarAndType vartype = new VarAndType(pv, kjt);
	    excTypes2Vars.put(excKind, vartype);
	}
	return ((VarAndType)excTypes2Vars.get(excKind)).getProgramVariable();
    }

    public VarAndType[] getProgVarsForExcTypes() {
	Iterator it=excTypes2Vars.values().iterator();
	VarAndType[] result=new VarAndType[excTypes2Vars.size()];
	for (int i=0; i<result.length; i++) {
	    result[i]=(VarAndType)it.next();
	}
	return result;
    }

    private void initializeProgVarsForExcTypes() {
	excTypes2Vars=new HashMap();
    }

    private void initializeBooleanTerm(JavaInfo javaInfo) {
	trueTerm=javaInfo.getTypeConverter().convertToLogicElement
	    (new BooleanLiteral(true)); //%%%%
    }


    private void translateExcThrown(String excKind, 
				    KeyOclExpressionTranslator exprTransl, 
				    Node n, 
				    Node appliedTo) {

	TermFactory tf = exprTransl.termFactory();
	TogetherFacade togetherFacade = exprTransl.togetherFacade();
	if (trueTerm==null) {
	    initializeBooleanTerm(togetherFacade.javaInfo());		
	}
	KeyOclConstraintTranslator constraintTranslator 
	    = exprTransl.constraintTranslator();
	KeyTranslationResult resultTranslation;
	
	KeyTranslationResult objectTranslation = (KeyTranslationResult) 
	    exprTransl.getTranslationForNode(appliedTo);
	Term objectTerm = objectTranslation.getTerm();
	Sort objectSort = objectTerm.sort();
	ProgramVariable pv = getVariableForExcType(excKind, exprTransl);
	Term resultTerm = tf.createEqualityTerm(tf.createVariableTerm(pv), 
						trueTerm);
	

	
 	resultTranslation = new KeyTranslationResult();	    	    	    
 	resultTranslation.setTerm(resultTerm);

	exprTransl.setTranslationForNode(n, resultTranslation);
	
    }



    public boolean translateOCL2DL(KeyOclExpressionTranslator exprTransl, 
				Node n, Node appliedTo,
				String featureName,
				Vector parameterNodes) {

	if (featureName.equals("excThrown")) {
	    if (parameterNodes.size()==0) {
		translateExcThrown("Exception", exprTransl, n, appliedTo);
	    } else {
		String nodeString=n.toString();
		String paramName
		    =nodeString.substring(nodeString.indexOf("'")+1,
					  nodeString.lastIndexOf("'"));
		translateExcThrown(paramName, exprTransl, n, appliedTo);
	    }
	    return true;
	}
	return false;
    }

    public boolean transformOCL(String constraint) {
	String result=constraint;
	int okuntil=0;
	while (okuntil<=result.length()) {
	    int i=result.indexOf("excThrown(", okuntil)+10;
	    if (i-10==-1) break;
	    int j=result.indexOf(")",i);
	    if (result.substring(i).trim().charAt(0)!='\'') {
		result=result.substring(0, i)+"'Exception'"+result.substring(j);
	    }
	    okuntil=j+1;
	}
	this.constraint=result;
	return true;
    }

    public String getOCLConstraint() {
	return constraint;
    }

    public static void main(String[] args) {
	ExcThrownMod em=new ExcThrownMod();
	em.transformOCL("excThrown() sd dsf sdaf  excThrown() sad excThrown(   )"
			+" sadd ds excThrown('MyExc') and excThrown(   'DsF'   )"
			+" sdfsaa excThrown()");
	System.out.println(em.getOCLConstraint());
    }

}
