// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.ocl.ocldltranslator.detailedocl;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import java.io.*;

public class XMIInfo extends DefaultHandler{

    protected org.apache.xerces.parsers.SAXParser xmlParser;
    protected InputSource src;
    String currentId="";
    boolean modelElementName=false;
    String booleanId;
    String stringId;


    public XMIInfo(String xmiFileName) {
	InputSource src = new InputSource(xmiFileName);
	xmlParser = new org.apache.xerces.parsers.SAXParser();
	xmlParser.setContentHandler(this);
	try {
	    xmlParser.parse(src);
	} catch (SAXException se) {
	    System.err.println("Parse error for XMIInfo in " + src);
	    se.printStackTrace();
	    System.exit(-1);
	} catch (IOException ioe){
	    System.err.println("I/O exception at " + src);
	    ioe.printStackTrace();
	} finally {
	    try {
		xmlParser.reset();
	    } catch (Exception e){
		System.err.println("Error while setting back parser.");
		e.printStackTrace();
	    }
	}
    }

    public void startElement(java.lang.String uri, java.lang.String localName, 
			     java.lang.String qName, Attributes attributes) {
       	for (int i=0; i<attributes.getLength(); i++) {	    
	    if (attributes.getLocalName(i).equals("xmi.id")) {
		currentId=attributes.getValue(i);
	    }
	}
	if (localName.equals("Foundation.Core.ModelElement.name")) {
	    modelElementName=true;
	}
    }

    public void endElement(String namespaceURI, String localName,
			   String qName) {      
	if (localName.equals("Foundation.Core.ModelElement.name")) {
	    modelElementName=false;
	}
    }

    public void characters(char[] ch, int start, int length) {
	if (modelElementName) {
	    StringBuffer sb=new StringBuffer().append(ch,start,length);
	    if (sb.toString().equals("boolean")) {
		booleanId=""+currentId;
	    }
	    if (sb.toString().equals("String")) {
		stringId=""+currentId;
	    }
	}
    }

    public String booleanId() {
	return booleanId;
    }


    public String stringId() {
	return stringId;
    }

}
