// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.ocl.ocldltranslator.heuristics;

import tudresden.ocl.*;
import tudresden.ocl.parser.analysis.*;
import java.util.Vector;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Iterator;
import java.util.ListIterator;
import java.util.Enumeration;

import tudresden.ocl.parser.node.*;
import tudresden.ocl.check.types.*;

import de.uka.ilkd.key.ocl.ocldltranslator.*;
import de.uka.ilkd.key.ocl.ocldltranslator.control.*;

/**
 * A heuristic that forces OCL features correpsonding to attributes, assciation ends  or 
 * methods to be translated into attributes and methods within the corresponding context 
 * classes in the DL.
 */ 

public class AttributesAndMethodsAsFunctionSymbolsHeuristic extends KeyTranslationHeuristic {

    /** 
     * Creates an AttributesAndMethodsAsFunctionSymbolsHeuristic instance.
     */
    public AttributesAndMethodsAsFunctionSymbolsHeuristic(){
	super();
    }

    /**
     * @return the name of this translation heuristic.
     */
    public String getHeuristicName(){
	return "Attributes and Methods as Functionsymbols - Heuristic";
    }
    
    // ================ Start the computation of the translation types =====================
	    
    public void determineTranslationTypesFor(OclTree constraintTree, 
					     AExpression expression, 
					     Map currentMarking,
					     KeyOclExpressionTranslator exprTranslator){
		
	this.tree = constraintTree;
	this.expression = expression;
	this.exprTranslator = exprTranslator;

	this.varMap = new HashMap();
	nodeTranslationTypeMap = currentMarking;
	
	expression.apply(this);

	
    }
  

    // ================ The main part of the heuristic ===================================    
    

    // =========================================================================================================
    // ================================= The feature calls on UML model types ==================================
    // =========================================================================================================



    protected void translateFeatureCallonModelType(Node n, String featureName, Node appliedTo, 
						   Vector parameterNodes, Vector iteratorNames, 
						   boolean  featureCallWithAtPre, boolean featureCallWithArrow, 
						   boolean isMethodCall){
	log("Translating feature call on UML Model type : " + featureName);
	log("Applied on node :" + appliedTo.toString());
	
	TranslationType resultTranslationType;
	
	boolean isUMLModelTypeFeature = (!featureName.equals("oclIsKindOf") &&
					 !featureName.equals("oclIsTypeOf") && 
					 !featureName.equals("oclAsType") && 
					 !featureName.equals("oclIsNew"));
	if (isUMLModelTypeFeature) {
	    // We have a feature call for a feature in the UML model 
	    // (attribute, association end, methodcall) ...
	    
	  
	    log("Is attribute or method or assication of some UML model type == " + isUMLModelTypeFeature);
	    
	    log("Marked node for nonrigid-function-symbol translation  :" + n.toString());
	    resultTranslationType = getTranslationTypeForNode(n);
	    resultTranslationType.addMark(TranslationMark.AS_NONRIGID_FUNCTION_SYMBOL);
	    
	    // Finally set the new translation result ...
	    setTranslationTypeForNode(n,resultTranslationType);
	}

    }
    
    // =========================================================================================================
    // ================================= The feature calls on OCLType ==========================================
    // =========================================================================================================
    
    protected void translateFeatureCallonOclType(Node n, String featureName, Node appliedTo, Vector parameterNodes, Vector iteratorNames, boolean  featureCallWithAtPre, boolean featureCallWithArrow ){
	log("Feature call on OclType : " + featureName);
	log("Applied on node :" + appliedTo.toString());
	
	Type resultType = tree.getNodeType(n);
	OclType oclType = (OclType) tree.getNodeType(appliedTo);
	Type typeObjectToCurrentOCLType = oclType.getType();
	

	TranslationType resultTranslationType;

	log("Result type is :" + resultType);
	log("Current type object to OCLType is :" + typeObjectToCurrentOCLType);
	
	boolean oclTypeIsUMLModelType = (!typeObjectToCurrentOCLType.equals(Basic.INTEGER) &&
					 !typeObjectToCurrentOCLType.equals(Basic.REAL) &&
					 !typeObjectToCurrentOCLType.equals(Basic.BOOLEAN) &&
					 !typeObjectToCurrentOCLType.equals(Basic.STRING) &&
					 !typeObjectToCurrentOCLType.equals(Basic.ENUMERATION) &&
					 !(typeObjectToCurrentOCLType instanceof Collection) &&
					 !(typeObjectToCurrentOCLType instanceof OclType) &&
					 !(typeObjectToCurrentOCLType instanceof OclState) );
	
	log("Current type object is UML modeltype :" + oclTypeIsUMLModelType);

	boolean featureIsClassFeature = ( !featureName.equals("allInstances") && 
					   !featureName.equals("name") && 
					   !featureName.equals("attributes") &&
					   !featureName.equals("associationEnds") &&
					   !featureName.equals("operations") &&
					   !featureName.equals("supertypes") &&
					   !featureName.equals("allSupertypes") );

	log("Current feature call on type object is a class feature call :" + featureIsClassFeature );


	if(oclTypeIsUMLModelType && featureIsClassFeature){
	    // Handle a method or attribute ...
	    log("Marked node for nonrigid-function-symbol translation  :" + n.toString());
	    
	    resultTranslationType = getTranslationTypeForNode(n);
	    resultTranslationType.addMark(TranslationMark.AS_NONRIGID_FUNCTION_SYMBOL);
	    
	    // Finally set the new translation result ...
	    setTranslationTypeForNode(n,resultTranslationType); 
	    
	}

    }

}







