// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.ocl.ocldltranslator.heuristics;

import tudresden.ocl.*;
import tudresden.ocl.parser.analysis.*;
import java.util.Vector;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Iterator;
import java.util.ListIterator;
import java.util.Enumeration;

import de.uka.ilkd.key.TogetherFacade;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;

import tudresden.ocl.parser.node.*;
import tudresden.ocl.check.types.*;

import de.uka.ilkd.key.ocl.ocldltranslator.*;
import de.uka.ilkd.key.ocl.ocldltranslator.control.*;

/**
 * THIS IS A KeY SPECIFIC HEURISTIC!
 *
 * A heuristic that forces OCL features correpsonding to attributes or methods to be
 * translated into attributes and methods within the corresponding context classes in
 * the DL.
 * Association(ends) will be in general represented by non-rigid function symbols with only
 * one exception: An association end r1 has multiplicity 0..1 or 1 and the JavaFile
 * that corresponds to the corresponding source class (in which the navigation starts) 
 * contains an attribute with the same name. In this case the association end will be
 * translated by use of this attribute. Note, that this treatmentis correct!
 */ 

public class AttributesAndMethodsAsFunctionSymbolsHeuristicForKey extends KeyTranslationHeuristic {
    
    /** 
     * Creates an AttributesAndMethodsAsFunctionSymbolsHeuristicForKey instance.
     */
    public AttributesAndMethodsAsFunctionSymbolsHeuristicForKey(){
	super();
    }

    /**
     * @return the name of this translation heuristic.
     */
    public String getHeuristicName(){
	return "Attributes and Methods as Functionsymbols For KeY - Heuristic";
    }
    
    // ================ Start the computation of the translation types =====================
	    
    public void determineTranslationTypesFor(OclTree constraintTree, 
					     AExpression expression, 
					     Map currentMarking,
					     KeyOclExpressionTranslator exprTranslator){
		
	this.tree = constraintTree;
	this.expression = expression;
	this.exprTranslator = exprTranslator;
	
	this.varMap = new HashMap();
	nodeTranslationTypeMap = currentMarking;
	
	expression.apply(this);

	
    }
  

    // ================ The main part of the heuristic ===================================    
    

    // =========================================================================================================
    // ================================= The feature calls on UML model types ==================================
    // =========================================================================================================



    protected void translateFeatureCallonModelType(Node n, String featureName, Node appliedTo, 
						   Vector parameterNodes, Vector iteratorNames, 
						   boolean  featureCallWithAtPre, boolean featureCallWithArrow, 
						   boolean isMethodCall){
	log("Translating feature call on UML Model type : " + featureName);
	log("Applied on node :" + appliedTo.toString());
	
	TranslationType resultTranslationType;
	
	// get the return type of the feature call ...
	Type returnType = tree.getNodeType(n);
	
	log("Return type = " + returnType.toString());

	
	
	boolean isUMLModelTypeFeature = (!featureName.equals("oclIsKindOf") &&
					 !featureName.equals("oclIsTypeOf") && 
					 !featureName.equals("oclAsType") && 
					 !featureName.equals("oclIsNew"));
	if (isUMLModelTypeFeature) {
	    // We have a feature call for a feature in the UML model 
	    // (attribute, association ends, methodcall) ...

	    // WE only mark features that belong to association ends with the following
	    // properties:
	    // - They have a multiplicity greater than 1
	    // - or (otherwise) there is no attribute in the corresponding 
	    //                  Java file for that class with the same name.

	    boolean featureIsAssociation = false;
	    // Check whether the given feature is an association ...
	   
	    // CURRENTLY THIS IS A SIMPLE WORK AROUND !!!
	    // Unfortunately the OCL Parser allows no suitable access to the UML model the
	    // constraint has been formulated over.
	    // Therefore we would have to build our own model representation.
	    // In order to get a simple work around we do the following ...
	    // An association will be represented as a non-rigid function symbol only
	    // when it has a multiplicity greater than 1, e.g. has a collection value,
	    // or there is no attribute with the same name in the corresponding Java class 
	    // (by use if the JavaInfo Interface!)
	    // Otherwise, we'll use simpliy treat it as an attribute of the class.
	    
	    boolean noAttributeInClass = false;
	    
	    // get the model type the feature belongs to ...
	    Type classType = tree.getNodeType(appliedTo);
	    String className = classType.toString();
//System.err.println("className="+className);            
	    KeYJavaType clsReference =  exprTranslator.getKeYJavaTypeForNonCollectionType(className); 

			
	    
	    // Now check whether there is some attribute with the same name declared within the class ...
	    // Note, that we refer here to the JAVA implementation instead of the UML model!
	    TogetherFacade togetherFacade = exprTranslator.getTogetherFacade();
	    ProgramVariable attributeInClass = togetherFacade.getAttribute(featureName , clsReference);
	    noAttributeInClass = (attributeInClass == null);

	    featureIsAssociation = !isMethodCall && ((returnType instanceof Collection) || 
						     (!(returnType instanceof Collection) && noAttributeInClass));
	    
	    log("Feature is association  == " + featureIsAssociation);

	    if (featureIsAssociation) {
		// Mark the corresponding node for a representation by a non-rigid function symbol ...
				
		log("Marked node for nonrigid-function-symbol translation  :" + n.toString());
		resultTranslationType = getTranslationTypeForNode(n);
		resultTranslationType.addMark(TranslationMark.AS_NONRIGID_FUNCTION_SYMBOL);
		
		// Finally set the new translation result ...
		setTranslationTypeForNode(n,resultTranslationType);
	    }
	}

    }
   

}
















