// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.ocl.ocldltranslator.test;

import tudresden.ocl.OclTree;
import tudresden.ocl.check.*;
import tudresden.ocl.check.types.*;

import de.uka.ilkd.key.ocl.ocldltranslator.*;
import de.uka.ilkd.key.ocl.ocldltranslator.heuristics.*;
import de.uka.ilkd.key.ocl.ocldltranslator.control.*;
import de.uka.ilkd.key.ocl.ocldltranslator.normalize.*;
import de.uka.ilkd.key.ocl.ocldltranslator.transform.*;


public class KeyTypeCheckerFactory extends TypeCheckerFactory {
    
    boolean expressionMode = false;

    /**
     * Creates a new factory that delivers (bug-fixed) type checkers for
     * the KeY-System.
     * Moreover the TypeCheckers run in constraint mode, that means that
     * the top-level expression of a constraint will be checked whether
     * it is a boolean expression.
     */
    public KeyTypeCheckerFactory(){
	super();
	expressionMode = false;
    }
    

     /**
     * Creates a new factory that delivers (bug-fixed) type checkers for
     * the KeY-System.
     * Moreover the TypeCheckers run in constraint mode (that means that
     * the top-level expression of a constraint will be checked whether
     * it is a boolean expression) iff the given parameter is false.
     * Otherwise the boolean check for the top-level expression will
     * be omitted, which is necessary for the test environment of the
     * OCL-DL translation since it should also be possible to test the 
     * translation of ARBITRARY OCL expressions.
     */
    public KeyTypeCheckerFactory(boolean exprMode){
	super();
	expressionMode = exprMode;
    }


    public TypeQueryable getTypeQueryable(
      NameBoundQueryable nbq, OclTree tree, TypeFactory factory) {
	TypeChecker result;
	if (nbq==null) {
	    if (!expressionMode){
		
		// Here we use a bug fixed TypeChecker instance instead of a TypeChecker instance ...
		// result = new KeyTypeChecker(tree, factory);
		
		result = new KeyTypeChecker(tree, factory);

	    } else {
		// Here we use a KeyExpressionTypeChecker instance instead of a TypeChecker instance ...
		result = new KeyExpressionTypeChecker(tree, factory);
	    }
	    return result;
	} else {
	    return (TypeQueryable) nbq;
	}
    }

    
    public NameBoundQueryable getNameBoundQueryable(TypeQueryable tq, OclTree tree, TypeFactory factory) {
	TypeChecker result;
	if (tq==null) {
	    if (!expressionMode){
		// Here we use a bug fixed TypeChecker instance instead of a TypeChecker instance ...
		// result = new KeyTypeChecker(tree, factory);

		result = new KeyTypeChecker(tree, factory);
	    } else {
		// Here we use a KeyExpressionTypeChecker instance instead of a TypeChecker instance ...
		result = new KeyExpressionTypeChecker(tree, factory);
	    }
	    return result;
	} else {
	    return tq;
	}
    }
    
}




