// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.ocl.ocldltranslator.test;

import java.util.*;

import tudresden.ocl.*;
import tudresden.ocl.parser.analysis.*;
import tudresden.ocl.parser.node.*;
import tudresden.ocl.check.types.*;

import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.*;
import java.io.*;

import de.uka.ilkd.key.ocl.ocldltranslator.*;
import de.uka.ilkd.key.ocl.ocldltranslator.heuristics.*;
import de.uka.ilkd.key.ocl.ocldltranslator.control.*;
import de.uka.ilkd.key.ocl.ocldltranslator.normalize.*;
import de.uka.ilkd.key.ocl.ocldltranslator.transform.*;

/**
 * This class represents a single test case for the transformation of OCL constraints
 * in DL formulas as used in the KeY-System.
 */
public class TransformationTestCase {
   
    protected TransformationTestSuite testSuite;
    
    protected String name;
    protected String description;
    protected String constraint;
    protected String expression;
    

    protected Map options;
    
    public TransformationTestCase(TransformationTestSuite tSuite) {
	testSuite = tSuite;
	name = "";
	constraint = "";
	description = "";
	expression = null;

	options = new HashMap();
    }
    

    public void setName(String aName){
	name = aName;
    }

    public String getName(){
	return name;
    }

    public void setDescription(String aDescr){
	description = aDescr;
    }

    public String getDescription(){
	return description;
    }

    public void setConstraint(String aConstraint){
	constraint = aConstraint;
    }

    public String getConstraint(){
	return constraint;
    }

    public void setExpression(String aExpression){
	expression = aExpression;
    }

    public String getExpression(){
	return expression;
    }

    public boolean isOCLExpression(){
	return (expression != null);
    }

    
    
    public TransformationTestSuite getTestSuite(){
	return testSuite;
    }
   
    public void setOption(String optionName, String optionValue){
	options.put(optionName, optionValue);
    }
    
    /**
     * @return the value for the fiven option if specified in the test suite or 'null' otherwise.
     */
    public String getValueForOption(String optionName){
	String optionValue;
	if(options.containsKey(optionName)){
	    optionValue = (String) options.get(optionName);
	} else {
	    optionValue = null;
	}
	return optionValue;
	
    }

}


