// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

//
// Offers the functionality of tudresden.ocl.ASTViewer 
// but decoupled from the graphical user interface issues
//

package de.uka.ilkd.key.ocl.tudresden;


// Imports from the original parser
import tudresden.ocl.parser.OclParserException;
import tudresden.ocl.parser.node.Start;
import tudresden.ocl.parser.parser.ParserException;
import tudresden.ocl.check.TypeQueryable;
import tudresden.ocl.*;

import de.uka.ilkd.key.util.KeYResourceManager;
import de.uka.ilkd.key.ocl.ocldltranslator.*;
import de.uka.ilkd.key.ocl.ocldltranslator.test.*;
import de.uka.ilkd.key.casetool.together.UMLOCLTogetherModel;
import de.uka.ilkd.key.casetool.together.OCLTogetherModelFacade;

import java.io.*;
import java.net.*;
import javax.swing.tree.*;

// Communication with the user
import com.togethersoft.openapi.ide.IdeAccess;
import com.togethersoft.openapi.ide.project.*;
import com.togethersoft.openapi.ide.window.IdeDialogType;
import  de.uka.ilkd.key.casetool.together.scripts.menuextension.ModelSourceSettings;
import  de.uka.ilkd.key.gui.ProofSettings;

public class TUDresdenParserAdapter{

    static String pathSep = File.separator;

    OclTree oclTree;
    // attributes to store syntax errors found by the OCL parser
    private String errorMessageToUser = "";
    private boolean errorOccurred = false;


    private void printURLStream(URL url) {
	try {
	    InputStream ins=url.openStream();
	    char c;
	    while ((c=(char)ins.read())!=-1) {
		System.out.print(c);
	    }
	    System.out.println();
	    ins.close();
	} catch (Exception e) {
	    System.out.println("Fehler: "+e);
	}
    }

    // parses the constraint on the model expressed in xmifile
    // ensures that in the same directory as xmifileURL an auxialiary
    // file uml.dtd can be found
    public OclTree doParse1(String constraintIdent, String xmifile, 
			    String constraint, String containingPackage){
    	oclTree = null;
	String xmifileURL = "file:"+xmifile;
	// copying the uml.dtd if needed
	String dtdTargetPath = (new File(xmifile)).getParent();
	String resourceName="uml.dtd";
	String dtdTargetName=resourceName;
	KeYResourceManager.getManager().copyIfNotExists
	    (this, resourceName, dtdTargetPath+File.separator+dtdTargetName);
    
	// parsing the constraint
	try {
	    // Modify the constraint in order to get a parsable version of the constraint.
	    // This is mainly necessary for constraints on constructors (that start with a capital letter!)
	    ConstraintModifier cModifier = new ConstraintModifier();
	    String modifiedConstraint = cModifier.modifyConstraintText(constraint);
	    System.out.println(modifiedConstraint);

	    tudresden.ocl.check.types.ModelFacade mf;
	    //mf = tudresden.ocl.check.types.xmifacade.XmiParser.createModel(xmiURL, xmifileURL);
	    
	    // getting the directory containing the TCC-project...
// 	    System.out.println(">>>>>>>>>>>>>> "+IdeProjectManagerAccess
// 			       .getProjectManager().getActiveProject().getFileName());

	    ModelSourceSettings mss
		= ProofSettings.THE_SETTINGS.getModelSourceSettings();
	    if ("0".equals(mss.getModelSource())) {
		if (!(new File(xmifile)).isFile()) {
		    throw new RuntimeException
			("XMI file "+xmifile
			 +" does not exist.\n"
			 +"Please use File | Export | Model to XMI File "
			 +"\nto create the XMI model file in the project directory."
			 +"\nAs alternative, you can use the TogetherCC model "
			 +"directly by "
			 +"\nchoosing `TogetherCC data structure as UML model source"
			 +"\nfrom the KeY menu.");
		}
		URL xmiURL = new URL(xmifileURL);
		mf = tudresden.ocl.check.types.xmifacade.XmiParser.
		    createModel(xmiURL, xmifileURL);
	    } else {
		// getting the directory containing the TCC-project...
		String tprFile = IdeProjectManagerAccess
		    .getProjectManager().getActiveProject().getFileName();
		String projectRoot = tprFile.substring(0, tprFile.lastIndexOf(pathSep));
		
		mf = new OCLTogetherModelFacade((new UMLOCLTogetherModel(projectRoot)).getUMLOCLClassifiers());
		//		mf = new TJModelFacade(containingPackage, projectRoot);
	    }
	   
	    oclTree=OclTree.createTree(modifiedConstraint, mf, 
				       new KeyTypeCheckerFactory());
	    if (oclTree.getRoot()!=null) {
		displayTree(constraintIdent, oclTree.getRoot(), oclTree);
		//createTextFile();
	    }
    	} catch (OclParserException e) {
	    showMessage("OclParserException occurred");
	    showMessage(e);
	    setErrorMessageToUser(constraintIdent,
				  "Syntax Error. OCLParser Exception occurred",
				  true);
	    e.printStackTrace();
	    //doLex();
    	} catch (java.net.MalformedURLException e) {
	    showMessage("Malformed URL. Please check the used file:\n"
				       +xmifile);
	    
	} catch (org.xml.sax.SAXException e) {
	    showMessage("Error in XMI file "+xmifile
				       +"\n"+e.getMessage());
	} catch (java.io.IOException e) {
	    showMessage("Error reading model file "
				       +"\n"+e.getMessage());
	}
    	return oclTree;
  	}

    // displayTree generates the errorMessages iff a syntactical 
    // error occurred
  protected void displayTree(String constraintIdent, Start root, TypeQueryable tq) {
      Visualizer vis=new Visualizer(true, tq);
      if (root == null){showMessage("doParse1(): displayTree(): root is null");}
      root.apply(vis);
      if (vis == null){showMessage("doParse1(): displayTree(): vis is null");}
      //if (tree==null) 
      //tree=new JTree( vis.getRootNode() );
      //} else {
      TreeModel model=new DefaultTreeModel(vis.getRootNode());
      //  tree.setModel( model );
      //makeVisible(tree, model, vis.getRootNode());
      if (vis.getErrorMessage() == null){
	  setErrorMessageToUser(constraintIdent,"No syntax error found", false);
      }else{
	  setErrorMessageToUser(constraintIdent," Syntax error: " + 
				vis.getErrorMessage(), true );
      }
      //}
  }

    //   public void doLex() {
    //       //    Lexer lexer=new Lexer(
    //     tudresden.ocl.parser.lexer.Lexer lexer=new tudresden.ocl.parser.lexer.Lexer(
    //       new PushbackReader( new StringReader(text.getText()) )
    //     );
    //     Token t;
    //     DefaultMutableTreeNode dtm=new DefaultMutableTreeNode("lexer output");
    //     try {
    //       do {
    //         t=lexer.next();
    //         String text=t.getClass().getName()+" "+t.toString();
    //         dtm.add(new DefaultMutableTreeNode(text));
    //       } while (! (t instanceof EOF) );
    //     }
    //     catch (Exception e) {}
    //     tree.setModel(
    //       new DefaultTreeModel(dtm)
    //     );
    //   }

    public void createTextFile() {
	//     if (oclTree!=null) {
	//       try {
	//         ASTTextGenerator textGen=new ASTTextGenerator();
	//         oclTree.apply(textGen);

	// 	FileWriter fw=new FileWriter("d:\\temp\\ocl.txt");
	//         fw.write(textGen.getText());
	//         fw.close();
	//       }
	//       catch (IOException e) {
	// 	  //        message.setText("error writing to file: "+e.getMessage());
	//         messageToUser("error writing to file: "+e.getMessage());
	//        }
	//     }
    }

    protected void showMessage(Exception e) {
	String s=e.getMessage();
	if (s==null) {
	    //      message.setText(e.getClass().getName()+" without message");
	    messageToUser(e.getClass().getName()+" without message");
	    e.printStackTrace(System.out);
	    return;
	} else {
	    showMessage(s);
	}
	if (e instanceof ParserException) {
	    ParserException pe=(ParserException)e;
	    String index=s.substring( s.indexOf(",")+1, s.indexOf("]") );
	    int ix=Integer.parseInt(index);
	    //text.setCaretPosition(ix);
	    //text.setSelectionStart(ix);
	    //text.setSelectionEnd(ix+1);
	    messageToUser(" Set Selection on Position " + ix);
	}
    }

    protected void showMessage(String s) {
	if (s==null || s.equals("")) {
	    //      message.setText(" ");
	} else {
	    StringBuffer sb=new StringBuffer();
	    for (int i=0; i<s.length(); i++) {
		char c=s.charAt(i);
		if (! Character.isISOControl(c) ) {
		    sb.append(c);
		}
	    }
	    //      message.setText(sb.toString());
	    messageToUser(sb.toString());
	}
    }


    private void messageToUser(String msg){
	System.err.println("Msg from TUDresdenParserAdapter: " + msg);
    }

    private void setErrorMessageToUser(String culprit, String msg, 
				       boolean occurrenceFlag){
	errorMessageToUser = culprit + ": "+ msg;
	errorOccurred = occurrenceFlag;
    }


    public String getErrorMessageToUser(){
    	return errorMessageToUser;
    }

    public boolean getErrorOccurred(){
    	return errorOccurred;
    }

}
