// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.parser;

import antlr.*;

public class AmbigiousDeclException extends antlr.SemanticException {
    String cat;
    String ambigious_symbol;
    Token t;
    
    public AmbigiousDeclException(String cat, Token t) {
	super("Declared twice or more");
	this.cat = cat;
	this.ambigious_symbol = t.getText();
	this.line = t.getLine();
	this.column = t.getColumn();
    }

    public AmbigiousDeclException(String cat, String ambigious_symbol,
				  String filename, int line, int column) {
	super("Declared twice or more");
	this.cat = cat;
	this.ambigious_symbol = ambigious_symbol;
	this.line = line;
	this.column = column;
    }
    /**
     * Returns a clean error message (no line number/column information)
     */
    public String getErrorMessage ()
    {
	String errmsg = cat+"\n  "+ambigious_symbol+"\n";
	return errmsg + "declared twice";
    }

    /**
     * Returns a clean error message (no line number/column information)
     */
    public String getMessage ()
    {
	return getErrorMessage();
    }
    
    /**
     * Returns a string representation of this exception.
     */
    public String toString() {
	return "("+this.getLine()+", "+this.getColumn()+"):\n\t"
	    +getErrorMessage();
    }
}
