// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.

package de.uka.ilkd.key.parser;


import java.io.Reader;

import antlr.*;

import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.gui.AbbrevMap;


/** Defaut implementation of SimpleTermParser. This class wraps the
 * default KeY-Term-Parser in the SimpleTermParser interface.
 *
 * @author Hubert Schmid
 */

public final class DefaultTermParser implements SimpleTermParser {

    /* --- methods --- */

    public Term parse(Reader in, Sort sort, JavaInfo javaInfo,
                      Namespace varNS, Namespace func_ns, Namespace sort_ns,
                      AbbrevMap scm)
        throws ParserException
    {
        try {
            TermParser parser
                = new TermParser(new KeYLexer(in), "", TermFactory.DEFAULT,
                                 javaInfo, varNS, func_ns, sort_ns,scm);
            return parser.term();
        } catch (RecognitionException re) {
            throw new ParserException(re.getMessage(),
                                      new Location(re.getFilename(),
                                                   re.getLine(),
                                                   re.getColumn()));
        } catch (TokenStreamException tse) {
            throw new ParserException(tse.getMessage(), null);
        }
    }

}
