// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.parser;

import antlr.*;

public class KeYSemanticException extends antlr.SemanticException {
    String cat;
    String filename;
    Token t;
    
    public KeYSemanticException(String cat, Token t, String filename) {
	super("Semantic Error");
	this.cat      = cat;
	this.filename = filename;
	this.t = t;
	this.line     = t.getLine();
	this.column   = t.getColumn();
    }

    /**
     * Returns a clean error message (no line number/column information)
     */
    public String getErrorMessage ()
    {
	String errmsg = cat+"\n Occured when reading symbol: ";
	return errmsg;
    }

    /**
     * Returns a clean error message (no line number/column information)
     */
    public String getMessage ()
    {
	return getErrorMessage();
    }
    
    /**
     * Returns a string representation of this exception.
     */
    public String toString() {
	return filename+"("+this.getLine()+", "+this.getColumn()+"): "
	    +getErrorMessage();
    }
}
