// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

/* -*-Antlr-*- */

header {
	package de.uka.ilkd.key.parser;
}

/**
 * The common lexer for declarations, terms, formulae, Taclets, etc.
 */

class KeYLexer extends Lexer;

options {
	k=2;
}

tokens {
	JAVA = "java";
	ARRAY = "array";
	INITIALIZER = "initializer";
	STATEMENT = "statement";
        EXPRESSION = "expression";
        STATEMENTEXPRESSION = "statementexpression";
	BRANCH = "branch";
	NONSIMPLE = "nonsimple";
	LEFTHANDSIDE = "lefthandside";
	SIMPLE = "simple";
	NOTSIMPLE  = "notsimple";
	LOOPINIT   = "loopinit";
	GUARD      = "guard";
	FORUPDATES = "forupdates";
	TYPEOF = "typeof";
        TYPE = "type";
        JAVABOOL="javaboolean";
	OBJECT = "object";
        JAVABYTE = "javabyte";
        JAVASHORT = "javashort";
        JAVAINT = "javaint";
        JAVALONG = "javalong";
        JAVACHAR = "javachar";
        JAVABYTESHORTINT = "javabyteshortint";
        ANYJAVATYPE = "anyjavatype";
        NUMBER = "number";
        INTLITERAL = "intliteral";
        LONGLITERAL = "longliteral";
	GENERIC = "generic";
        EXTENDS = "extends";
        ONEOF = "oneof";
	VARIABLE = "variable";
        SWITCH = "switch";
	BOOLEANVARIABLE = "booleanvariable";
	CATCH = "catch";
	METHODREFERENCE = "methodreference";
	CONSTRUCTORREFERENCE = "constructorreference";
	SPECIAL = "special";
	CLASSTYPE = "classtype";
	INTERFACETYPE = "interfacetype";
	ARRAYTYPE = "arraytype";
	METHODBODY = "methodbody";
	VARIABLEINIT = "variableinitializer";
	LABEL = "label";
	PROGRAM = "program";
	MULTIPLEVARDECL = "multiplevardecl";
	SVLIST   = "svlist";
	FORALL = "all";
	EXISTS = "ex";
	TRUE = "true";
	FALSE = "false";
	FIND = "find";
	IF = "if";
	VARCOND = "varcond";
	NONINTERACTIVE = "noninteractive";
	RECURSIVE = "recursive";
        DISPLAYNAME = "displayname";
        HELPTEXT = "helptext";
	NEW = "new";
	NOT = "not";
	FREE = "free";
	IN = "in";
        DEPENDING = "depending";
        ON = "on";
	CLOSE = "close";
	GOAL = "goal";
	REPLACEWITH = "replacewith";
	ADD = "add";
	ADDRULES = "addrules";
	ADDPROGVARS = "addprogvars";
	HEURISTICS = "heuristics";
	SORTS = "sorts";
	SCHEMA = "schema";
	VARIABLES = "variables";
	IMPLICIT = "implicit";
	FIELD = "field";
	REFERENCE = "reference";
	NEXT = "next";
	LENGTH = "length";
    	RIGID = "rigid";
	PREDICATES = "predicates";
	FUNCTIONS = "functions";
	FORMULA = "formula";
	RULES = "rules";
        PROOF = "proof";
        RULE = "rule";
        TERM = "term";
        INST = "inst";
        IFSEQFORMULA = "ifseqformula";
        INTERACTIVE = "interactive";
        HEUR = "heur";
        PROBLEM = "problem";
	IN_TYPE = "#inType";
	IS_OBJECT = "#isObject";
	IS_CLONABLE = "#isClonable";
	META_IN_LONG ="#inLong";
	META_IN_INT ="#inInt";
	META_IN_CHAR ="#inChar";
	META_IN_SHORT ="#inShort";
	META_IN_BYTE ="#inByte";
	META_ADD = "#add";
	META_SUB = "#sub";
	META_MUL = "#mul";
	META_DIV = "#div";
	META_MOD = "#mod";
	META_LESS = "#less";
	META_GREATER = "#greater";
	META_LEQ = "#leq";
	META_GEQ = "#geq";
	META_EQ = "#eq";
	META_INT_UNARY_MINUS ="#JavaIntUnaryMinus";
	META_INT_ADD ="#JavaIntAdd";
	META_INT_SUB ="#JavaIntSub";
	META_INT_MUL ="#JavaIntMul";
	META_INT_DIV ="#JavaIntDiv";
	META_INT_MOD ="#JavaIntMod";
	META_INT_AND ="#JavaIntAnd";
	META_INT_OR  ="#JavaIntOr";
	META_INT_XOR ="#JavaIntXor";
	META_INT_COMPLEMENT ="#JavaIntComplement";
	META_INT_SHIFTRIGHT ="#JavaIntShiftRight";
	META_INT_SHIFTLEFT ="#JavaIntShiftLeft";
	META_INT_UNSIGNEDSHIFTRIGHT ="#JavaIntUnsignedShiftRight";
	META_LONG_UNARY_MINUS ="#JavaLongUnaryMinus";
	META_LONG_ADD ="#JavaLongAdd";
	META_LONG_SUB ="#JavaLongSub";
	META_LONG_MUL ="#JavaLongMul";
	META_LONG_DIV ="#JavaLongDiv";
	META_LONG_MOD ="#JavaLongMod";
	META_LONG_AND ="#JavaLongAnd";
	META_LONG_OR  ="#JavaLongOr";
	META_LONG_XOR ="#JavaLongXor";
	META_LONG_COMPLEMENT ="#JavaLongComplement";
	META_LONG_SHIFTRIGHT ="#JavaLongShiftRight";
	META_LONG_SHIFTLEFT ="#JavaLongShiftLeft";
	META_LONG_UNSIGNEDSHIFTRIGHT ="#JavaLongUnsignedShiftRight";
        META_MODULO_BYTE = "#moduloByte";
        META_MODULO_SHORT = "#moduloShort";
        META_MODULO_INTEGER = "#moduloInteger";
        META_MODULO_LONG = "#moduloLong";	
        STATIC_ANALYSIS = "#staticanalysis";	
        META_LENGTH = "#lengthReference";	
        WARY_ALL = "waryAll";	
        WARY_EX = "waryEx";	
        SAME_UPDATE_LEVEL = "sameUpdateLevel";
}

protected
VOCAB
   :       '\3'..'\377'
   ;



SEMI
options {
  paraphrase = "`;'";
} :	';'
    ;

SLASH
options {
  paraphrase = "`/'";
} :	'/'
    ;

COLON
    options {
    paraphrase = "`:'";
} :    ':'
    ;

GUARD
    options {
    paraphrase = "`?='";
} :    "?="
    ;

CONDUP
    options {
    paraphrase = "`/->'";
} :    "/->"
    ;

ASSIGN
    options {
    paraphrase = "`:='";
} :    ":="
    ;

DOT
options {
  paraphrase = "`.'";
}
	:	'.'
	;

COMMA
options {
  paraphrase = "`,'";
}
	:	','
	;


LPAREN
options {
  paraphrase = "`('";
}
	:	'('
	;

RPAREN
options {
    paraphrase = "`)'";
}
:	')'
    ;


LBRACE
options {
  paraphrase = "`{'";
}
	:	'{'
	;

RBRACE
options {
    paraphrase = "`}'";
}
:	'}'
    ;

LBRACKET
options {
    paraphrase = "`['";
}
:	'['
    ;

RBRACKET
options{
    paraphrase = "']'";
}
:	']'
	;

AT
options {
    paraphrase = "`at'";
}
:	'@'
	;

OR
options {
  paraphrase = "`|'";
}
	:	'|'
	;

AND
options {
  paraphrase = "`&'";
}
	:	'&'
	;

NOT
options {
  paraphrase = "`!'";
}
	:	'!'
	;

IMP
options {
  paraphrase = "`->'";
}
	:	"->"
	;

EQV
options {
  paraphrase = "`<->'";
}
	:	"<->"
	;

EQUALS
options {
  paraphrase = "`='";
}
	:	"="
	;

SEQARROW
options {
  paraphrase = "`==>'";
}
	:	"==>"
	;

CTRL
options { 
  paraphrase = "'^'";
}
	:	'^'
	;


ARITH_OP
options {
  paraphrase = "java operator";
}
	:	 "%" | "*" | "-"
	;

WS
options {
  paraphrase = "white space";
}
	:	(' '
	|	'\t'
	|	'\n'  { newline(); }
	|	'\r')
		{ $setType(Token.SKIP); }
	;


/*CARD_LITERAL
options {
    paraphrase = "a non-negative number";
}
    :
    (DIGIT)+
    ;
*/

STRING_LITERAL
options {
  paraphrase = "a string in double quotes";
}
    : '"'! ( ESC | ~('"'|'\\') )* '"'! ;


CHAR_LITERAL 
options {
  paraphrase = "a char in single quotes";
}
    : '\''(('!'..'z' '\'') | ("\\u" HEX '\'')) ;


protected
ESC
    :	'\\'
    (	'n'         { $setText("\n"); }
	|	'r' { $setText("\r"); }
	|	't' { $setText("\t"); }
	|	'b' { $setText("\b"); }
	|	'f' { $setText("\f"); }
	|	'"' { $setText("\""); }
	|	'\'' { $setText("'"); }
	|	'\\' { $setText("\\"); }
    )
    ;

DIAMOND
    : '<'! (WS)* BLOCK (WS)* '>'!
    ;

BOX
    : '['! (WS)* BLOCK (WS)* ']'!
    ;



protected
BLOCK
 :
	'{'
	(
		options {
			greedy = false; // exit upon '}'
		}
	:
		(
			options {
				generateAmbigWarnings = false; // shut off newline warning
			}
		:	'\r' '\n'	{newline();}
		|	'\r' 		{newline();}
		|	'\n'		{newline();}
		)
	|	BLOCK
	|	SL_COMMENT
	|	ML_COMMENT
	|	QUOTED_STRING_LITERAL
	|	.
	)*
	'}'
   ;

protected
QUOTED_STRING_LITERAL
options {
  paraphrase = "a string with double quotes";
}
    : '"' ('\\' . | ~('"'|'\\') )* '"' ;



SL_COMMENT
options {
  paraphrase = "a comment";
}

	:
	"//"
	(~'\n')* '\n'
	{ $setType(Token.SKIP); newline(); }
	;

ML_COMMENT
options {
  paraphrase = "a comment";
}
	:
	"/*"
	(
	        '\n' { newline(); }
	 |	'*' ~'/'
	 |	~'*'
	)*
	"*/"
	{ $setType(Token.SKIP);  }
	;

protected
DIGIT
options {
  paraphrase = "a digit";
}
	:	'0'..'9'
	;


protected 
HEX
options {
    paraphrase = "a hexadeciamal number";
}
:	('a'..'f'|'A'..'F'|DIGIT )
	('a'..'f'|'A'..'F'|DIGIT )
	('a'..'f'|'A'..'F'|DIGIT )
	('a'..'f'|'A'..'F'|DIGIT )
    ;

protected
LETTER
options {
    paraphrase = "a letter";
}
:	'a'..'z'|'A'..'Z' 
    ;


protected IDCHAR 
options {
	paraphrase = "an admissible character for identifiers";
}
	: LETTER | DIGIT | '_' | '+' | '~'
	;


IDENT
options {
    testLiterals = true;
    paraphrase = "an identifer";
}

:       ("#" (IDCHAR)*) | IDCHAR (IDCHAR | (AT "pre"))*
	;


/*NUM_LITERAL
options {
    paraphrase = "a number";
}
    : "-"(DIGIT)+ | (DIGIT)+    ;

*/
