// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

/* -*-antlr-*- */
header {

    package de.uka.ilkd.key.parser;

    import java.io.*;
    import java.util.Vector;
    import java.util.HashMap;
    import java.util.HashSet;
    import java.util.Iterator;

    import antlr.*;

    import de.uka.ilkd.key.logic.*;
    import de.uka.ilkd.key.logic.sort.*;
    import de.uka.ilkd.key.logic.op.*;
    import de.uka.ilkd.key.rule.metaconstruct.*;
    import de.uka.ilkd.key.java.JavaInfo;
    import de.uka.ilkd.key.java.JavaReader;
    import de.uka.ilkd.key.java.SchemaJavaReader;
    import de.uka.ilkd.key.java.visitor.ProgramVariableCollector;
    import de.uka.ilkd.key.proof.*;
    import de.uka.ilkd.key.rule.*;
    import de.uka.ilkd.key.java.visitor.*;
    import de.uka.ilkd.key.gui.AbbrevMap;
    import de.uka.ilkd.key.java.recoderext.KeYCrossReferenceServiceConfiguration;

}

/** 
 * A parser for first-order terms and formulae.
 */  

class ProblemParser extends TacletParser;
options {
	importVocab=KeYLexer;
	k = 2;
	defaultErrorHandler = false;
}


{

    static final Sort[] AN_ARRAY_OF_SORTS = new Sort[0];
    static final Term[] AN_ARRAY_OF_TERMS = new Term[0];
    protected boolean minus=false;
    protected JavaInfo javaInfo;
    protected Namespace var_ns  = new Namespace();
    protected Namespace func_ns = new Namespace();
    protected Namespace sort_ns = new Namespace();
    protected Namespace heur_ns = new Namespace();
    protected AbbrevMap scm = new AbbrevMap();
    // these variables are set if a file is read in step by
    // step. This used when reading in LDTs because of cyclic
    // dependencies.
    protected boolean skip_functions;
    protected boolean skip_predicates;
    protected boolean skip_sorts;
    protected boolean skip_schemavariables;
    protected boolean skip_heuristics;
    protected UseInsteadMap useInstead=new UseInsteadMap();	

    // if this is used then we can capture parts of the input for later use
    protected DeclPicker capturer = null;

    protected int lineOffset=0;
    protected int colOffset=0;
	

    protected SetOfTaclet taclets = SetAsListOfTaclet.EMPTY_SET;

    private static ListOfRulevariable NO_RULE_VARIABLES =
	SLListOfRulevariable.EMPTY_LIST;
    private Term a=null;

    protected TermFactory tf = TermFactory.DEFAULT;
    boolean schematicJava;	
    boolean allowSchemaVars;

    public boolean errors = false;


    public ProblemParser(TokenStream lexer, 
			 JavaInfo javaInfo) {
	this(lexer);
	this.javaInfo = javaInfo;
    }

    public ProblemParser(TokenStream lexer, 
			 JavaInfo javaInfo, UseInsteadMap uim) {
	this(lexer, javaInfo);
	this.useInstead=uim;
    }

    public ProblemParser(TokenStream lexer, 
			 String filename,
			 TermFactory tf,
			 JavaInfo javaInfo) {
	this(lexer, javaInfo);
	setFilename(filename);
	this.tf = tf;	
    }
	

    public ProblemParser(TokenStream lexer, 
			 String filename,
			 TermFactory tf,
			 JavaInfo javaInfo,
			 Namespace var_ns,
			 Namespace func_ns,
			 Namespace sort_ns,
			 Namespace heur_ns) {
	this(lexer, javaInfo);
	setFilename(filename);
	this.tf = tf;

	this.func_ns = func_ns;
	this.sort_ns = sort_ns;
	this.var_ns  = var_ns;
	this.heur_ns = heur_ns;	
    }

    public ProblemParser(TokenStream lexer, 
			 String filename,
			 JavaInfo javaInfo,
			 Namespace var_ns,
			 Namespace func_ns,
			 Namespace sort_ns,
			 Namespace heur_ns) {

	this(lexer, filename, TermFactory.DEFAULT, 
	     javaInfo, var_ns, func_ns, sort_ns, heur_ns);
    }

    public ProblemParser(TokenStream lexer, 
			 JavaInfo javaInfo,			
			 Namespace var_ns,
			 Namespace func_ns,
			 Namespace sort_ns,
			 Namespace heur_ns) {
	this(lexer, "(no file)", javaInfo, var_ns, func_ns, sort_ns, heur_ns);
    }


    public ProblemParser(TokenStream lexer, 
			 JavaInfo javaInfo,
			 NamespaceSet ns) {

	this(lexer, javaInfo, ns.variables(), ns.functions(), 
	ns.sorts(), ns.heuristics());
	if (lexer instanceof DeclPicker) {
          capturer = (DeclPicker) lexer;
	}
    }

    public ProblemParser(TokenStream lexer, 
			 JavaInfo javaInfo,
			 NamespaceSet ns,
			 UseInsteadMap uim) {
	this (lexer, javaInfo, ns);
	this.useInstead=uim;
    }


    protected void setSkips(boolean svars, boolean funcs, 
			    boolean preds, boolean sorts,
			    boolean heurs) {

	skip_schemavariables = svars;
	skip_functions       = funcs;
	skip_predicates      = preds;
	skip_sorts           = sorts;
	skip_heuristics      = heurs;

    }
    
    public Term toZNotation(String s, boolean minus){
	Term[] t1=new Term[1];
	Operator v;
	Term a;
	v=(Function) func_ns.lookup(new Name("#"));
	t1[0] = tf.createFunctionTerm((Function)v);
	v = (Function) func_ns.lookup(new Name(s.substring(0,1)));
	a = tf.createFunctionTerm((Function)v,t1);
	for(int i=1;i<s.length();i++){
		t1[0]=a;
		v = (Function)func_ns.lookup(new Name(s.substring(i,i+1)));
		a = tf.createFunctionTerm((Function)v,t1);
	}
	if(minus){
		t1[0]=a;
		v = (Function) func_ns.lookup(new Name("neg"));
		a = tf.createFunctionTerm((Function)v,t1);
	}
	t1[0]=a;
	v = (Function) func_ns.lookup(new Name("Z"));
	a = tf.createFunctionTerm((Function)v,t1);
	return a;
    }

    protected void skipFuncs() {
	skip_functions = true;
    }
    


    protected boolean isNumber(String s){
	      boolean num=true;
          if(s==null){
            num=false;
          }else{
	        for(int i=s.length()-1;i>=0;i--){
		     if(s.charAt(i)<'0'||s.charAt(i)>'9')
			    num=false;
	        }
         }
	     return num;
    }

    protected void skipPreds() {
	skip_predicates = true;
    }

    protected void skipVars() {
	skip_schemavariables = true;
    }

    protected void skipSorts() {
	skip_sorts = true;
    }

    protected void skipHeur() {
	skip_heuristics = true;
    }
	
    /** returns the java information object */
    public JavaInfo getJavaInfo() {
	return javaInfo;
    }


    /** parses a problem but without reading the declarations of
     * sorts, functions and predicates. These have to be given
     * explicitly.
     * @param func_ns the Namespace with the function and predicate
     * symbols to be used
     * @param sort_ns the Namespace with the sorts to be used
     * @param heur_ns the Namespace with the heuristics to be used
     * the heuristics of the current problem file will be added	
     */ 
    public Term parseProblemGivenNamespaces
	(Namespace func_ns, 
	Namespace sort_ns,
	Namespace heur_ns) 
	throws antlr.RecognitionException, antlr.TokenStreamException
	
	{
	// restore known state
	setSkips(false, false, false, false, false);

	// set namespaces
	this.func_ns = func_ns;
	this.sort_ns = sort_ns;
	this.heur_ns = heur_ns;
	// skip decls
	skipSorts();
	skipFuncs();
	skipPreds();	
	
	return problem();
    }

    public void addSort(Sort s) {
	sort_ns.add(s);
    }

    public void addSorts(Namespace ns) {
	IteratorOfNamed it=ns.elements().iterator();
	while (it.hasNext()) {
	    addSort((Sort)it.next());
	}
    }

    public void addFunction(Function f) {
	func_ns.add(f);
    }


    public void addFunctions(Namespace ns) {
	IteratorOfNamed it=ns.elements().iterator();
	while (it.hasNext()) {
	    addFunction((Function)it.next());
	}
    }

    public Namespace getSorts() {
	return sort_ns;
    }

    public Namespace getVariables() {
	return var_ns;
    }

    public Namespace getFunctions() {
	return func_ns;
    }

    public Namespace getHeuristics() {
	return heur_ns;
    }

    public NamespaceSet namespaces() {
	NamespaceSet ns = new NamespaceSet();
	ns.setVariables(getVariables());
	ns.setFunctions(getFunctions());
	ns.setSorts(getSorts());
	ns.setHeuristics(getHeuristics());
	return ns;
    }

    public SetOfTaclet getTaclets() {
	return taclets;
    }

    protected int getLine() {
	int line = -1;
	try {
	    line = LT(0).getLine()+lineOffset;
	} 
	catch (TokenStreamException e) {
	    System.err.println("No further token in stream");
	}
	return line;
    }   

    protected int getColumn() {
	int 	col = -1;
	try {
	    col = LT(0).getColumn()+colOffset;
	} 
	catch (TokenStreamException e) {
	    System.err.println("No further token in stream");
	}
	return col;
    }       

    protected TermSymbol progVar(Token id,Sort s) throws NotDeclException{
	TermSymbol v = (TermSymbol)
	    var_ns.lookup(new Name(id.getText()));

	if (v == null) {
		throw new NotDeclException("program variable", id,
						getFilename());
	}
	return v;
    }

    protected static HashSet progVars(JavaBlock jb) {
	if (jb==JavaBlock.EMPTY_JAVABLOCK) {
		return new HashSet();
	}	
       ProgramVariableCollector pvc
	  = new ProgramVariableCollector(jb.program());
	pvc.start();
	return pvc.result();
    }

    protected void bindProgVars(HashSet progVars) {
	var_ns=new Namespace(var_ns);
	Iterator it=progVars.iterator();
	while (it.hasNext()) {
	    var_ns.add((ProgramVariable)it.next());
	}
    }

    protected void unbindProgVars() {
	var_ns = var_ns.parent();
    }


    protected LogicVariable bindVar(Token id,Sort s) {
	LogicVariable v=new LogicVariable(new Name(id.getText()), s);
	var_ns=var_ns.extended(v);
	return v;
    }
	
    protected void unbindVars() {
	var_ns = var_ns.parent();
    }




    protected Term termForParsedVariable(ParsableVariable v) 
	throws antlr.SemanticException 
	{
	    if ( v instanceof LogicVariable ) {
		return tf.createVariableTerm((LogicVariable)v);
	    } else if ( allowSchemaVars
			&& v instanceof SchemaVariable ) {
		return tf.createVariableTerm((SchemaVariable)v);
	    } else if ( v instanceof ProgramVariable ) {
		return tf.createVariableTerm((ProgramVariable)v);
	    }
	    if ( allowSchemaVars ) {
		throw new antlr.SemanticException(
						  ""+ v +" is not a program or logic or schema variable");
	    } else {
		throw new antlr.SemanticException(
						  ""+ v +" is not a logic variable "
						  +"or program variable");
	    }
	}



    private TacletBuilder createTacletBuilderFor
                  (Object find, boolean sameUpdatePrefix) 
	throws InvalidFindException {
        if ( sameUpdatePrefix && !( find instanceof Term ) )
		throw new InvalidFindException
		    ("\"sameUpdateLevel\" may only be used for rewrite taclets:"+find,
		     getFilename(), getLine(), getColumn());
	if ( find == null ) {
	    return new NoFindTacletBuilder();
	} else if ( find instanceof Term ) {
	    return new RewriteTacletBuilder().setFind((Term)find)
	                     .setSameUpdatePrefix(sameUpdatePrefix);
	} else if ( find instanceof Sequent ) {
	    Sequent findSeq = (Sequent) find;
	    if ( findSeq == Sequent.EMPTY_SEQUENT ) {
		return new NoFindTacletBuilder();
	    } else if (   findSeq.antecedent().size() == 1
			  && findSeq.succedent().size() == 0 ) {
		Term findFma = findSeq.antecedent().get(0).formula();
		return new AntecTacletBuilder().setFind(findFma);
	    } else if (   findSeq.antecedent().size() == 0
			  && findSeq.succedent().size() == 1 ) {
		Term findFma = findSeq.succedent().get(0).formula();
		return new SuccTacletBuilder().setFind(findFma);
	    } else {
		throw new InvalidFindException
		    ("Unknown find-sequent (perhaps null?):"+findSeq,
		     getFilename(), getLine(), getColumn());	
	    }
	} else {
	    throw new IllegalArgumentException("createTacletBuilderFor called with"
				      		 + find.getClass().getName() + "object");
	}
    }	    



    private void addGoalTemplate(TacletBuilder b,
                                 String id,
				 Object rwObj,
				 Sequent addSeq,
				 ListOfTaclet addRList,
				 SetOfSchemaVariable pvs) 
	throws SemanticException
	{
	    TacletGoalTemplate gt = null;
	    if ( rwObj == null ) {
		// there is no replacewith, so we take
		gt = new TacletGoalTemplate(addSeq,
					  addRList,
					  NO_RULE_VARIABLES,
					  pvs);
	    } else {
		if ( b instanceof NoFindTacletBuilder ) {
		    // there is a replacewith without a find.
		    throw
			new UnfittingReplacewithException
			("Replacewith without find", getFilename(),
			 getLine(), getColumn());
		} else if ( b instanceof SuccTacletBuilder
			    || b instanceof AntecTacletBuilder ) {
		    if ( rwObj instanceof Sequent ) {
			gt = new AntecSuccTacletGoalTemplate(addSeq,
							   addRList,
							   NO_RULE_VARIABLES,
							   (Sequent)rwObj, pvs); 
		    } else {
			throw new UnfittingReplacewithException
			    ("Replacewith in a Antec-or SuccTaclet has "+
			     "to contain a sequent (not a term)", 
			     getFilename(), getLine(), getColumn());
		    }
		} else if ( b instanceof RewriteTacletBuilder ) {
		    if ( rwObj instanceof Term ) {
			gt = new RewriteTacletGoalTemplate(addSeq,
							 addRList,
							 NO_RULE_VARIABLES,
							 (Term)rwObj, pvs);  
		    } else {
			throw new UnfittingReplacewithException
			    ("Replacewith in a RewriteTaclet has "+
			     "to contain a term (not a sequent)", 
			     getFilename(), getLine(), getColumn());
		    }
		}
	    }
            gt.setName(id);
	    b.addTacletGoalTemplate(gt);
	}
							
	
    public static void main(String[] args) {
	ProblemParser p = 
           new ProblemParser(new KeYLexer(
              new BufferedReader(new InputStreamReader(System.in))));
	try {
	    Term fma = p.problem();
	    System.out.println("Parsed problem:\n"+fma);
	    System.out.println("Parsed Taclets:\n"+p.getTaclets());
	} catch (TokenStreamException e) {
	    System.out.println("Lexing error");
	    e.printStackTrace();
	} catch (RecognitionException e) {
	    System.out.println("Parsing error");
	    e.printStackTrace();
	}
    }


    protected JavaBlock getJavaBlock(Token t) throws antlr.SemanticException {
	String s=t.getText();
	de.uka.ilkd.key.java.JavaReader jr;
	try {
		if (schematicJava) {
		    jr = new de.uka.ilkd.key.java.SchemaRecoder2KeY
			(getJavaInfo().rec2key(),
			 getJavaInfo().getTypeConverter(), 
			 new NamespaceSet(var_ns, func_ns, sort_ns, heur_ns));
		    ((SchemaJavaReader)jr).setSVNamespace(var_ns);
		    return jr.readBlockWithEmptyContext(s);
		} else {
		    jr = new de.uka.ilkd.key.java.Recoder2KeY(
					(KeYCrossReferenceServiceConfiguration)
					getJavaInfo().getKeYProgModelInfo()
						.getServConf(),
					getJavaInfo().rec2key(),
					new NamespaceSet(var_ns, func_ns, 
							sort_ns, heur_ns),
					getJavaInfo().getTypeConverter());
		    return jr.readBlockWithProgramVariables(var_ns, s);	
		}
	} catch (de.uka.ilkd.key.java.PosConvertException e) {
		  lineOffset=e.getLine()-1;
		  colOffset=e.getColumn();
		  throw new JavaParserException(e.getMessage(), t, 
			getFilename(), lineOffset, colOffset);  		  
				
	} catch (de.uka.ilkd.key.java.ConvertException e) {
		e.printStackTrace(); ///
		if (s.length()>10) {
			s=s.substring(0,10)+"...";
		}
		e.printStackTrace(); ///
		if (e.parseException()!=null
		    &&	e.parseException().currentToken != null
		    &&	e.parseException().currentToken.next != null) {		  
		  lineOffset=e.parseException().currentToken.next.beginLine;
		  colOffset=e.parseException().currentToken.next.beginColumn;
		  e.parseException().currentToken.next.beginLine=getLine()-1;
		  e.parseException().currentToken.next.beginColumn=getColumn();
		  throw new JavaParserException(e.parseException().getMessage(), t, 
			getFilename(), -1, -1);  // row/columns already in text
		}		

		throw new JavaParserException(e.getMessage(), t, getFilename());
	}		
    }
}

top : {} a=problem;

problem returns [ Term a = null ]
{
   Taclet s = null;
   String st = null;if (capturer != null) capturer.mark();
}
   :  (st=java)
      {  }
      decls
      { var_ns.add(getJavaInfo().programVariables()); }
      (
         "rules" LBRACE
         { allowSchemaVars = true; 
           schematicJava=true;
                        }
         ( 
            s=taclet SEMI
            {
               try {
                  taclets=taclets.addUnique(s);
               } catch(de.uka.ilkd.key.collection.NotUniqueException e) {
                  throw new IllegalArgumentException(e.toString());
               }
            }
         )*
         RBRACE
      ) ?
      { allowSchemaVars = false; 
        schematicJava=false;
      }
      (PROBLEM {if (capturer != null) capturer.capture();} 
       LBRACE  a=formula RBRACE)?
   ;


java returns [String s = null]
	{StringBuffer b=new StringBuffer(); } :
	((JAVA LBRACE (l:STRING_LITERAL {b.append(l.getText());} 
			| SLASH {b.append("/");})+
	{s=b.toString();} RBRACE))?
;

proof [ProblemLoader prl] :
   ( PROOF LBRACE
        pseudosexpr[prl] 
     RBRACE
   )?
;

pseudosexpr [ProblemLoader prl] { char eid='0'; } :
   LPAREN RPAREN
   |
   LPAREN eid=expreid 
          (str:STRING_LITERAL { prl.beginExpr(eid,str.getText()); } )?
          ( pseudosexpr[prl] )* 
   { prl.endExpr(eid); }
   RPAREN
   
;

expreid returns [ char eid = '0' ]:
   FORMULA { eid='f'; }
   |
   TERM    { eid='t'; }
   |
   RULE    { eid= 'r'; }
   |
   BRANCH  { eid='b'; }
   |
   INST    { eid='i'; }
   |
   IFSEQFORMULA { eid='q'; }
   |
   INTERACTIVE	{ eid='a'; }
   |
   HEUR		{ eid='h'; }
   |
   IDENT
;
