// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

/* -*-antlr-*- */
header {

    package de.uka.ilkd.key.parser;

    import java.io.*;
    import antlr.*;
    import de.uka.ilkd.key.logic.*;
    import de.uka.ilkd.key.logic.sort.*;
    import de.uka.ilkd.key.logic.op.*;
    import de.uka.ilkd.key.rule.*;
    import de.uka.ilkd.key.gui.AbbrevMap;

    import de.uka.ilkd.key.rule.metaconstruct.*;
    import de.uka.ilkd.key.java.JavaReader;
    import de.uka.ilkd.key.java.JavaInfo;
    import java.util.Vector;
    import java.util.HashSet;
    import java.util.Iterator;
    import java.util.HashMap;
    import de.uka.ilkd.key.java.visitor.ProgramVariableCollector;
    import de.uka.ilkd.key.proof.UseInsteadMap;
    import de.uka.ilkd.key.java.Recoder2KeY;
    import de.uka.ilkd.key.java.Services;
    import de.uka.ilkd.key.java.recoderext.KeYCrossReferenceServiceConfiguration;
}

/** 
 * A parser for first-order terms and formulae.
 */  

class TermParser extends DeclParser;
options {
	importVocab=KeYLexer;
	k = 2;
	defaultErrorHandler=false;
}


{

    static final Sort[] AN_ARRAY_OF_SORTS = new Sort[0];
    static final Term[] AN_ARRAY_OF_TERMS = new Term[0];
    protected boolean minus=false;

    protected Namespace var_ns;
    protected Namespace func_ns;
    protected Namespace sort_ns;
    protected Namespace heur_ns;
    protected JavaInfo javaInfo;
    protected AbbrevMap scm;

    // these variables are set if a file is read in step by
    // step. This used when reading in LDTs because of cyclic
    // dependencies.
    protected boolean skip_functions;
    protected boolean skip_predicates;
    protected boolean skip_sorts;
    protected boolean skip_schemavariables;
    protected boolean skip_heuristics;

    protected int lineOffset=0;
    protected int colOffset=0;

    protected TermFactory tf;
    protected JavaReader javaReader;

    protected UseInsteadMap useInstead=new UseInsteadMap();	
    boolean schematicJava = false;	

    public boolean errors = false;


    public TermParser(TokenStream lexer,					
		      String filename,
		      TermFactory tf,
		      JavaReader jr,
		      Namespace var_ns,
		      Namespace func_ns,
		      Namespace sort_ns,
	              AbbrevMap scm) {
	this(lexer,2);
	setFilename(filename);
	this.javaReader = jr;
	this.tf = tf;
	this.var_ns = var_ns;
	this.func_ns = func_ns;
	this.sort_ns = sort_ns;
    	this.scm = scm;
    }

    public TermParser(TokenStream lexer,
		      String filename,
		      JavaReader jr,
		      Namespace var_ns,
		      Namespace func_ns,
		      Namespace sort_ns,
	              AbbrevMap scm) {

	this(lexer, filename, TermFactory.DEFAULT, jr, 
	     var_ns, func_ns, sort_ns, scm);
    }

    public TermParser(TokenStream lexer,
		      String filename,
		      TermFactory tf,
		      JavaInfo javaInfo,
		      Namespace var_ns,
		      Namespace func_ns,
		      Namespace sort_ns,
	              AbbrevMap scm) {

	this(lexer, filename, tf, getRecoder2KeY(javaInfo),	
	     var_ns, func_ns, sort_ns,scm);
    }

    static Recoder2KeY getRecoder2KeY(JavaInfo javaInfo) {
	 return new de.uka.ilkd.key.java.Recoder2KeY
		((KeYCrossReferenceServiceConfiguration)
			javaInfo.getKeYProgModelInfo().getServConf(),
		 javaInfo.rec2key(), javaInfo.getTypeConverter());	
    }

    public TermParser(TokenStream lexer,
		      String filename,
		      JavaInfo javaInfo,
		      Namespace var_ns,
		      Namespace func_ns,
		      Namespace sort_ns,
        	      AbbrevMap scm) {		

	this(lexer, filename, TermFactory.DEFAULT, getRecoder2KeY(javaInfo),
	     var_ns, func_ns, sort_ns, scm);
    }

    protected int getLine() {
	int line = -1;
	try {
	    line = LT(0).getLine()+lineOffset;
	} catch (TokenStreamException e) {
	    System.err.println("No further token in stream");
	}
	return line;
    }   


    public Term toZNotation(String s, boolean minus){
	Term[] t1=new Term[1];
	Operator v;
	Term a;
	v=(Function) func_ns.lookup(new Name("#"));
	t1[0] = tf.createFunctionTerm((Function)v);
	v = (Function) func_ns.lookup(new Name(s.substring(0,1)));
	a = tf.createFunctionTerm((Function)v,t1);
	for(int i=1;i<s.length();i++){
		t1[0]=a;
		v = (Function)func_ns.lookup(new Name(s.substring(i,i+1)));
		a = tf.createFunctionTerm((Function)v,t1);
	}
	if(minus){
		t1[0]=a;
		v = (Function) func_ns.lookup(new Name("neg"));
		a = tf.createFunctionTerm((Function)v,t1);
	}
	t1[0]=a;
	v = (Function) func_ns.lookup(new Name("Z"));
	a = tf.createFunctionTerm((Function)v,t1);
	return a;
    }

    protected int getColumn() {
	int 	col = -1;
	try {
	    col = LT(0).getColumn()+colOffset;
	} 
	catch (TokenStreamException e) {
	    System.err.println("No further token in stream");
	}
	return col;
    }   

    protected LogicVariable bindVar(Token id,Sort s) {
	LogicVariable v=new LogicVariable(new Name(id.getText()), s);
	var_ns=var_ns.extended(v);
	return v;
    }

    protected TermSymbol progVar(Token id,Sort s) throws NotDeclException{
	TermSymbol v = (TermSymbol)
	    var_ns.lookup(new Name(id.getText()));

	if (v == null) {
		throw new NotDeclException("program variable", id,
						getFilename());
	}
	return v;
    }


    /** returns the java information object */
    public JavaInfo getJavaInfo() {
	if (javaInfo==null) {
           javaInfo=javaReader.createJavaInfo(new Services());
        }
	return javaInfo;
    }


    protected boolean isNumber(String s){
	      boolean num=true;
          if(s==null){
            num=false;
          }else{
	        for(int i=s.length()-1;i>=0;i--){
		     if(s.charAt(i)<'0'||s.charAt(i)>'9')
			    num=false;
	        }
         }
	     return num;
    }


	
    protected void unbindVars() {
	var_ns = var_ns.parent();
    }

    protected void bindProgVars(HashSet progVars) {
	var_ns=new Namespace(var_ns);
	Iterator it=progVars.iterator();
	while (it.hasNext()) {
	    var_ns.add((ProgramVariable)it.next());
	}
    }

    protected void unbindProgVars() {
	var_ns = var_ns.parent();
    }

    protected static HashSet progVars(JavaBlock jb) {
	if (jb==JavaBlock.EMPTY_JAVABLOCK) {
		return new HashSet();
	}	
	ProgramVariableCollector pvc
	    = new ProgramVariableCollector(jb.program());
	pvc.start();
	return pvc.result();
    }


    protected Term termForParsedVariable(ParsableVariable v) 
	throws antlr.SemanticException 
	{

	    if ( v instanceof LogicVariable ) {
		return tf.createVariableTerm((LogicVariable)v);
	    } else if ( v instanceof ProgramVariable ) {
		return tf.createVariableTerm((ProgramVariable)v);
	    } else if ( v instanceof Metavariable ) {
		return tf.createFunctionTerm((Metavariable)v);
	    } else {
		throw new antlr.SemanticException(
						  ""+ v +" is not a logic variable");
	    }
	}

      

    protected JavaBlock getJavaBlock(Token t) throws antlr.SemanticException {
	String s=t.getText();
	try {
		return javaReader.readBlockWithProgramVariables(var_ns, s);
	} catch (de.uka.ilkd.key.java.PosConvertException e) {
		  lineOffset=e.getLine()-1;
		  colOffset=e.getColumn()+1;
		  throw new JavaParserException(e.getMessage(), t, 
			getFilename(), lineOffset, colOffset);  		  				
	} catch (de.uka.ilkd.key.java.ConvertException e) {	
		if (e.parseException()!=null) {		  		  
		  lineOffset=e.parseException().currentToken.next.beginLine;		  		 
		  colOffset=e.parseException().currentToken.next.beginColumn;
		  e.parseException().currentToken.next.beginLine=getLine()-1;
		  e.parseException().currentToken.next.beginColumn=getColumn();
		  throw new JavaParserException(e.parseException().getMessage(), t, 
			getFilename(), -1, -1);  // row/columns already in text
		}		
		throw new JavaParserException(e.getMessage(), t, getFilename(), 0, 1);
	}
    }

}

/**
 * In the special but important case of Taclets, we don't yet know
 * whether we are going to have a term or a formula, and it is hard
 * to decide a priori what we are looking at.  So the `term' 
 * non-terminal will recognize a term or a formula.  The `formula'
 * reads a formula/term and throws an error if it wasn't a formula.
 * This gives a rather late error message. */

top {Term t;} : t=formula;

formula returns [Term a = null] 
	:
		a = term 
		{
			if ( a.sort() != Sort.FORMULA ) {
				throw new antlr.SemanticException(
					"Just Parsed a Term where a Formula was expected");
			}
		}
	;


term returns [Term a = null] 
	:
		a=term20 
	;

term20 returns [Term a = null] {Term a1;}
	:   a=term30 
		(EQV a1=term20 
			{ a = tf.createJunctorTerm(Op.EQV,new Term[]{a,a1});} ) ?;
term30 returns [Term a = null] {Term a1;}
	:   a=term40 
		(IMP a1=term30 
			{ a = tf.createJunctorTerm(Op.IMP,new Term[]{a,a1});} ) ?;
term40 returns [Term a = null] {Term a1;}
	:   a=term50 
		(OR a1=term40 
			{ a = tf.createJunctorTerm(Op.OR, new Term[]{a,a1});} ) ?;
term50 returns [Term a = null] {Term a1;}
	:   a=term60 
		(AND a1=term50 
			{ a = tf.createJunctorTerm(Op.AND,new Term[]{a,a1});} ) ?;

term60 returns [Term a = null] 
{
    Term a1,a2;
	Sort s;
}
	: (LPAREN IDENT (COMMA IDENT COLON IDENT)? GUARD) => a=condterm
	|   a = term70
	|   a=bindingterm
	|   a=dl_term
	|   NOT a1=term60 { a = tf.createJunctorTerm(Op.NOT,new Term[]{a1}); }
	|   CTRL a1 = term60 { a = tf.createJunctorTerm(Op.CTRL,new Term[]{a1}); }

;


term70 returns [Term a = null] {Term a1;}
	:
        a=term80
		(EQUALS a1=term80
			{ a = tf.createEqualityTerm(a,a1);} ) ?;
	

term80 returns [Term a = null]
	:
     (IDENT (LBRACKET funcpredvarterm RBRACKET)* DOT IDENT)=> a=attrterm
    |   AT a = abbreviation
    |   ARITH_OP {minus = !minus; a=term80();}
    |   a = funcpredvarterm
    |   LPAREN a = term RPAREN
    |   "true" { a = tf.createJunctorTerm(Op.TRUE); }
    |   "false" { a = tf.createJunctorTerm(Op.FALSE); }
    ;
    exception // for rule
      catch [RecognitionException ex] {
        throw new RecognitionException("Not a valid logical term.\n"+ex.toString());
    }



abbreviation returns [Term a=null]
{ 
	String sc = null;
}
	:
	(	sc = id
		{
			a =  scm.getTerm(sc);
		    if(a==null){
				throw new NotDeclException
				("abbreviation", sc, 
				getFilename(), getLine(), getColumn());
			}
			
                
		}
)
        ;

condterm returns [Term a = null]
{ Term cond1_1 = null;
  Term cond1_2 = null;
  Term cond2_1 = null;
  Term cond2_2 = null;
  Term c_then = null;
  Term c_else = null;
  ParsableVariable v=null;
}
       : 
   (LPAREN term80 COMMA)=>(
	    LPAREN cond1_1=term80 COMMA cond1_2=term80 COLON c_else=term60 GUARD 
	cond2_1=term80 COMMA cond2_2=term80 COLON c_then=term60 RPAREN 
   {
	a = tf.createConjCondTerm(new Term[]{cond1_1, cond1_2, c_else, 
	  cond2_1, cond2_2, c_then});
   }
   )
   | LPAREN cond1_1=term80 GUARD cond2_1=term80 RPAREN 
    (

	DOT v = var_id CONDUP c_then = term60 {
		  a = tf.createIfElseTerm((ProgramVariable)v, cond1_1, 
					  cond2_1, c_then);
	        }	
	|
	c_then = term60 COLON c_else = term60 {
		  a = tf.createIfElseTerm(cond1_1, cond2_1, c_then, c_else);
		}
     ) 
;

/**
 * this is a hook that can be redefined by inheriting grammars to
 * introduce other sorts of binding behaviour for quantifiers or
 * substitution terms.  In particular, the Taclet parser will hook in
 * here to allow quantification of schema variables.
 */

bindingterm returns [Term a = null]
	:
    a = local_bindingterm             
    ;


/** here is the standard behaviour, that declares a new variable
  * with local scope in a quantifier */

local_bindingterm returns [Term a = null]
{
	Operator op = null;
	LogicVariable v = null;
	Term a1;
	Sort s;
}
	:
		(
			"all" { op = Op.ALL; }
		|   "ex"  { op = Op.EX;  }
		)

		v0:IDENT COLON s=sort_id DOT {v = bindVar(v0,s);} a1=term60
		{
			a = tf.createQuantifierTerm((Quantifier)op, v,a1);
			unbindVars();
		}
	| a = update_or_substitution
	;

     
        



update_or_substitution returns [Term result = null]
{
} :

    ( LBRACE IDENT COLON ) => result = substitutionterm |
                              result = updateterm
  ; 


substitutionterm returns [Term result = null]
{ Sort s;
  LogicVariable v = null;
  Term a1, a2;
  boolean waryAll = false;
  boolean waryEx = false;
} : 

	LBRACE v1:IDENT COLON s=sort_id a1=term
                     ( "waryAll" { waryAll = true; } |
                       "waryEx"  { waryEx  = true; } )?
    RBRACE 
        {v = bindVar(v1, s);} 
	a2=term60 { 
		    if ( waryAll )
		        result = tf.createSubstitutionTerm ( Op.WARY_SUBST_ALL,
		                                             v, a1, a2 );
		    else if ( waryEx )
		        result = tf.createSubstitutionTerm ( Op.WARY_SUBST_EX,
		                                             v, a1, a2 );
		    else
		        result = tf.createSubstitutionTerm ( Op.SUBST,
		                                             v, a1, a2 );
		    unbindVars();
	      	  }
;


updateterm returns [Term result = null]
{
  Term a0 = null;
  Term a1, a2;
  TermSymbol pv;
  Vector subs = new Vector();
  Vector index = new Vector();
  ArraySort as;
  int n=0;
} :
	LBRACE 
	  ((IDENT (LBRACKET funcpredvarterm RBRACKET)* DOT IDENT) => a0 = attrterm 
   |
	  v1:IDENT (LBRACKET a0=term80 RBRACKET {index.add(a0);n++;})*)
          ASSIGN a1=term60 {
				if(n>0){
			 		as=ArraySortImpl.getArraySort(a1.sort());
			    		for(int i=1;i<n;i++){
			        		as=ArraySortImpl.getArraySort(as);
			    		}
			    		pv = progVar(v1, as);
			    		a0 = tf.createFunctionTerm(pv);
			    		a0 = tf.createArrayTerm(Op.ARRAY,a0,(Term[])index.toArray(new Term[index.size()]));
			    		n=0;
					}else if (a0 == null){
						pv = progVar(v1,a1.sort());
						a0 = tf.createFunctionTerm(pv);
			     		}
			     
			     subs.add(a0); 
			     subs.add(a1); 
			   }
	  (COMMA a0 = term60 ASSIGN a1 = term60 { subs.add(a0); subs.add(a1); })*
	RBRACE a2=term60 {
		subs.add(a2);
		result = tf.createUpdateTerm((Term[])subs.toArray(new Term[subs.size()]));			
		subs = new Vector();
	   }
;


/**
 * this is a hook that can be redefined by inheriting grammars to
 * redefine the sorts of things that can appear inside a diamond or
 * box.  In particular, in Taclets, schema variables and pre/suffixes
 * might occur.
 */

dl_term returns [Term a = null]
	:
		a = normal_dl_term
	;


/** here is the standard behaviour, only accepts a program. */

normal_dl_term returns [Term a = null]
{	JavaBlock jb=null;
	Term a1;
}
    :
		d1:DIAMOND 
		{
			jb=getJavaBlock(d1);
			HashSet progVars=progVars(jb);
			bindProgVars(progVars);
		}
		        a1=term60	
		{
			a = tf.createDiamondTerm(jb, a1);
			unbindProgVars();
		}
	|
		d2:BOX 
		{
			jb=getJavaBlock(d2);
			HashSet progVars=progVars(jb);
			bindProgVars(progVars);
		}
		        a1=term60	
		{
			a = tf.createBoxTerm(jb, a1);
			unbindProgVars();
		}
    ;


attrterm returns [Term a = null]
{	Vector vec=new Vector();
	ParsableVariable v=null;
	Term t=null;
	ParsableVariable v0=null;
    Sort so=null;

} : (t=funcpredvarterm { so=t.sort();
        } (DOT^ v=attrid[so] {vec.add(v); if(v!=null) so=v.sort(); })+) {
	for (int i=0; i<vec.size(); i++) {
		if(vec.get(i)!=null){
			t=tf.createAttributeTerm((ProgramVariable)vec.get(i), t);
		}
	}
	a=t;
    };

attrid [Sort s] returns [ParsableVariable v = null]
{

} : attr:IDENT 
{
  v = getJavaInfo().getAttribute(attr.getText(), (ObjectSort)s);
  if ( v == null && !("length").equals(attr.getText()) ) {
     	throw new NotDeclException
		("Attribute ", attr.getText(),getFilename(), 
		  getLine(), getColumn());

  }
  
};

funcpredvarterm returns [Term a = null]
{
	Operator vf;
	int n = 0;
	Vector al = null;
	Term t;
	Sort as=null;
}
	:
		({
		String s=LT(0)+" ";
		s=LT(1)+" "; 
		s=LT(2)+" ";
		s=LT(3)+" ";        
		if(LT(1)!=null && isNumber(LT(1).getText())){
			if(LT(0)==null || LT(2)==null){
			    a=toZNotation(LT(1).getText(), minus);
			    minus=false;  		
			    match(IDENT);
			    return a;
			}else if(!LT(0).getText().equals("(")|| !LT(2).getText().equals("(")){ 
			    a=toZNotation(LT(1).getText(), minus);
			    minus=false;  		
			    match(IDENT);
			    return a;
			}
		}  
		}
		|		
		 CHAR_LITERAL {
				String s=LT(0).getText();
				int intVal=0;
				if (s.length()==3) {
					intVal=(int)LT(0).getText().charAt(1);
				} else {
					try {
					intVal = Integer.parseInt(s.substring(3,s.length()-1),16);
					} catch (NumberFormatException ex) {
					 throw new KeYSemanticException
					   ("Expected '"+s+"' to be a character", LT(0), getFilename());
					} 		
				}
				t=toZNotation(""+intVal,false);
		     		vf = (Function) func_ns.lookup(new Name("C"));
				return a=tf.createFunctionTerm((Function)vf, t.sub(0));
			      }
 		) 
		vf = varfunc_id
		(
			{ vf.arity() > 0 }?
				LPAREN 
				t=term 
				{
					al = new Vector();
					n=1;
					al.add(t);
				}
				( COMMA 
					t=term 
					{
						n++;
						al.add(t);
					}
				)* RPAREN	

                )?
                (
                 (LBRACKET)=>  
                   {vf instanceof ParsableVariable}? 
		   {a = termForParsedVariable((ParsableVariable)vf);}
		   {as = a.sort();} 
            {as instanceof ArraySort || as == ProgramSVSort.VARIABLE}?
                   (LBRACKET
                       t=term60
                       {
                         if (as!=null && as!=ProgramSVSort.VARIABLE){
                           if (!(as instanceof ArraySort)){
                                    throw new NotDeclException
                                    ("array "+vf.name()+" declared with a different"+
                                    " dimension than required here", vf.name().toString(), 
                                    getFilename(), getLine(), getColumn());
                           } else {
                                    as = ((ArraySort) as).elementSort();
                           } 
                         }
                         a=tf.createArrayTerm(Op.ARRAY,a,t);
                       }
                     RBRACKET)+                 
                |                
		 {  if (vf instanceof ParsableVariable) {
		      a = termForParsedVariable((ParsableVariable)vf);
		    } else if (vf instanceof Function) {
			if ( al == null ) {	
				a = tf.createFunctionTerm((Function)vf);
			} else {
				a = tf.createFunctionTerm((Function)vf,
					   (Term[])al.toArray(AN_ARRAY_OF_TERMS));
				}

		    } else if (vf instanceof MetaOperator) {

			if ( al == null ) {	
				a = tf.createMetaTerm((MetaOperator)vf, new Term[0]);
			} else {	
				a = tf.createMetaTerm((MetaOperator)vf,
					   (Term[])al.toArray(AN_ARRAY_OF_TERMS));
			}
		    }
		    if (minus) {
	              Term[] t1 = new Term[1];
        	      t1[0] = a;
               	      vf = (Function) func_ns.lookup(new Name("~m"));
                      a = tf.createFunctionTerm((Function)vf,t1);
                      minus = false;
            	    }	
	         }
		)
		;
		
	



varfunc_id returns [Operator v = null] 
{ 
	String varfunc_name = null;
}
	:
	(	v = expr_op	
		| varfunc_name = id
		{
			v = (TermSymbol) var_ns.lookup(new Name(varfunc_name));
			if (v == null) {
			   v = (Function) func_ns.lookup(new Name(varfunc_name));
			   if(v==null) {
                           	v = (ProgramVariable)
                                	getJavaInfo().programVariables().lookup(
                                        	new Name(varfunc_name));
                                if(v==null) {
					throw new NotDeclException
					("variable or function", varfunc_name, 
					getFilename(), getLine(), getColumn());
                                }
			   }
			}
                
		}
		| META_INT_UNARY_MINUS 
		{ v = Op.META_INT_UNARY_MINUS ; }
		| META_LONG_UNARY_MINUS 
		{ v = Op.META_LONG_UNARY_MINUS ; }
		| META_ADD 
		{ v = Op.META_ADD; }
		| META_SUB
		{ v = Op.META_SUB; }
		| META_MUL
		{ v = Op.META_MUL; }
		| META_DIV 
		{ v = Op.META_DIV; }
		| META_MOD
		{ v = Op.META_MOD; }
		| META_LESS
		{ v = Op.META_LESS; }
		| META_GREATER
		{ v = Op.META_GREATER; }
		| META_LEQ
		{ v = Op.META_LEQ; }
		| META_GEQ
		{ v = Op.META_GEQ; }
		| META_EQ
		{ v = Op.META_EQ; }
		| META_INT_ADD 
		{ v = Op.META_INT_ADD; }
		| META_INT_SUB 
		{ v = Op.META_INT_SUB; }
		| META_INT_MUL 
		{ v = Op.META_INT_MUL; }
		| META_INT_DIV 
		{ v = Op.META_INT_DIV; }
		| META_INT_MOD 
		{ v = Op.META_INT_MOD; }
		| META_INT_AND 
		{ v = Op.META_INT_AND; }
		| META_INT_OR
		{ v = Op.META_INT_OR; }
		| META_INT_XOR
		{ v = Op.META_INT_XOR; }
		| META_INT_COMPLEMENT
		{ v = Op.META_INT_COMPLEMENT; }
		| META_INT_SHIFTRIGHT
		{ v = Op.META_INT_SHIFTRIGHT; }
		| META_INT_SHIFTLEFT
		{ v = Op.META_INT_SHIFTLEFT; }
		| META_INT_UNSIGNEDSHIFTRIGHT
		{ v = Op.META_INT_UNSIGNEDSHIFTRIGHT; }
		| META_LONG_ADD 
		{ v = Op.META_LONG_ADD; }
		| META_LONG_SUB 
		{ v = Op.META_LONG_SUB; }
		| META_LONG_MUL 
		{ v = Op.META_LONG_MUL; }
		| META_LONG_DIV 
		{ v = Op.META_LONG_DIV; }
		| META_LONG_MOD 
		{ v = Op.META_LONG_MOD; }
		| META_LONG_AND 
		{ v = Op.META_LONG_AND; }
		| META_LONG_OR
		{ v = Op.META_LONG_OR; }
		| META_LONG_XOR
		{ v = Op.META_LONG_XOR; }
		| META_LONG_COMPLEMENT
		{ v = Op.META_LONG_COMPLEMENT; }
		| META_LONG_SHIFTRIGHT
		{ v = Op.META_LONG_SHIFTRIGHT; }
		| META_LONG_SHIFTLEFT
		{ v = Op.META_LONG_SHIFTLEFT; }
		| META_LONG_UNSIGNEDSHIFTRIGHT
		{ v = Op.META_LONG_UNSIGNEDSHIFTRIGHT; }
		| META_IN_LONG 
		{ v = Op.META_IN_LONG; }
		| META_IN_INT
		{ v = Op.META_IN_INT; }
		| META_IN_CHAR
		{ v = Op.META_IN_CHAR; }
		| META_IN_SHORT
		{ v = Op.META_IN_SHORT; }
		| META_IN_BYTE 
		{ v = Op.META_IN_BYTE; }
		| META_MODULO_BYTE
		{ v = Op.META_MODULO_BYTE; }
		| META_MODULO_SHORT
		{ v = Op.META_MODULO_SHORT; }
		| META_MODULO_INTEGER
		{ v = Op.META_MODULO_INTEGER; }
		| META_MODULO_LONG 
		{ v = Op.META_MODULO_LONG; }	
		| STATIC_ANALYSIS
		{ v = Op.STATIC_ANALYSIS; }
		| META_LENGTH
		{ v = Op.META_LENGTH; })
	;


expr_op returns [Operator v = null] 
{
	ParsableVariable left_var0 = null;
	ParsableVariable right_var0 = null;
	ParsableVariable unary_var0 = null;
}	
	:

	IN_TYPE LPAREN ((left_var0=var_id ((id:ARITH_OP | id1:SLASH | id2:IDENT) right_var0 = var_id)?) |
			   ( "-" unary_var0 = var_id) )  RPAREN
	{  
           if (id==null) id=id1;	
	   if (unary_var0!=null) {
	      // can only be unary minus
	      SchemaVariable unary_var = (SchemaVariable) unary_var0;
	      Sort type = de.uka.ilkd.key.logic.sort.Sort.FORMULA;
	      return new InTypeOperator
		(type, new de.uka.ilkd.key.java.expression.operator.Negative(unary_var));
	   }
	   else {
	      SchemaVariable left_var = (SchemaVariable) left_var0;
	      Sort type = de.uka.ilkd.key.logic.sort.Sort.FORMULA;
	      if (right_var0 != null) {
	         SchemaVariable right_var = (SchemaVariable) right_var0;
    	         String op_str = (id == null ? id2.getText() : id.getText());
           	   if ("+".equals(op_str)) {
		     return new InTypeOperator
		       (type, new de.uka.ilkd.key.java.expression.operator.Plus(left_var, right_var));
	         } else if ("-".equals(op_str)) {
		     return new InTypeOperator
		       (type, new de.uka.ilkd.key.java.expression.operator.Minus(left_var, right_var));
	         } else if ("*".equals(op_str)) {
		     return new InTypeOperator
		       (type, new de.uka.ilkd.key.java.expression.operator.Times(left_var, right_var));
	         } else if ("/".equals(op_str)) {
	    	     return new InTypeOperator
		       (type, new de.uka.ilkd.key.java.expression.operator.Divide(left_var, right_var));
		} else if ("%".equals(op_str)) {
	    	     return new InTypeOperator
		       (type, new de.uka.ilkd.key.java.expression.operator.Modulo(left_var, right_var));
  	         } 
	      }
	      else {
	         return new InTypeOperator(type, left_var);
	      }
 	   }
	}
	| IS_OBJECT  LPAREN (left_var0=var_id )  RPAREN
	{
 	   SchemaVariable var = (SchemaVariable) left_var0;
	   return new IsObject(de.uka.ilkd.key.logic.sort.Sort.FORMULA, var);
	}
	| IS_CLONABLE  LPAREN (left_var0=var_id )  RPAREN
	{
 	   SchemaVariable var = (SchemaVariable) left_var0;
	   return new IsClonable(de.uka.ilkd.key.logic.sort.Sort.FORMULA, var);
	}
;

var_id returns [ParsableVariable v = null]
	:
		id:IDENT 
		{   
			v = (ParsableVariable) var_ns.lookup(new Name(id.getText()));
			if (v == null) {
				throw new NotDeclException("variable", id, 
							   getFilename());
			}
		}
	;


