// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.proof;

import de.uka.ilkd.key.logic.PosInOccurrence;
import de.uka.ilkd.key.logic.Constraint;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.rule.*;

public class BuiltInRuleAppIndex implements java.io.Serializable {

    private BuiltInRuleIndex index;
    
    public BuiltInRuleAppIndex(BuiltInRuleIndex index) {
	this.index = index;
    }
    
    
    /** 
     * returns a list of built-in rules application applicable
     * for the given goal and position
     */
    public ListOfRuleApp getBuiltInRule(Goal            goal, 
					PosInOccurrence pos, 
					Constraint      userConstraint) {

	ListOfRuleApp result = SLListOfRuleApp.EMPTY_LIST;
	IteratorOfBuiltInRule it = index.rules().iterator();

	while (it.hasNext()) {
	    BuiltInRule bir = it.next();
	    result = bir.isApplicable(goal, pos, userConstraint) ?
		result.prepend(new BuiltInRuleApp(bir, pos, userConstraint)) :
		result;
	}

	return result;
    }


    /** 
     * returns a copy of this index
     */
    public BuiltInRuleAppIndex copy() {
	return new BuiltInRuleAppIndex(index.copy());
    }

}
