// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.proof;

import javax.swing.DefaultComboBoxModel;
import de.uka.ilkd.key.gui.AbbrevMap;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.parser.*;
import de.uka.ilkd.key.java.JavaInfo;
import java.io.*;
import de.uka.ilkd.key.rule.RuleApp;
import de.uka.ilkd.key.rule.TacletApp;
import de.uka.ilkd.key.rule.inst.*;
import de.uka.ilkd.key.rule.IfFormulaInstantiation;
import de.uka.ilkd.key.rule.IfFormulaInstDirect;
import de.uka.ilkd.key.rule.ListOfIfFormulaInstantiation;
import de.uka.ilkd.key.rule.IteratorOfIfFormulaInstantiation;


public class IfChoiceModel extends DefaultComboBoxModel {


    private static final String manualText="Manual Input";
    private String manualInput;
    //    private RuleApp app;
    private Term ifFma;

    private ListOfIfFormulaInstantiation candidates;
    

   /** namespace of variables */
    private Namespace var_ns;
    /** namespace of functions */
    private Namespace func_ns;
    /** namespace of sorts */
    private Namespace sort_ns;
    /** map containing shortcuts for terms*/
	private AbbrevMap scm;
    /** the java information object */
    private JavaInfo javaInfo;


    public IfChoiceModel ( Term                         p_ifFma,
			   ListOfIfFormulaInstantiation p_candidates,
			   JavaInfo                     p_javaInfo,
			   Namespace                    p_var_ns,
			   Namespace                    p_func_ns, 
			   Namespace                    p_sort_ns ,
			   AbbrevMap                  scm) {
	super ( createIfInsts ( p_candidates ) );
	
	ifFma       = p_ifFma;
	candidates  = p_candidates;
	javaInfo    = p_javaInfo;
	var_ns      = p_var_ns;
	func_ns     = p_func_ns;
	sort_ns     = p_sort_ns;
	this.scm    = scm;

	addElement(manualText);
	manualInput = "";
    }


    public String manualText() {
	return manualText;
    }

    public void setInput(String s) {
	manualInput = s;
    }

    public Term ifFma() {
	return ifFma;
    }
    

    public static Object[] createIfInsts ( ListOfIfFormulaInstantiation p_candidates ) {
	Object[]                         res = new Object [ p_candidates.size () ];
	IteratorOfIfFormulaInstantiation it  = p_candidates.iterator ();
	int                              i   = 0;

	while ( it.hasNext () )
	    res[i++] = it.next ();

	return res;
    }


    /** creates a new Termparser that parses the given string
     * @param s the String to parse 
     */
    private TermParser stringParser(String s) {
	return new TermParser(new KeYLexer(new StringReader(s)),
			      "", TermFactory.DEFAULT, javaInfo,
			      var_ns, func_ns, sort_ns, scm);
    }


    /** parses the given string that represents a term
     * @param s the String to parse 
     */
    public Term parseFormula(String s) throws antlr.ANTLRException {
	TermParser p = stringParser(s);
	return p.formula();
    }


    /**
     * @param pos int describing the position of the if-sequent 
     *   (only required for error message)
     */
    public IfFormulaInstantiation getSelection(int pos) 
	throws SVInstantiationParserException, MissingInstantiationException {
	Object o = getSelectedItem();
	if (o != manualText) {
	    return (IfFormulaInstantiation)o;
	}
	try {
 	    if (manualInput == null || "".equals(manualInput)) {
		throw new MissingInstantiationException("'if'-formula: " + ifFma, pos, -1, true);
	    }

	    return new IfFormulaInstDirect ( new ConstrainedFormula ( parseFormula(manualInput),
								      Constraint.BOTTOM ) );
	} catch (antlr.RecognitionException are) {
 	    throw new SVInstantiationParserException
 		( manualInput, pos+are.getLine(), are.getColumn(), 
		  "Problem occured parsing a manual input"
 		  + " of an 'if'-sequent.\n" +  are.getMessage(), true);
	} catch (antlr.ANTLRException e) {
 	    throw new SVInstantiationParserException
 		( manualInput, pos, -1, "Problem occured parsing a manual input"
 		  +" of an 'if'-sequent.\n" +  e.getMessage(), true);
 	} 
    }

}
