// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.proof.init;

import de.uka.ilkd.key.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.proof.*;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.parser.*;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.Recoder2KeY;
import de.uka.ilkd.key.casetool.*;
import de.uka.ilkd.key.ocl.ocldltranslator.detailedocl.*;
import de.uka.ilkd.key.cspec.*;
import de.uka.ilkd.key.gui.ProofSettings;
import java.io.*;
import java.util.*;

/**
 * Represents a proof obligation originating from a request of
 * computing the specification of a method as input for the prover.
 * @author Andr&eacute; Platzer
 * @version 1.0, 2003-02-24
 * @see de.uka.ilkd.key.cspec.ComputeSpecification
 */
public class ComputeSpecificationPO extends OCLPostPO {

    //@xxx duplicate data storage here and in super class. But still keep private
    private ReprModelMethod methRepr;
    private String precondition;
    private String postcondition;
    private String xmifile;

    /**
     * creates a new representation of a proof obligation from a
     * request of computing the specification.
     * @param name the name used to identify the proof obligation
     * @param aMethRepr the representation of a model method
     * originating this proof obligation. That is the method to
     * compute a specification for.
     * @param precondition the precondition already specified for the
     * method (or "true" if none).
     * @param postcondition the precondition already specified for the
     * method (or "true" if none).
     * @param xmiFileName The name of an XMI file name that can be
     * used to read the underlying model.
     */
    public ComputeSpecificationPO(String name, ReprModelMethod aMethRepr,
		     String precondition, 
		     String postcondition, 
		     String xmiFileName) {
	super(name, aMethRepr, precondition, postcondition, xmiFileName);
	this.methRepr=aMethRepr;
	this.precondition=precondition;
	this.postcondition=postcondition;
	this.xmifile=xmiFileName;
    }

    /**
     * reads the proof obligation, i.e., invokes the OCL-DL-translation on
     * the OCL constraint of this OCL proof obligation.
     */
    public void read(ModStrategy mod) {
	final InitConfig initConfig = getInitConfig();
	if (initConfig==null) {
	    throw new IllegalStateException("KeYFile: InitConfig not set.");
	}
	Debug.out("1) program variables are", initConfig.javaInfo().programVariables());
	Debug.out("1) variables are", initConfig.varNS());
	
	final ExcerptMethodCallMod excerpt = new ExcerptMethodCallMod();
	{
	    //@internal the following lines will read the Java sources, thus extract programVariables and inform excerpt of the method call. @xxx is there a better way?
	    OCLMod mods[] = ProofSettings.THE_SETTINGS
		.getOCLRefinementSettings().getOCLMods();
	    final List l = new ArrayList(mods.length + 1);
	    l.add(excerpt);
	    l.addAll(Arrays.asList(mods));
	    mods = (OCLMod[]) l.toArray(new OCLMod[0]);
	    final Object forgetAbout = generateMethodPO(initConfig.createFacadeForTransl(),
							mods,
							methRepr, 
							"prepost",
							precondition, 
							postcondition, xmifile);
	}

	Debug.out("2) program variables are", initConfig.javaInfo().programVariables());
	Debug.out("2) variables are", initConfig.varNS());

	final JavaBlock methodCallJavaBlock = excerpt.getMethodCall();
	final Term resultTerm = new ComputeSpecification()
	    .createSpecificationComputationTerm(methodCallJavaBlock, initConfig.javaInfo().programVariables());
	//@todo resultTerm = precondition -> resultTerm (translating away OCL)
	setProofObligation(resultTerm);
    }

}
