// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.proof.init;


/** represents an entity read to produce an environment to read a proof
 * obligation. Environment means the initial configuration of a prover 
 * containing namespaces and Java model.
 */
public interface EnvInput {

    /** returns the initial configuration that has been the result of 
     * reading the input.
     */
    public InitConfig getInitConfig();

    /** sets the initial configuration the read environment input should be
     * added to.
     */
    public void addTo(InitConfig i);

    /** reads the input using the given modification strategy, i.e.,
     * parts of the input do not modify the initial configuration while
     * others do.
     */
    public void read(ModStrategy mod) throws ProofInputException;

    /** returns the name of this input
     */
    public String name();

    /** returns the total numbers of chars that can be read in this input.
     */
    public int getNumberOfChars();

}
