// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.proof.init;

import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.proof.*;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.parser.*;
import de.uka.ilkd.key.java.JavaInfo;
import java.io.*;
import de.uka.ilkd.key.gui.Main;
import java.util.Vector;
import de.uka.ilkd.key.java.Recoder2KeY;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.Services;


/** represents an input from a .key user problem file producing environment
 *  as well as a proof obligation.
 */
public class KeYUserProblemFile extends KeYFile {

    private Term problemTerm = null;
    private String problemHeader = "";


    /** creates a new representation of a KeYSortFile with the given name,
     * a rule source representing the physical source of the input, and
     * a graphical representation to call back in order to report the progress
     * while reading.
     */
    public KeYUserProblemFile(String name, File file, Main main) {
	super(name, file, main);
    }


    /** reads the whole .key file and modifies the initial configuration
     * assigned to this object according to the given modification strategy. 
     * Throws an exception if no initial configuration has been set yet.
     */
    public void read(ModStrategy mod) throws ProofInputException {
	if (initConfig==null) {
	    throw new IllegalStateException("KeYFile: InitConfig not set.");
	}
	try {
	    //	    Services copy=initConfig.getServices().copy();
	    Services copy=initConfig.getServices();
	    CountingBufferedInputStream cinp = 
		new CountingBufferedInputStream
		    (getNewStream(),main,getNumberOfChars()/100);
            DeclPicker lexer = new DeclPicker(new KeYLexer(cinp));
	    problemParser = new ProblemParser(lexer, copy.getJavaInfo(), 
	        initConfig.namespaces().copy());
                
            problemTerm = problemParser.problem();
	    initConfig.add(problemParser.namespaces(), mod);

            problemHeader = lexer.getText();
            problemHeader = 
               problemHeader.substring(0, problemHeader.lastIndexOf("problem"));
            
	    if (mod.modifyTaclets()) initConfig.add(problemParser.getTaclets());
	    if (mod.modifyJavaInfo()) initConfig.setServices(copy);
	} catch (antlr.ANTLRException e) {
	    throw new ProofInputException(e);
        }
    }
    

    public Proof[] getPO() {
        if (problemTerm == null) throw new IllegalStateException(
           "KeYUserProblemFile: getPO() without read()");
	return new Proof[] {
            new Proof(name(), problemTerm, problemHeader,
                      initConfig.createTacletIndex(), 
		      initConfig.createBuiltInRuleIndex())
        };
    }
    
}
