// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.proof.init;

import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.proof.*;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.parser.*;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.Recoder2KeY;
import de.uka.ilkd.key.casetool.*;
import de.uka.ilkd.key.TogetherFacade;
import de.uka.ilkd.key.casetool.together.*;
import de.uka.ilkd.key.ocl.tudresden.TUDresdenParserAdapter;
import de.uka.ilkd.key.ocl.ocldltranslator.*;
import de.uka.ilkd.key.ocl.ocldltranslator.detailedocl.*;
import de.uka.ilkd.key.gui.ProofSettings;
import tudresden.ocl.*;

import java.io.*;

public class OCLInvInhMethPO extends OCLProofOblInput {

    private ReprModelMethod currOpRepr;
    private String preText;
    private String postText;
    private String invParentText;
    private String invText;
    private String xmifile;

    private Term prePO  = null;
    private Term postPO = null;


    public OCLInvInhMethPO(ReprModelMethod aMethRepr, String xmifile, 
			   String preText, String invText, 
			   String invParentText, String postText) {	
	super("BehavioralSubtyping "+aMethRepr.getName());
	this.currOpRepr=aMethRepr;
	this.xmifile=xmifile;
	this.invParentText=invParentText;
	this.preText=preText;
	this.postText=postText;
	this.invText=invText;
    }


    public void readJava() {
	if (initConfig==null) {
	    throw new IllegalStateException("InitConfig not set.");
	}
	Recoder2KeY r2k = new Recoder2KeY
	    (initConfig.javaInfo().getKeYProgModelInfo().getServConf(), 
	     null, "",  
	     initConfig.javaInfo().rec2key(), 
	     initConfig.javaInfo().getTypeConverter(),
	     initConfig.namespaces());
	String[] cus = currOpRepr.getContainingClass().getClassesInPackage();
	r2k.readCompilationUnitsAsFiles(cus);
    }

    public void read(ModStrategy mod) {
	if (initConfig==null) {
	    throw new IllegalStateException("InitConfig not set.");
	}
	TogetherFacade po2prover = initConfig.createFacadeForTransl();
	prePO = generateMethodPO(
	    po2prover, currOpRepr, "purepre", "(" 
	    + FunctionalityOnModel.normalizeConstraint(preText) + " and " 
	    + FunctionalityOnModel.normalizeConstraint(invText) + ") implies " 
	    + FunctionalityOnModel.normalizeConstraint(invParentText), "", 
	    xmifile);
	postPO = generateMethodPO(
	    po2prover, currOpRepr, "prepostaspl", 
	    FunctionalityOnModel.normalizeConstraint(preText) + " and " 
	    + FunctionalityOnModel.normalizeConstraint(invText),  "(" 
	    + FunctionalityOnModel.normalizeConstraint(postText) + " and " 
	    + FunctionalityOnModel.normalizeConstraint(invParentText) 
	    + ") implies " + 
	    FunctionalityOnModel.normalizeConstraint(invText), xmifile);
    }


    public Proof[] getPO() {
        if (prePO == null) throw new IllegalStateException("No prePO term");
        if (postPO == null) throw new IllegalStateException("No postPO term");
        String s = createProofHeader(currOpRepr.getContainingClass()
				     .getRootDirectory());
        return new Proof[] {
           new Proof("pre conditions "+currOpRepr.getName(), prePO, s,
                     initConfig.createTacletIndex(),
		     initConfig.createBuiltInRuleIndex()),
           new Proof("post conditions "+currOpRepr.getName(), postPO, s,
                     initConfig.createTacletIndex(),
		     initConfig.createBuiltInRuleIndex())
        };
    }

}
