// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.proof.init;

import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.proof.*;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.parser.*;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.Recoder2KeY;
import java.io.*;
import de.uka.ilkd.key.TogetherFacade;
import de.uka.ilkd.key.casetool.together.*;
import de.uka.ilkd.key.casetool.*;
import de.uka.ilkd.key.ocl.tudresden.TUDresdenParserAdapter;
import de.uka.ilkd.key.ocl.ocldltranslator.*;
import de.uka.ilkd.key.ocl.ocldltranslator.detailedocl.*;
import de.uka.ilkd.key.gui.ProofSettings;
import tudresden.ocl.*;


/** represents an OCL invariant proof obligation as input to the prover.
 */
public abstract class OCLInvPO extends OCLProofOblInput {

    protected String xmifile;
    protected ReprModelClass aClRepr;

    /** creates a new OCL invariant proof obligation having a given name,
     * a xmifile to read the model from, and a representation of a 
     * model class where the OCL proof obligation has been initiated.
     */
    public OCLInvPO(String name, String xmifile, ReprModelClass aClRepr) {
	super(name);
	this.xmifile=xmifile;
	this.aClRepr=aClRepr;
    }

    /** reads the underlying Java model for the generated DL proof obligation.
     */
    public void readJava() {
	if (initConfig==null) {
	    throw new IllegalStateException("InitConfig not set.");
	}
	Recoder2KeY r2k = new Recoder2KeY
	    (initConfig.javaInfo().getKeYProgModelInfo().getServConf(), 
	     null, "",  
	     initConfig.javaInfo().rec2key(), 
	     initConfig.javaInfo().getTypeConverter(),
	     initConfig.namespaces());
	String[] cus = aClRepr.getClassesInPackage();
	r2k.readCompilationUnitsAsFiles(cus);
    }




}
