// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.proof.init;

import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.proof.*;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.parser.*;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.Recoder2KeY;
import de.uka.ilkd.key.casetool.*;
import java.io.*;

/** represents a proof obligation originating from an OCL pre/post constraint
 * as input for the prover.
 */
public class OCLPostPO extends OCLProofOblInput {

    private ReprModelMethod methRepr;
    private String poPreText;
    private String poPostText;
    private String xmifile;
    
    private Term proofObl;

    /** creates a new representation of a proof obligation from an OCL 
     * constraint with the given name, originating from a representation 
     * of a model method and having the given Strings as text of the pre
     * condition and of the post condition. The given XMI file name can be used 
     * to read the underlying model.
     */
    public OCLPostPO(String name, ReprModelMethod aMethRepr,
		     String poPreText, 
		     String postText, 
		     String xmifile) {
	super(name+" "+aMethRepr.getName());
	this.methRepr=aMethRepr;
	this.poPreText=poPreText;
	this.poPostText=postText;
	this.xmifile=xmifile;
    }

    /** reads the underlying Java model and modifies the initial configuration
     * of this object. If the latter is not yet defined an exception is
     * thrown.
     */
    public void readJava() {
	if (initConfig==null) {
	    throw new IllegalStateException("InitConfig not set.");
	}
	Recoder2KeY r2k = new Recoder2KeY
	    (initConfig.javaInfo().getKeYProgModelInfo().getServConf(), 
	     null, "",  
	     initConfig.javaInfo().rec2key(), 
	     initConfig.javaInfo().getTypeConverter(),
	     initConfig.namespaces());
	String[] cus = methRepr.getContainingClass().getClassesInPackage();
	r2k.readCompilationUnitsAsFiles(cus);
	Debug.out("Initialize JavaInfo");
    }

    /**
     * Set the result of reading the OCL input.
     */
    protected final void setProofObligation(Term theResultingProofObligation) {
	this.proofObl = theResultingProofObligation;
    }


    /** reads the proof obligation, i.e., invokes the OCL-DL-translation on
     * the OCL constraint of this OCL proof obligation.
     */
    public void read(ModStrategy mod) {
	if (initConfig==null) {
	    throw new IllegalStateException("InitConfig not set.");
	}
        setProofObligation(generateMethodPO(initConfig.createFacadeForTransl(), 
                                    methRepr, "prepost", poPreText, poPostText, 
                                    xmifile)
			   );
    }


    public Proof[] getPO() {
        if (proofObl == null) throw 
           new IllegalStateException("No proofObl term");
        String s = createProofHeader(methRepr.getContainingClass()
				     .getRootDirectory());
	return new Proof[] {
           new Proof(name(), proofObl, s, 
		     initConfig.createTacletIndex(),
		     initConfig.createBuiltInRuleIndex())
        };
    }


}
