// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.proof.init;

import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.proof.*;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.parser.*;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.Recoder2KeY;
import de.uka.ilkd.key.casetool.*;
import java.io.*;
import de.uka.ilkd.key.TogetherFacade;
import de.uka.ilkd.key.casetool.together.*;
import de.uka.ilkd.key.ocl.tudresden.TUDresdenParserAdapter;
import de.uka.ilkd.key.ocl.ocldltranslator.*;
import de.uka.ilkd.key.ocl.ocldltranslator.detailedocl.*;
import de.uka.ilkd.key.gui.ProofSettings;
import tudresden.ocl.*;

/** represents an OCL proof obligation input to the prover for proving the 
 * structural subtyping property.
 */
public class OCLStructSubPO extends OCLInvPO {

    private String invTextParent;
    private String invText;
    
    private Term proofObl = null;

    /** creates a new OCL proof obligation input representing a structural
     * subtyping property proof obligation using the given name of the PO, 
     * an XMI file name for reading the UML model from, a representation 
     * of the model class the PO is invoked from and two strings, one with
     * the OCL invariant of the superclass and one with that of the subclass.
     */
    public OCLStructSubPO(String name, String xmifile, ReprModelClass aClRepr,
			  String invTextParent, 
			  String invText) {
	super(name, xmifile, aClRepr);
	this.invText=invText;
	this.invTextParent=invTextParent;
    }


    /** reads the proof obligation, i.e., invokes the OCL-DL-Translation.
     * The initial configuration is modified according to the given 
     * modification strategy.
     */
    public void read(ModStrategy mod) throws ProofInputException {
	if (initConfig==null) {
	    throw new IllegalStateException("InitConfig not set.");
	}

	// generating proofobligation
	//      invText -> invTextParent
	TUDresdenParserAdapter aAdapter = new TUDresdenParserAdapter();

	OclTree invPOAsTree 
	    = aAdapter.doParse1
	    (aClRepr.getClassName() + " invPO",  
	     xmifile, "context "+aClRepr.getClassName() 
	     + " inv: " 
	     + FunctionalityOnModel.normalizeConstraint(invText) 
	     + " implies " 
	     + FunctionalityOnModel.normalizeConstraint(invTextParent),
	     aClRepr.getContainingPackage());

	TogetherFacade po2prover;
	OCLTreeToDLAggregator aOCLTreeToDLAggregator;
	po2prover = initConfig.createFacadeForTransl();
	OCLMod[] mods=ProofSettings.THE_SETTINGS.
	    getOCLRefinementSettings().getOCLMods();
	aOCLTreeToDLAggregator =  getAggregator(po2prover, mods, null);
	if ( !aAdapter.getErrorOccurred()){
	    proofObl = aOCLTreeToDLAggregator.
		treeToTerm(invPOAsTree, true, true);
	} else {
	    throw new ProofInputException(
               "Invariants are syntactically incorrect");
	}
    }


    public Proof[] getPO() {
        if (proofObl == null) throw 
           new IllegalStateException("No proofObl term");
        String s = createProofHeader(aClRepr.getRootDirectory());
	return new Proof[] {
           new Proof(name(), proofObl, s, 
		     initConfig.createTacletIndex(),
		     initConfig.createBuiltInRuleIndex())
        };
    }

}
