// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.proof.init;

import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.proof.Proof;


/**
 * Represents something that produces an input proof obligation for the 
 * prover. A .key file or a proof obligation generated from a CASE tool may 
 * be instances. An instance can be called to set an initial configuration,
 * that is modified during reading the input (using <code>setConfig</code>), 
 * to read the Java model 
 * belonging to the input (using <code>readJava</code>), to read the rest 
 * of the input (using <code>read</code>), and to get the read proof 
 * obligation (using <code>getProblemTerm</code>). During reading the input
 * given initial configuration may become modified. The modification
 * can be controlled by a modification strategy.
 */
public interface ProofOblInput {

    /** returns the proof obligation term as result of the proof obligation
     * input. If there is still no input available because nothing has
     * been read yet null is returned.
     */
    public Proof[] getPO();

    /** starts reading the input and modifies the InitConfig of this 
     * object with respect to the given modification
     * strategy. 
     */
    public void read(ModStrategy mod) throws ProofInputException;

    /** reads the Java model that belongs to this input.
     */
    public void readJava() throws ProofInputException;

    /** set the initial configuration used to read an input. It may become
     * modified during reading depending on the modification strategy used
     * for reading.
     */
    public void setConfig(InitConfig i);
    
    /** returns the name of the proof obligation input.
     */
    public String name();

}
