// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.rule;

import de.uka.ilkd.key.logic.IteratorOfConstrainedFormula;
import de.uka.ilkd.key.logic.ConstrainedFormula;
import de.uka.ilkd.key.logic.Sequent;
import de.uka.ilkd.key.logic.Semisequent;


/**
 * Instantiation of an if-formula that is a formula of an existing
 * sequent.
 */

public class IfFormulaInstSeq implements IfFormulaInstantiation {

    /**
     * Sequent and formula
     */ 
    private Sequent            seq;
    private boolean antec;	// formula is in antecedent?
    private ConstrainedFormula cf;

    public IfFormulaInstSeq(Sequent p_seq, boolean antec, 
                            ConstrainedFormula p_cf ) {
	seq = p_seq;
        this.antec = antec;
	cf  = p_cf;
    }


    public IfFormulaInstSeq(Sequent seq, int formulaNr ) {
        this(seq, seq.numberInAntec(formulaNr), seq.getFormulabyNr(formulaNr));
    }



    /**
     * @return the cf this is pointing to
     */
    public ConstrainedFormula getConstrainedFormula () {
	return cf;
    }    

    /**
     * Create a list with all formulas of a given semisequent
     */
    public static ListOfIfFormulaInstantiation createList ( Sequent     p_s,
							    boolean antec ) {
	
	ListOfIfFormulaInstantiation res = SLListOfIfFormulaInstantiation.EMPTY_LIST;
	IteratorOfConstrainedFormula  it;
        if (antec) it = p_s.antecedent().iterator ();
           else it = p_s.succedent().iterator ();
	while ( it.hasNext () ) {
	    res = res.prepend(new IfFormulaInstSeq (p_s, antec, it.next()));
	}

	return res;

    }

    public String toString () {
	return getConstrainedFormula ().toString ();
    }

    public boolean equals ( Object p_obj ) {
	if (!(p_obj instanceof IfFormulaInstSeq)) {
	    return false;
	}
	return seq == ((IfFormulaInstSeq)p_obj).seq &&
	    cf == ((IfFormulaInstSeq)p_obj).cf &&
            antec == ((IfFormulaInstSeq)p_obj).antec;
    }
    
    public boolean inAntec() {
       return antec;
    }

}
