// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.rule;

import de.uka.ilkd.key.rule.inst.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.util.Debug;


/** Simple container class containing the information resulting from a
 * Taclet.match-call */

public class MatchConditions {

    public static final MatchConditions EMPTY_MATCHCONDITIONS =
	new MatchConditions ( SVInstantiations.EMPTY_SVINSTANTIATIONS,
			      Constraint.BOTTOM,
			      SetAsListOfMetavariable.EMPTY_SET );

    private SVInstantiations   instantiations   = SVInstantiations.EMPTY_SVINSTANTIATIONS;
    private Constraint         constraint       = Constraint.BOTTOM;
    private SetOfMetavariable  newMetavariables = SetAsListOfMetavariable.EMPTY_SET;

    
    public MatchConditions ( SVInstantiations   p_instantiations,
			     Constraint         p_constraint,
			     SetOfMetavariable  p_newMetavariables) {
	instantiations   = p_instantiations;
	constraint       = p_constraint;
	newMetavariables = p_newMetavariables;
    }


    public SVInstantiations   getInstantiations   () {
	return instantiations;
    }

    public MatchConditions    setInstantiations   ( SVInstantiations   p_instantiations ) {
	if ( instantiations == p_instantiations )
	    return this;
	else
	    return new MatchConditions ( p_instantiations, constraint, newMetavariables );
    }

    public Constraint         getConstraint       () {
	return constraint;
    }

    public MatchConditions    setConstraint       ( Constraint         p_constraint ) {
	if ( constraint == p_constraint )
	    return this;
	else
	    return new MatchConditions ( instantiations, p_constraint, newMetavariables );
    }

    public MatchConditions    join                ( Constraint         p_constraint ) {
	return setConstraint ( constraint.join ( p_constraint ) );
    }

    public MatchConditions    unify               ( Term p0, Term p1 ) {
	return setConstraint ( constraint.unify ( p0, p1 ) );
    }


    public SetOfMetavariable getNewMetavariables () {
	return newMetavariables;
    }

    public MatchConditions    setNewMetavariables ( SetOfMetavariable  p_newMetavariables ) {
	if ( newMetavariables == p_newMetavariables )
	    return this;
	else
	    return new MatchConditions ( instantiations, constraint, p_newMetavariables );
    }

    
    public MatchConditions    addNewMetavariable  ( Metavariable       p_mv ) {
	return new MatchConditions ( instantiations, constraint, newMetavariables.add ( p_mv ) );
    }    

}
