// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

/** this interface has to be implemented by all classes that want to
 * act as a rule in the calculus. */
package de.uka.ilkd.key.rule;

import de.uka.ilkd.key.proof.Goal;
import de.uka.ilkd.key.proof.ListOfGoal;
import de.uka.ilkd.key.java.JavaInfo;
import java.io.Serializable;
import de.uka.ilkd.key.logic.Name;


public interface Rule extends Serializable {

    /** the rule is applied on the given goal using the
     * information of rule application. 
     * @param goal the goal that the rule application should refer to.
     * @param javaInfo the JavaInfo with the necessary information
     * about the java programs 
     * @param the rule application that is executed.
     */
    public ListOfGoal apply(Goal goal, JavaInfo javaInfo, RuleApp ruleApp);
    
    /** 
     * the name of the rule
     */
    public Name name();

    /** returns the display name of the rule 
     */
    public String displayName();

} 
