// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

/* rule application with specific information how and where the rule
 * has to be applied 
*/

package de.uka.ilkd.key.rule;

import de.uka.ilkd.key.proof.Goal;
import de.uka.ilkd.key.proof.RuleAppIndex;
import de.uka.ilkd.key.proof.ListOfGoal;
import de.uka.ilkd.key.java.JavaInfo;
import java.io.Serializable;
import de.uka.ilkd.key.logic.PosInOccurrence;
import de.uka.ilkd.key.logic.Constraint;

public interface RuleApp extends Serializable {

    /**
     * returns the rule of this rule application
     */
    public Rule rule();

    /**
     * returns the PositionInOccurrence (representing a ConstrainedFormula and
     * a position in the corresponding formula) of this rule application
     */
    public PosInOccurrence posInOccurrence();

    /**
     * returns the constraint under which a rule is applicable
     */
    public Constraint       constraint ();


    /** applies the specified rule at the specified position 
     * if all schema variables have been instantiated
     * @param goal the Goal where to apply the rule
     * @param javaInfo the JavaInfo encapsulating all java information
     * @return list of new created goals 
     */
    public ListOfGoal execute(Goal goal, JavaInfo javaInfo);

    /** returns true if all variables are instantiated 
     * @return true if all variables are instantiated 
     */
    public boolean complete();

}
