// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.rule;

import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.Constraint;
import de.uka.ilkd.key.logic.PosInOccurrence;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.proof.Goal;
import de.uka.ilkd.key.proof.ListOfGoal;
import de.uka.ilkd.key.proof.SLListOfGoal;
import de.uka.ilkd.key.proof.decproc.*;

/**
 * Invokes the ESC Simplify to solve an integer problem. 
 */
public class SimplifyIntegerRule implements BuiltInRule {

    private Name name = new Name("Decision Procedure \"Simplify\"");
    private boolean showResults = true;
    

    public SimplifyIntegerRule() {      
	this(true);
    }

    public SimplifyIntegerRule(boolean resultWindow) {      
	showResults = resultWindow;
    }

    /**
     * returns true iff a rule is applicable at the given
     * position. This does not necessary mean that a rule application
     * will change the goal (this decision is made due to performance
     * reasons)
     */
    public boolean isApplicable(Goal            goal, 				
				PosInOccurrence pio,
				Constraint      userConstraint) {
	if (pio == null) {
	    return true;
	}
	return false;
    }

    public ListOfGoal apply(Goal     goal, 
			    JavaInfo javaInfo, 
			    RuleApp  ruleApp) {

	BuiltInRuleApp birApp = (BuiltInRuleApp)ruleApp;
	DecisionProcedureSimplify dps = 
	    new DecisionProcedureSimplify(goal, 
					  birApp.userConstraint());
	SimplifyResult result = dps.run();
	
	if (result.getResult()) {
	    ListOfGoal goals = SLListOfGoal.EMPTY_LIST;
	    if (result.getConstraint()!=Constraint.BOTTOM) {		
		goals = goal.split(1);
		goals.head().addClosureConstraint(result.getConstraint());		
		return goals;
	    } else {
		return SLListOfGoal.EMPTY_LIST;
	    }
	} else {
	    if (showResults) {
		birApp.setErrorMessage
		    (displayName() + " was not successful\n" + result.getText());	
	    }
	    return null;
	}
    }


    public Name name() {
	return name;
    }

    public String displayName() {
	return name().toString();
    }
    
    public String toString() {
	return name().toString();
    }       
}
