// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

/** this class is used to parse in Taclet from a file that are used by tests */
package de.uka.ilkd.key.rule;

import de.uka.ilkd.key.parser.*;
import de.uka.ilkd.key.proof.*;
import java.io.*;
import de.uka.ilkd.key.gui.ProofSettings;
import de.uka.ilkd.key.gui.AbbrevMap;
import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.declaration.*;
import de.uka.ilkd.key.logic.ldt.LDT;
import de.uka.ilkd.key.logic.ldt.*;
import de.uka.ilkd.key.logic.op.Function;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.proof.init.*;

public class TacletForTests {


    public static final String testRules = System.getProperty("key.home")+
        File.separator+"system"+File.separator+"proofExamples"+
        File.separator+"testrules.key";
    public static String standardFile = testRules;
    public static Namespace sorts = new Namespace();
    public static Namespace variables = new Namespace();
    public static Namespace functions = new Namespace();
    public static Namespace heuristics = new Namespace();
    public static AbbrevMap scm = new AbbrevMap();
    public static TacletIndex rules= null;
    public static Proof[] problems;
    public static ProblemInitializer pi=null;
    public static JavaInfo javaInfo;
    public static Recoder2KeY r2k;
    public static File lastFile=null;

    
    public static void parse(File file) {
	try {	    
	    if (!file.equals(lastFile)) {
		pi.BASE_CONFIG=null;	
		pi=ProblemInitializer.problemInitializerForTests
		    (new KeYFile("Test",file));	   
		problems=pi.setUpTest();
		sorts = pi.initConfig().sortNS();
		functions = pi.initConfig().funcNS();
		heuristics = pi.initConfig().heurNS();
		variables = pi.initConfig().varNS();
		rules = pi.initConfig().createTacletIndex();	    
		javaInfo=pi.initConfig().javaInfo();
		lastFile=file;
	    }
	} catch (Exception e) {
	    System.err.println("Exception occured while parsing "+file+"\n");
	    e.printStackTrace();
	    System.exit(-1);
	}

    }

    public static JavaInfo javaInfo() {
	if (javaInfo==null) parse();
	return javaInfo;
    }

    public static void setStandardFile(String filename) {
	standardFile=filename;
    }

    public static Proof[] problems() {
	return problems;
    }

    public static void parse(String filename) {
	parse(new File(filename));
    }

    public static void parse() {
	parse(standardFile);
    }
    
    public static NoPosTacletApp getTaclet(String filename, String name) {
	parse(filename);
	return rules.lookup(new Name(name));
    }

    public static NoPosTacletApp getTaclet(String name) {
	return rules.lookup(new Name(name));
    }
    
    public static AbbrevMap getAbbrevs(){
        return scm;
    }

    public static Namespace getSorts() {
	return sorts;
    }

    public static TacletIndex getRules() {
	return rules;
    }


    public static Namespace getHeuristics() {
	return heuristics;
    }

    public static Namespace getFunctions() {
	return functions;
    }


    public static Namespace getVariables() {
	return variables;
    }

    public static Function funcLookup(String name) {
	return (Function)functions.lookup(new Name(name));
    }

    public static SchemaVariable svLookup(String name) {
	return (SchemaVariable)variables.lookup(new Name(name));
    }

    public static Sort sortLookup(String name) {
	return (Sort)sorts.lookup(new Name(name));
    }


    private static Term parseTerm(String termstr, JavaInfo ji) {
	if (termstr.equals("")) return null;
	try {	    
	    StringReader br=new StringReader(termstr);
	    TermParser parser = new TermParser(new KeYLexer(br), 
					       "No file. " +
					       "TacletForTests.parseTerm("
					       +termstr+")",
					       TermFactory.DEFAULT, 
					       ji,
					       TacletForTests.getVariables(),
					       TacletForTests.getFunctions(), 
					       TacletForTests.getSorts(),
					       TacletForTests.getAbbrevs());
	    return parser.term();
	} catch (Exception e) {
	    System.err.println("Exception during parsing!");
	    e.printStackTrace();
	    return null;
	}
	
    }

    public static JavaInfo getJavaInfo() {	
	return javaInfo().copy();
    }

    public static Term parseTerm(String termstr, NamespaceSet set) {
	if (termstr.equals("")) return null;
	try {	    
	    StringReader br=new StringReader(termstr);
	    TermParser parser = new TermParser(new KeYLexer(br), 
					       "No file. " +
					       "TacletForTests.parseTerm("
					       +termstr+")",
					       TermFactory.DEFAULT, 
					       getJavaInfo(),
					       set.variables(),
					       set.functions(), 
					       set.sorts(),
					       new de.uka.ilkd.key.gui.AbbrevMap());
	    return parser.term();
	} catch (Exception e) {
	    System.err.println("Exception during parsing!");
	    e.printStackTrace();
	    return null;
	}
	
    }

    public static Term parseTerm(String termstr) {
	return parseTerm(termstr, javaInfo());
    }

    public static ProgramElement parsePrg(String prgString) {
	r2k=new Recoder2KeY("/usr/lib/java/jre/lib/rt.jar",
			    javaInfo().rec2key(), javaInfo.getTypeConverter());
	return r2k.readBlockWithEmptyContext(prgString).program();
    }

    public static ProgramElement parseExpr(String type, String prgString) {
	r2k=new Recoder2KeY("/usr/lib/java/jre/lib/rt.jar",
			    javaInfo().rec2key(), javaInfo.getTypeConverter());
	ProgramElement pe=r2k.readBlockWithEmptyContext("{"+type
	       					+" x="+prgString+";}").program();
        LocalVariableDeclaration ld
	    =(LocalVariableDeclaration)((StatementBlock)pe).getStatementAt(0);

	return ld.getVariableSpecifications().
	    getVariableSpecification(0).getInitializer();

    }


    public static Proof parseTermAsInitialProof(String termstr) {
	ConstrainedFormula cf=new ConstrainedFormula(parseTerm(termstr));
	Semisequent succ=Semisequent.EMPTY_SEMISEQUENT.insert(0,cf).semisequent();
	Sequent s=Sequent.createSuccSequent(succ);
	Proof proof = new Proof();
	proof.setRoot(new Node(proof, s));
	return proof;
    }

}
