// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.rule;

import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.logic.op.SimultaneousUpdateOp;
import de.uka.ilkd.key.logic.ListOfTerm;
import de.uka.ilkd.key.logic.SLListOfTerm;
import de.uka.ilkd.key.logic.IteratorOfTerm;
import de.uka.ilkd.key.util.Debug;

/** this class represents a pair of an update operator and the term
 * that is used to update the variable(attribute) of update operator
 */

public class UpdatePair {

    /** the update terms */
    private final ListOfTerm updateTerms;

    public UpdatePair(Term update) {

	Debug.assertTrue(update.op() instanceof SimultaneousUpdateOp, 
			 "updatepair:: simultaneous update expected");
	
	ListOfTerm result = SLListOfTerm.EMPTY_LIST;

	for (int i = update.arity()-2; i>=0; i--) {
	    result = result.prepend(update.sub(i));	    
	}

	updateTerms = result;
    }

    public int arity() {
	return updateTerms.size();
    }

    public ListOfTerm subs() {
	return updateTerms;
    }

    private boolean equalEntries(UpdatePair cmp) {
	for (int i = 0; i<arity(); i++) {
	    if (!subs().equals(cmp.subs())) {
		return false;
	    }
	}
	return true;
    }

    public boolean equals(Object o) {
	if (!(o instanceof UpdatePair)) {
	    return false;
	}
	UpdatePair cmp = (UpdatePair) o;	
	return cmp.arity()==arity() && equalEntries(cmp);
    }

    public String toString() {
	StringBuffer result = new StringBuffer("{");
	IteratorOfTerm it = updateTerms.iterator();
	for (;it.hasNext(); result.append(",")) {
	    result.append(""+it.next());
	    result.append(":=");
	    result.append(""+it.next());
	}
	result.append("}");
	return result.toString();
    }
}
