// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.rule.inst;

import de.uka.ilkd.key.java.ProgramElement;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.logic.PosInProgram;

/** This class is used to store the information about a matched
 * context of a dl formula. (the pi and omega part)
 */

public class ContextInstantiationEntry extends InstantiationEntry {

    /** the prefix and suffix instantiation */
    private ContextTermInstantiation inst;
   
    /** creates a new ContextInstantiationEntry 
     * @param ctxt the SchemaVariable that is
     * instantiated
     * @param PosInProgram pi the PosInProgram describing the position
     * of the first statement after the prefix
     * @param PosInProgram omega the PosInProgram describing the position
     * of the statement just before the suffix starts
     */
    ContextInstantiationEntry(SchemaVariable ctxt, PosInProgram pi,
			      PosInProgram omega, ProgramElement pe) {
	super(ctxt);
	inst = new ContextTermInstantiation(pi, omega, pe);
    }

    /** returns the depth of the position of the first statement after the prefix
     * @return the depth of the position of the first statement after the prefix
     */
    public int prefixDepth() {
	return inst.prefixDepth();
    }


    /**
     * returns the length of the context measured on the level of the
     * beginning of the context.
     */
    public int contextWidth() {
	return inst.contextWidth();
    }

    /** returns the position of the first statement after the prefix
     * @return the position of the first statement after the prefix
     */
    public PosInProgram prefix() {
	return inst.prefix();
    }


    /** returns the position of the statement just before the suffix
     * starts  
     * @return the position of the statement just before the suffix
     * starts 
     */
    public PosInProgram suffix() {
	return inst.suffix();
    }

    /** returns the context program with an ignorable part between prefix
     * and suffix position
     */
    public ProgramElement contextProgram() {
	return inst.programElement();
    }


    /** returns the instantiation of the SchemaVariable 
     * @return  the instantiation of the SchemaVariable 
    */
    public Object getInstantiation() {
	return inst;
    }

    /** toString */
    public String toString() {
	return "["+getSchemaVariable()+",\npi:"+prefix()+"\nomega:"+suffix()+"\n]";
    }

}
