// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.rule.inst;

import de.uka.ilkd.key.java.ProgramElement;
import java.io.Serializable;
import de.uka.ilkd.key.logic.PosInProgram;
/** this class is creates if the scheme given by a context term has
 * matched to a java program. The ContextTermInstantiation class
 * stores the instantiations of the prefix and the suffix. 
 */
public class ContextTermInstantiation implements Serializable {
    
    /** the end position of the prefix omega */
    private PosInProgram prefixEnd;

    /** the start position of the suffix omega */
    private PosInProgram suffixStart;

    /** the whole program element this context term inst refers to */
    private ProgramElement programElement;
    
    /** creates a ContextTermInstantiation of a context term 
     * @param prefixEnd the PosInProgram describing the first
     * statement after the end of the prefix
     * @param suffixStart the PosInProgram describing the statement
     * just before the suffix begins
     */
    public ContextTermInstantiation(PosInProgram prefixEnd,
				    PosInProgram suffixStart,
				    ProgramElement pe) {
	this.prefixEnd = prefixEnd;
	this.suffixStart = suffixStart;
	this.programElement = pe;
    }

    /** returns the end position of the prefix 
     * @return the end position of the prefix 
     */
    public PosInProgram prefix() {
	return prefixEnd;
    }

    /** returns the depth of the end position of the prefix 
     * @return the depth of the end position of the prefix 
     */
    public int prefixDepth() {
	return (prefixEnd == null ? -1 : prefixEnd.depth());
    }

    /**
     * returns the length of the context measured on the level of the
     * beginning of the context.
     */
    public int contextWidth() {
	return ((suffixStart.get(prefixEnd.depth()-1)).intValue() -
	    prefixEnd.get(prefixEnd.depth()-1).intValue())+1;  
    }

    /** returns the PosInProgram describing the statement
     * just before the suffix begins
     */
    public PosInProgram suffix() {
	return suffixStart;
    }
    /**
     * returns the program element this context term instantiation refers to
     * @return returns the program element this context term instantiation 
     * refers to
     */
    public ProgramElement programElement() {
	return programElement;
    }

    /** toString */
    public String toString() {
	String result = "ContextTermInstantiation:\n";
	result += "Prefix ends before "+prefixEnd.toString();
	result += "\nSuffix starts after "+suffixStart.toString();
	result += "\nRefers to Program "+programElement;
	return result+"\n";
    }
}
