// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.rule.metaconstruct;

import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.Expression;
import de.uka.ilkd.key.java.Statement;
import de.uka.ilkd.key.java.ProgramElement;
import de.uka.ilkd.key.java.ArrayOfExpression;
import de.uka.ilkd.key.java.statement.MethodBodyStatement;
import de.uka.ilkd.key.java.reference.MethodReference;
import de.uka.ilkd.key.java.reference.VariableReference;
import de.uka.ilkd.key.java.reference.TypeRef;
import de.uka.ilkd.key.java.reference.ReferencePrefix;
import de.uka.ilkd.key.java.reference.SuperReference;
import de.uka.ilkd.key.java.declaration.Modifier;
import de.uka.ilkd.key.java.declaration.MethodDeclaration;
import de.uka.ilkd.key.java.declaration.LocalVariableDeclaration;
import de.uka.ilkd.key.java.declaration.VariableSpecification;
import de.uka.ilkd.key.java.declaration.ParameterDeclaration;
import de.uka.ilkd.key.java.StatementBlock;
import de.uka.ilkd.key.java.ArrayOfStatement;
import de.uka.ilkd.key.java.visitor.ProgVarReplaceVisitor;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.logic.op.ProgramVariable;
import de.uka.ilkd.key.logic.op.ProgramMethod;
import de.uka.ilkd.key.rule.inst.SVInstantiations;
import de.uka.ilkd.key.logic.*;

import java.util.HashMap;

public class ExpandMethodBody extends ProgramMetaConstruct {

    public ExpandMethodBody(SchemaVariable sv) {
	super(new Name("expand-method-body"), sv);
    }


    /** 
     * Replaces the MethodBodyStatement shortcut with the full body,
     * performs prefix adjustments in the body (execution context).
     * @param javaInfo the JavaInfo with all necessary information 
     * about the java programs
     * @param svInst the instantiations esp. of the inner and outer label 
     * @return the transformed program
     */
    public ProgramElement symbolicExecution(ProgramElement pe,
					    JavaInfo javaInfo,
					    SVInstantiations svInst) {

	MethodBodyStatement mbs = (MethodBodyStatement) pe;
        MethodReference mr = mbs.getMethodReference();

        ProgramMethod pm = mr.method(javaInfo, mbs.getBodySource());

	MethodDeclaration methDecl = pm.getMethodDeclaration();
	int params = methDecl.getParameterDeclarationCount();


	StatementBlock body = (StatementBlock) mbs.getBody(javaInfo);	

	ReferencePrefix newCalled;
	if (!mbs.isStatic(javaInfo) &&
	    mbs.getBodySource()!=mbs.getDesignatedContextType(javaInfo)) {
		ProgramVariable newCalledPV = new ProgramVariable
		    (getNewCalledName(mbs),
		     mbs.getBodySource());
		newCalled=newCalledPV;
		VariableSpecification newCalledSpec 
		    = new VariableSpecification(newCalledPV, 
						(Expression)mbs.getDesignatedContext(), 
						mbs.getBodySource());
		Statement s = new LocalVariableDeclaration
		    (new TypeRef(mbs.getBodySource()), newCalledSpec);
		body = insertStatementInBlock(new Statement[]{s}, 
					      (StatementBlock) body);	    
	} else {
	    newCalled=mbs.getDesignatedContext();
	}

	SetExecutionContextVisitor repl = 
           new SetExecutionContextVisitor(body,
                                          newCalled,
                                          mbs.isStatic(javaInfo),
					  javaInfo);
        repl.start();
        Statement result = (Statement) repl.result(); 

	// at this point all arguments should be program variables
	ArrayOfExpression argsAsParam = mr.getArguments();

	HashMap map = new HashMap();
	for (int i = 0; i < argsAsParam.size(); i++) {
	    map.put(methDecl.getParameterDeclarationAt(i).
		    getVariableSpecification().getProgramVariable(), 
		    argsAsParam.getExpression(i));
	}
	
	ProgVarReplaceVisitor paramRepl = 
	    new ProgVarReplaceVisitor(result, map); 
	paramRepl.start();	
	return (Statement) paramRepl.result();
        
    }


    private String getReferencePrefixName(ReferencePrefix rp) {
	if (rp instanceof ProgramVariable) {
	    String s=((ProgramVariable)rp).name().toString();
	    if (s.lastIndexOf("::")>0) {
		s=s.substring(s.lastIndexOf("::")+1);
	    }
	    return s;
	} else if (rp instanceof VariableReference) {
	    return getReferencePrefixName(rp.getReferencePrefix())+"_"
		+((VariableReference)rp).getProgramElementName();
	}   
	return "this";       
    }
    
    private ProgramElementName getNewCalledName(MethodBodyStatement mbs) {
	ReferencePrefix rp=mbs.getDesignatedContext();
	return new ProgramElementName(mbs.getBodySource().getSort()+"::"
				      +getReferencePrefixName(rp));
    }


    /** inserts the given statements at the begin of the block 
     * @param stmnt array of Statement those have to be inserted
     * @param b the Statementblock where to insert
     */
    private StatementBlock insertStatementInBlock(Statement[] stmnt, 
						  StatementBlock b) {
	
	Statement[] block = new Statement[b.getStatementCount()+
					  stmnt.length];
	for (int j = 0; j < stmnt.length; j++) {
	    block[j] = stmnt[j];
	}
	for (int i = 0; i < b.getStatementCount(); i++) {
	    block[i+stmnt.length] = b.getStatementAt(i);
	}
	return new StatementBlock(new ArrayOfStatement(block));	
    }

}
