// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001 Universitaet Karlsruhe, Germany
//                    and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General 
// Public License. See LICENSE.TXT for details.
//

package de.uka.ilkd.key.rule.metaconstruct;

import de.uka.ilkd.key.java.Services;
import de.uka.ilkd.key.java.abstraction.*;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.ProgramElement;
import de.uka.ilkd.key.java.Expression;
import de.uka.ilkd.key.java.Statement;
import de.uka.ilkd.key.java.StatementBlock;
import de.uka.ilkd.key.java.ArrayOfStatement;
import de.uka.ilkd.key.java.ArrayOfExpression;
import de.uka.ilkd.key.java.NonTerminalProgramElement;
import de.uka.ilkd.key.java.expression.operator.CopyAssignment;
import de.uka.ilkd.key.java.statement.MethodCallStatement;
import de.uka.ilkd.key.java.reference.*;
import de.uka.ilkd.key.java.reference.MethodReference;
import de.uka.ilkd.key.java.declaration.Modifier;
import de.uka.ilkd.key.java.declaration.MethodDeclaration;
import de.uka.ilkd.key.java.declaration.LocalVariableDeclaration;
import de.uka.ilkd.key.java.declaration.VariableSpecification;
import de.uka.ilkd.key.java.declaration.ParameterDeclaration;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;
import de.uka.ilkd.key.util.ExtList;
import de.uka.ilkd.key.java.statement.*;
import de.uka.ilkd.key.java.expression.*;
import de.uka.ilkd.key.java.expression.operator.*; 
import de.uka.ilkd.key.java.expression.literal.*;
import de.uka.ilkd.key.rule.metaconstruct.ProgramMetaConstruct;
import de.uka.ilkd.key.proof.TacletInstantiationsTableModel;
import de.uka.ilkd.key.java.visitor.ProgVarReplaceVisitor;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.logic.op.*;
import de.uka.ilkd.key.rule.inst.SVInstantiations;
import de.uka.ilkd.key.util.Debug;
import java.util.Vector;
 

/** 
 * Symbolically executes a method invocation
 */ 
public class MethodCall extends ProgramMetaConstruct {
    
    private final SchemaVariable result;

    private MethodReference methRef;
    private ProgramMethod pm;
    private ReferencePrefix refPrefix;
    private ReferencePrefix newContext;
    private JavaInfo javaInfo;
    private ProgramVariable pvar;
    private ProgramVariable prefix;
    private ProgramVariable paramVars[];

    /** creates the methodcall-MetaConstruct 
     * @param result the SchemaVariable that is used to keep the result
     * @param body the ProgramElement contained by the meta construct 
     */
    public MethodCall(SchemaVariable result, ProgramElement body) {
	super(new Name("method-call"), body);
	this.result = result;
    }

    /** creates the methodcall-MetaConstruct 
     * @param body the ProgramElement contained by the meta construct 
     */
    public MethodCall(ProgramElement body) {
	super(new Name("method-call"), body);
	this.result = null;
    }


    /** gets an array of expression and returns a list of types */
    private ListOfKeYJavaType getTypes(JavaInfo javaInfo, 
				       ArrayOfExpression args) {
	ListOfKeYJavaType result = SLListOfKeYJavaType.EMPTY_LIST; 
	for (int i = args.size()-1; i >= 0 ; i--) {
	    Expression argument = args.getExpression(i);
	    result = result.prepend
		(javaInfo.getTypeConverter().getKeYJavaType(argument));
	}
	return result;
    }




    /** performs the program transformation needed for symbolic
     * program execution 
     * @param javaInfo the JavaInfo with all necessary information 
     * about the java programs
     * @param svInst the instantiations esp. of the inner and outer label 
     * @return the transformed program
     */
    public ProgramElement symbolicExecution(ProgramElement pe,
					    JavaInfo javaInfo,
					    SVInstantiations svInst) {
	this.javaInfo=javaInfo;
	if (result != null) {
	    pvar = (ProgramVariable) svInst.getInstantiation(result);
	}
	methRef = (MethodReference) pe;
	refPrefix = methRef.getReferencePrefix();
	StatementBlock ourBody;
	Statement[] a=null;
	if(refPrefix instanceof VariableReference || refPrefix instanceof ProgramVariable){
	    String vname="_var" + TacletInstantiationsTableModel.
				globalVariableNameCnt++;
	    Statement s;
	    VariableSpecification prefixSpec; 
	    if(refPrefix instanceof VariableReference){
		prefix = new ProgramVariable(new ProgramElementName(vname),
							     ((VariableReference)refPrefix).
							     getKeYJavaType());
		prefixSpec = new VariableSpecification(prefix,((VariableReference)refPrefix).
						getKeYJavaType());
		s = new LocalVariableDeclaration
		    (new TypeRef(((VariableReference)refPrefix).getKeYJavaType()), prefixSpec);
	    }else{
		prefix = new ProgramVariable(new ProgramElementName(vname),
					     ((ProgramVariable)refPrefix).
					     getKeYJavaType());	
		prefixSpec = new VariableSpecification(prefix,((ProgramVariable)refPrefix).
						getKeYJavaType());
		s = new LocalVariableDeclaration
		    (new TypeRef(((ProgramVariable)refPrefix).getKeYJavaType()), prefixSpec);
	    }	
	    a=new Statement[2];
	    a[0] = s;
	    a[1] = new CopyAssignment(prefix,(Expression)refPrefix);
	}
        KeYJavaType t = null;
	pm = methRef.method(javaInfo);

        newContext = methRef.getReferencePrefix();

	if (refPrefix instanceof TypeRef) {
	    t = ((TypeRef)refPrefix).getKeYJavaType();
	    if (t == null) {
		//System.err.println("CHECK: t==null");            
		t = javaInfo.getKeYJavaTypeByClassName
		    (((TypeRef)refPrefix).getName());
	    }
	} else if (refPrefix instanceof ProgramVariable) {
	    t = ((ProgramVariable)refPrefix).getKeYJavaType();
	} else if (refPrefix instanceof FieldReference) {
	    t = ((FieldReference)refPrefix).getProgramVariable().getKeYJavaType();
	} else if (refPrefix instanceof SuperReference) {
	    t = javaInfo.getSuperclass(((SuperReference)refPrefix).
				       getExecutionContext().
				       getTypeReference().getKeYJavaType());	
	} else if (refPrefix instanceof MethodReference) {	   
	    return evaluateMethodReference((MethodReference)refPrefix, methRef);
	} else {
	    throw new de.uka.ilkd.key.util.NotSupported
		("Unsupported method invocation mode\n"+
		 refPrefix.getClass());
	}			    


	if (pm == null) {
	    Debug.out("methodcall:No implementation available for ", pe);
	    if ("<prepare>".equals(methRef.getName().toString()) &&
		("Object".equals(t.getFullName()))) {
		// %%HACK for 9.9.
		ProgramElementName prepareName = new ProgramElementName("<prepare>");
		pm = new ProgramMethod
		    (new MethodDeclaration
			(new Modifier[]
			    {new de.uka.ilkd.key.java.declaration.modifier.Protected()}, 
			 null, prepareName, new ParameterDeclaration[0], 
			 null, new StatementBlock(), false), 
		     javaInfo.getSuperclass(t), null);
		methRef = new MethodReference(new ArrayOfExpression(new Expression[0]),
					      prepareName, ((SuperReference)refPrefix).
					      getExecutionContext().getRuntimeInstance(), pm);
	    } else {
		return pe;
	    }
	}

        if (!de.uka.ilkd.key.gui.Main.standalone)
            de.uka.ilkd.key.casetool.together.ModelUtil.hiliteMethod(t, pm);

	StatementBlock result = null;
	// Static invocation mode
	
	Statement[] paramDecl = createParamAssignments(t); 
	
	if (!pm.getMethodDeclaration().isAbstract()) {
	    methRef = new MethodReference(new ArrayOfExpression(paramVars),
					  methRef.getProgramElementName(),
					  newContext, pm);
	} else {
	    methRef = new MethodReference(new ArrayOfExpression(paramVars),
					  methRef.getProgramElementName(),
					  newContext, null);
	}

	if (pm.isStatic()) {
	    //            newContext = new TypeRef((KeYJavaType)t);
	    // this was too aggressive
	    // - we did not transform the logical formula
	    // - what about side effects in prefix? (OK, we should take care of them 
	    //  beforehand, but anyway...)
	    ourBody = new StatementBlock(new MethodBodyStatement(t, methRef,  newContext));
	    result = insertStatementInBlock
		(paramDecl, new StatementBlock
		    (new MethodCallStatement(pvar, ourBody, methRef)));
	    // Instance invocation mode
	} else  {
	    if (pm.isPrivate()) { // private methods are bound statically
		ourBody = makeMbs(t);
		result = insertStatementInBlock(paramDecl, ourBody);
	    } else if (refPrefix instanceof SuperReference) { 
		ourBody = new StatementBlock
		    (new MethodBodyStatement
			(t, 
			 new MethodReference(methRef.getArguments(),
					     methRef.getProgramElementName(),
					     null,
					     pm),
			 ((SuperReference)refPrefix).
			 getExecutionContext().getRuntimeInstance()));
		result = insertStatementInBlock(paramDecl, ourBody); 
	    } else {
		// Hack to make virtual method dispatching work
		methRef = new MethodReference(methRef.getArguments(),
                                              methRef.getProgramElementName(),
                                              newContext,
                                              null);
		ifCascadeTop = ifCascadeBottom = null;
		ifBodys = new Vector();
		ifConds = new Vector();
		javaInfo.getKeYProgModelInfo().findImplementations
		    (this, 
		     (KeYJavaType)t, methRef.getName(),
		     getTypes(javaInfo, methRef.getArguments()));
		if(ifBodys.size() == 1){
		    result = insertStatementInBlock
			(paramDecl, new StatementBlock((Statement) ifBodys.get(0)));
		} else {
		    if (ifBodys.size() != 0) {
			ifCascadeBottom = ifCascadeTop 
			    = new If((Expression)ifConds.get(0), 
				     new Then((Statement)ifBodys.get(0)));
			for(int i=1;i<ifBodys.size()-1;i++){
			    ifCascadeBottom.addBranch
				(new Else
				    (ifCascadeBottom 
				     = new If((Expression)ifConds.get(i), 
					      new Then((Statement)ifBodys.get(i)))));
			}
			ifCascadeBottom.addBranch
			    (new Else((Statement)ifBodys.get(ifBodys.size()-1)));
			result = insertStatementInBlock
			    (paramDecl, new StatementBlock(ifCascadeTop));
		    }
		    
		}
	    }
	}
	
	if(a != null) {
	    result = insertStatementInBlock(a, result);
	}	

	ProgVarReplaceVisitor paramRepl = new ProgVarReplaceVisitor(result);
	paramRepl.start();	
	return (Statement) paramRepl.result();
    }


    private ProgramElement evaluateMethodReference(MethodReference p,
						   MethodReference s) {
	KeYJavaType type=p.method(javaInfo).getKeYJavaType();
	String vname="_var" + TacletInstantiationsTableModel.
	    globalVariableNameCnt++;
	ProgramVariable newPV = new ProgramVariable
	    (new ProgramElementName(vname), type);
	VariableSpecification newSpec 
	    = new VariableSpecification(newPV, p, type);
	Statement s0 = new LocalVariableDeclaration(new TypeRef(type), newSpec);
	Statement s1;
	MethodReference mr = new MethodReference
	    (s.getArguments(), s.getProgramElementName(), newPV, null);
	if (result!=null) {
	    s1 = new CopyAssignment(pvar, mr);
	} else {
	    s1=mr;
	} 
	return new StatementBlock(new ArrayOfStatement(new Statement[]{s0,s1}));
    }


    //***************** Dynamic Binding Construction Utilities ***************

    private If ifCascadeTop = null;
    private If ifCascadeBottom = null;
    private Vector ifBodys = null;
    private Vector ifConds = null;


    private StatementBlock makeMbs(KeYJavaType t) {
	StatementBlock result;
	ExtList cmp     = new ExtList();
	ExtList excType = new ExtList();
	ExtList npe     = new ExtList();
	excType.add
	    (new TypeRef(javaInfo.getTypeByName("java.lang.NullPointerException")));
	cmp.add(prefix);
	cmp.add(NullLiteral.NULL);
	npe.add(new Throw(new New(excType, null)));
	Statement[] npeBlock = {new If(new Equals(cmp),new Then(npe))};
	result=new StatementBlock(new MethodCallStatement
	    (pvar,
	     new StatementBlock(new MethodBodyStatement(t, methRef, newContext)),
	     methRef));
	if (prefix!=null) result = insertStatementInBlock(npeBlock,result);
	return result;
    }


    public Expression makeIOf(Type t) {
	return new Instanceof((Expression) refPrefix, 
			      new TypeRef((KeYJavaType)t));
    }


    public Expression makeEIOf(Type t) {
	return new ExactInstanceof((ProgramVariable) refPrefix, 
				   new TypeRef((KeYJavaType)t));
    }

    public Statement[] createParamAssignments(KeYJavaType t){
	
	MethodBodyStatement mbs       = new MethodBodyStatement(t, 
								methRef, 
								newContext);
	MethodDeclaration methDecl    = pm.getMethodDeclaration();
	int params                    = methDecl.getParameterDeclarationCount();
	Statement[] paramDecl         = new Statement[params];
	paramVars = new ProgramVariable[params];
	for (int i = 0; i < params; i++) {
	    ParameterDeclaration parDecl = 
		methDecl.getParameterDeclarationAt(i);
	    VariableSpecification originalSpec = 
		parDecl.getVariableSpecification();
	    VariableSpecification newSpec = 
		new VariableSpecification
		    (originalSpec.getProgramVariable(), 
		     originalSpec.getDimensions(), 
		     methRef.getArgumentAt(i),
		     originalSpec.getType());
	    paramVars[i] = (ProgramVariable)newSpec.getProgramVariable();
	    paramDecl[i] = new LocalVariableDeclaration
		(parDecl.getModifiers(), parDecl.getTypeReference(), newSpec);  
	} 
	return paramDecl;
    }

    public Expression disj(Vector v) {
	v.remove(null);
	if (v.size() == 0) return null;
	Expression e = makeIOf((Type)v.get(0));
	for (int i=1; i< v.size(); i++) 
	    e = new LogicalOr(e, makeIOf((Type)v.get(i)));
	return e;
    }


    public void addBranch(Expression ifCond, KeYJavaType t) {
	ifConds.add(ifCond);
	ifBodys.add(makeMbs(t));
    }

    /** inserts the given statements at the begin of the block 
     * @param stmnt array of Statement those have to be inserted
     * @param b the Statementblock where to insert
     */
    private StatementBlock insertStatementInBlock(Statement[] stmnt, 
						  StatementBlock b) {
	
	Statement[] block = new Statement[b.getStatementCount()+
					 stmnt.length];
	for (int j = 0; j < stmnt.length; j++) {
	    block[j] = stmnt[j];
	}
	for (int i = 0; i < b.getStatementCount(); i++) {
	    block[i+stmnt.length] = b.getStatementAt(i);
	}
	return new StatementBlock(new ArrayOfStatement(block));	
    }


}
