// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.rule.metaconstruct;

import de.uka.ilkd.key.casetool.together.TgFacade;

import de.uka.ilkd.key.java.Recoder2KeY;
import de.uka.ilkd.key.java.ProgramElement;
import de.uka.ilkd.key.java.Expression;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.abstraction.ArrayType;
import de.uka.ilkd.key.java.abstraction.Field;
import de.uka.ilkd.key.java.abstraction.ListOfField;
import de.uka.ilkd.key.java.abstraction.IteratorOfField;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;
import de.uka.ilkd.key.java.abstraction.PrimitiveType;
import de.uka.ilkd.key.java.abstraction.Type;

import de.uka.ilkd.key.java.declaration.ClassDeclaration;
import de.uka.ilkd.key.java.declaration.LocalVariableDeclaration;
import de.uka.ilkd.key.java.declaration.FieldSpecification;

import de.uka.ilkd.key.java.expression.literal.*;
import de.uka.ilkd.key.java.expression.operator.Instanceof;
import de.uka.ilkd.key.java.expression.operator.New;

import de.uka.ilkd.key.java.reference.*;

import de.uka.ilkd.key.logic.op.ProgramVariable;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.rule.inst.SVInstantiations;

import de.uka.ilkd.key.util.Debug;

public class PostWork extends ProgramMetaConstruct {
    
    private final SchemaVariable newObjectSV;
    
     /** creates a typeof ProgramMetaConstruct 
     * @param expr the instance of expression contained by 
     * the meta construct 
     */
    public PostWork(SchemaVariable newObjectSV,
		    Expression expr) {
	super("post-work", expr); 
	this.newObjectSV = newObjectSV;
    
    }

    /** performs the program transformation needed for symbolic
     * program transformation 
     * @return the transformated program
     */
    public ProgramElement symbolicExecution(ProgramElement pe,
					    JavaInfo javaInfo,
					    SVInstantiations svInst) {       
	ClassDeclaration classDecl = (ClassDeclaration)
	    ((New)pe).getTypeReference().getKeYJavaType().getJavaType();
	ProgramVariable newObject = (ProgramVariable) 
	    svInst.getInstantiation(newObjectSV);
	ProgramVariable initialized = find
	    ("<initialized>", javaInfo.getImplicitFields(classDecl));
	
	return assign(attribute(newObject, initialized), 
		      BooleanLiteral.TRUE);
    }

    /**
     * retrieves a field with the given name out of the list
     */
    private final ProgramVariable find(String name, ListOfField fields) {
	IteratorOfField it = fields.iterator();
	while (it.hasNext()) {
	    Field field = it.next();
	    if (name.equals(field.getName())) {
		return (ProgramVariable)
		    ((FieldSpecification)field).getProgramVariable();
	    }
	}
	return null;
    }


}
