// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.rule.metaconstruct;

import java.util.Stack;
import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.abstraction.ClassType;
import de.uka.ilkd.key.java.abstraction.ListOfClassType;
import de.uka.ilkd.key.java.abstraction.IteratorOfClassType;
import de.uka.ilkd.key.java.reference.*;
import de.uka.ilkd.key.java.expression.operator.*;
import de.uka.ilkd.key.java.expression.ParenthesizedExpression;
import de.uka.ilkd.key.util.ExtList;
import de.uka.ilkd.key.java.statement.MethodBodyStatement;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.java.visitor.CreatingASTVisitor;
import de.uka.ilkd.key.logic.op.ProgramVariable;
import de.uka.ilkd.key.logic.op.ProgramMethod;


/**
 * Walks through a java AST in depth-left-first-order. This visitor
 * replaces ThisReferences by a variable this.
 */
public class SetExecutionContextVisitor extends CreatingASTVisitor {

    private ProgramElement result=null;

    private ReferencePrefix newPrefix = null;
    
    private boolean staticCall;

    private JavaInfo javaInfo = null;

    private boolean fatherIsMethodReference = false;

    /** create the  ThisReplaceVisitor
     * @param root the ProgramElement where to begin
     */
    public SetExecutionContextVisitor(Statement st, 
                                      ReferencePrefix newPrefix,
                                      boolean staticCall) {
	this(st, newPrefix, staticCall, null);
    }

    /** create the  ThisReplaceVisitor
     * @param root the ProgramElement where to begin
     */
    public SetExecutionContextVisitor(Statement st, 
                                      ReferencePrefix newPrefix,
                                      boolean staticCall,
				      JavaInfo javaInfo) {
	super(st);
	this.newPrefix  = newPrefix;
        this.staticCall = staticCall;
	this.javaInfo = javaInfo;
    }

    /** the action that is performed just before leaving the node the
     * last time 
     */
    protected void doAction(ProgramElement node) {
//System.err.println("Visiting "+node.getClass()+" "+node);
	node.visit(this);
    }
    
    /** starts the walker*/
    public void start() {	
	stack.push(new ExtList());		
	walk(root());
	ExtList el=(ExtList)stack.peek();
	int i=0;
	while (!(el.get(i) instanceof ProgramElement)) {
	    i++;
	}
	result=(ProgramElement) ((ExtList)stack.peek()).get(i);
    }

    public ProgramElement result() { 	
	return result;
    }
    
    /** walks through the AST. While keeping track of the current node
     * @param node the JavaProgramElement the walker is at 
     */
    protected void walk(ProgramElement node) {	
	if (node instanceof MethodReference) {
	    fatherIsMethodReference = true;
	} 	    
	stack.push(new ExtList());
	super.walk(node);	
    }


// ************ HERE'S THE MEAT *********************************

    public void performActionOnThisReference(ThisReference x) {
	addChild(newPrefix);
	changed();
    }

    public void performActionOnSuperReference(SuperReference x) {
	// we are allowed to substitute "super.a" by "((Superclass)this).a"
	// iff "a" ist is a field reference (and not a method reference)
	ExtList changeList = (ExtList)stack.peek();
	if (!fatherIsMethodReference) {
	    TypeCast cast = new TypeCast((Expression)newPrefix,
					 new TypeRef(javaInfo.getSuperclass
						     (((ProgramVariable)newPrefix).
						      getKeYJavaType())));
	    addChild(new ParenthesizedExpression(cast));
	    changed();
	} else {
	    fatherIsMethodReference = false;
	    if (changeList.getFirst() == CHANGED) {
		changeList.removeFirst();        
		addChild(new SuperReference(changeList));
		changed();
	    } else {
		doDefaultAction(x);
	    }
	}
    }

    public void performActionOnMethodReference(MethodReference mr) {

       MethodReference newmr = mr;
       ExtList changeList = (ExtList)stack.peek();
       if (changeList.getFirst() == CHANGED) { // process change in children
           changeList.removeFirst();        
           ReferencePrefix rp=null;                         
           if (newmr.getReferencePrefix()!=null) {
               rp = (ReferencePrefix)changeList.get(0);
           }
           changeList.remove(rp);
	   ProgramMethod pm = mr.isResolved() ? mr.method(null) : null;
           newmr=new MethodReference(changeList, rp, pm);
       }

       if (newmr.getReferencePrefix() == null) {
          newmr = (MethodReference) newmr.setReferencePrefix(newPrefix);
       }

       if (newmr==mr) {
           doDefaultAction(mr);
       } else {
           addChild(newmr);
           changed();
       }
       fatherIsMethodReference = false;
    }

    public void performActionOnFieldReference(FieldReference fr) {

	FieldReference newfr = fr;

       ExtList changeList = (ExtList)stack.peek();
       if (changeList.getFirst() == CHANGED) { // process change in children	    
	  changeList.removeFirst();	    
	  if (fr.getReferencePrefix() != null) {
	     newfr = new FieldReference((ProgramVariable)changeList.get(1), 
		                        (ReferencePrefix)changeList.get(0));
	  } else {
	     newfr = new FieldReference((ProgramVariable)changeList.get(0),
		                        null);
	  }
       }

       if (newfr.getReferencePrefix() == null)
          newfr = (FieldReference) newfr.setReferencePrefix(newPrefix);

       if (newfr==fr) {
	   doDefaultAction(fr);
       } else {
	   addChild(newfr);
	   changed();
       }
    }


}
