package de.uka.ilkd.key.rule.metaconstruct;

import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.logic.op.SchemaVariable;
import de.uka.ilkd.key.logic.op.ProgramVariable;
import de.uka.ilkd.key.java.*;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;
import de.uka.ilkd.key.java.declaration.ClassDeclaration;
import de.uka.ilkd.key.java.reference.ExecutionContext;
import de.uka.ilkd.key.java.reference.SpecialConstructorReference;
import de.uka.ilkd.key.java.reference.SuperConstructorReference;
import de.uka.ilkd.key.java.reference.TypeRef;
import de.uka.ilkd.key.java.expression.operator.New;
import de.uka.ilkd.key.rule.inst.SVInstantiations;

/**
 * The constructor call meta construct is used to handle a allocation expression
 * like <code>new Class(...)</code>. Thereby it replaces the allocation 
 * expression by a method reference to an implict method called
 * <code>&lt;init&gt;</code> that is mainly the constructor but in its
 * normalform.
 */
public class SpecialConstructorCall extends ConstructorBodyNormalform {

    /** 
     * creates the metaconstruct
     */
    public SpecialConstructorCall(ProgramElement consRef) {
	super(new Name("special-constructor-call"), consRef);
    }

    /** 
     * transforms the constructor's body into its normalform and
     * returns the result. Thereby all necessary references are
     * resolved.     
     */
    public ProgramElement symbolicExecution
	(ProgramElement pe, JavaInfo javaInfo, SVInstantiations svInst) {

	SpecialConstructorReference constructorReference = 
	    (SpecialConstructorReference) pe;

	ExecutionContext executionContext = 
	    constructorReference.getExecutionContext();

	if (constructorReference instanceof SuperConstructorReference) {
	    KeYJavaType superType = javaInfo.getSuperclass
		(executionContext.getTypeReference().getKeYJavaType());
	    if (!(superType.getJavaType() instanceof ClassDeclaration)) { 
		// %%% testing Throwable is a bad HACK for 9.9. REMOVE
		// no implementation available
		return pe;
	    } else {
		executionContext = 
		    new ExecutionContext(new TypeRef(superType),
					 executionContext.getRuntimeInstance());
	    }
	}

	ProgramElement methodReference = createConstructorReference
	    (constructorReference.getArguments(), executionContext, javaInfo);
	
	if (methodReference == null) {
	    return pe;
	} else {
	    return methodReference;
	}
	
    }

}
