// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.rule.metaconstruct;

import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.expression.literal.*;
import de.uka.ilkd.key.logic.op.AbstractMetaOperator;
import de.uka.ilkd.key.logic.op.InstanceofSymbol;
import de.uka.ilkd.key.logic.op.Function;
import de.uka.ilkd.key.logic.op.ProgramArray;
import de.uka.ilkd.key.logic.TermFactory;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.rule.inst.SVInstantiations;
import de.uka.ilkd.key.logic.Name;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;
import de.uka.ilkd.key.logic.op.ProgramVariable;
import de.uka.ilkd.key.logic.sort.Sort;
import de.uka.ilkd.key.logic.op.Equality;
import de.uka.ilkd.key.logic.op.Op;
import de.uka.ilkd.key.logic.op.Operator;

public class StaticAnalysis extends AbstractMetaOperator {


    public StaticAnalysis() {
	super(new Name("#staticanalysis"), 1);
    }


    /**
     * checks whether the top level structure of the given @link Term
     * is syntactically valid, given the assumption that the top level
     * operator of the term is the same as this Operator. The
     * assumption that the top level operator and the term are equal
     * is NOT checked.  
     * @return true iff the top level structure of
     * the @link Term is valid.
     */
    public boolean validTopLevel(Term term) {
	// a meta operator accepts almost everything
	return  term.arity()==arity();
    }

    private Boolean calcInstanceof(Term term, SVInstantiations svInst, 
				   JavaInfo javaInfo) {
	Term instOfTerm = term.sub(0);
	Sort depSort = (Sort) ((InstanceofSymbol)instOfTerm.op())
	    .getSortDependingOn();
	KeYJavaType type = javaInfo.getKeYJavaType(depSort);		

	KeYJavaType objType = javaInfo.getTypeConverter()
		.getKeYJavaType(instOfTerm.sub(0));
	
	if (type==javaInfo.getNullType()) { //only null is instance of Null
	    return new Boolean(objType==javaInfo.getNullType());
	}
	if (objType==javaInfo.getNullType()) { // in Java, null is no 
	    // instance of anything
	    return new Boolean(false);
	} else if (javaInfo.isSubtype(objType, type)) {
	    return new Boolean(true);	    
	} else if (!(javaInfo.isSubtype(type, objType))){
	    return new Boolean(false);		
	} else if ((instOfTerm.sub(0).op() instanceof Function) // constant
		   && (!javaInfo.isSubtype(objType, type))) {
	    return new Boolean(false);
	} else { // no decision
	    return null;
	}

    }


    /** calculates the resulting term. */
    public Term calculate(Term term, SVInstantiations svInst, JavaInfo javaInfo) 
	throws NumberFormatException {
	Operator trueLitOp = javaInfo.getTypeConverter()
			.convertToLogicElement(BooleanLiteral.TRUE).op();
	if (term.sub(0).op() instanceof Equality &&
	    term.sub(0).sub(0).op() instanceof InstanceofSymbol &&
	    term.sub(0).sub(1).op()==trueLitOp) {

	    Boolean res=calcInstanceof(term.sub(0), svInst, javaInfo);
	    if  (res==null){ // no decision
		return term.sub(0);
	    } else if (res.booleanValue()) {
		Term argTerm = term.sub(0).sub(0).sub(0);
		Term nulleq = TermFactory.DEFAULT.createEqualityTerm
		    (argTerm, javaInfo.getNullTerm());
		return TermFactory.DEFAULT.createJunctorTerm(Op.NOT, nulleq);

	    } else {
		return TermFactory.DEFAULT.createJunctorTerm(Op.FALSE);
	    }	    
	}
	return null;
    }

}
