// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.rule.metaconstruct;
import de.uka.ilkd.key.java.Recoder2KeY;
import de.uka.ilkd.key.java.ProgramElement;
import de.uka.ilkd.key.java.Expression;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.abstraction.Type;
import de.uka.ilkd.key.java.abstraction.ArrayType;
import de.uka.ilkd.key.java.abstraction.PrimitiveType;
import de.uka.ilkd.key.java.reference.*;
import de.uka.ilkd.key.java.expression.literal.*;
import de.uka.ilkd.key.java.expression.operator.Instanceof;
import de.uka.ilkd.key.java.expression.operator.New;
import de.uka.ilkd.key.logic.op.ProgramVariable;
import de.uka.ilkd.key.casetool.together.TgFacade;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.java.abstraction.KeYJavaType;
import de.uka.ilkd.key.rule.inst.SVInstantiations;

public class TypeOf extends ProgramMetaConstruct {
    
     /** creates a typeof ProgramMetaConstruct 
     * @param expr the instance of expression contained by 
     * the meta construct 
     */
    public TypeOf(Expression expr) {
	super("#typeof", expr); 
	
    }

    /** performs the program transformation needed for symbolic
     * program transformation 
     * @return the transformated program
     */
    public ProgramElement symbolicExecution(ProgramElement pe,
					    JavaInfo javaInfo,
					    SVInstantiations insts) {
	KeYJavaType kjt;
	kjt = javaInfo.getTypeConverter()
	    .getKeYJavaType((Expression)pe);
	if (!(kjt.getJavaType() instanceof PrimitiveType)) {
	    if (kjt.getJavaType() instanceof ArrayType) {
		return new TypeRef(kjt, ((ArrayType)kjt.
					 getJavaType()).getDimension());
	    }
	}
	if (kjt == null) {
		throw new IllegalArgumentException
		    ("TypeOf could not handle "+pe);
	}

	return new TypeRef(kjt);
    }
}
