// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//
package de.uka.ilkd.key.rule.metaconstruct.arith;

import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.java.expression.literal.*;
import java.math.BigInteger;
import de.uka.ilkd.key.logic.op.AbstractMetaOperator;
import de.uka.ilkd.key.logic.TermFactory;
import de.uka.ilkd.key.logic.Term;
import de.uka.ilkd.key.rule.inst.SVInstantiations;
import de.uka.ilkd.key.logic.Name;
/** this class implements the interface for
 * MetaAdderators. MetaAdderators are used to do complex term
 * transformation when applying a taclet. Often these transformation
 * caanot be described with the taclet scheme (or trying to do so would
 * result in a huge number of rules)
 */
public class MetaJavaLongUnsignedShiftRight extends AbstractMetaOperator {

    private int arity;

    private TermFactory termFactory = TermFactory.DEFAULT;

    public MetaJavaLongUnsignedShiftRight() {
	super(new Name("#JavaLongUnsignedShiftRight"), 2);
    }


    /**
     * checks whether the top level structure of the given @link Term
     * is syntactically valid, given the assumption that the top level
     * operator of the term is the same as this Operator. The
     * assumption that the top level operator and the term are equal
     * is NOT checked.  
     * @return true iff the top level structure of
     * the @link Term is valid.
     */
    public boolean validTopLevel(Term term) {
	// a meta operator accepts almost everything
	return term.op() instanceof MetaJavaLongUnsignedShiftRight && term.arity()==arity();
    }


    /** calculates the resulting term. */
    public Term calculate(Term term, SVInstantiations svInst, JavaInfo javaInfo) {
	Term arg1 = term.sub(0);
	Term arg2 = term.sub(1);
	BigInteger intArg1=null;
	BigInteger intArg2=null;

	intArg1 = new
	    BigInteger(convertToDecimalString(arg1, javaInfo));
	intArg2 = new
	    BigInteger(convertToDecimalString(arg2, javaInfo));
	Long longResult = new Long(intArg1.longValue() >>> intArg2.longValue());
	
	IntLiteral lit = new IntLiteral(longResult.toString());
	return javaInfo.getTypeConverter().convertToLogicElement(lit);

    }

}
