// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.rule.strategy;

import de.uka.ilkd.key.rule.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.proof.Proof;
import de.uka.ilkd.key.proof.Node;
import de.uka.ilkd.key.proof.Goal;
import de.uka.ilkd.key.proof.ListOfGoal;
import de.uka.ilkd.key.proof.IteratorOfGoal;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.util.Debug;


/**
 * Strategy to apply simplifier rules as soon as possible
 */
public class SimplifierStrategy extends TrackChangesStrategy {

    ListOfHeuristic heur              = SLListOfHeuristic.EMPTY_LIST;


    public SimplifierStrategy ( Namespace p_heur_ns ) {
	Named h = p_heur_ns.lookup ( new Name ( "simplify_updates" ) );

	if ( h != null ) {
	    heur = heur.prepend ( (Heuristic)h );
	} else
	    Debug.out ( "Didn't find the heuristic \"simplify_updates\" I am part of" );
    }


    /**
     * @return Iterator of heuristics that include this strategy
     */
    public IteratorOfHeuristic heuristics () {
	return heur.iterator ();
    }


    /**
     * @param p_priority Number of strategies for which
     * "recommendRuleApp" has already been called without result
     * @param p_heuristics List of heuristics chosen by the user
     * @param p_goals List of goals a possible rule application should
     * be searched for
     * @return The RuleApp that should be executed next (this strategy
     * thinks), or null
     */
    public RuleAppRecommendation recommendRuleApp ( int             p_priority,
						    ListOfHeuristic p_heuristics,
						    ListOfGoal      p_goals,
						    JavaInfo        p_javaInfo,
						    Constraint      p_userConstraint ) {
	IteratorOfGoal           it = p_goals.iterator ();
	Goal                     goal;
	Node                     node;
	PosInOccurrence          pos;
	UpdateSimplificationRule rule;

	while ( it.hasNext () ) {
	    goal = it.next ();
	    node = goal.node ();

	    rule = new UpdateSimplificationRule ();

	    while ( ( pos = topNewFormula ( node ) ) != null ) {
		if ( rule.isApplicable ( goal, pos, p_userConstraint ) ) {
		    BuiltInRuleApp b = new BuiltInRuleApp ( rule,
							    pos,
							    p_userConstraint );
		    return new RuleAppRecommendation ( goal, b );		    
		}
		popNewFormula ( node );
	    }
	}

	return null;
    }
}
