// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.rule.strategy;

import de.uka.ilkd.key.rule.*;
import de.uka.ilkd.key.logic.*;
import de.uka.ilkd.key.proof.Node;
import de.uka.ilkd.key.proof.Goal;
import de.uka.ilkd.key.proof.Proof;
import de.uka.ilkd.key.proof.ListOfGoal;
import de.uka.ilkd.key.proof.IteratorOfGoal;
import de.uka.ilkd.key.java.JavaInfo;
import de.uka.ilkd.key.util.Debug;
import de.uka.ilkd.key.rule.strategy.proofevent.*;


/**
 * Strategy to apply any matching taclet
 */
public class StdStrategy extends MarkNodesStrategy {

    ListOfHeuristic heur                 = SLListOfHeuristic.EMPTY_LIST;

    ListOfHeuristic activeHeuristics     = SLListOfHeuristic.EMPTY_LIST;


    public StdStrategy ( Namespace p_heur_ns ) {}


    /**
     * StdStrategy is always enabled
     * @return Iterator of heuristics that include this strategy
     */
    public IteratorOfHeuristic heuristics () {
	return heur.iterator ();
    }


    /**
     * @param p_priority Number of strategies for which
     * "recommendRuleApp" has already been called without result
     * @param p_heuristics List of heuristics chosen by the user
     * @param p_goals List of goals a possible rule application should
     * be searched for
     * @return The RuleApp that should be executed next (this strategy
     * thinks), or null
     */
    public RuleAppRecommendation recommendRuleApp ( int             p_priority,
						    ListOfHeuristic p_heuristics,
						    ListOfGoal      p_goals,
						    JavaInfo        p_javaInfo,
						    Constraint      p_userConstraint ) {
	IteratorOfGoal               it = p_goals.iterator ();
	IteratorOfConstrainedFormula it2;
	Goal                         goal;
	Node                         node;
	ConstrainedFormula           cf;
	Sequent                      seq;
	IteratorOfTacletApp          appIt;
	PosInOccurrence              pos;

	while ( it.hasNext () ) {
	    goal = it.next ();
	    node = goal.node ();

	    if ( !isMarkedNode ( node ) ) {
		seq  = node.sequent ();

		it2  = seq.iterator ();
		while ( it2.hasNext () ) {
		    cf    = it2.next ();
		    pos   = new PosInOccurrence ( cf,
						  PosInTerm.TOP_LEVEL,
						  seq );

		    appIt = findTacletApps ( activeHeuristics,
					     goal, pos, true, true,
					     p_javaInfo, p_userConstraint );
		    if ( appIt.hasNext () )
			return new RuleAppRecommendation ( goal, appIt.next () );
		}

		markNode ( node );
	    }
	    
	}

	return null;
    }


    /**
     * Settings like the set of active heuristics or the user
     * constraint may have changed
     */
    public void settingsChange ( ListOfHeuristic p_heuristics,
				 Proof           p_proof,
				 JavaInfo        p_javaInfo,
				 Constraint      p_userConstraint ) {

	// HACK: remove heuristics of strategies
	activeHeuristics           = SLListOfHeuristic.EMPTY_LIST;
	IteratorOfHeuristic heurIt = p_heuristics.iterator ();
	while ( heurIt.hasNext () ) {
	    Heuristic h = heurIt.next ();
	    if ( !h.name ().toString ().equals ( "closure" ) &&
		 !h.name ().toString ().equals ( "replace_known" ) &&
		 !h.name ().toString ().equals ( "beta" ) &&
		 !h.name ().toString ().equals ( "alpha_concrete" ) &&
		 !h.name ().toString ().equals ( "gamma" ) &&
		 !h.name ().toString ().equals ( "gamma_destructive" ) )
		activeHeuristics = activeHeuristics.prepend ( h );
	}

	super.settingsChange ( p_heuristics,
			       p_proof,
			       p_javaInfo,
			       p_userConstraint );
    }

}
