// This file is part of KeY - Integrated Deductive Software Design
// Copyright (C) 2001-2003 Universitaet Karlsruhe, Germany
//                         and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General Public License. 
// See LICENSE.TXT for details.
//

package de.uka.ilkd.key.rule.strategy;

import de.uka.ilkd.key.rule.Heuristic;
import de.uka.ilkd.key.rule.ListOfHeuristic;
import de.uka.ilkd.key.rule.IteratorOfHeuristic;
import de.uka.ilkd.key.rule.RuleApp;
import de.uka.ilkd.key.proof.ListOfGoal;
import de.uka.ilkd.key.proof.ListOfNode;
import de.uka.ilkd.key.proof.Node;
import de.uka.ilkd.key.proof.Proof;
import de.uka.ilkd.key.rule.strategy.proofevent.RuleAppInfo;
import de.uka.ilkd.key.logic.Constraint;
import de.uka.ilkd.key.logic.SequentChangeInfo;
import de.uka.ilkd.key.java.JavaInfo;


/**
 * Interface representing knowledge source for taclet applications
 * following some strategy
 */
public interface Strategy {

    /**
     * @return Iterator of heuristics that include this strategy
     */
    public IteratorOfHeuristic   heuristics       ();

    /**
     * @param p_priority Number of strategies for which
     * "recommendRuleApp" has already been called without result
     * @param p_heuristics List of heuristics chosen by the user
     * @param p_goals List of goals a possible rule application should
     * be searched for
     * @return The RuleApp that should be executed next (this strategy
     * thinks), or null
     */
    public RuleAppRecommendation recommendRuleApp ( int             p_priority,
						    ListOfHeuristic p_heuristics,
						    ListOfGoal      p_goals,
						    JavaInfo        p_javaInfo,
						    Constraint      p_userConstraint );

    /**
     * A new node has been added to the proof (and we don't know how
     * exactly)
     */
    public void                  nodeAdded        ( Node        p_node );

    /**
     * Tell the strategy object that a node has been modified
     */
    public void                  nodeChanged      ( Node              p_node,
						    SequentChangeInfo p_sci );

    /**
     * Tell the strategy object that the proof has been expanded,
     * adding the nodes <code>p_newLeaves</code> as new leaves below
     * <code>p_oldLeaf</code>
     */
    public void                  proofExpanded    ( Node       p_oldLeaf,
						    ListOfNode p_newLeaves );

    /**
     * Tell the strategy object that a rule has been applied
     */
    public void                  ruleApplied      ( RuleAppInfo       p_rai );

    /**
     * Settings like the set of active heuristics or the user
     * constraint may have changed
     */
    public void                  settingsChange   ( ListOfHeuristic p_heuristics,
						    Proof           p_proof,
						    JavaInfo        p_javaInfo,
						    Constraint      p_userConstraint );

}
