include
	prelude.gf;
	syntax.Eng.gf;
        core.Abs.gf ;
	oclLibrary.Abs.gf ;
--	res.Eng.gf;

--
-- general NL
--

-- the gentive form here is perhaps not obvious?
oper conjoinNP : Str -> (n1,n2 : NounPhrase) -> NounPhrase = \and, n1, n2 -> 
		{s = \\c => n1.s!c ++ and ++ n2.s!c; n = Pl; p = P3};
		
	appFunSg : Function -> NounPhrase -> NounPhrase = \mother,john -> 
	    {s = table {
		GenP => nonExist ;
		_  => artDef ++ mother.s ! Sg ! Nom ++ mother.s2 ++ john.s ! GenSP };
     n = Sg ;
     p = P3
    } ;
    
  funX : CommNoun -> Str -> Function = \mother,prep -> 
    {s = \\n => mother.s ! n  ; s2 = prep} ;
  funXReg : Str -> Str -> Function = \mother,prep -> funX (nounReg mother) prep;

  -- for handling ind.valued fuctions with "postfix notation" (e.g. the string converted to ...)
  postfixIndFun : Str -> NounPhrase -> NounPhrase = \f, np ->  { s = \\c =>
		np.s ! c ++ f;
		n=np.n;
		p = np.p
  };
		
  constant : Str -> NounPhrase = \c -> nameNounPhrase (nameReg c);
  numeral : Str -> NounPhrase = constant;



--
-- Classes
--

lintype ClassL  = CommNoun ** {s42 : Str}; -- let's hope CN has less than 42 "s"-field
lincat Class = ClassL;
oper class2id : ClassL -> Str = \c -> c.s42;
	class2cn: ClassL -> CommNoun = \c -> {s = c.s; g = c.g};

oper mkClassNoHum : Str -> Str -> ClassL = \noun, classId -> 
		(cnNoHum (nounReg noun)) ** {s42 = classId};
	mkClassHum : Str -> Str -> ClassL = \noun, classId -> 
		(cnHum (nounReg noun)) ** {s42 = classId};
	mkClass = mkClassNoHum;

--
-- Subtyping 
--
lin coerce sub super proof obj = obj;
    subTypeTrans a b c pab pbc= {s = ["every"] ++ a.s!Sg!Nom ++ ["is a"] ++ c.s!Sg!Nom ++ "," 
    	++ "since" ++ pab.s ++ "and" ++ pbc.s}; 
   subTypeRefl a = {s = "a" ++ a.s!Sg!Nom ++ ["is a"] ++ a.s!Sg!Nom ++ ["is a"] ++ a.s!Sg!Nom};  

printname coerce = "[coercion]";



--
-- Instances
--

lintype Inst = NounPhrase; 
lincat Instance = Inst;
oper test : Str -> Inst = \t -> {s = \\c => t; n=Sg; p=P3};
lindef Instance = test; -- \t -> {s = \\c => t; n=Sg; p=P3};
oper 
	np2inst : NounPhrase -> Inst = \np -> np;    
	inst2np : Inst -> NounPhrase = \inst -> inst;

-- should we have "True" as the first argument to predAdjective?
oper equalTo : Inst -> Inst -> Sentence = \a,b ->
	(predVerbPhrase a (predAdjective True (complAdj {s="equal"; s2="to"} b)));
     notEqualTo : Inst -> Inst -> Sentence = \a,b ->
	(predVerbPhrase a (predAdjective False (complAdj {s=["equal"]; s2="to"} b)));
    littleAdj : AdjDegr = mkAdjDegr "little" "less" "least";
    greatAdj : AdjDegr = adjDegrReg "great";
     lessThan : Inst -> Inst -> Sentence = \a,b -> -- a is less than b
	      predVerbPhrase a (predAdjective True (comparAdjPhrase littleAdj b));
     greaterThan : Inst -> Inst -> Sentence = \a, b -> -- a is greater than b
	      predVerbPhrase a (predAdjective True (comparAdjPhrase greatAdj b));
     
     instFunOf : Str -> Inst -> Inst = \size, object ->
	       np2inst (appFunSg (funOfReg size) (inst2np object));
     instInfix : Str -> Inst -> Inst -> Inst = \plus, a, b ->
	       np2inst (conjoinNP plus (inst2np a) (inst2np b));	       
     instPrefix : Str -> Inst -> Inst = \not, a -> np2inst { 
		s = \\c => not ++ (inst2np a).s!c;
		n = (inst2np a).n;
		p = (inst2np a).p};
	       
    conjoinInst : Str -> Inst -> Inst -> Inst = \and, a, b ->
		np2inst (conjoinNP and (inst2np a) (inst2np b));
	      
	


lin atPre _ i = {s = \\c => i.s!c ++ ["at the start of the operation"]; n=i.n; p=i.p};

--
-- Attributes / queries (Features)
--

oper mkAttr : Inst -> Str -> Inst = \obj, attr ->
	np2inst(appFunSg (funOfReg attr) obj);

     -- handle attributes like "isMarried"
     mkIsAttr : Inst -> Str -> Sentence = \obj, married ->
	       (predVerbPhrase (inst2np obj) (predAdjective True (ss married)));
     -- arbitrary boolean-valued attributes
     -- the property prop holds for obj
     mkSentAttr : Inst -> Str -> Sentence = \obj, prop ->
		(predVerbPhrase (constant (["the property"] ++ prop))
		(complTransVerb True (mkTransVerb (regVerbP3 "hold") "for") obj));

-- build list of arguments/parameters to queries:	
lintype  ArgList = {s : Str; empty : Bool};
oper argNil : ArgList = {s = ""; empty = True};
	argCons : Inst -> ArgList -> ArgList = 
		\x, xs -> {s = case xs.empty of {
			True => (inst2np x).s!NomP;
			False => xs.s ++ "," ++ (inst2np x).s!NomP
		}; empty = False};

	mkQuery : Inst -> Str -> ArgList -> Inst = \obj, query, args -> 
		np2inst	(appFunSg (funOfReg (query ++ "(" ++ args.s ++ ")")) obj);
     -- handle queries like "isMarried()"
     mkIsQuery : Inst -> Str -> Sentence = \obj, married ->
	       (predVerbPhrase (inst2np obj) (predAdjective True (ss married)));
     -- arbitrary boolean-valued queries
     -- the property prop(x,y,z) holds for obj
     mkSentQuery : Inst -> Str -> ArgList -> Sentence = \obj, query, args ->
		(predVerbPhrase (constant (["the property"] ++ query ++ "(" ++ args.s ++ ")"))
		(complTransVerb True (mkTransVerb (regVerbP3 "hold") "for") obj));

-- associations
   mkAssoc : Str -> Inst -> Inst = \thing, obj -> -- the set of the things associated with obj
	   postfixIndFun (["associated with"] ++ (inst2np obj).s!AccP)
	    (np2inst ((appFunSg (funOfReg "set") (defNounPhrase Pl (nounReg thing)))));
			

--
-- Sentences 
--
lincat Sent = Sentence;

oper s2sent : Str -> Sentence =  \str -> {s = str};
lin	bool2sent b = equalTo b (constant "true");
	sent2bool s = (np2inst (appFunSg (funOfReg ["truth value of the condition that"]) 
		(constant s.s)));

printname sent2bool = "[use a sentence as a boolean]";
	  bool2sent = "[use a boolean as a sentence]";


lin atPreSent s = s2sent (s.s ++ ["at the start of the operation"]);

--
-- self & result
--
lin self c _ = np2inst (detNounPhrase theSgDet c);
lin result _ _ = np2inst (detNounPhrase theSgDet (nounReg "result"));


--
-- pre- and postconditions
--
oper beginBulletList : Str = ""; -- "\\begin{itemize}";
	 bullet : Str = "(*)"; --"\\item";
	endBulletList : Str = ""; --"\\end{itemize}";
	wrapBullets : Str -> Str = \str -> beginBulletList ++ str ++ endBulletList;

param ListSize = LSempty | LSone | LSmany;
oper LSsucc : ListSize => ListSize = table {LSempty => LSone; LSone => LSmany; LSmany => LSmany};
	LS2Bool : ListSize => Bool = table {LSempty => False; _ => True};


lintype ConstraintBodyL = {s0, s1, s2 : Str; invC, preC, postC : ListSize};
lincat ConstraintBody = ConstraintBodyL;   -- one string for inv, pre and post, respectively


lin
	emptyC = {s0,s1,s2 = ""; invC, preC, postC = LSempty};
	invC i cs = {s0 = bullet ++ i.s ++ cs.s0; invC = LSsucc ! cs.invC;
		s1 = cs.s1; preC = cs.preC;
		s2 = cs.s2; postC = cs.postC};
	preC p cs = {s0 = cs.s0 ; 
		s1 = bullet ++ p.s ++ cs.s1; 
		s2 = cs.s2; invC = cs.invC; preC = LSsucc ! cs.preC; postC = cs.postC};
	postC p cs = {s0 = cs.s0; s1 = cs.s1; invC = cs.invC; preC = cs.preC;
		s2 = bullet ++ p.s ++ cs.s2; postC = LSsucc ! cs.postC};

	invHC s = s;

oper 
	LSand1 : (l1,l2,l3 : ListSize) -> Str = \l1, l2, l3 -> -- "and" between inv and pre?
		LSand1' (LS2Bool ! l1) (LS2Bool ! l2) (LS2Bool ! l3);
	LSand1' : (b1,b2,b2 : Bool) -> Str = \b1, b2, b3 ->
		if_then_else Str (orB (andB b1 b2) (andB b1 b3)) "and" "";
	LSand2 : (l2,l3 : ListSize) -> Str = \l2, l3 ->    -- "then" between pre and post?
		LSand2' (LS2Bool ! l2) (LS2Bool ! l3);
	LSand2' : (b2,b3 : Bool) -> Str = \b2, b3 ->
		if_then_else Str (andB b2 b3) "then" "";

oper cb2str : ConstraintBodyL -> Str = 
	\cb -> (case cb.invC of {
		 LSempty => "";
		 LSone => ["this invariant should hold :"] ++ wrapBullets cb.s0;
		 LSmany => ["these invariants should hold :"] ++ wrapBullets cb.s0
	}) ++ LSand1 cb.invC cb.preC cb.postC ++ 
	(case cb.preC of {
		LSempty => "";
		LSone => ["given this precondition :"] ++ wrapBullets cb.s1;
		LSmany => ["given these preconditions :"] ++ wrapBullets cb.s1
	}) ++ LSand2 cb.preC cb.postC ++ 
	(case cb.postC of {
		LSempty => "";
		LSone => ["this postcondition should hold :"] ++ wrapBullets cb.s2;
		LSmany => ["these postconditions should hold : "] ++ wrapBullets cb.s2
	});


	strip2varCB : {v0 ,v1 : Str} ** ConstraintBodyL  -> ConstraintBodyL = \cb -> 
		{s0=cb.s0; s1=cb.s1; s2=cb.s2; invC=cb.invC; preC=cb.preC; postC=cb.postC};

--
-- constraint
--

lintype ConstraintL = {s:Str};
lincat Constraint = ConstraintL;

oper contextOp00 : Str -> Str -> ConstraintBodyL -> ConstraintL = \class, op, body ->
	ss (["for the operation"] ++ op ++ ["of the class"] ++ class ++ ["we have that"] ++ cb2str body);
	contextOp10 : (class, op, args : Str) -> ConstraintBodyL -> ConstraintL =
		\class, op, args, body -> ss (["for the operation"] ++ op ++ "(" ++ args ++ ")" ++ 
			["of the class"] ++ class ++ ["we have that"] ++ cb2str body);
	contextOp01 : (class, op,ret : Str) -> ConstraintBodyL -> ConstraintL = 
		\class, op, ret, body -> ss (["for the operation"] ++ op
		++ "()" ++ ":" ++ ret  ++ ["of the class"] ++ class ++ ["we have that"] ++ cb2str body);
	contextOp11 : (class, op, args, ret : Str) -> ConstraintBodyL -> ConstraintL = 
		\class, op, args, ret, body -> ss (["for the operation"] ++ op
		++ "(" ++ args ++ ")" ++ ":" ++ ret  ++ ["of the class"] ++ class ++ ["we have that"] ++ cb2str body);

	contextClass : Str -> ConstraintBodyL -> ConstraintL = 
		\class, body -> ss (["for the class"] ++ class ++ ["we have that"] ++ cb2str body);		


--
-- let expressions
--

lincat LetExp = Inst ** {s10 : Str; listSize : ENumber}; -- Inst + the list of bindings

lin letExp2Inst _ le = np2inst {
	s = \\c => "define" ++ le.s10 ++ "in" ++ le.s!c;
	n = le.n; p = le.p
	};

lin letSingleAttr _ _ defn body = {
		s = body.s; n = body.n; p = body.p; 
		s10 = body.v ++ "as" ++ defn.s ! NomP;
		listSize = E1};	
	letConsAttr _ _ defn body = {
		s = body.s; n = body.n; p = body.p; 
		s10 = case body.listSize of {
			E1 => body.v ++ "as" ++ defn.s!NomP ++ "and" ++ body.s10;
			_ => body.v ++ "as" ++ defn.s!NomP ++ "," ++ body.s10
		};
		listSize = eNext body.listSize
	};
	
lincat LetExpSent = Sentence ** {s10 : Str; listSize : ENumber}; -- Sent + the list of bindings

lin letExp2Sent le = s2sent ("define" ++ le.s10 ++ "in" ++ le.s);

lin letSingleAttrS _ defn body = {
		s = body.s; 
		s10 = body.v ++ "as" ++ defn.s ! NomP;
		listSize = E1};	
	letConsAttrS _ defn body = {
		s = body.s; 
		s10 = case body.listSize of {
			E1 => body.v ++ "as" ++ defn.s!NomP ++ "and" ++ body.s10;
			_ => body.v ++ "as" ++ defn.s!NomP ++ "," ++ body.s10
		};
		listSize = eNext body.listSize
	};


