// This file is part of KeY - Integrated Deductive Software Design 
// Copyright (C) 2001 Universitaet Karlsruhe, Germany
//                    and Chalmers University of Technology, Sweden          
//
// The KeY system is protected by the GNU General 
// Public License. See LICENSE.TXT for details.
//
package tudresden.ocl.check;

import tudresden.ocl.parser.analysis.*;
import tudresden.ocl.parser.node.*;
import tudresden.ocl.check.types.*;
import tudresden.ocl.*;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.ArrayList;
import java.util.List;

import tudresden.ocl.parser.analysis.*;
import tudresden.ocl.parser.node.*;
import tudresden.ocl.check.types.*;
import tudresden.ocl.check.*;
import tudresden.ocl.*;

import de.uka.ilkd.key.ocl.ocldltranslator.*;
import de.uka.ilkd.key.ocl.ocldltranslator.heuristics.*;
import de.uka.ilkd.key.ocl.ocldltranslator.control.*;
import de.uka.ilkd.key.ocl.ocldltranslator.normalize.*;
import de.uka.ilkd.key.ocl.ocldltranslator.transform.*;

/**
 * A modified type checker that is used for the KeY system.
 * This type checker class removes some 'bugs' in the TU Dresden type checker.
 */
public class KeyTypeChecker extends tudresden.ocl.check.TypeChecker {
    public final static String KEY_PREFIX_LOWERCASE= "keyprefix_";
    public final static String KEY_PREFIX_UPPERCASE= "Keyprefix_";

    public KeyTypeChecker(OclTree tree, TypeFactory tf) {
	super(tree,tf);
	// First remove possible KeY-specific prefixes for name tokens ...
	KeyPrefixDeletion kpd = new KeyPrefixDeletion();
        tree.apply(kpd);
    }
 
    public void outALiteralCollection(ALiteralCollection lc) {
	// First do the old stuff ...
	super.outALiteralCollection(lc);
	// Now correct the result if necessary ...
	if (lc.getExpressionListOrRange()==null) {
	
	// There are problems with the the types for literals for empty collections.
	 // Here is a workaround for iterate ...
	     
	     Node currentNode = lc.parent();
	     boolean iterateFound = false;
	     AIterateDeclarator itDecl = null;
	     while(currentNode != null && !iterateFound){
		 iterateFound = (currentNode instanceof AIterateDeclarator);
		 if (iterateFound) { 
		     itDecl = (AIterateDeclarator) currentNode;
		 }
		 currentNode = currentNode.parent();
	     }
	     if (iterateFound) {
		 // Use the type of the declaration of the accumulator variable instead ...
		 ADeclaratorTypeDeclaration accTypeDecl = (ADeclaratorTypeDeclaration) itDecl.getAccType();
		 APathSimpleTypeSpecifier psts = (APathSimpleTypeSpecifier) accTypeDecl.getSimpleTypeSpecifier();
		 OclType oclTypeObj = (OclType) ntm.get(psts);
		 Type accType =  oclTypeObj.getType();
		 ntm.put(lc, accType);
		 // System.err.println("KEY-TYPE-CHECKER -----------> Changed the type of an empty collection literal to :"
				//     +accType.toString());

	     }
	 
	 }
	 
  }
    
    // Some inner class ...
    
    /**
     * A class the removes key specific prefixes used for the names of some tokens
     * in order to garantuee the possibility of parsing with the current OCL parser.
     * E.g. method names have to begin with lowercase letters for the parser, but in
     * UML they actually can start with uppercase letters also (Constructor methods !)
     */
    class KeyPrefixDeletion extends DepthFirstAdapter {
 
	public void caseTName(TName node) {
	    // Handle names that have to start with lowercase letters for the parser ...
	    
	    String text = node.getText();
	    if (text.startsWith(KEY_PREFIX_LOWERCASE)){
		String newText = text.substring(KEY_PREFIX_LOWERCASE.length());
		node.setText(newText);
		System.err.println("KEY-TYPE-CHECKER (KeyPrefixDeletion) "+ 
				   "-----------> Removed TName Key-Prefix "+
				   "and set new name to :" + newText);
	    }
	}
    }
 


}





